@@ -54,9 +54,10 @@ extern "C" {
 
 /* general return codes */
 #define SLURM_SUCCESS   0
+#define ESPANK_SUCCESS 0
 #define SLURM_ERROR    -1
 
-enum {
+typedef enum {
 	/* General Message error codes */
 	SLURM_UNEXPECTED_MSG_ERROR = 			1000,
 	SLURM_COMMUNICATIONS_CONNECTION_ERROR,
@@ -71,6 +72,7 @@ enum {
 	SLURM_MPI_PLUGIN_PRELAUNCH_SETUP_FAILED,
 	SLURM_PLUGIN_NAME_INVALID,
 	SLURM_UNKNOWN_FORWARD_ADDR,
+	SLURM_COMMUNICATIONS_MISSING_SOCKET_ERROR,
 
 	/* communication failures to/from slurmctld */
 	SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR =     1800,
@@ -212,6 +214,20 @@ enum {
 	ESLURM_RESERVATION_NO_SKIP,
 	ESLURM_RESERVATION_USER_GROUP,
 	ESLURM_PARTITION_ASSOC =			2130,
+	ESLURM_IN_STANDBY_USE_BACKUP,
+
+	/* SPANK errors */
+	ESPANK_ERROR = 					3000,
+	ESPANK_BAD_ARG,
+	ESPANK_NOT_TASK,
+	ESPANK_ENV_EXISTS,
+	ESPANK_ENV_NOEXIST,
+	ESPANK_NOSPACE,
+	ESPANK_NOT_REMOTE,
+	ESPANK_NOEXIST,
+	ESPANK_NOT_EXECD,
+	ESPANK_NOT_AVAIL,
+	ESPANK_NOT_LOCAL,
 
 	/* slurmd error codes */
 	ESLURMD_PIPE_ERROR_ON_TASK_SPAWN =		4000,
@@ -245,6 +261,7 @@ enum {
 	ESLURMD_STEP_SUSPENDED,
 	ESLURMD_STEP_NOTSUSPENDED,
 	ESLURMD_INVALID_SOCKET_NAME_LEN =		4030,
+	ESLURMD_CONTAINER_RUNTIME_INVALID,
 
 	/* slurmd errors in user batch job */
 	ESCRIPT_CHDIR_FAILED =			4100,
@@ -293,6 +310,8 @@ enum {
 	/* plugin and custom errors */
 	ESLURM_MISSING_TIME_LIMIT       = 8000,
 	ESLURM_INVALID_KNL,
+	ESLURM_PLUGIN_INVALID,
+	ESLURM_PLUGIN_INCOMPLETE,
 
 	/* REST errors */
 	ESLURM_REST_INVALID_QUERY = 9000,
@@ -305,7 +324,9 @@ enum {
 	ESLURM_DATA_PTR_NULL,
 	ESLURM_DATA_CONV_FAILED,
 	ESLURM_DATA_REGEX_COMPILE,
-};
+	ESLURM_DATA_UNKNOWN_MIME_TYPE,
+	ESLURM_DATA_TOO_LARGE,
+} slurm_err_t;
 
 /* look up an errno value */
 char * slurm_strerror(int errnum);
