@@ -387,11 +387,11 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 		force = p[i].max_share & SHARED_FORCE;
 		val = p[i].max_share & (~SHARED_FORCE);
 		if (val == 0)
-		        fprintf(fp, " Shared=EXCLUSIVE");
+		        fprintf(fp, " OverSubscribe=EXCLUSIVE");
 		else if (force) {
-		        fprintf(fp, " Shared=FORCE:%u", val);
+		        fprintf(fp, " OverSubscribe=FORCE:%u", val);
 		} else if (val != 1)
-		        fprintf(fp, " Shared=YES:%u", val);
+		        fprintf(fp, " OverSubscribe=YES:%u", val);
 
 		if (p[i].state_up == PARTITION_UP)
 	                fprintf(fp, " State=UP");
@@ -408,6 +408,24 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 			fprintf(fp, " TRESBillingWeights=%s",
 			        p[i].billing_weights_str);
 
+		if (p[i].resume_timeout == INFINITE16)
+	                fprintf(fp, " ResumeTimeout=INFINITE");
+		else if (p[i].resume_timeout != NO_VAL16)
+	                fprintf(fp, " ResumeTimeout=%d",
+				p[i].resume_timeout);
+
+		if (p[i].suspend_timeout == INFINITE16)
+	                fprintf(fp, " SuspendTimeout=INFINITE");
+		else if (p[i].suspend_timeout != NO_VAL16)
+	                fprintf(fp, " SuspendTimeout=%d",
+				p[i].suspend_timeout);
+
+		if (p[i].suspend_time == INFINITE)
+	                fprintf(fp, " SuspendTime=INFINITE");
+		else if (p[i].suspend_time != NO_VAL)
+	                fprintf(fp, " SuspendTime=%d",
+				p[i].suspend_time);
+
 		fprintf(fp, "\n");
 	}
 
@@ -489,7 +507,7 @@ void slurm_print_ctl_conf ( FILE* out,
 			      select_title);
 
 }
-extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
+extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 {
 	List ret_list = NULL;
 	config_key_pair_t *key_pair;
@@ -522,8 +540,14 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AccountingStorageLoc");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->accounting_storage_loc);
+	key_pair->name = xstrdup("AccountingStorageExternalHost");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->accounting_storage_ext_host);
+	list_append(ret_list, key_pair);
+
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("AccountingStorageParameters");
+	key_pair->value =
+		xstrdup(slurm_ctl_conf_ptr->accounting_storage_params);
 	list_append(ret_list, key_pair);
 
 	snprintf(tmp_str, sizeof(tmp_str), "%u",
@@ -549,9 +573,16 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AccountingStoreJobComment");
-	key_pair->value = xstrdup(
-		(slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJC) ? "Yes" : "No");
+	key_pair->name = xstrdup("AccountingStoreFlags");
+	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJC)
+		xstrfmtcat(key_pair->value, "%sjob_comment",
+			   key_pair->value ? "," : "");
+	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJE)
+		xstrfmtcat(key_pair->value, "%sjob_env",
+			   key_pair->value ? "," : "");
+	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJS)
+		xstrfmtcat(key_pair->value, "%sjob_script",
+			   key_pair->value ? "," : "");
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -595,6 +626,11 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->authalttypes);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("AuthAltParameters");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->authalt_params);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("AuthInfo");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->authinfo);
@@ -612,6 +648,16 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(*key_pair));
+	key_pair->name = xstrdup("BcastExclude");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->bcast_exclude);
+	list_append(ret_list, key_pair);
+
+	key_pair = xmalloc(sizeof(*key_pair));
+	key_pair->name = xstrdup("BcastParameters");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->bcast_parameters);
+	list_append(ret_list, key_pair);
+
 	slurm_make_time_str((time_t *)&slurm_ctl_conf_ptr->boot_time,
 			    tmp_str, sizeof(tmp_str));
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -803,12 +849,12 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	list_append(ret_list, key_pair);
 
 	if (slurm_ctl_conf_ptr->hash_val != NO_VAL) {
-		if (slurm_ctl_conf_ptr->hash_val == slurm_get_hash_val())
+		if (slurm_ctl_conf_ptr->hash_val == slurm_conf.hash_val)
 			snprintf(tmp_str, sizeof(tmp_str), "Match");
 		else {
 			snprintf(tmp_str, sizeof(tmp_str),
 				 "Different Ours=0x%x Slurmctld=0x%x",
-				 slurm_get_hash_val(),
+				 slurm_conf.hash_val,
 				 slurm_ctl_conf_ptr->hash_val);
 		}
 		key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -842,6 +888,11 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("InteractiveStepOptions");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->interactive_step_opts);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("JobAcctGatherFrequency");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->job_acct_gather_freq);
@@ -961,11 +1012,6 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->launch_type);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("Layouts");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->layouts);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("Licenses");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->licenses);
@@ -1096,11 +1142,6 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->mpi_params);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MsgAggregationParams");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->msg_aggr_params);
-	list_append(ret_list, key_pair);
-
 	if (cluster_flags & CLUSTER_FLAG_MULTSD) {
 		key_pair = xmalloc(sizeof(config_key_pair_t));
 		key_pair->name = xstrdup("MULTIPLE_SLURMD");
@@ -1428,16 +1469,6 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->route_plugin);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("SallocDefaultCommand");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->salloc_default_command);
-	list_append(ret_list, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("SbcastParameters");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->sbcast_parameters);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SchedulerParameters");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->sched_params);
@@ -1455,6 +1486,11 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->schedtype);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("ScronParameters");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->scron_params);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SelectType");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->select_type);
@@ -1695,11 +1731,11 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	if (slurm_ctl_conf_ptr->suspend_time == 0) {
-		snprintf(tmp_str, sizeof(tmp_str), "NONE");
+	if (slurm_ctl_conf_ptr->suspend_time == INFINITE) {
+		snprintf(tmp_str, sizeof(tmp_str), "INFINITE");
 	} else {
-		snprintf(tmp_str, sizeof(tmp_str), "%d sec",
-			 ((int)slurm_ctl_conf_ptr->suspend_time - 1));
+		snprintf(tmp_str, sizeof(tmp_str), "%u sec",
+			 slurm_ctl_conf_ptr->suspend_time);
 	}
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SuspendTime");
@@ -1717,6 +1753,11 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("SwitchParameters");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->switch_param);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SwitchType");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->switch_type);
@@ -1828,8 +1869,7 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
  * RET SLURM_SUCCESS on success, otherwise return SLURM_ERROR with errno set
  * NOTE: free the response using slurm_free_ctl_conf
  */
-int
-slurm_load_ctl_conf (time_t update_time, slurm_ctl_conf_t **confp)
+int slurm_load_ctl_conf(time_t update_time, slurm_conf_t **confp)
 {
 	int rc;
 	slurm_msg_t req_msg;
@@ -1890,8 +1930,7 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 					    req_msg.flags);
 		} else {
 			this_addr = "localhost";
-			slurm_set_addr(&req_msg.address,
-				       (uint16_t)slurm_get_slurmd_port(),
+			slurm_set_addr(&req_msg.address, slurm_conf.slurmd_port,
 				       this_addr);
 		}
 	} else {
@@ -1901,11 +1940,10 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 		 *  Set request message address to slurmd on localhost
 		 */
 		gethostname_short(this_host, sizeof(this_host));
-		this_addr = slurm_conf_get_nodeaddr(this_host);
+		this_addr = slurm_conf_get_nodeaddr(this_host, NULL);
 		if (this_addr == NULL)
 			this_addr = xstrdup("localhost");
-		slurm_set_addr(&req_msg.address,
-			       (uint16_t)slurm_get_slurmd_port(),
+		slurm_set_addr(&req_msg.address, slurm_conf.slurmd_port,
 			       this_addr);
 		xfree(this_addr);
 	}
@@ -1917,11 +1955,11 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 	if ((rc != 0) || !resp_msg.auth_cred) {
 		error("slurm_slurmd_info: %m");
 		if (resp_msg.auth_cred)
-			g_slurm_auth_destroy(resp_msg.auth_cred);
+			auth_g_destroy(resp_msg.auth_cred);
 		return SLURM_ERROR;
 	}
 	if (resp_msg.auth_cred)
-		g_slurm_auth_destroy(resp_msg.auth_cred);
+		auth_g_destroy(resp_msg.auth_cred);
 
 	switch (resp_msg.msg_type) {
 	case RESPONSE_SLURMD_STATUS:
@@ -2078,8 +2116,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 			      key_pair->name,
 			      key_pair->value);
 		} else {
-			if ((!xstrcasecmp(key_pair->name, "ChosLoc")) ||
-			    (!xstrcasecmp(key_pair->name, "Epilog")) ||
+			if ((!xstrcasecmp(key_pair->name, "Epilog")) ||
 			    (!xstrcasecmp(key_pair->name, "EpilogSlurmctld")) ||
 			    (!xstrcasecmp(key_pair->name,
 					  "HealthCheckProgram")) ||
@@ -2090,8 +2127,6 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 			    (!xstrcasecmp(key_pair->name, "ResumeProgram")) ||
 			    (!xstrcasecmp(key_pair->name, "ResvEpilog")) ||
 			    (!xstrcasecmp(key_pair->name, "ResvProlog")) ||
-			    (!xstrcasecmp(key_pair->name,
-					  "SallocDefaultCommand")) ||
 			    (!xstrcasecmp(key_pair->name, "SrunEpilog")) ||
 			    (!xstrcasecmp(key_pair->name, "SrunProlog")) ||
 			    (!xstrcasecmp(key_pair->name, "SuspendProgram")) ||
@@ -2146,11 +2181,10 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 		if (!xstrcasecmp(key_pair->name, "AccountingStorageBackupHost") ||
 		    !xstrcasecmp(key_pair->name, "AccountingStorageEnforce") ||
 		    !xstrcasecmp(key_pair->name, "AccountingStorageHost") ||
-		    !xstrcasecmp(key_pair->name, "AccountingStorageLoc") ||
 		    !xstrcasecmp(key_pair->name, "AccountingStoragePort") ||
 		    !xstrcasecmp(key_pair->name, "AccountingStorageType") ||
 		    !xstrcasecmp(key_pair->name, "AccountingStorageUser") ||
-		    !xstrcasecmp(key_pair->name, "AccountingStoreJobComment") ||
+		    !xstrcasecmp(key_pair->name, "AccountingStoreFlags") ||
 		    !xstrcasecmp(key_pair->name, "AcctGatherEnergyType") ||
 		    !xstrcasecmp(key_pair->name, "AcctGatherFilesystemType") ||
 		    !xstrcasecmp(key_pair->name, "AcctGatherInterconnectType") ||
