@@ -80,8 +80,7 @@
 /* check signature */
 #define _assert_bitstr_valid(name) do { \
 	xassert((name) != NULL); \
-	xassert(_bitstr_magic(name) == BITSTR_MAGIC \
-			    || _bitstr_magic(name) == BITSTR_MAGIC_STACK); \
+	xassert(_bitstr_magic(name) == BITSTR_MAGIC); \
 } while (0)
 
 /* check bit position */
@@ -97,15 +96,6 @@
 	xassert((bit) <= 0x40000000); 	\
 } while (0)
 
-/*
- * external macros
- */
-
-/* allocate a bitstring on the stack */
-/* XXX bit_decl does not check if nbits overflows word 1 */
-#define	bit_decl(name, nbits) \
-	(name)[_bitstr_words(nbits)] = { BITSTR_MAGIC_STACK, (nbits) }
-
 /*
  * Define slurm-specific aliases for use by plugins, see slurm_xlator.h
  * for details.
@@ -138,6 +128,7 @@ strong_alias(bit_fmt_full,	slurm_bit_fmt_full);
 strong_alias(bit_unfmt,		slurm_bit_unfmt);
 strong_alias(bitfmt2int,	slurm_bitfmt2int);
 strong_alias(bit_fmt_hexmask,	slurm_bit_fmt_hexmask);
+strong_alias(bit_fmt_hexmask_trim, slurm_bit_fmt_hexmask_trim);
 strong_alias(bit_unfmt_hexmask,	slurm_bit_unfmt_hexmask);
 strong_alias(bit_fmt_binmask,	slurm_bit_fmt_binmask);
 strong_alias(bit_unfmt_binmask,	slurm_bit_unfmt_binmask);
@@ -619,37 +610,37 @@ bit_equal(bitstr_t *b1, bitstr_t *b2)
 
 
 /*
- * b1 &= b2
+ * b1 &= b2 as many bits as both bitstr_t have
  *   b1 (IN/OUT)	first string
  *   b2 (IN)		second bitstring
  */
 void
 bit_and(bitstr_t *b1, bitstr_t *b2)
 {
-	bitoff_t bit;
+	bitoff_t bit, bit_cnt;
 
 	_assert_bitstr_valid(b1);
 	_assert_bitstr_valid(b2);
-	xassert(_bitstr_bits(b1) == _bitstr_bits(b2));
 
-	for (bit = 0; bit < _bitstr_bits(b1); bit += sizeof(bitstr_t)*8)
+	bit_cnt = MIN(_bitstr_bits(b1), _bitstr_bits(b2));
+	for (bit = 0; bit < bit_cnt; bit += sizeof(bitstr_t)*8)
 		b1[_bit_word(bit)] &= b2[_bit_word(bit)];
 }
 
 /*
- * b1 &= ~b2
+ * b1 &= ~b2 as many bits as both bitstr_t have
  * b1 (IN/OUT)
  * b2 (IN)
  */
 void bit_and_not(bitstr_t *b1, bitstr_t *b2)
 {
-	bitoff_t bit;
+	bitoff_t bit, bit_cnt;
 
 	_assert_bitstr_valid(b1);
 	_assert_bitstr_valid(b2);
-	xassert(_bitstr_bits(b1) == _bitstr_bits(b2));
 
-	for (bit = 0; bit < _bitstr_bits(b1); bit += sizeof(bitstr_t)*8)
+	bit_cnt = MIN(_bitstr_bits(b1), _bitstr_bits(b2));
+	for (bit = 0; bit < bit_cnt; bit += sizeof(bitstr_t)*8)
 		b1[_bit_word(bit)] &= ~b2[_bit_word(bit)];
 }
 
@@ -669,37 +660,37 @@ bit_not(bitstr_t *b)
 }
 
 /*
- * b1 |= b2
+ * b1 |= b2 as many bits as both bitstr_t have
  *   b1 (IN/OUT)	first bitmap
  *   b2 (IN)		second bitmap
  */
 void
 bit_or(bitstr_t *b1, bitstr_t *b2)
 {
-	bitoff_t bit;
+	bitoff_t bit, bit_cnt;
 
 	_assert_bitstr_valid(b1);
 	_assert_bitstr_valid(b2);
-	xassert(_bitstr_bits(b1) == _bitstr_bits(b2));
 
-	for (bit = 0; bit < _bitstr_bits(b1); bit += sizeof(bitstr_t)*8)
+	bit_cnt = MIN(_bitstr_bits(b1), _bitstr_bits(b2));
+	for (bit = 0; bit < bit_cnt; bit += sizeof(bitstr_t)*8)
 		b1[_bit_word(bit)] |= b2[_bit_word(bit)];
 }
 
 /*
- * b1 |= ~b2
+ * b1 |= ~b2 as many bits as both bitstr_t have
  * b1 (IN/OUT)
  * b2 (IN)
  */
 void bit_or_not(bitstr_t *b1, bitstr_t *b2)
 {
-	bitoff_t bit;
+	bitoff_t bit, bit_cnt;
 
 	_assert_bitstr_valid(b1);
 	_assert_bitstr_valid(b2);
-	xassert(_bitstr_bits(b1) == _bitstr_bits(b2));
 
-	for (bit = 0; bit < _bitstr_bits(b1); bit += sizeof(bitstr_t)*8)
+	bit_cnt = MIN(_bitstr_bits(b1), _bitstr_bits(b2));
+	for (bit = 0; bit < bit_cnt; bit += sizeof(bitstr_t)*8)
 		b1[_bit_word(bit)] |= ~b2[_bit_word(bit)];
 }
 
@@ -1362,22 +1353,17 @@ int32_t *bitstr2inx(bitstr_t *b)
 	return bit_inx;
 }
 
-/* bit_fmt_hexmask
- *
- * Given a bitstr_t, allocate and return a string in the form of:
- *                         "0x0123ABC\0"
- *                            ^     ^
- *                            |     |
- *                           MSB   LSB
- *   bitmap (IN)  bitmap to format
- *   RETURN       formatted string
- */
-char * bit_fmt_hexmask(bitstr_t * bitmap)
+/* If trim_output is true, strip off leading zeros from result. */
+static char *_bit_fmt_hexmask(bitstr_t *bitmap, bool trim_output)
 {
 	char *retstr, *ptr;
 	char current;
 	bitoff_t i;
-	bitoff_t bitsize = bit_size(bitmap);
+	bitoff_t bitsize;
+	if (trim_output)
+		bitsize = bit_fls(bitmap) + 1;
+	else
+		bitsize = bit_size(bitmap);
 
 	/* 4 bits per ASCII '0'-'F' */
 	bitoff_t charsize = (bitsize + 3) / 4;
@@ -1405,6 +1391,30 @@ char * bit_fmt_hexmask(bitstr_t * bitmap)
 	return retstr;
 }
 
+/* bit_fmt_hexmask
+ *
+ * Given a bitstr_t, allocate and return a string in the form of:
+ *                         "0x0123ABC\0"
+ *                            ^     ^
+ *                            |     |
+ *                           MSB   LSB
+ *   bitmap (IN)  bitmap to format
+ *   RETURN       formatted string
+ */
+char *bit_fmt_hexmask(bitstr_t *bitmap)
+{
+	return _bit_fmt_hexmask(bitmap, false);
+}
+
+/* bit_fmt_hexmask_trim
+ *
+ * Same as bit_fmt_hexmask() except leading zeros are stripped from the output.
+ */
+char *bit_fmt_hexmask_trim(bitstr_t *bitmap)
+{
+	return _bit_fmt_hexmask(bitmap, true);
+}
+
 /* bit_unfmt_hexmask
  *
  * Given a hex mask string "0x0123ABC\0", convert to a bitstr_t *
