@@ -36,6 +36,8 @@
 
 #include "config.h"
 
+#define _ISOC99_SOURCE	/* needed for lrint */
+
 #include <math.h>
 #include <regex.h>
 
@@ -44,6 +46,7 @@
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/read_config.h"
+#include "src/common/timers.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
@@ -68,6 +71,42 @@ static regex_t bool_pattern_float_re;
 static pthread_mutex_t init_mutex = PTHREAD_MUTEX_INITIALIZER;
 static bool initialized = false; /* protected by init_mutex */
 
+typedef struct {
+	int (*serialize)(char **dest, const data_t *src,
+			 data_serializer_flags_t flags);
+	int (*deserialize)(data_t **dest, const char *src, size_t length);
+} serializer_funcs_t;
+
+/*
+ * Must be synchronized with serializer_funcs_t above.
+ */
+static const char *syms[] = {
+	"serializer_p_serialize",
+	"serializer_p_deserialize",
+};
+
+#define SERIALIZER_MIME_TYPES_SYM "mime_types"
+/* serializer plugin state */
+static serializer_funcs_t *plugins = NULL;
+static int g_context_cnt = -1;
+static plugin_context_t **g_context = NULL;
+
+/* serializer plugrack state */
+static plugin_handle_t *plugin_handles = NULL;
+static char **plugin_types = NULL;
+static size_t plugin_count = 0;
+static plugrack_t *rack = NULL;
+
+#define PMT_MAGIC 0xaaaa8031
+typedef struct {
+	int magic;
+	const char *mime_type; /* never free - const data from plugins */
+	int index; /* plugin index in g_context[] */
+} plugin_mime_type_t;
+
+/* list of all of the known mime types */
+static List mime_types_list = NULL;
+
 #define DATA_MAGIC 0x1992189F
 #define DATA_LIST_MAGIC 0x1992F89F
 #define DATA_LIST_NODE_MAGIC 0x1921F89F
@@ -94,14 +133,14 @@ static void _check_magic(const data_t *data);
 static void _release(data_t *data);
 static void _release_data_list_node(data_list_t *dl, data_list_node_t *dn);
 
-static int _dump_regex_error(int rc, const regex_t *preg)
+static void _dump_regex_error(int rc, const regex_t *preg)
 {
 	char *buffer = NULL;
 	size_t len = regerror(rc, preg, NULL, 0);
 
 	if (len == 0) {
 		error("%s: unknown regex error code: %d", __func__, rc);
-		return SLURM_ERROR;
+		return;
 	}
 
 	buffer = xmalloc(len);
@@ -113,7 +152,6 @@ static int _dump_regex_error(int rc, const regex_t *preg)
 		error("%s: unexpected failure to get regex error", __func__);
 
 	xfree(buffer);
-	return SLURM_ERROR;
 }
 
 static bool _regex_quick_match(const char *str, const regex_t *preg)
@@ -139,8 +177,10 @@ static bool _regex_quick_match(const char *str, const regex_t *preg)
 	}
 }
 
-extern void data_destroy_static(void)
+extern void data_fini(void)
 {
+	int rc;
+
 	slurm_mutex_lock(&init_mutex);
 
 	if (initialized) {
@@ -150,11 +190,201 @@ extern void data_destroy_static(void)
 		regfree(&bool_pattern_int_re);
 		regfree(&bool_pattern_float_re);
 	}
+	if (initialized && rack) {
+		/* cleanup plugins if any were loaded */
+		FREE_NULL_LIST(mime_types_list);
+
+		for (int i = 0; (g_context_cnt > 0) && (i < g_context_cnt);
+		     i++) {
+			if (g_context[i] &&
+			    plugin_context_destroy(g_context[i]))
+				fatal_abort("%s: unable to unload plugin",
+					    __func__);
+		}
+
+		for (size_t i = 0; i < plugin_count; i++) {
+			plugrack_release_by_type(rack, plugin_types[i]);
+			xfree(plugin_types[i]);
+		}
+
+		if ((rc = plugrack_destroy(rack)))
+			fatal_abort("unable to clean up serializer plugrack: %s",
+				    slurm_strerror(rc));
+		rack = NULL;
+
+		xfree(plugin_handles);
+		xfree(plugin_types);
+		xfree(plugins);
+		xfree(g_context);
+		plugin_count = 0;
+		g_context_cnt = -1;
+	}
 
 	slurm_mutex_unlock(&init_mutex);
 }
 
-extern int data_init_static(void)
+static bool _plugin_loaded(const char *plugin)
+{
+	if (plugin_count <= 0)
+		return false;
+
+	for (int i = 0; i < plugin_count; i++)
+		if (!xstrcasecmp(plugin, plugin_types[i]))
+			return true;
+
+	return false;
+}
+
+static void _plugrack_foreach(const char *full_type, const char *fq_path,
+			      const plugin_handle_t id, void *arg)
+{
+	if (_plugin_loaded(full_type)) {
+		log_flag(DATA, "%s: serializer plugin type %s already loaded",
+			 __func__, full_type);
+		/* effectively a no-op if already loaded */
+		return;
+	}
+
+	plugin_count++;
+	xrecalloc(plugin_handles, plugin_count, sizeof(*plugin_handles));
+	xrecalloc(plugin_types, plugin_count, sizeof(*plugin_types));
+
+	plugin_types[plugin_count - 1] = xstrdup(full_type);
+	plugin_handles[plugin_count - 1] = id;
+
+	log_flag(DATA, "%s: serializer plugin type:%s path:%s",
+		 __func__, full_type, fq_path);
+}
+
+static void _find_plugins(const char *plugin_list, plugrack_foreach_t listf)
+{
+	int rc;
+
+	if (!rack)
+		rack = plugrack_create("serializer");
+
+	if ((rc = plugrack_read_dir(rack, slurm_conf.plugindir)))
+		fatal("%s: plugrack_read_dir(%s) failed: %s",
+		      __func__, slurm_conf.plugindir, slurm_strerror(rc));
+
+	if (listf && !xstrcasecmp(plugin_list, "list")) {
+		/* call list function ptr and then load all */
+		plugrack_foreach(rack, listf, NULL);
+		plugin_list = NULL;
+	}
+
+	if (!plugin_list) {
+		/* no filter specified: load them all */
+		plugrack_foreach(rack, _plugrack_foreach, NULL);
+	} else if (plugin_list[0] == '\0') {
+		log_flag(DATA, "not loading any serializer plugins");
+	} else {
+		/* Caller provided which plugins they want */
+		char *type, *last = NULL, *pl;
+
+		pl = xstrdup(plugin_list);
+		type = strtok_r(pl, ",", &last);
+		while (type) {
+			/*
+			 * Permit both prefix and no-prefix for
+			 * plugin names.
+			 */
+			if (!xstrncmp(type, "serializer/", 11))
+				type += 11;
+			type = xstrdup_printf("serializer/%s", type);
+
+			_plugrack_foreach(type, NULL, PLUGIN_INVALID_HANDLE,
+					  NULL);
+
+			xfree(type);
+			type = strtok_r(NULL, ",", &last);
+		}
+
+		xfree(pl);
+	}
+
+	for (size_t i = 0; i < plugin_count; i++) {
+		if (plugin_handles[i] == PLUGIN_INVALID_HANDLE) {
+			plugin_handles[i] = plugrack_use_by_type(
+				rack, plugin_types[i]);
+
+			if (plugin_handles[i] == PLUGIN_INVALID_HANDLE)
+				fatal("Unable to find plugin: %s",
+				      plugin_types[i]);
+		}
+	}
+}
+
+static int _register_mime_types(List mime_types_list, size_t plugin_index,
+				const char **mime_type)
+{
+	while (*mime_type) {
+		plugin_mime_type_t *pmt = xmalloc(sizeof(*pmt));
+
+		pmt->index = plugin_index;
+		pmt->mime_type = *mime_type;
+		pmt->magic = PMT_MAGIC;
+
+		list_append(mime_types_list, pmt);
+
+		log_flag(DATA, "registered serializer plugin %s for %s",
+			 plugin_types[plugin_index], pmt->mime_type);
+
+		mime_type++;
+	}
+
+	return SLURM_SUCCESS;
+}
+
+static void _load_plugins(void)
+{
+	if (!mime_types_list) {
+		mime_types_list = list_create(xfree_ptr);
+
+		/* Load serializer plugins */
+		xassert(g_context_cnt == -1);
+		g_context_cnt = 0;
+
+		xrecalloc(plugins, (plugin_count + 1),
+			  sizeof(serializer_funcs_t));
+		xrecalloc(g_context, (plugin_count + 1),
+			  sizeof(plugin_context_t *));
+
+	}
+
+	for (size_t i = 0; (i < plugin_count); i++) {
+		const char **mime_types;
+
+		if (plugin_handles[i] == PLUGIN_INVALID_HANDLE)
+			fatal("Invalid plugin to load?");
+
+		if (plugins[i].serialize) {
+			log_flag(DATA, "%s: serializer plugin type %s already loaded",
+				 __func__, plugin_types[i]);
+			continue;
+		}
+
+		if (plugin_get_syms(plugin_handles[i], ARRAY_SIZE(syms),
+				    syms, (void **)&plugins[g_context_cnt])
+		    < ARRAY_SIZE(syms))
+			fatal("Incomplete plugin detected");
+
+		mime_types = plugin_get_sym(plugin_handles[i],
+					    SERIALIZER_MIME_TYPES_SYM);
+		if (!mime_types)
+			fatal_abort("%s: unable to load %s from plugin",
+				    __func__, SERIALIZER_MIME_TYPES_SYM);
+		_register_mime_types(mime_types_list, i, mime_types);
+
+		g_context_cnt++;
+
+		if (plugins[i].serialize)
+			log_flag(DATA, "%s: serializer plugin type %s loaded",
+				 __func__, plugin_types[i]);
+	}
+}
+
+extern int data_init(const char *plugin_list, plugrack_foreach_t listf)
 {
 	int rc = SLURM_SUCCESS;
 	int reg_rc; /* regex rc */
@@ -162,35 +392,48 @@ extern int data_init_static(void)
 	slurm_mutex_lock(&init_mutex);
 
 	if (initialized)
-		goto cleanup;
+		goto load_plugins;
 	initialized = true;
 
 	if (!rc && (reg_rc = regcomp(&bool_pattern_null_re, bool_pattern_null,
 			      REG_EXTENDED)) != 0) {
-		rc = _dump_regex_error(reg_rc, &bool_pattern_null_re);
+		_dump_regex_error(reg_rc, &bool_pattern_null_re);
+		rc = ESLURM_DATA_REGEX_COMPILE;
 	}
 
 	if (!rc && (reg_rc = regcomp(&bool_pattern_true_re, bool_pattern_true,
 			      REG_EXTENDED)) != 0) {
-		rc = _dump_regex_error(reg_rc, &bool_pattern_true_re);
+		_dump_regex_error(reg_rc, &bool_pattern_true_re);
+		rc = ESLURM_DATA_REGEX_COMPILE;
 	}
 
 	if (!rc && (reg_rc = regcomp(&bool_pattern_false_re, bool_pattern_false,
 			      REG_EXTENDED)) != 0) {
-		rc = _dump_regex_error(reg_rc, &bool_pattern_false_re);
+		_dump_regex_error(reg_rc, &bool_pattern_false_re);
+		rc = ESLURM_DATA_REGEX_COMPILE;
 	}
 
 	if (!rc && (reg_rc = regcomp(&bool_pattern_int_re, bool_pattern_int,
 			      REG_EXTENDED)) != 0) {
-		rc = _dump_regex_error(reg_rc, &bool_pattern_int_re);
+		_dump_regex_error(reg_rc, &bool_pattern_int_re);
+		rc = ESLURM_DATA_REGEX_COMPILE;
 	}
 
 	if (!rc && (reg_rc = regcomp(&bool_pattern_float_re, bool_pattern_float,
 			      REG_EXTENDED)) != 0) {
-		rc = _dump_regex_error(reg_rc, &bool_pattern_float_re);
+		_dump_regex_error(reg_rc, &bool_pattern_float_re);
+		rc = ESLURM_DATA_REGEX_COMPILE;
+	}
+
+load_plugins:
+
+	if (!rc && (!plugin_list || plugin_list[0] != '\0')) {
+		_find_plugins(plugin_list, listf);
+
+		if (plugin_count > 0)
+			_load_plugins();
 	}
 
-cleanup:
 	slurm_mutex_unlock(&init_mutex);
 
 	return rc;
@@ -199,7 +442,7 @@ cleanup:
 static data_list_t *_data_list_new(void)
 {
 	data_list_t *dl = xmalloc(sizeof(*dl));
-	xassert((dl->magic = DATA_LIST_MAGIC));
+	dl->magic = DATA_LIST_MAGIC;
 
 	log_flag(DATA, "%s: new data list (0x%"PRIXPTR")",
 		 __func__, (uintptr_t) dl);
@@ -278,6 +521,7 @@ static void _release_data_list_node(data_list_t *dl, data_list_node_t *dn)
 
 	if (dn == dl->begin) {
 		/* at the beginning */
+		xassert(!prev);
 		dl->begin = dn->next;
 
 		if (dl->end == dn) {
@@ -287,10 +531,12 @@ static void _release_data_list_node(data_list_t *dl, data_list_node_t *dn)
 	} else if (dn == dl->end) {
 		/* at the end */
 		xassert(!dn->next);
+		xassert(prev);
 		dl->end = prev;
 		prev->next = NULL;
 	} else {
 		/* somewhere in middle */
+		xassert(prev);
 		xassert(prev != dn);
 		xassert(prev->next == dn);
 		xassert(dl->begin != dn);
@@ -302,7 +548,7 @@ static void _release_data_list_node(data_list_t *dl, data_list_node_t *dn)
 	FREE_NULL_DATA(dn->data);
 	xfree(dn->key);
 
-	xassert((dn->magic = ~DATA_LIST_NODE_MAGIC));
+	dn->magic = ~DATA_LIST_NODE_MAGIC;
 	xfree(dn);
 }
 
@@ -317,8 +563,9 @@ static void _release_data_list(data_list_t *dl)
 	_check_data_list_magic(dl);
 
 	if (!n) {
+		xassert(!dl->count);
 		xassert(!dl->end);
-		return;
+		goto finish;
 	}
 
 	xassert(dl->end);
@@ -335,9 +582,10 @@ static void _release_data_list(data_list_t *dl)
 
 #ifndef NDEBUG
 	xassert(count == init_count);
-	xassert(!(dl->count = 0));
-	xassert((dl->magic = ~DATA_LIST_MAGIC));
 #endif
+
+finish:
+	dl->magic = ~DATA_LIST_MAGIC;
 	xfree(dl);
 }
 
@@ -349,7 +597,7 @@ static void _release_data_list(data_list_t *dl)
 static data_list_node_t *_new_data_list_node(data_t *d, const char *key)
 {
 	data_list_node_t *dn = xmalloc(sizeof(*dn));
-	xassert((dn->magic = DATA_LIST_NODE_MAGIC));
+	dn->magic = DATA_LIST_NODE_MAGIC;
 	_check_magic(d);
 
 	dn->data = d;
@@ -406,7 +654,7 @@ static void _data_list_prepend(data_list_t *dl, data_t *d, const char *key)
 data_t *data_new(void)
 {
 	data_t *data = xmalloc(sizeof(*data));
-	xassert((data->magic = DATA_MAGIC));
+	data->magic = DATA_MAGIC;
 	data->type = DATA_TYPE_NULL;
 
 	log_flag(DATA, "%s: new data (0x%"PRIXPTR")",
@@ -417,7 +665,9 @@ data_t *data_new(void)
 
 static void _check_magic(const data_t *data)
 {
-	xassert(data);
+	if (!data)
+		return;
+
 	xassert(data->type > DATA_TYPE_NONE);
 	xassert(data->type < DATA_TYPE_MAX);
 	xassert(data->magic == DATA_MAGIC);
@@ -438,22 +688,21 @@ static void _release(data_t *data)
 	switch (data->type) {
 	case DATA_TYPE_LIST:
 		_release_data_list(data->data.list_u);
-		xassert(!(data->data.list_u = 0));
 		break;
 	case DATA_TYPE_DICT:
 		_release_data_list(data->data.dict_u);
-		xassert(!(data->data.dict_u = 0));
 		break;
 	case DATA_TYPE_STRING:
 		xfree(data->data.string_u);
 		break;
 	default:
 		/* other types don't need to be freed */
-		xassert(!(data->data.int_u = 0));
 		break;
 	}
 
 	data->type = DATA_TYPE_NONE;
+	/* always zero data in debug mode */
+	xassert(memset(&data->data, 0, sizeof(data->data)));
 }
 
 extern void data_free(data_t *data)
@@ -467,18 +716,18 @@ extern void data_free(data_t *data)
 	_check_magic(data);
 	_release(data);
 
-	xassert((data->magic = 0) == 0);
+	data->magic = ~DATA_MAGIC;
 	xfree(data);
 }
 
 extern data_type_t data_get_type(const data_t *data)
 {
+	if (!data)
+		return DATA_TYPE_NONE;
+
 	_check_magic(data);
 
-	if (data)
-		return data->type;
-	else
-		return DATA_TYPE_NONE;
+	return data->type;
 }
 
 extern data_t *data_set_float(data_t *data, double value)
@@ -561,6 +810,23 @@ extern data_t *data_set_string(data_t *data, const char *value)
 	return data;
 }
 
+extern data_t *data_set_string_own(data_t *data, char *value)
+{
+	_check_magic(data);
+
+	if (!data || !value)
+		return NULL;
+
+	log_flag(DATA, "%s: set data (0x%"PRIXPTR") to string: %s",
+		 __func__, (uintptr_t) data, value);
+
+	data->type = DATA_TYPE_STRING;
+	/* take ownership of string */
+	data->data.string_u = value;
+
+	return data;
+}
+
 extern data_t *data_set_dict(data_t *data)
 {
 	_check_magic(data);
@@ -590,7 +856,7 @@ extern data_t *data_set_list(data_t *data)
 		 __func__, (uintptr_t) data);
 
 	data->type = DATA_TYPE_LIST;
-	data->data.dict_u = _data_list_new();
+	data->data.list_u = _data_list_new();
 
 	return data;
 }
@@ -600,7 +866,6 @@ extern data_t *data_list_append(data_t *data)
 	data_t *ndata = NULL;
 	_check_magic(data);
 
-	xassert(data->type == DATA_TYPE_LIST);
 	if (!data || data->type != DATA_TYPE_LIST)
 		return NULL;
 
@@ -618,7 +883,6 @@ extern data_t *data_list_prepend(data_t *data)
 	data_t *ndata = NULL;
 	_check_magic(data);
 
-	xassert(data->type == DATA_TYPE_LIST);
 	if (!data || data->type != DATA_TYPE_LIST)
 		return NULL;
 
@@ -631,11 +895,44 @@ extern data_t *data_list_prepend(data_t *data)
 	return ndata;
 }
 
+static data_for_each_cmd_t _data_list_join(const data_t *src, void *arg)
+{
+	data_t *dst = (data_t *) arg;
+	_check_magic(src);
+	_check_magic(dst);
+	xassert(data_get_type(dst) == DATA_TYPE_LIST);
+
+	log_flag(DATA, "%s: list join data (0x%"PRIXPTR") to (0x%"PRIXPTR")",
+		 __func__, (uintptr_t) src, (uintptr_t) dst);
+
+	 data_copy(data_list_append(dst), src);
+
+	 return DATA_FOR_EACH_CONT;
+}
+
+extern data_t *data_list_join(const data_t **data, bool flatten_lists)
+{
+	data_t *dst = data_set_list(data_new());
+
+	for (size_t i = 0; data[i]; i++) {
+		if (flatten_lists && (data_get_type(data[i]) == DATA_TYPE_LIST))
+			(void) data_list_for_each_const(data[i],
+							_data_list_join, dst);
+		else /* simple join */
+			_data_list_join(data[i], dst);
+	}
+
+	return dst;
+}
+
 const data_t *data_key_get_const(const data_t *data, const char *key)
 {
 	const data_list_node_t *i;
 
 	_check_magic(data);
+	if (!data)
+		return NULL;
+
 	xassert(data->type == DATA_TYPE_DICT);
 	if (!key || data->type != DATA_TYPE_DICT)
 		return NULL;
@@ -666,6 +963,9 @@ data_t *data_key_get(data_t *data, const char *key)
 	data_list_node_t *i;
 
 	_check_magic(data);
+	if (!data)
+		return NULL;
+
 	xassert(data->type == DATA_TYPE_DICT);
 	if (!key || data->type != DATA_TYPE_DICT)
 		return NULL;
@@ -696,6 +996,10 @@ data_t *data_key_set(data_t *data, const char *key)
 	data_t *d;
 
 	_check_magic(data);
+
+	if (!data)
+		return NULL;
+
 	xassert(data->type == DATA_TYPE_DICT);
 	xassert(key && key[0]);
 	if (!key || !key[0] || data->type != DATA_TYPE_DICT)
@@ -731,6 +1035,9 @@ bool data_key_unset(data_t *data, const char *key)
 	data_list_node_t *i;
 
 	_check_magic(data);
+	if (!data)
+		return false;
+
 	xassert(data->type == DATA_TYPE_DICT);
 	if (!key || data->type != DATA_TYPE_DICT)
 		return NULL;
@@ -793,7 +1100,18 @@ int64_t data_get_int(const data_t *data)
 	return data->data.int_u;
 }
 
-const char *data_get_string(const data_t *data)
+extern char *data_get_string(data_t *data)
+{
+	_check_magic(data);
+
+	if (!data)
+		return NULL;
+
+	xassert(data->type == DATA_TYPE_STRING);
+	return data->data.string_u;
+}
+
+extern const char *data_get_string_const(const data_t *data)
 {
 	_check_magic(data);
 
@@ -810,7 +1128,7 @@ int data_get_string_converted(const data_t *d, char **buffer)
 	char *_buffer = NULL;
 
 	if (!d || !buffer)
-		return SLURM_ERROR;
+		return ESLURM_DATA_PTR_NULL;
 
 	if (data_get_type(d) != DATA_TYPE_STRING) {
 		/* copy the data and then convert it to a string type */
@@ -820,24 +1138,27 @@ int data_get_string_converted(const data_t *d, char **buffer)
 		    DATA_TYPE_STRING)
 			_buffer = xstrdup(data_get_string(dclone));
 		FREE_NULL_DATA(dclone);
-	} else
-		_buffer = xstrdup(data_get_string(d));
+	} else {
+		_buffer = xstrdup(data_get_string_const(d));
+		if (!_buffer)
+			_buffer = xstrdup("");
+	}
 
 	if (_buffer) {
 		*buffer = _buffer;
 		return SLURM_SUCCESS;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 extern int data_copy_bool_converted(const data_t *d, bool *buffer)
 {
 	_check_magic(d);
-	int rc = SLURM_ERROR;
+	int rc = ESLURM_DATA_CONV_FAILED;
 
 	if (!d || !buffer)
-		return rc;
+		return ESLURM_DATA_PTR_NULL;
 
 	if (data_get_type(d) != DATA_TYPE_BOOL) {
 		data_t *dclone = data_new();
@@ -862,7 +1183,7 @@ extern int data_get_bool_converted(data_t *d, bool *buffer)
 	_check_magic(d);
 
 	if (!d || !buffer)
-		return SLURM_ERROR;
+		return ESLURM_DATA_PTR_NULL;
 
 	/* assign value if converted successfully */
 	rc = data_copy_bool_converted(d, buffer);
@@ -878,7 +1199,7 @@ extern int data_get_int_converted(const data_t *d, int64_t *buffer)
 	int rc = SLURM_SUCCESS;
 
 	if (!d || !buffer)
-		return SLURM_ERROR;
+		return ESLURM_DATA_PTR_NULL;
 
 	if (data_get_type(d) != DATA_TYPE_INT_64) {
 		data_t *dclone = data_new();
@@ -887,7 +1208,7 @@ extern int data_get_int_converted(const data_t *d, int64_t *buffer)
 		    DATA_TYPE_INT_64)
 			*buffer = data_get_int(dclone);
 		else
-			rc = SLURM_ERROR;
+			rc = ESLURM_DATA_CONV_FAILED;
 		FREE_NULL_DATA(dclone);
 	} else {
 		*buffer = data_get_int(d);
@@ -1140,10 +1461,10 @@ static int _convert_data_string(data_t *data)
 		return SLURM_SUCCESS;
 	}
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 
@@ -1184,10 +1505,10 @@ static int _convert_data_force_bool(data_t *data)
 		data_set_bool(data, data->data.int_u != 0);
 		return SLURM_SUCCESS;
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 static int _convert_data_null(data_t *data)
@@ -1204,15 +1525,15 @@ static int _convert_data_null(data_t *data)
 			data_set_null(data);
 			return SLURM_SUCCESS;
 		} else {
-			return SLURM_ERROR;
+			return ESLURM_DATA_CONV_FAILED;
 		}
 	case DATA_TYPE_NULL:
 		return SLURM_SUCCESS;
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 static int _convert_data_bool(data_t *data)
@@ -1236,15 +1557,15 @@ static int _convert_data_bool(data_t *data)
 			data_set_bool(data, false);
 			return SLURM_SUCCESS;
 		} else {
-			return SLURM_ERROR;
+			return ESLURM_DATA_CONV_FAILED;
 		}
 	case DATA_TYPE_BOOL:
 		return SLURM_SUCCESS;
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 static int _convert_data_int(data_t *data)
@@ -1265,18 +1586,21 @@ static int _convert_data_int(data_t *data)
 			} else { /* failed */
 				debug2("%s: sscanf of int failed: %s", __func__,
 				       data->data.string_u);
-				return SLURM_ERROR;
+				return ESLURM_DATA_CONV_FAILED;
 			}
 		} else {
-			return SLURM_ERROR;
+			return ESLURM_DATA_CONV_FAILED;
 		}
+	case DATA_TYPE_FLOAT:
+		data_set_int(data, lrint(data_get_float(data)));
+		return SLURM_SUCCESS;
 	case DATA_TYPE_INT_64:
 		return SLURM_SUCCESS;
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 static int _convert_data_float(data_t *data)
@@ -1297,29 +1621,43 @@ static int _convert_data_float(data_t *data)
 			} else { /* failed */
 				error("%s: sscanf of double failed: %s",
 				      __func__, data->data.string_u);
-				return SLURM_ERROR;
+				return ESLURM_DATA_CONV_FAILED;
 			}
 		} else {
-			return SLURM_ERROR;
+			return ESLURM_DATA_CONV_FAILED;
 		}
+	case DATA_TYPE_INT_64:
+		if (data_get_int(data) == INFINITE64)
+			data_set_float(data, HUGE_VAL);
+		else if (data_get_int(data) == NO_VAL64)
+			data_set_float(data, NAN);
+		else /* attempt normal fp conversion */
+			data_set_float(data, data_get_int(data));
+		return SLURM_SUCCESS;
 	case DATA_TYPE_FLOAT:
 		return SLURM_SUCCESS;
 	default:
-		return SLURM_ERROR;
+		return ESLURM_DATA_CONV_FAILED;
 	}
 
-	return SLURM_ERROR;
+	return ESLURM_DATA_CONV_FAILED;
 }
 
 extern data_type_t data_convert_type(data_t *data, data_type_t match)
 {
 	_check_magic(data);
+
+	if (!data)
+		return DATA_TYPE_NONE;
+
 	/*
 	 * This currently only works on primitive types and doesn't
 	 * apply to dictionaries or lists.
 	 */
-	xassert(data_get_type(data) != DATA_TYPE_DICT);
-	xassert(data_get_type(data) != DATA_TYPE_LIST);
+	if (data_get_type(data) == DATA_TYPE_DICT)
+		return DATA_TYPE_NONE;
+	if (data_get_type(data) == DATA_TYPE_LIST)
+		return DATA_TYPE_NONE;
 
 	switch (match) {
 	case DATA_TYPE_STRING:
@@ -1391,10 +1729,10 @@ extern size_t data_convert_tree(data_t *data, const data_type_t match)
 
 	switch (data_get_type(data)) {
 	case DATA_TYPE_DICT:
-		data_dict_for_each(data, _convert_dict_entry, &args);
+		(void)data_dict_for_each(data, _convert_dict_entry, &args);
 		break;
 	case DATA_TYPE_LIST:
-		data_list_for_each(data, _convert_list_entry, &args);
+		(void)data_list_for_each(data, _convert_list_entry, &args);
 		break;
 	default:
 		if (match == data_convert_type(data, match))
@@ -1502,7 +1840,8 @@ extern bool data_check_match(const data_t *a, const data_t *b, bool mask)
 		return (data_get_type(b) == DATA_TYPE_NULL);
 	case DATA_TYPE_STRING:
 		// TODO: should we have a case insensitive compare?
-		return !xstrcmp(data_get_string(a), data_get_string(b));
+		return !xstrcmp(data_get_string_const(a),
+				data_get_string_const(b));
 	case DATA_TYPE_BOOL:
 		return (data_get_bool(a) == data_get_bool(b));
 	case DATA_TYPE_INT_64:
@@ -1518,15 +1857,59 @@ extern bool data_check_match(const data_t *a, const data_t *b, bool mask)
 	}
 }
 
-extern const data_t *data_resolve_dict_path(const data_t *data, const char *path)
+extern data_t *data_resolve_dict_path(data_t *data, const char *path)
+{
+	data_t *found = data;
+	char *save_ptr = NULL;
+	char *token = NULL;
+	char *str;
+
+	_check_magic(data);
+
+	if (!data)
+		return NULL;
+
+	str = xstrdup(path);
+
+	token = strtok_r(str, "/", &save_ptr);
+	while (token && found) {
+		xstrtrim(token);
+
+		if (data_get_type(found) != DATA_TYPE_DICT)
+			found = NULL;
+
+		if (found) {
+			found = data_key_get(found, token);
+			token = strtok_r(NULL, "/", &save_ptr);
+		}
+	}
+	xfree(str);
+
+	if (found)
+		log_flag(DATA, "%s: data (0x%"PRIXPTR") resolved dictionary path \"%s\" to (0x%"PRIXPTR")",
+			 __func__, (uintptr_t) data, path, (uintptr_t) found);
+	else
+		log_flag(DATA, "%s: data (0x%"PRIXPTR") failed to resolve dictionary path \"%s\"",
+			 __func__, (uintptr_t) data, path);
+
+	return found;
+}
+
+extern const data_t *data_resolve_dict_path_const(const data_t *data,
+						  const char *path)
 {
 	const data_t *found = data;
 	char *save_ptr = NULL;
 	char *token = NULL;
-	char *str = xstrdup(path);
+	char *str;
 
 	_check_magic(data);
 
+	if (!data)
+		return NULL;
+
+	str = xstrdup(path);
+
 	token = strtok_r(str, "/", &save_ptr);
 	while (token && found) {
 		xstrtrim(token);
@@ -1556,10 +1939,15 @@ extern data_t *data_define_dict_path(data_t *data, const char *path)
 	data_t *found = data;
 	char *save_ptr = NULL;
 	char *token = NULL;
-	char *str = xstrdup(path);
+	char *str;
 
 	_check_magic(data);
 
+	if (!data)
+		return NULL;
+
+	str = xstrdup(path);
+
 	token = strtok_r(str, "/", &save_ptr);
 	while (token && found) {
 		xstrtrim(token);
@@ -1588,6 +1976,9 @@ extern data_t *data_define_dict_path(data_t *data, const char *path)
 
 data_t *data_copy(data_t *dest, const data_t *src)
 {
+	if (!src || !dest)
+		return NULL;
+
 	_check_magic(src);
 	_check_magic(dest);
 
@@ -1596,7 +1987,7 @@ data_t *data_copy(data_t *dest, const data_t *src)
 
 	switch (data_get_type(src)) {
 	case DATA_TYPE_STRING:
-		return data_set_string(dest, data_get_string(src));
+		return data_set_string(dest, data_get_string_const(src));
 	case DATA_TYPE_BOOL:
 		return data_set_bool(dest, data_get_bool(src));
 	case DATA_TYPE_INT_64:
@@ -1647,8 +2038,8 @@ extern int data_retrieve_dict_path_string(const data_t *data, const char *path,
 	int rc;
 
 	_check_magic(data);
-	if (!(d = data_resolve_dict_path(data, path)))
-		return SLURM_ERROR;
+	if (!(d = data_resolve_dict_path_const(data, path)))
+		return ESLURM_DATA_PATH_NOT_FOUND;
 
 	rc = data_get_string_converted(d, ptr_buffer);
 
@@ -1665,8 +2056,8 @@ extern int data_retrieve_dict_path_bool(const data_t *data, const char *path,
 	int rc;
 
 	_check_magic(data);
-	if (!(d = data_resolve_dict_path(data, path)))
-		return SLURM_ERROR;
+	if (!(d = data_resolve_dict_path_const(data, path)))
+		return ESLURM_DATA_PATH_NOT_FOUND;
 
 	rc = data_copy_bool_converted(d, ptr_buffer);
 
@@ -1684,8 +2075,8 @@ extern int data_retrieve_dict_path_int(const data_t *data, const char *path,
 	int rc;
 
 	_check_magic(data);
-	if (!(d = data_resolve_dict_path(data, path)))
-		return SLURM_ERROR;
+	if (!(d = data_resolve_dict_path_const(data, path)))
+		return ESLURM_DATA_PATH_NOT_FOUND;
 
 	rc = data_get_int_converted(d, ptr_buffer);
 
@@ -1716,3 +2107,102 @@ extern char *data_type_to_string(data_type_t type)
 			return "INVALID";
 	}
 }
+
+static int _find_serializer_full_type(void *x, void *key)
+{
+	plugin_mime_type_t *pmt = x;
+	const char *mime_type = key;
+
+	xassert(pmt->magic == PMT_MAGIC);
+
+	if (!xstrcasecmp(mime_type, pmt->mime_type))
+		return 1;
+
+	return 0;
+}
+
+static plugin_mime_type_t *_find_serializer(const char *mime_type)
+{
+	if (!xstrcmp("*/*", mime_type)) {
+		/*
+		 * default to JSON if client will accept anything to avoid
+		 * finding url-encoded or any other less suitable plugin first.
+		 */
+		plugin_mime_type_t *pmt = _find_serializer(MIME_TYPE_JSON);
+
+		if (pmt) {
+			return pmt;
+		} else {
+			/* JSON must not be loaded - try first thing we find */
+			return list_peek(mime_types_list);
+		}
+
+	} else {
+		const int len = strlen(mime_type);
+
+		/* check if client gave {TYPE} / * */
+		if ((len > 3) && (mime_type[len] == '*') &&
+		    (mime_type[len - 1] == '*')) {
+			debug("%s: Partial mime_type globbing not supported", __func__);
+			return NULL;
+		}
+	}
+
+	return list_find_first(mime_types_list, _find_serializer_full_type,
+			       (void *) mime_type);
+}
+
+extern int data_g_serialize(char **dest, const data_t *src,
+			    const char *mime_type,
+			    data_serializer_flags_t flags)
+{
+	DEF_TIMERS;
+	int rc;
+	plugin_mime_type_t *pmt = NULL;
+
+	xassert(dest && (*dest == NULL));
+
+	pmt = _find_serializer(mime_type);
+	if (!pmt)
+		return ESLURM_DATA_UNKNOWN_MIME_TYPE;
+
+	xassert(pmt->magic == PMT_MAGIC);
+
+	START_TIMER;
+	rc = (*(plugins[pmt->index].serialize))(dest, src, flags);
+	END_TIMER2(__func__);
+
+	return rc;
+}
+
+extern int data_g_deserialize(data_t **dest, const char *src, size_t length,
+			      const char *mime_type)
+{
+	DEF_TIMERS;
+	int rc;
+	plugin_mime_type_t *pmt = NULL;
+
+	xassert(dest && (*dest == NULL));
+
+	pmt = _find_serializer(mime_type);
+	if (!pmt)
+		return ESLURM_DATA_UNKNOWN_MIME_TYPE;
+
+	xassert(pmt->magic == PMT_MAGIC);
+
+	START_TIMER;
+	rc = (*(plugins[pmt->index].deserialize))(dest, src, length);
+	END_TIMER2(__func__);
+
+	return rc;
+}
+
+extern const char *data_resolve_mime_type(const char *mime_type)
+{
+	plugin_mime_type_t *pmt = _find_serializer(mime_type);
+
+	if (!pmt)
+		return NULL;
+
+	return pmt->mime_type;
+}
