@@ -34,6 +34,79 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+/*
+ * The data_t struct exists to provide a generic and type safe way to work with
+ * complex data types in a tree. All data_t pointers and helpers are re-entrant
+ * but not thread safe.
+ *
+ * data_t ptr always has a root instance created by data_new() and cleaned up by
+ * data_free(). Do not use data_free() on a child data_t ptr as it will
+ * eventually cause a double free error. All the helper functions exist to
+ * manipulate the data_t struct. Never directly edit the contents of the data_t
+ * struct without one of the helpers. Most helper calls will return a data_t
+ * pointer which is a child of the existing tree and will be cleaned up by the
+ * root.
+ *
+ * To use data_t, data_init() must be called before anything else. It is safe to
+ * call this multiple times but not advised before every usage as it may be
+ * slow. data_init() is designed to allow calls for a specific plugin
+ * requirement which will not prevent loading of other plugins by the other
+ * calls to data_init(). data_fini() needs to be called after all data
+ * operations are complete is only for testing for memory leaks.
+ *
+ * data_t has very *strict* typing that is based on JSON. All of the possible
+ * types are in data_type_t. The caller is required to verify the type of the
+ * data_t pointer is correct before calling the helper function to retrieve the
+ * contents. If the source data is provided by a user (or is just unknown), then
+ * one of the many data_*convert*() functions must be used to ensure that the
+ * data_t pointer is of the correct type. These convert functions will generally
+ * allow conversion betwen all of the types except DICT and LIST (as converting
+ * between them is not well defined).
+ *
+ * There are helpers to iterate over all members of LIST and DICT data_t
+ * pointers. These are all function pointer based helpers that will call a given
+ * function pointer on each item being iterated over. The return value is
+ * operational command allowing the function pointer to inform the caller how to
+ * proceed. There are purposefully no iterators for data_t pointers to avoid any
+ * form of dangling pointers.
+ *
+ * data_t uses a plugin interface for serialization of the data to common
+ * formats. These plugins require 3rd party libraries and may not have been
+ * compiled. Any code should expect this possiblity as data_init() will fail if
+ * the plugin is not found.
+ *
+ * Example usage:
+ *
+ * //Global init requiring JSON serializer
+ * if (data_init(MIME_TYPE_JSON_PLUGIN, NULL)) fatal("failed");
+ * //Create root data entry:
+ * data_t *ex = data_new();
+ * //Set data entry to be a dictionary type
+ * data_set_dict(ex);
+ * //Set key test1 to be string "test1 value"
+ * data_set_string(data_key_set(ex, "test1"), "test1 value");
+ * //Set key test2 to be integer 12345
+ * data_set_int(data_key_set(ex, "test2"), 12345);
+ * // serialize into JSON string
+ * char *json = NULL;
+ * data_g_serialize(&json, ex, MIME_TYPE_JSON, DATA_SER_FLAGS_PRETTY);
+ * // cleanup the example
+ * FREE_NULL_DATA(ex);
+ * // log the json
+ * debug("example json: %s", json);
+ * // deserialise the JSON back into data_t
+ * data_g_deserialize(&ex, json, strlen(json), MIME_TYPE_JSON);
+ * xfree(json);
+ * // verify contents
+ * xassert(data_get_type(ex) == DATA_TYPE_DICT);
+ * xassert(!xstrcmp(data_get_string(data_key_get(ex, "test1"), "test1 value"));
+ * xassert(data_get_int(data_key_get(ex, "test2") == 12345);
+ * // cleanup tree
+ * FREE_NULL_DATA(ex);
+ * // release all global memory and plugins
+ * data_fini();
+ */
+
 #ifndef _DATA_H
 #define _DATA_H
 
@@ -42,6 +115,7 @@
 #include <stddef.h>
 
 #include "src/common/list.h"
+#include "src/common/plugrack.h"
 
 /*
  * The possible types of data.
@@ -128,12 +202,25 @@ typedef data_for_each_cmd_t (*DataDictForF) (const char *key, data_t *data, void
 typedef data_for_each_cmd_t (*DataDictForFConst) (const char *key, const data_t *data, void *arg);
 
 /*
- * Initialize static structs needed by data functions.
- * WARNING: must be called only once before any data commands
+ * Initialize static structs needed by data functions and
+ * 	load serializer plugins
+ *
+ * It is safe to call this function multiple times with different plugins as
+ * they will be loaded only once.
+ *
+ * IN plugins - comma delimited list of plugins or "list"
+ * 	pass NULL to load all found or "" to load none of them
+ *
+ * IN listf - function to call if plugins="list" (may be NULL)
  * RET SLURM_SUCCESS or error
  */
-extern int data_init_static(void);
-extern void data_destroy_static(void);
+extern int data_init(const char *plugins, plugrack_foreach_t listf);
+/*
+ * Cleanup global memory used by data_t helpers and unload all plugins
+ *
+ * WARNING: must be called only once after all data commands complete
+ */
+extern void data_fini(void);
 
 /*
  * Create new data struct.
@@ -203,6 +290,27 @@ extern data_t *data_set_int(data_t *data, int64_t value);
  */
 extern data_t *data_set_string(data_t *data, const char *value);
 
+/*
+ * Set data to string type with given value.
+ * IN data structure to modify
+ * IN value value to set (takes ownership of value and will xfree())
+ * RET data ptr or NULL on error
+ */
+extern data_t *data_set_string_own(data_t *data, char *value);
+
+/*
+ * Set data to string type with given formatted value.
+ * IN data structure to modify
+ * IN fmt - printf format field
+ */
+#define data_set_string_fmt(data, fmt, ...)          \
+	do {                                         \
+		char *str = NULL;                    \
+		xstrfmtcat(str, fmt, ##__VA_ARGS__); \
+		if (!data_set_string_own(data, str)) \
+			xfree(str);                  \
+	} while (0)
+
 /*
  * Detect data type and if possible, change to correct type.
  * WARNING: command is currently only useful for to/from DATA_TYPE_STRING.
@@ -272,11 +380,18 @@ extern int64_t data_get_int(const data_t *data);
 extern int data_get_int_converted(const data_t *d, int64_t *ptr_buffer);
 
 /*
- * Get data as string and force conversion if possible.
+ * Get data as string
  * IN data data to convert into a string
- * RET data converted to string or NULL on failure
+ * RET data string or NULL on failure
  */
-extern const char *data_get_string(const data_t *data);
+extern char *data_get_string(data_t *data);
+
+/*
+ * Get const data as string
+ * IN data data to convert into a string
+ * RET data string or NULL on failure
+ */
+extern const char *data_get_string_const(const data_t *data);
 
 /*
  * Get data as string and force conversion if possible.
@@ -364,6 +479,16 @@ extern data_t *data_list_append(data_t *data);
  */
 extern data_t *data_list_prepend(data_t *data);
 
+/*
+ * Copy and join array of data into a single list.
+ * IN data - array of data objects (list type only) to copy/merge into a single
+ * data list. Last entry must be NULL.
+ * IN flatten_lists - if data is a list, add items in list as if there were
+ * separate data items in data. Will only flatten 1 level.
+ * RET new data object with merged lists.
+ */
+extern data_t *data_list_join(const data_t **data, bool flatten_lists);
+
 /*
  * Get number of entities in list
  * IN data data object to count list entities
@@ -432,8 +557,16 @@ extern bool data_check_match(const data_t *a, const data_t *b, bool mask);
  * IN path /key/path/to/value to search
  * RET ptr to data of value or NULL if not found
  */
-extern const data_t *
-	data_resolve_dict_path(const data_t *data, const char *path);
+extern data_t *data_resolve_dict_path(data_t *data, const char *path);
+
+/*
+ * resolve out path based on data object full of dictionary keys
+ * IN data dictionary of dictionaries to search
+ * IN path /key/path/to/value to search
+ * RET const ptr to data of value or NULL if not found
+ */
+extern const data_t *data_resolve_dict_path_const(const data_t *data,
+						   const char *path);
 
 /*
  * create (or resolve) out path based on data object full of dictionary keys
@@ -482,4 +615,55 @@ extern int data_retrieve_dict_path_int(const data_t *data, const char *path,
  */
 extern data_t *data_copy(data_t *dest, const data_t *src);
 
+typedef enum {
+	DATA_SER_FLAGS_NONE = 0, /* defaults to compact currently */
+	DATA_SER_FLAGS_COMPACT = 1 << 1,
+	DATA_SER_FLAGS_PRETTY = 1 << 2,
+} data_serializer_flags_t;
+
+/*
+ * Define common MIME types to make it easier for serializer callers.
+ *
+ * WARNING: There is no guarantee that plugins for these types
+ * will be loaded at any given time.
+ */
+#define MIME_TYPE_YAML "application/x-yaml"
+#define MIME_TYPE_YAML_PLUGIN "serializer/yaml"
+#define MIME_TYPE_JSON "application/json"
+#define MIME_TYPE_JSON_PLUGIN "serializer/json"
+#define MIME_TYPE_URL_ENCODED "application/x-www-form-urlencoded"
+#define MIME_TYPE_URL_ENCODED_PLUGIN "serializer/url-encoded"
+
+/*
+ * Serialize data in src into string dest
+ * IN/OUT dest - ptr to NULL string ptr to set with output data.
+ * 	caller must xfree(dest) if set.
+ * IN src - populated data ptr to serialize
+ * IN mime_type - serialize data into the given mime_type
+ * IN flags - optional flags to specify to serilzier to change presentation of
+ * 	data
+ * RET SLURM_SUCCESS or error
+ */
+extern int data_g_serialize(char **dest, const data_t *src,
+			    const char *mime_type,
+			    data_serializer_flags_t flags);
+
+/*
+ * Deserialize string in src into data dest
+ * IN/OUT dest - ptr to NULL data ptr to set with output data.
+ * 	caller must FREE_NULL_DATA(dest) if set.
+ * IN src - string to deserialize
+ * IN length - number of bytes in src
+ * IN mime_type - deserialize data using given mime_type
+ * RET SLURM_SUCCESS or error
+ */
+extern int data_g_deserialize(data_t **dest, const char *src, size_t length,
+			      const char *mime_type);
+
+/*
+ * Check if there is a plugin loaded that can handle the requested mime type
+ * RET ptr to best matching mime type or NULL if none can match
+ */
+extern const char *data_resolve_mime_type(const char *mime_type);
+
 #endif /* _DATA_H */
