@@ -45,7 +45,6 @@
 
 #include "src/common/hostlist.h"
 #include "src/common/job_resources.h"
-#include "src/common/layouts_mgr.h"
 #include "src/common/log.h"
 #include "src/common/pack.h"
 #include "src/common/xassert.h"
@@ -103,7 +102,7 @@ extern int build_job_resources(job_resources_t *job_resrcs,
 			continue;
 		node_ptr = node_record_table + i;
 
-		socks = node_ptr->config_ptr->sockets;
+		socks = node_ptr->config_ptr->tot_sockets;
 		cores = node_ptr->config_ptr->cores;
 
 		if ((sock_inx < 0) ||
@@ -134,6 +133,7 @@ extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 {
 	int cpu_count = 0, i;
 	uint32_t last_cpu_cnt = NO_VAL;
+	int j, i_first, i_last, node_cpu_count;
 
 	if (job_resrcs_ptr->nhosts == 0)
 		return cpu_count;	/* no work to do */
@@ -151,9 +151,25 @@ extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 	job_resrcs_ptr->cpu_array_value = xcalloc(job_resrcs_ptr->nhosts,
 						  sizeof(uint16_t));
 
-	for (i=0; i<job_resrcs_ptr->nhosts; i++) {
-		if (job_resrcs_ptr->cpus[i] != last_cpu_cnt) {
-			last_cpu_cnt = job_resrcs_ptr->cpus[i];
+	i_first = bit_ffs(job_resrcs_ptr->node_bitmap);
+	if (i_first != -1)
+		i_last = bit_fls(job_resrcs_ptr->node_bitmap);
+	else
+		i_last = -2;
+
+	for (i = i_first, j = 0; i <= i_last; i++) {
+		if (!bit_test(job_resrcs_ptr->node_bitmap, i))
+			continue;
+
+		/*
+		 * This needs to be the threads per core count to handle
+		 * allocations correctly.
+		 */
+		node_cpu_count = job_resources_get_node_cpu_cnt(
+			job_resrcs_ptr, j, i);
+
+		if (node_cpu_count != last_cpu_cnt) {
+			last_cpu_cnt = node_cpu_count;
 			job_resrcs_ptr->cpu_array_value[
 				job_resrcs_ptr->cpu_array_cnt]
 				= last_cpu_cnt;
@@ -164,7 +180,9 @@ extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 			job_resrcs_ptr->cpu_array_reps[
 				job_resrcs_ptr->cpu_array_cnt-1]++;
 		}
-		cpu_count += last_cpu_cnt;
+		/* This needs to be the full amount for accounting */
+		cpu_count += job_resrcs_ptr->cpus[j];
+		j++;
 	}
 	return cpu_count;
 }
@@ -278,7 +296,7 @@ extern int valid_job_resources(job_resources_t *job_resrcs,
 			continue;
 		node_ptr = node_record_table + i;
 
-		socks = node_ptr->config_ptr->sockets;
+		socks = node_ptr->config_ptr->tot_sockets;
 		cores = node_ptr->config_ptr->cores;
 
 		if (sock_cnt >= job_resrcs->sock_core_rep_count[sock_inx]) {
@@ -538,13 +556,92 @@ extern void log_job_resources(void *void_job_ptr)
 	info("====================");
 }
 
-extern void pack_job_resources(job_resources_t *job_resrcs_ptr, Buf buffer,
+extern void pack_job_resources(job_resources_t *job_resrcs_ptr, buf_t *buffer,
 			       uint16_t protocol_version)
 {
 	int i;
 	uint32_t core_cnt = 0, sock_recs = 0;
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
+		if (job_resrcs_ptr == NULL) {
+			uint32_t empty = NO_VAL;
+			pack32(empty, buffer);
+			return;
+		}
+
+		pack32(job_resrcs_ptr->nhosts, buffer);
+		pack32(job_resrcs_ptr->ncpus, buffer);
+		pack32(job_resrcs_ptr->node_req, buffer);
+		packstr(job_resrcs_ptr->nodes, buffer);
+		pack8(job_resrcs_ptr->whole_node, buffer);
+		pack16(job_resrcs_ptr->threads_per_core, buffer);
+		pack16(job_resrcs_ptr->cr_type, buffer);
+
+		if (job_resrcs_ptr->cpu_array_reps)
+			pack32_array(job_resrcs_ptr->cpu_array_reps,
+				     job_resrcs_ptr->cpu_array_cnt, buffer);
+		else
+			pack32_array(job_resrcs_ptr->cpu_array_reps, 0, buffer);
+
+		if (job_resrcs_ptr->cpu_array_value)
+			pack16_array(job_resrcs_ptr->cpu_array_value,
+				     job_resrcs_ptr->cpu_array_cnt, buffer);
+		else
+			pack16_array(job_resrcs_ptr->cpu_array_value,
+				     0, buffer);
+
+		if (job_resrcs_ptr->cpus)
+			pack16_array(job_resrcs_ptr->cpus,
+				     job_resrcs_ptr->nhosts, buffer);
+		else
+			pack16_array(job_resrcs_ptr->cpus, 0, buffer);
+
+		if (job_resrcs_ptr->cpus_used)
+			pack16_array(job_resrcs_ptr->cpus_used,
+				     job_resrcs_ptr->nhosts, buffer);
+		else
+			pack16_array(job_resrcs_ptr->cpus_used, 0, buffer);
+
+		if (job_resrcs_ptr->memory_allocated)
+			pack64_array(job_resrcs_ptr->memory_allocated,
+				     job_resrcs_ptr->nhosts, buffer);
+		else
+			pack64_array(job_resrcs_ptr->memory_allocated,
+				     0, buffer);
+
+		if (job_resrcs_ptr->memory_used)
+			pack64_array(job_resrcs_ptr->memory_used,
+				     job_resrcs_ptr->nhosts, buffer);
+		else
+			pack64_array(job_resrcs_ptr->memory_used, 0, buffer);
+
+		xassert(job_resrcs_ptr->cores_per_socket);
+		xassert(job_resrcs_ptr->sock_core_rep_count);
+		xassert(job_resrcs_ptr->sockets_per_node);
+
+		for (i=0; i < job_resrcs_ptr->nhosts; i++) {
+			core_cnt += job_resrcs_ptr->sockets_per_node[i]
+				* job_resrcs_ptr->cores_per_socket[i] *
+				job_resrcs_ptr->sock_core_rep_count[i];
+			sock_recs += job_resrcs_ptr->
+				     sock_core_rep_count[i];
+			if (sock_recs >= job_resrcs_ptr->nhosts)
+				break;
+		}
+		i++;
+		pack16_array(job_resrcs_ptr->sockets_per_node,
+			     (uint32_t) i, buffer);
+		pack16_array(job_resrcs_ptr->cores_per_socket,
+			     (uint32_t) i, buffer);
+		pack32_array(job_resrcs_ptr->sock_core_rep_count,
+			     (uint32_t) i, buffer);
+
+		xassert(job_resrcs_ptr->core_bitmap);
+		xassert(job_resrcs_ptr->core_bitmap_used);
+		pack_bit_str_hex(job_resrcs_ptr->core_bitmap, buffer);
+		pack_bit_str_hex(job_resrcs_ptr->core_bitmap_used,
+				 buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		if (job_resrcs_ptr == NULL) {
 			uint32_t empty = NO_VAL;
 			pack32(empty, buffer);
@@ -628,14 +725,77 @@ extern void pack_job_resources(job_resources_t *job_resrcs_ptr, Buf buffer,
 }
 
 extern int unpack_job_resources(job_resources_t **job_resrcs_pptr,
-				Buf buffer, uint16_t protocol_version)
+				buf_t *buffer, uint16_t protocol_version)
 {
 	char *bit_fmt = NULL;
 	uint32_t empty, tmp32;
 	job_resources_t *job_resrcs;
 
 	xassert(job_resrcs_pptr);
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
+		safe_unpack32(&empty, buffer);
+		if (empty == NO_VAL) {
+			*job_resrcs_pptr = NULL;
+			return SLURM_SUCCESS;
+		}
+
+		job_resrcs = xmalloc(sizeof(struct job_resources));
+		job_resrcs->nhosts = empty;
+		safe_unpack32(&job_resrcs->ncpus, buffer);
+		safe_unpack32(&job_resrcs->node_req, buffer);
+		safe_unpackstr_xmalloc(&job_resrcs->nodes, &tmp32, buffer);
+		safe_unpack8(&job_resrcs->whole_node, buffer);
+		safe_unpack16(&job_resrcs->threads_per_core, buffer);
+		safe_unpack16(&job_resrcs->cr_type, buffer);
+
+		safe_unpack32_array(&job_resrcs->cpu_array_reps,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->cpu_array_reps);
+		job_resrcs->cpu_array_cnt = tmp32;
+
+		safe_unpack16_array(&job_resrcs->cpu_array_value,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->cpu_array_value);
+
+		if (tmp32 != job_resrcs->cpu_array_cnt)
+			goto unpack_error;
+
+		safe_unpack16_array(&job_resrcs->cpus, &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->cpus);
+		if (tmp32 != job_resrcs->nhosts)
+			goto unpack_error;
+		safe_unpack16_array(&job_resrcs->cpus_used, &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->cpus_used);
+
+		safe_unpack64_array(&job_resrcs->memory_allocated,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->memory_allocated);
+		safe_unpack64_array(&job_resrcs->memory_used, &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->memory_used);
+
+		safe_unpack16_array(&job_resrcs->sockets_per_node,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->sockets_per_node);
+		safe_unpack16_array(&job_resrcs->cores_per_socket,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->cores_per_socket);
+		safe_unpack32_array(&job_resrcs->sock_core_rep_count,
+				    &tmp32, buffer);
+		if (tmp32 == 0)
+			xfree(job_resrcs->sock_core_rep_count);
+
+		unpack_bit_str_hex(&job_resrcs->core_bitmap, buffer);
+		unpack_bit_str_hex(&job_resrcs->core_bitmap_used,
+				   buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&empty, buffer);
 		if (empty == NO_VAL) {
 			*job_resrcs_pptr = NULL;
@@ -1659,137 +1819,19 @@ extern int job_resources_node_inx_to_cpu_inx(job_resources_t *job_resrcs_ptr,
 	return node_offset;
 }
 
-extern int adapt_layouts(job_resources_t *job_resrcs_ptr, uint32_t cpu_freq_max,
-			 uint32_t node_id, char* node_name, bool new_value)
+extern uint16_t job_resources_get_node_cpu_cnt(job_resources_t *job_resrcs_ptr,
+					       int job_node_inx,
+					       int sys_node_inx)
 {
-	int i, k = 0, bit_inx = 0, core_cnt = 0;
-	uint32_t max_watts, zero = 0, der;
-	uint32_t core_num,val = 0;
-	uint16_t num_freq;
-	char temp[128], ename[128], keyname[128];
-	uint32_t data[2], vals[2];
-	int num_counts = 0, activate = 0;
-	int *desalloc_cores;
+	uint16_t cpu_count = job_resrcs_ptr->cpus[job_node_inx];
 
-	xassert(job_resrcs_ptr);
-
-	for (i = 0; i < job_resrcs_ptr->nhosts; i++) {
-		if (job_resrcs_ptr->sock_core_rep_count[i] <= node_id) {
-			bit_inx += job_resrcs_ptr->sockets_per_node[i] *
-				   job_resrcs_ptr->cores_per_socket[i] *
-				   job_resrcs_ptr->sock_core_rep_count[i];
-			node_id -= job_resrcs_ptr->sock_core_rep_count[i];
-		} else {
-			bit_inx += job_resrcs_ptr->sockets_per_node[i] *
-				   job_resrcs_ptr->cores_per_socket[i] *
-				   node_id;
-			core_cnt = job_resrcs_ptr->sockets_per_node[i] *
-				   job_resrcs_ptr->cores_per_socket[i];
-			break;
-		}
-	}
-	if (core_cnt < 1) {
-		error("count_job_resources_node: core_cnt=0");
-		return 0;
+	if (((job_resrcs_ptr->cr_type & CR_CORE) ||
+	     (job_resrcs_ptr->cr_type & CR_SOCKET)) &&
+	    (job_resrcs_ptr->threads_per_core <
+	     node_record_table_ptr[sys_node_inx].vpus)) {
+		cpu_count /= node_record_table_ptr[sys_node_inx].vpus;
+		cpu_count *= job_resrcs_ptr->threads_per_core;
 	}
 
-	i = bit_size(job_resrcs_ptr->core_bitmap);
-	if ((bit_inx + core_cnt) > i) {
-		error("count_job_resources_node: offset > bitmap size "
-		      "(%d >= %d)", (bit_inx + core_cnt), i);
-		return 0;
-	}
-
-	layouts_entity_get_kv("power", node_name, "NumFreqChoices",
-			      &num_freq, L_T_UINT16);
-	layouts_entity_get_mkv("power", node_name,
-			       "CoresCount,LastCore", data,
-			       (sizeof(uint32_t)*2),L_T_UINT32);
-	if (cpu_freq_max != 0) {
-		for (i = 1; i < num_freq + 1; i++) {
-			sprintf(temp, "Cpufreq%d", i);
-			layouts_entity_pullget_kv("power", node_name,
-						  temp, &val, L_T_UINT32);
-			if (val == cpu_freq_max) {
-				k = i;
-				break;
-			}
-		}
-	}
-
-	desalloc_cores = xcalloc(core_cnt, sizeof(int));
-	for (i = 0; i < core_cnt; i++) {
-		/*core_num=LastCore+1-CoresCount*/
-		core_num = data[1] + 1 - data[0] + i;
-		sprintf(ename, "virtualcore%u", core_num);
-		if (bit_test(job_resrcs_ptr->core_bitmap, bit_inx++)) {
-			if (new_value) {
-				if (cpu_freq_max != 0 && k != 0) {
-					sprintf(keyname, 
-						"Cpufreq%dWatts", k);
-					layouts_entity_get_kv("power",
-							ename, keyname,
-							&max_watts, L_T_UINT32);
-				} else {
-					layouts_entity_get_kv("power",
-							ename, "MaxCoreWatts",
-							&max_watts, L_T_UINT32);
-				}
-				layouts_entity_set_kv("power",
-						    ename, "CurrentCorePower",
-						    &max_watts, L_T_UINT32);
-			} else {
-				layouts_entity_set_kv("power",
-						    ename, "CurrentCorePower",
-						    &zero, L_T_UINT32);
-				desalloc_cores[num_counts] = i;
-				num_counts++;
-			}
-		} else {
-			layouts_entity_get_mkv("power", ename,
-					  "CurrentCorePower,IdleCoreWatts",
-					  vals, 
-					  (sizeof(uint32_t)*2) ,L_T_UINT32);
-			if (new_value) {
-				if (vals[0] == 0) {
-					layouts_entity_set_kv(
-							  "power",
-							  ename,
-							  "CurrentCorePower",
-							  &vals[1],
-							  L_T_UINT32);
-				}
-			} else {
-				if (vals[1] != vals[0]) {
-					activate = 1;
-				} else {
-					desalloc_cores[num_counts] = i;
-					num_counts++;
-					layouts_entity_set_kv(
-							  "power",
-							  ename,
-							  "CurrentCorePower",
-							  &zero,
-							  L_T_UINT32);		
-					layouts_entity_get_kv("power",
-							  ename,
-							  "CurrentCorePower",
-							  &der, L_T_UINT32);
-				}	
-			}
-		}
-	}
-
-	if (activate == 1) {
-		for (i = 0; i < num_counts; i++) {
-			core_num = data[1] + 1- data[0] + desalloc_cores[i];
-			sprintf(ename, "virtualcore%u", core_num);
-			layouts_entity_set_kv("power", ename,
-						  "CurrentCorePower", &vals[1],
-						  L_T_UINT32);	
-		}
-	}
-	xfree(desalloc_cores);
-
-	return 1;
+	return cpu_count;
 }
