@@ -2,6 +2,7 @@
  *  list.c
  *****************************************************************************
  *  Copyright (C) 2001-2002 The Regents of the University of California.
+ *  Copyright (C) 2021 NVIDIA Corporation.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
  *
@@ -64,10 +65,14 @@ strong_alias(list_transfer_max,	slurm_list_transfer_max);
 strong_alias(list_prepend,	slurm_list_prepend);
 strong_alias(list_find_first,	slurm_list_find_first);
 strong_alias(list_delete_all,	slurm_list_delete_all);
+strong_alias(list_delete_first,	slurm_list_delete_first);
+strong_alias(list_delete_ptr,	slurm_list_delete_ptr);
 strong_alias(list_for_each,	slurm_list_for_each);
 strong_alias(list_for_each_max,	slurm_list_for_each_max);
 strong_alias(list_flush,	slurm_list_flush);
+strong_alias(list_flush_max,	slurm_list_flush_max);
 strong_alias(list_sort,		slurm_list_sort);
+strong_alias(list_flip,		slurm_list_flip);
 strong_alias(list_push,		slurm_list_push);
 strong_alias(list_pop,		slurm_list_pop);
 strong_alias(list_peek,		slurm_list_peek);
@@ -86,6 +91,7 @@ strong_alias(list_delete_item,	slurm_list_delete_item);
  *  Constants  *
  ***************/
 #define LIST_MAGIC 0xDEADBEEF
+#define LIST_ITR_MAGIC 0xDEADBEFF
 
 #define list_alloc() xmalloc(sizeof(struct xlist))
 #define list_free(_l) xfree(l)
@@ -104,25 +110,22 @@ struct listNode {
 };
 
 struct listIterator {
+	unsigned int          magic;        /* sentinel for asserting validity   */
 	struct xlist         *list;         /* the list being iterated           */
 	struct listNode      *pos;          /* the next node to be iterated      */
 	struct listNode     **prev;         /* addr of 'next' ptr to prv It node */
 	struct listIterator  *iNext;        /* iterator chain for list_destroy() */
-#ifndef NDEBUG
-	unsigned int          magic;        /* sentinel for asserting validity   */
-#endif /* !NDEBUG */
 };
 
 struct xlist {
+	unsigned int          magic;        /* sentinel for asserting validity   */
 	struct listNode      *head;         /* head of the list                  */
 	struct listNode     **tail;         /* addr of last node's 'next' ptr    */
+	struct listNode      *tail_ptr;     /* tail ptr of the list  */
 	struct listIterator  *iNext;        /* iterator chain for list_destroy() */
 	ListDelF              fDel;         /* function to delete node data      */
 	int                   count;        /* number of nodes in list           */
 	pthread_mutex_t       mutex;        /* mutex to protect access to list   */
-#ifndef NDEBUG
-	unsigned int          magic;        /* sentinel for asserting validity   */
-#endif /* !NDEBUG */
 };
 
 typedef struct listNode * ListNode;
@@ -152,13 +155,14 @@ list_create (ListDelF f)
 {
 	List l = list_alloc();
 
+	l->magic = LIST_MAGIC;
 	l->head = NULL;
 	l->tail = &l->head;
+	l->tail_ptr = l->head;
 	l->iNext = NULL;
 	l->fDel = f;
 	l->count = 0;
 	slurm_mutex_init(&l->mutex);
-	xassert((l->magic = LIST_MAGIC));      /* set magic via assert abuse */
 
 	return l;
 }
@@ -172,14 +176,14 @@ list_destroy (List l)
 	ListNode p, pTmp;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	i = l->iNext;
 	while (i) {
-		xassert(i->magic == LIST_MAGIC);
+		xassert(i->magic == LIST_ITR_MAGIC);
+		i->magic = ~LIST_ITR_MAGIC;
 		iTmp = i->iNext;
-		xassert((i->magic = ~LIST_MAGIC)); /* clear magic via assert abuse */
 		list_iterator_free(i);
 		i = iTmp;
 	}
@@ -191,7 +195,7 @@ list_destroy (List l)
 		list_node_free(p);
 		p = pTmp;
 	}
-	xassert((l->magic = ~LIST_MAGIC));     /* clear magic via assert abuse */
+	l->magic = ~LIST_MAGIC;
 	slurm_mutex_unlock(&l->mutex);
 	slurm_mutex_destroy(&l->mutex);
 	list_free(l);
@@ -205,8 +209,8 @@ list_is_empty (List l)
 	int n;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 	n = l->count;
 	slurm_mutex_unlock(&l->mutex);
 
@@ -224,8 +228,8 @@ int list_count(List l)
 	if (!l)
 		return 0;
 
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 	n = l->count;
 	slurm_mutex_unlock(&l->mutex);
 
@@ -262,8 +266,8 @@ list_append (List l, void *x)
 
 	xassert(l != NULL);
 	xassert(x != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 	v = _list_append_locked(l, x);
 	slurm_mutex_unlock(&l->mutex);
 
@@ -341,8 +345,8 @@ list_prepend (List l, void *x)
 
 	xassert(l != NULL);
 	xassert(x != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = _list_node_create(l, &l->head, x);
 	slurm_mutex_unlock(&l->mutex);
@@ -360,9 +364,8 @@ list_find_first (List l, ListFindF f, void *key)
 
 	xassert(l != NULL);
 	xassert(f != NULL);
-	xassert(key != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	for (p = l->head; p; p = p->next) {
 		if (f(p->data, key)) {
@@ -386,8 +389,8 @@ list_remove_first (List l, ListFindF f, void *key)
 	xassert(l != NULL);
 	xassert(f != NULL);
 	xassert(key != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	pp = &l->head;
 	while (*pp) {
@@ -414,8 +417,8 @@ list_delete_all (List l, ListFindF f, void *key)
 
 	xassert(l != NULL);
 	xassert(f != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	pp = &l->head;
 	while (*pp) {
@@ -435,6 +438,70 @@ list_delete_all (List l, ListFindF f, void *key)
 	return n;
 }
 
+int list_delete_first(List l, ListFindF f, void *key)
+{
+	ListNode *pp;
+	void *v;
+	int n = 0;
+
+	xassert(l != NULL);
+	xassert(f != NULL);
+	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
+
+	pp = &l->head;
+	while (*pp) {
+		int rc = f((*pp)->data, key);
+
+		if (rc > 0) {
+			if ((v = _list_node_destroy(l, pp))) {
+				if (l->fDel)
+					l->fDel(v);
+			}
+			n = 1;
+			break;
+		} else if (rc < 0) {
+			n = -1;
+			break;
+		} else {
+			pp = &(*pp)->next;
+		}
+	}
+	slurm_mutex_unlock(&l->mutex);
+
+	return n;
+}
+
+/* list_delete_ptr()
+ */
+int list_delete_ptr(List l, void *key)
+{
+	ListNode *pp;
+	void *v;
+	int n = 0;
+
+	xassert(l);
+	xassert(key);
+	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
+
+	pp = &l->head;
+	while (*pp) {
+		if ((*pp)->data == key) {
+			if ((v = _list_node_destroy(l, pp))) {
+				if (l->fDel)
+					l->fDel(v);
+				n = 1;
+				break;
+			}
+		} else
+			pp = &(*pp)->next;
+	}
+	slurm_mutex_unlock(&l->mutex);
+
+	return n;
+}
+
 /* list_for_each()
  */
 int
@@ -459,8 +526,8 @@ int list_for_each_max(List l, int *max, ListForF f, void *arg,
 
 	xassert(l != NULL);
 	xassert(f != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	for (p = l->head; (*max == -1 || n < *max) && p; p = p->next) {
 		n++;
@@ -479,21 +546,23 @@ int list_for_each_max(List l, int *max, ListForF f, void *arg,
 	return n;
 }
 
-/* list_flush()
- */
-int
-list_flush (List l)
+int list_flush(List l)
+{
+	return list_flush_max(l, -1);
+}
+
+int list_flush_max(List l, int max)
 {
 	ListNode *pp;
 	void *v;
 	int n = 0;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	pp = &l->head;
-	while (*pp) {
+	for (int i = 0; (max < 0 || i < max) && *pp; i++) {
 		if ((v = _list_node_destroy(l, pp))) {
 			if (l->fDel)
 				l->fDel(v);
@@ -514,8 +583,8 @@ list_push (List l, void *x)
 
 	xassert(l != NULL);
 	xassert(x != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = _list_node_create(l, &l->head, x);
 	slurm_mutex_unlock(&l->mutex);
@@ -575,7 +644,48 @@ list_sort(List l, ListCmpF f)
 	 * to the head of the list.
 	 */
 	for (i = l->iNext; i; i = i->iNext) {
-		xassert(i->magic == LIST_MAGIC);
+		xassert(i->magic == LIST_ITR_MAGIC);
+		i->pos = i->list->head;
+		i->prev = &i->list->head;
+	}
+
+	slurm_mutex_unlock(&l->mutex);
+}
+
+/*
+ * list_flip - not called list_reverse due to collision with MariaDB
+ */
+void list_flip(List l)
+{
+	ListNode old_head, prev = NULL, curr, next = NULL;
+	ListIterator i;
+
+	xassert(l);
+	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
+
+	if (l->count <= 1) {
+		slurm_mutex_unlock(&l->mutex);
+		return;
+	}
+
+	old_head = curr = l->head;
+	while (curr) {
+		next = curr->next;
+		curr->next = prev;
+		prev = curr;
+		curr = next;
+	}
+	l->head = prev;
+	l->tail = &old_head->next;
+	l->tail_ptr = old_head;
+
+	/*
+	 * Reset all iterators on the list to point
+	 * to the head of the list.
+	 */
+	for (i = l->iNext; i; i = i->iNext) {
+		xassert(i->magic == LIST_ITR_MAGIC);
 		i->pos = i->list->head;
 		i->prev = &i->list->head;
 	}
@@ -591,8 +701,8 @@ list_pop (List l)
 	void *v;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = _list_pop_locked(l);
 	slurm_mutex_unlock(&l->mutex);
@@ -608,8 +718,8 @@ list_peek (List l)
 	void *v;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = (l->head) ? l->head->data : NULL;
 	slurm_mutex_unlock(&l->mutex);
@@ -617,6 +727,24 @@ list_peek (List l)
 	return v;
 }
 
+/*
+ * list_peek_last()
+ */
+void *list_peek_last(List l)
+{
+	void *v;
+
+	xassert(l != NULL);
+	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
+
+	v = l->tail_ptr ? l->tail_ptr->data : NULL;
+
+	slurm_mutex_unlock(&l->mutex);
+
+	return v;
+}
+
 /* list_enqueue()
  */
 void *
@@ -626,8 +754,8 @@ list_enqueue (List l, void *x)
 
 	xassert(l != NULL);
 	xassert(x != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = _list_node_create(l, l->tail, x);
 	slurm_mutex_unlock(&l->mutex);
@@ -643,8 +771,8 @@ list_dequeue (List l)
 	void *v;
 
 	xassert(l != NULL);
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	v = _list_node_destroy(l, &l->head);
 	slurm_mutex_unlock(&l->mutex);
@@ -662,15 +790,15 @@ list_iterator_create (List l)
 	xassert(l != NULL);
 	i = list_iterator_alloc();
 
+	i->magic = LIST_ITR_MAGIC;
 	i->list = l;
-	slurm_mutex_lock(&l->mutex);
 	xassert(l->magic == LIST_MAGIC);
+	slurm_mutex_lock(&l->mutex);
 
 	i->pos = l->head;
 	i->prev = &l->head;
 	i->iNext = l->iNext;
 	l->iNext = i;
-	xassert((i->magic = LIST_MAGIC));      /* set magic via assert abuse */
 
 	slurm_mutex_unlock(&l->mutex);
 
@@ -683,9 +811,9 @@ void
 list_iterator_reset (ListIterator i)
 {
 	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->magic == LIST_ITR_MAGIC);
 	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
 
 	i->pos = i->list->head;
 	i->prev = &i->list->head;
@@ -701,12 +829,12 @@ list_iterator_destroy (ListIterator i)
 	ListIterator *pi;
 
 	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->magic == LIST_ITR_MAGIC);
 	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
 
 	for (pi = &i->list->iNext; *pi; pi = &(*pi)->iNext) {
-		xassert((*pi)->magic == LIST_MAGIC);
+		xassert((*pi)->magic == LIST_ITR_MAGIC);
 		if (*pi == i) {
 			*pi = (*pi)->iNext;
 			break;
@@ -714,30 +842,38 @@ list_iterator_destroy (ListIterator i)
 	}
 	slurm_mutex_unlock(&i->list->mutex);
 
-	xassert((i->magic = ~LIST_MAGIC));     /* clear magic via assert abuse */
+	i->magic = ~LIST_ITR_MAGIC;
 	list_iterator_free(i);
 }
 
-/* list_next()
- */
-void *
-list_next (ListIterator i)
+static void * _list_next_locked(ListIterator i)
 {
 	ListNode p;
 
-	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
-	xassert(i->list->magic == LIST_MAGIC);
-
 	if ((p = i->pos))
 		i->pos = p->next;
 	if (*i->prev != p)
 		i->prev = &(*i->prev)->next;
 
+	return (p ? p->data : NULL);
+}
+
+/* list_next()
+ */
+void *list_next (ListIterator i)
+{
+	void *rc;
+
+	xassert(i != NULL);
+	xassert(i->magic == LIST_ITR_MAGIC);
+	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
+
+	rc = _list_next_locked(i);
+
 	slurm_mutex_unlock(&i->list->mutex);
 
-	return (p ? p->data : NULL);
+	return rc;
 }
 
 /* list_peek_next()
@@ -748,9 +884,9 @@ list_peek_next (ListIterator i)
 	ListNode p;
 
 	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->magic == LIST_ITR_MAGIC);
 	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
 
 	p = i->pos;
 
@@ -768,9 +904,9 @@ list_insert (ListIterator i, void *x)
 
 	xassert(i != NULL);
 	xassert(x != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->magic == LIST_ITR_MAGIC);
 	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
 
 	v = _list_node_create(i->list, i->prev, x);
 	slurm_mutex_unlock(&i->list->mutex);
@@ -788,9 +924,14 @@ list_find (ListIterator i, ListFindF f, void *key)
 	xassert(i != NULL);
 	xassert(f != NULL);
 	xassert(key != NULL);
-	xassert(i->magic == LIST_MAGIC);
+	xassert(i->magic == LIST_ITR_MAGIC);
 
-	while ((v = list_next(i)) && !f(v,key)) {;}
+	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->list->magic == LIST_MAGIC);
+
+	while ((v = _list_next_locked(i)) && !f(v, key)) {;}
+
+	slurm_mutex_unlock(&i->list->mutex);
 
 	return v;
 }
@@ -803,9 +944,9 @@ list_remove (ListIterator i)
 	void *v = NULL;
 
 	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
-	slurm_mutex_lock(&i->list->mutex);
+	xassert(i->magic == LIST_ITR_MAGIC);
 	xassert(i->list->magic == LIST_MAGIC);
+	slurm_mutex_lock(&i->list->mutex);
 
 	if (*i->prev != i->pos)
 		v = _list_node_destroy(i->list, i->prev);
@@ -822,7 +963,7 @@ list_delete_item (ListIterator i)
 	void *v;
 
 	xassert(i != NULL);
-	xassert(i->magic == LIST_MAGIC);
+	xassert(i->magic == LIST_ITR_MAGIC);
 
 	if ((v = list_remove(i))) {
 		if (i->list->fDel)
@@ -853,13 +994,16 @@ static void *_list_node_create(List l, ListNode *pp, void *x)
 	p = list_node_alloc();
 
 	p->data = x;
-	if (!(p->next = *pp))
+	if (!(p->next = *pp)) {
 		l->tail = &p->next;
+		l->tail_ptr = p;
+	}
+
 	*pp = p;
 	l->count++;
 
 	for (i = l->iNext; i; i = i->iNext) {
-		xassert(i->magic == LIST_MAGIC);
+		xassert(i->magic == LIST_ITR_MAGIC);
 		if (i->prev == pp)
 			i->prev = &p->next;
 		else if (i->pos == p->next)
@@ -893,12 +1037,14 @@ static void *_list_node_destroy(List l, ListNode *pp)
 		return NULL;
 
 	v = p->data;
-	if (!(*pp = p->next))
+	if (!(*pp = p->next)) {
 		l->tail = pp;
+		l->tail_ptr = p;
+	}
 	l->count--;
 
 	for (i = l->iNext; i; i = i->iNext) {
-		xassert(i->magic == LIST_MAGIC);
+		xassert(i->magic == LIST_ITR_MAGIC);
 		if (i->pos == p)
 			i->pos = p->next, i->prev = pp;
 		else if (i->prev == &p->next)
