@@ -54,6 +54,7 @@
 #  include <sys/prctl.h>
 #endif
 
+#include <arpa/inet.h>
 #include <errno.h>
 #include <poll.h>
 #include <pthread.h>
@@ -113,8 +114,6 @@ strong_alias(fatal,		slurm_fatal) __attribute__((noreturn));
 strong_alias(fatal_abort,	slurm_fatal_abort) __attribute__((noreturn));
 strong_alias(error,		slurm_error);
 strong_alias(spank_log,		slurm_spank_log);
-strong_alias(info,		slurm_info);
-strong_alias(verbose,		slurm_verbose);
 strong_alias(sched_error,	slurm_sched_error);
 strong_alias(sched_info,	slurm_sched_info);
 strong_alias(sched_verbose,	slurm_sched_verbose);
@@ -132,7 +131,6 @@ typedef struct {
 	log_options_t opt;
 	unsigned initialized:1;
 	uint16_t fmt;            /* Flag for specifying timestamp format */
-	uint64_t debug_flags;
 }	log_t;
 
 char *slurm_prog_name = NULL;
@@ -268,7 +266,8 @@ static int _fd_writeable(int fd)
 	if ((ufds.revents & POLLHUP) || fstat(fd, &stat_buf) ||
 	    ((S_ISSOCK(stat_buf.st_mode) &&
 	     (rc = recv(fd, &temp, 1, MSG_DONTWAIT) <= 0) &&
-	     ((rc == 0) || ((errno != EAGAIN) && (errno != EWOULDBLOCK))))))
+	     ((rc == 0) ||
+	      (errno && (errno != EAGAIN) && (errno != EWOULDBLOCK))))))
 		return -1;
 	else if ((ufds.revents & POLLNVAL)
 		 || (ufds.revents & POLLERR)
@@ -582,23 +581,9 @@ int log_alter(log_options_t opt, log_facility_t fac, char *logfile)
 	slurm_mutex_lock(&log_lock);
 	rc = _log_init(NULL, opt, fac, logfile);
 	slurm_mutex_unlock(&log_lock);
-	log_set_debug_flags();
 	return rc;
 }
 
-/* log_set_debug_flags()
- * Set or reset the debug flags based on the configuration
- * file or the scontrol command.
- */
-void log_set_debug_flags(void)
-{
-	uint64_t debug_flags = slurm_get_debug_flags();
-
-	slurm_mutex_lock(&log_lock);
-	log->debug_flags = debug_flags;
-	slurm_mutex_unlock(&log_lock);
-}
-
 /* reinitialize log data structures. Like log_init, but do not init
  * the log mutex
  */
@@ -691,7 +676,7 @@ static void _set_idbuf(char *idbuf, size_t size)
 	gettimeofday(&now, NULL);
 #if HAVE_SYS_PRCTL_H
 	if (prctl(PR_GET_NAME, thread_name, NULL, NULL, NULL) < 0) {
-		error("failed to get thread name: %m");
+		fprintf(stderr, "failed to get thread name: %m\n");
 		max_len = 0;
 		thread_name[0] = '\0';
 	}
@@ -707,6 +692,38 @@ static void _set_idbuf(char *idbuf, size_t size)
 		 thread_name, (void *) pthread_self());
 }
 
+/*
+ * _addr2fmt() - print an IP address from slurm_addr_t
+ */
+static char *_addr2fmt(slurm_addr_t *addr_ptr, char *buf, int buf_size)
+{
+	struct sockaddr_in *in = (struct sockaddr_in *) addr_ptr;
+	char addrbuf[INET6_ADDRSTRLEN];
+	uint16_t port = 0;
+
+	/*
+	 * NOTE: You will notice we put a %.0s in front of the string.
+	 * This is to handle the fact that we can't remove the addr_ptr
+	 * argument from the va_list directly. So when we call vsnprintf()
+	 * to handle the va_list this will effectively skip this argument.
+	 */
+	if (addr_ptr == NULL)
+		return "%.0sNULL";
+
+	if (addr_ptr->ss_family == AF_INET6) {
+		struct sockaddr_in6 *in6 = (struct sockaddr_in6 *) addr_ptr;
+		inet_ntop(AF_INET6, &in6->sin6_addr, addrbuf, INET6_ADDRSTRLEN);
+		port = ntohs(in6->sin6_port);
+		snprintf(buf, buf_size, "[%%.0s%s]:%d", addrbuf, port);
+		return buf;
+	}
+
+	inet_ntop(AF_INET, &in->sin_addr, addrbuf, INET_ADDRSTRLEN);
+	port = ntohs(in->sin_port);
+	snprintf(buf, buf_size, "%%.0s%s:%d", addrbuf, port);
+
+	return buf;
+}
 /*
  * jobid2fmt() - print a job ID as "JobId=..." including, as applicable,
  * the job array or hetjob component information with the raw jobid in
@@ -758,36 +775,11 @@ static char *_stepid2fmt(step_record_t *step_ptr, char *buf, int buf_size)
 	if (step_ptr->magic != STEP_MAGIC)
 		return " StepId=CORRUPT";
 
-	if (step_ptr->step_id == SLURM_EXTERN_CONT) {
-		return " StepId=Extern";
-	} else if (step_ptr->step_id == SLURM_BATCH_SCRIPT) {
-		return " StepId=Batch";
-	} else if (step_ptr->step_id == SLURM_PENDING_STEP) {
-		return " StepId=TBD";
-	} else {
-		snprintf(buf, buf_size, " StepId=%u", step_ptr->step_id);
-	}
-
-	return buf;
+	return log_build_step_id_str(&step_ptr->step_id, buf, buf_size,
+				     STEP_ID_FLAG_SPACE | STEP_ID_FLAG_NO_JOB);
 }
 
-/*
- * return a heap allocated string formed from fmt and ap arglist
- * returned string is allocated with xmalloc, so must free with xfree.
- *
- * args are like printf, with the addition of the following format chars:
- * - %m expands to strerror(errno)
- * - %M expand to time stamp, format is configuration dependent
- * - %pJ expands to "JobId=XXXX" for the given job_ptr, with the appropriate
- *       format for job arrays and hetjob components.
- * - %pS expands to "JobId=XXXX StepId=YYYY" for a given step_ptr.
- * - %t expands to strftime("%x %X") [ locally preferred short date/time ]
- * - %T expands to rfc2822 date time  [ "dd, Mon yyyy hh:mm:ss GMT offset" ]
- *
- * these formats are expanded first, leaving all others to be passed to
- * vsnprintf() to complete the expansion using the ap arglist.
- */
-static char *vxstrfmt(const char *fmt, va_list ap)
+extern char *vxstrfmt(const char *fmt, va_list ap)
 {
 	char	*intermediate_fmt = NULL;
 	char	*out_string = NULL;
@@ -822,7 +814,9 @@ static char *vxstrfmt(const char *fmt, va_list ap)
 				break;
 			case 'p':
 				switch (*(p + 2)) {
+				case 'A':
 				case 'J':
+				case 's':
 				case 'S':
 					is_our_format = true;
 					/*
@@ -866,6 +860,24 @@ static char *vxstrfmt(const char *fmt, va_list ap)
 			case 'p':
 				fmt++;
 				switch (*fmt) {
+				case 'A':	/* "%pA" -> "AAA.BBB.CCC.DDD:XXXX" */
+				{
+					void *ptr = NULL;
+					slurm_addr_t *addr_ptr;
+					va_list	ap_copy;
+
+					va_copy(ap_copy, ap);
+					for (int i = 0; i < cnt; i++ )
+						ptr = va_arg(ap_copy, void *);
+					addr_ptr = ptr;
+					xstrcat(intermediate_fmt,
+						_addr2fmt(
+							addr_ptr,
+							substitute_on_stack,
+							sizeof(substitute_on_stack)));
+					va_end(ap_copy);
+					break;
+				}
 				case 'J':	/* "%pJ" => "JobId=..." */
 				{
 					int i;
@@ -885,7 +897,34 @@ static char *vxstrfmt(const char *fmt, va_list ap)
 					va_end(ap_copy);
 					break;
 				}
-				/* "%pS" => "JobId=... StepId=..." */
+				/*
+				 * "%ps" => "StepId=... " on a
+				 * slurm_step_id_t
+				 */
+				case 's':
+				{
+					int i;
+					void *ptr = NULL;
+					slurm_step_id_t *step_id = NULL;
+					va_list	ap_copy;
+
+					va_copy(ap_copy, ap);
+					for (i = 0; i < cnt; i++ )
+						ptr = va_arg(ap_copy, void *);
+					step_id = ptr;
+					xstrcat(intermediate_fmt,
+						log_build_step_id_str(
+							step_id,
+							substitute_on_stack,
+							sizeof(substitute_on_stack),
+							STEP_ID_FLAG_PS));
+					va_end(ap_copy);
+					break;
+				}
+				/*
+				 * "%pS" => "JobId=... StepId=..." on a
+				 * step_record_t
+				 */
 				case 'S':
 				{
 					int i;
@@ -1136,6 +1175,7 @@ static void _log_msg(log_level_t level, bool sched, bool spank, const char *fmt,
 	char *pfx = "";
 	char *buf = NULL;
 	char *msgbuf = NULL;
+	char *eol = "\n";
 	int priority = LOG_INFO;
 
 	slurm_mutex_lock(&log_lock);
@@ -1145,6 +1185,9 @@ static void _log_msg(log_level_t level, bool sched, bool spank, const char *fmt,
 		_log_init(NULL, opts, 0, NULL);
 	}
 
+	if (log->opt.raw)
+		eol = "\r\n";
+
 	if (SCHED_LOG_INITIALIZED && sched &&
 	    (highest_sched_log_level > LOG_LEVEL_QUIET)) {
 		buf = vxstrfmt(fmt, args);
@@ -1223,15 +1266,15 @@ static void _log_msg(log_level_t level, bool sched, bool spank, const char *fmt,
 
 		fflush(stdout);
 		if (spank) {
-			_log_printf(log, log->buf, stderr, "%s\n", buf);
+			_log_printf(log, log->buf, stderr, "%s%s", buf, eol);
 		} else if (log->fmt == LOG_FMT_THREAD_ID) {
 			char tmp[64];
 			_set_idbuf(tmp, sizeof(tmp));
-			_log_printf(log, log->buf, stderr, "%s: %s%s\n",
-			            tmp, pfx, buf);
+			_log_printf(log, log->buf, stderr, "%s: %s%s%s",
+			            tmp, pfx, buf, eol);
 		} else {
-			_log_printf(log, log->buf, stderr, "%s: %s%s\n",
-			            log->argv0, pfx, buf);
+			_log_printf(log, log->buf, stderr, "%s: %s%s%s",
+			            log->argv0, pfx, buf, eol);
 		}
 		fflush(stderr);
 	}
@@ -1362,12 +1405,12 @@ void spank_log(const char *fmt, ...)
 	va_end(ap);
 }
 
-void info(const char *fmt, ...)
+void slurm_info(const char *fmt, ...)
 {
 	LOG_MACRO(LOG_LEVEL_INFO, false, fmt);
 }
 
-void verbose(const char *fmt, ...)
+void slurm_verbose(const char *fmt, ...)
 {
 	LOG_MACRO(LOG_LEVEL_VERBOSE, false, fmt);
 }
@@ -1434,3 +1477,100 @@ extern int get_sched_log_level(void)
 {
 	return MAX(highest_log_level, highest_sched_log_level);
 }
+
+/*
+ * log_build_step_id_str() - print a slurm_step_id_t as " StepId=...", with
+ * Batch and Extern used as appropriate.
+ */
+extern char *log_build_step_id_str(
+	slurm_step_id_t *step_id, char *buf, int buf_size, uint16_t flags)
+{
+	int pos = 0;
+
+	xassert(buf);
+	xassert(buf_size > 1);
+
+	buf[pos] = '\0';
+
+	if (flags & STEP_ID_FLAG_SPACE)
+		buf[pos++] = ' ';
+
+	/*
+	 * NOTE: You will notice we put a %.0s in front of the string if running
+	 * with %ps like interactions.
+	 * This is to handle the fact that we can't remove the step_id
+	 * argument from the va_list directly. So when we call vsnprintf()
+	 * to handle the va_list this will effectively skip this argument.
+	 */
+	if (flags & STEP_ID_FLAG_PS)
+		pos += snprintf(buf + pos, buf_size - pos, "%%.0s");
+
+	if (!(flags & STEP_ID_FLAG_NO_PREFIX))
+		pos += snprintf(buf + pos, buf_size - pos, "%s",
+				(!step_id || (step_id->step_id != NO_VAL)) ?
+				"StepId=" : "JobId=");
+
+	if (!step_id || !step_id->job_id) {
+		snprintf(buf + pos, buf_size - pos, "Invalid");
+		return buf;
+	}
+
+	if (step_id->job_id && !(flags & STEP_ID_FLAG_NO_JOB))
+		pos += snprintf(buf + pos, buf_size - pos,
+				"%u%s", step_id->job_id,
+				step_id->step_id == NO_VAL ? "" : ".");
+
+	if ((pos >= buf_size) || (step_id->step_id == NO_VAL))
+		return buf;
+
+	if (step_id->step_id == SLURM_BATCH_SCRIPT)
+		pos += snprintf(buf + pos, buf_size - pos, "batch");
+	else if (step_id->step_id == SLURM_EXTERN_CONT)
+		pos += snprintf(buf + pos, buf_size - pos, "extern");
+	else if (step_id->step_id == SLURM_INTERACTIVE_STEP)
+		pos += snprintf(buf + pos, buf_size - pos, "interactive");
+	else if (step_id->step_id == SLURM_PENDING_STEP)
+		pos += snprintf(buf + pos, buf_size - pos, "TDB");
+	else
+		pos += snprintf(buf + pos, buf_size - pos, "%u",
+				step_id->step_id);
+
+	if (pos >= buf_size)
+		return buf;
+
+	if (step_id->step_het_comp != NO_VAL)
+		snprintf(buf + pos, buf_size - pos, "+%u",
+			 step_id->step_het_comp);
+
+	return buf;
+}
+
+extern void _log_flag_hex(const void *data, size_t len, const char *fmt, ...)
+{
+	va_list ap;
+	char *prepend;
+	static const int hex_cols = 16, hex_rows = 16;
+
+	if (!data || !len)
+		return;
+
+	va_start(ap, fmt);
+	prepend = vxstrfmt(fmt, ap);
+	va_end(ap);
+
+	for (int i = 0; (i < len) && (i < (hex_cols * hex_rows)); ) {
+		int remain = len - i;
+		int print = (remain < hex_cols) ? remain : hex_cols;
+		char *phex = bytes_to_hex((data + i), print, " ");
+		char *pstr = bytes_to_printable((data + i), print, '.');
+
+		format_print(LOG_LEVEL_VERBOSE, "%s [%04d/%04zu] 0x%s \"%s\"",
+			     prepend, i, len, phex, pstr);
+
+		i += print;
+		xfree(phex);
+		xfree(pstr);
+	}
+
+	xfree(prepend);
+}
