@@ -68,7 +68,6 @@ const char *node_select_syms[] = {
 	"select_p_state_save",
 	"select_p_state_restore",
 	"select_p_job_init",
-	"select_p_node_ranking",
 	"select_p_node_init",
 	"select_p_job_test",
 	"select_p_job_begin",
@@ -76,7 +75,6 @@ const char *node_select_syms[] = {
 	"select_p_job_expand",
 	"select_p_job_resized",
 	"select_p_job_signal",
-	"select_p_job_mem_confirm",
 	"select_p_job_fini",
 	"select_p_job_suspend",
 	"select_p_job_resume",
@@ -118,26 +116,41 @@ typedef struct _plugin_args {
 	char *default_plugin;
 } _plugin_args_t;
 
-static char *_plugin_id2name(int plugin_id)
-{
-	static char id_str[16];
+typedef struct {
+	int id;
+	char *name;
+} plugin_id_name;
 
-	if (plugin_id == SELECT_PLUGIN_CONS_RES)
-		return "cons_res";
-	if (plugin_id == SELECT_PLUGIN_LINEAR)
-		return "linear";
-	if (plugin_id == SELECT_PLUGIN_SERIAL)
-		return "serial";
-	if (plugin_id == SELECT_PLUGIN_CRAY_LINEAR)
-		return "cray_aries+linear";
-	if (plugin_id == SELECT_PLUGIN_CRAY_CONS_RES)
-		return "cray_aries+cons_res";
-	if (plugin_id == SELECT_PLUGIN_CONS_TRES)
-		return "cons_tres";
-	if (plugin_id == SELECT_PLUGIN_CRAY_CONS_TRES)
-		return "cray_aries+cons_tres";
-	snprintf(id_str, sizeof(id_str), "%d", plugin_id);
-	return id_str;
+const plugin_id_name plugin_ids[] = {
+	{ SELECT_PLUGIN_CONS_RES, "cons_res" },
+	{ SELECT_PLUGIN_LINEAR, "linear" },
+	{ SELECT_PLUGIN_SERIAL, "serial" },
+	{ SELECT_PLUGIN_CRAY_LINEAR, "cray_aries+linear" },
+	{ SELECT_PLUGIN_CRAY_CONS_RES, "cray_aries+cons_res" },
+	{ SELECT_PLUGIN_CONS_TRES, "cons_tres" },
+	{ SELECT_PLUGIN_CRAY_CONS_TRES, "cray_aries+cons_tres" },
+};
+
+extern char *select_plugin_id_to_string(int plugin_id)
+{
+	for (int i = 0; i < ARRAY_SIZE(plugin_ids); i++)
+		if (plugin_id == plugin_ids[i].id)
+			return plugin_ids[i].name;
+
+	error("%s: unknown select plugin id: %d",
+	      __func__, plugin_id);
+	return NULL;
+}
+
+extern int select_string_to_plugin_id(const char *plugin)
+{
+	for (int i = 0; i < ARRAY_SIZE(plugin_ids); i++)
+		if (xstrcasecmp(plugin, plugin_ids[i].name))
+			return plugin_ids[i].id;
+
+	error("%s: unknown select plugin: %s",
+	      __func__, plugin);
+	return 0;
 }
 
 static int _load_plugins(void *x, void *arg)
@@ -200,13 +213,6 @@ extern int slurm_select_init(bool only_default)
 			      select_type);
 			fatal("Use SelectType=select/cray_aries");
 		}
-#else
-		/* if (!xstrcasecmp(select_type, "select/cray")) { */
-		/* 	fatal("Requested SelectType=select/cray " */
-		/* 	      "in slurm.conf, but not running on a native Cray " */
-		/* 	      "system.  If looking to run on a Cray " */
-		/* 	      "system natively use --enable-native-cray."); */
-		/* } */
 #endif
 	}
 
@@ -256,7 +262,7 @@ done:
 	slurm_mutex_unlock( &select_context_lock );
 	if (!working_cluster_rec) {
 		if (select_running_linear_based()) {
-			uint16_t cr_type = slurm_get_select_type_param();
+			uint16_t cr_type = slurm_conf.select_type_param;
 			if (cr_type & (CR_CPU | CR_CORE | CR_SOCKET)) {
 				fatal("Invalid SelectTypeParameters for "
 				      "%s: %s (%u), it can't contain "
@@ -319,7 +325,7 @@ again:
 		     (plugin_id == SELECT_PLUGIN_CRAY_CONS_TRES) ||
 		     (plugin_id == SELECT_PLUGIN_CRAY_LINEAR))) {
 			char *type = "select", *name = "select/cray_aries";
-			uint16_t save_params = slurm_get_select_type_param();
+			uint16_t save_params = slurm_conf.select_type_param;
 			uint16_t params[2];
 			int cray_plugin_id[2], cray_offset;
 
@@ -355,14 +361,14 @@ again:
 				goto end_it;	/* No match */
 
 			slurm_mutex_lock(&select_context_lock);
-			slurm_set_select_type_param(params[cray_offset]);
+			slurm_conf.select_type_param = params[cray_offset];
 			plugin_context_destroy(select_context[i]);
 			select_context[i] =
 				plugin_context_create(type, name,
 						      (void **)&ops[i],
 						      node_select_syms,
 						      sizeof(node_select_syms));
-			slurm_set_select_type_param(save_params);
+			slurm_conf.select_type_param = save_params;
 			slurm_mutex_unlock(&select_context_lock);
 			goto again;
 		}
@@ -455,21 +461,6 @@ extern int select_g_job_init(List job_list)
 		(job_list);
 }
 
-/*
- * Assign a 'node_rank' value to each of the node_ptr entries.
- * IN node_ptr - current node data
- * IN node_count - number of node entries
- * Return true if node ranking was performed, false if not.
- */
-extern bool select_g_node_ranking(node_record_t *node_ptr, int node_cnt)
-{
-	if (slurm_select_init(0) < 0)
-		return SLURM_ERROR;
-
-	return (*(ops[select_context_default].node_ranking))
-		(node_ptr, node_cnt);
-}
-
 /*
  * Note re/initialization of node record data structure
  * IN node_ptr - current node data
@@ -596,21 +587,6 @@ extern int select_g_job_signal(job_record_t *job_ptr, int signal)
 		(job_ptr, signal);
 }
 
-/*
- * Confirm that a job's memory allocation is still valid after a node is
- * restarted. This is an issue if the job is allocated all of the memory on a
- * node and that node is restarted with a different memory size than at the time
- * it is allocated to the job. This would mostly be an issue on an Intel KNL
- * node where the memory size would vary with the MCDRAM cache mode.
- */
-extern int select_g_job_mem_confirm(job_record_t *job_ptr)
-{
-	if (slurm_select_init(0) < 0)
-		return SLURM_ERROR;
-
-	return (*(ops[select_context_default].job_mem_confirm)) (job_ptr);
-}
-
 /*
  * Note termination of job is starting. Executed from slurmctld.
  * IN job_ptr - pointer to job being terminated
@@ -713,7 +689,7 @@ extern int select_g_step_finish(step_record_t *step_ptr, bool killing_step)
 }
 
 extern int select_g_select_nodeinfo_pack(dynamic_plugin_data_t *nodeinfo,
-					 Buf buffer,
+					 buf_t *buffer,
 					 uint16_t protocol_version)
 {
 	void *data = NULL;
@@ -741,7 +717,7 @@ extern int select_g_select_nodeinfo_pack(dynamic_plugin_data_t *nodeinfo,
 }
 
 extern int select_g_select_nodeinfo_unpack(dynamic_plugin_data_t **nodeinfo,
-					   Buf buffer,
+					   buf_t *buffer,
 					   uint16_t protocol_version)
 {
 	dynamic_plugin_data_t *nodeinfo_ptr = NULL;
@@ -758,7 +734,7 @@ extern int select_g_select_nodeinfo_unpack(dynamic_plugin_data_t **nodeinfo,
 		safe_unpack32(&plugin_id, buffer);
 		if ((i = select_get_plugin_id_pos(plugin_id)) == SLURM_ERROR) {
 			error("%s: select plugin %s not found", __func__,
-			      _plugin_id2name(plugin_id));
+			      select_plugin_id_to_string(plugin_id));
 			goto unpack_error;
 		} else {
 			 nodeinfo_ptr->plugin_id = i;
@@ -978,7 +954,7 @@ extern dynamic_plugin_data_t *select_g_select_jobinfo_copy(
  * RET         - slurm error code
  */
 extern int select_g_select_jobinfo_pack(dynamic_plugin_data_t *jobinfo,
-					Buf buffer,
+					buf_t *buffer,
 					uint16_t protocol_version)
 {
 	void *data = NULL;
@@ -1010,7 +986,7 @@ extern int select_g_select_jobinfo_pack(dynamic_plugin_data_t *jobinfo,
  * NOTE: returned value must be freed using select_g_free_jobinfo
  */
 extern int select_g_select_jobinfo_unpack(dynamic_plugin_data_t **jobinfo,
-					  Buf buffer,
+					  buf_t *buffer,
 					  uint16_t protocol_version)
 {
 	dynamic_plugin_data_t *jobinfo_ptr = NULL;
@@ -1027,7 +1003,7 @@ extern int select_g_select_jobinfo_unpack(dynamic_plugin_data_t **jobinfo,
 		safe_unpack32(&plugin_id, buffer);
 		if ((i = select_get_plugin_id_pos(plugin_id)) == SLURM_ERROR) {
 			error("%s: select plugin %s not found", __func__,
-			      _plugin_id2name(plugin_id));
+			      select_plugin_id_to_string(plugin_id));
 			goto unpack_error;
 		} else
 			jobinfo_ptr->plugin_id = i;
