@@ -73,8 +73,6 @@ typedef struct slurm_select_ops {
 	int		(*state_save)		(char *dir_name);
 	int		(*state_restore)	(char *dir_name);
 	int		(*job_init)		(List job_list);
-	int		(*node_ranking)		(node_record_t *node_ptr,
-						 int node_cnt);
 	int		(*node_init)		(node_record_t *node_ptr,
 						 int node_cnt);
 	int		(*job_test)		(job_record_t *job_ptr,
@@ -94,7 +92,6 @@ typedef struct slurm_select_ops {
 						 node_record_t *node_ptr);
 	int		(*job_signal)		(job_record_t *job_ptr,
 						 int signal);
-	int		(*job_mem_confirm)	(job_record_t *job_ptr);
 	int		(*job_fini)		(job_record_t *job_ptr);
 	int		(*job_suspend)		(job_record_t *job_ptr,
 						 bool indf_susp);
@@ -108,10 +105,10 @@ typedef struct slurm_select_ops {
 	int             (*step_finish)          (step_record_t *step_ptr,
 						 bool killing_step);
 	int		(*nodeinfo_pack)	(select_nodeinfo_t *nodeinfo,
-						 Buf buffer,
+						 buf_t *buffer,
 						 uint16_t protocol_version);
 	int		(*nodeinfo_unpack)	(select_nodeinfo_t **nodeinfo,
-						 Buf buffer,
+						 buf_t *buffer,
 						 uint16_t protocol_version);
 	select_nodeinfo_t *(*nodeinfo_alloc)	(void);
 	int		(*nodeinfo_free)	(select_nodeinfo_t *nodeinfo);
@@ -134,10 +131,10 @@ typedef struct slurm_select_ops {
 						 void *data);
 	select_jobinfo_t *(*jobinfo_copy)	(select_jobinfo_t *jobinfo);
 	int		(*jobinfo_pack)		(select_jobinfo_t *jobinfo,
-						 Buf buffer,
+						 buf_t *buffer,
 						 uint16_t protocol_version);
 	int		(*jobinfo_unpack)	(select_jobinfo_t **jobinfo_pptr,
-						 Buf buffer,
+						 buf_t *buffer,
 						 uint16_t protocol_version);
 	char *		(*jobinfo_sprint)	(select_jobinfo_t *jobinfo,
 						 char *buf, size_t size,
@@ -167,6 +164,17 @@ extern const char *node_select_syms[];
  * '0' = 0; '9' = 9; 'A' = 10; etc. */
 extern int select_char2coord(char coord);
 
+/*
+ * Give string name for plugin_id
+ */
+extern char *select_plugin_id_to_string(int plugin_id);
+
+/*
+ * Convert string name to plugin_id
+ * RET plugin_id or 0 on failure
+ */
+extern int select_string_to_plugin_id(const char *plugin);
+
 /*******************************************\
  * GLOBAL SELECT STATE MANAGEMENT FUNCIONS *
 \*******************************************/
@@ -244,7 +252,7 @@ extern dynamic_plugin_data_t *select_g_select_nodeinfo_alloc(void);
  * IN protocol_version - Version used for packing the record
  */
 extern int select_g_select_nodeinfo_pack(dynamic_plugin_data_t *nodeinfo,
-					 Buf buffer,
+					 buf_t *buffer,
 					 uint16_t protocol_version);
 
 /*
@@ -257,7 +265,7 @@ extern int select_g_select_nodeinfo_pack(dynamic_plugin_data_t *nodeinfo,
  * returned value
  */
 extern int select_g_select_nodeinfo_unpack(dynamic_plugin_data_t **nodeinfo,
-					   Buf buffer,
+					   buf_t *buffer,
 					   uint16_t protocol_version);
 
 /* Free the memory allocated for a select plugin node record */
@@ -292,14 +300,6 @@ extern int select_g_select_nodeinfo_get(dynamic_plugin_data_t *nodeinfo,
  */
 extern int select_g_update_node_config (int index);
 
-/*
- * Assign a 'node_rank' value to each of the node_ptr entries.
- * IN node_ptr - current node data
- * IN node_count - number of node entries
- * Return true if node ranking was performed, false if not.
- */
-extern bool select_g_node_ranking(node_record_t *node_ptr, int node_cnt);
-
 /******************************************************\
  * JOB SPECIFIC SELECT CREDENTIAL MANAGEMENT FUNCIONS *
 \******************************************************/
@@ -375,7 +375,7 @@ extern int select_g_select_jobinfo_free(dynamic_plugin_data_t *jobinfo);
  * RET         - slurm error code
  */
 extern int select_g_select_jobinfo_pack(dynamic_plugin_data_t *jobinfo,
-					Buf buffer,
+					buf_t *buffer,
 					uint16_t protocol_version);
 
 /* unpack a select job credential from a buffer
@@ -386,7 +386,7 @@ extern int select_g_select_jobinfo_pack(dynamic_plugin_data_t *jobinfo,
  * NOTE: returned value must be freed using select_g_select_jobinfo_free
  */
 extern int select_g_select_jobinfo_unpack(dynamic_plugin_data_t **jobinfo,
-					  Buf buffer,
+					  buf_t *buffer,
 					  uint16_t protocol_version);
 
 /* fill in a previously allocated select job credential
@@ -482,15 +482,6 @@ extern int select_g_job_fini(job_record_t *job_ptr);
  */
 extern int select_g_job_signal(job_record_t *job_ptr, int signal);
 
-/*
- * Confirm that a job's memory allocation is still valid after a node is
- * restarted. This is an issue if the job is allocated all of the memory on a
- * node and that node is restarted with a different memory size than at the time
- * it is allocated to the job. This would mostly be an issue on an Intel KNL
- * node where the memory size would vary with the MCDRAM cache mode.
- */
-extern int select_g_job_mem_confirm(job_record_t *job_ptr);
-
 /*
  * Suspend a job. Executed from slurmctld.
  * IN job_ptr - pointer to job being suspended
