@@ -44,6 +44,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <math.h>
 #include <netinet/in.h>
 #include <stdlib.h>
 #include <string.h>
@@ -112,9 +113,9 @@ strong_alias(unpackmem_array,	slurm_unpackmem_array);
 /* Basic buffer management routines */
 /* create_buf - create a buffer with the supplied contents, contents must
  * be xalloc'ed */
-Buf create_buf(char *data, uint32_t size)
+buf_t *create_buf(char *data, uint32_t size)
 {
-	Buf my_buf;
+	buf_t *my_buf;
 
 	if (size > MAX_BUF_SIZE) {
 		error("%s: Buffer size limit exceeded (%u > %u)",
@@ -122,7 +123,7 @@ Buf create_buf(char *data, uint32_t size)
 		return NULL;
 	}
 
-	my_buf = xmalloc_nz(sizeof(struct slurm_buf));
+	my_buf = xmalloc_nz(sizeof(*my_buf));
 	my_buf->magic = BUF_MAGIC;
 	my_buf->size = size;
 	my_buf->processed = 0;
@@ -136,9 +137,9 @@ Buf create_buf(char *data, uint32_t size)
  * create_mmap_buf - create an mmap()'d read-only buffer from
  * the supplied file.
  */
-Buf create_mmap_buf(char *file)
+buf_t *create_mmap_buf(const char *file)
 {
-	Buf my_buf;
+	buf_t *my_buf;
 	int fd;
 	struct stat f_stat;
 	void *data;
@@ -161,21 +162,21 @@ Buf create_mmap_buf(char *file)
 		return NULL;
 	}
 
-	my_buf = xmalloc_nz(sizeof(struct slurm_buf));
+	my_buf = xmalloc_nz(sizeof(*my_buf));
 	my_buf->magic = BUF_MAGIC;
 	my_buf->size = f_stat.st_size;
 	my_buf->processed = 0;
 	my_buf->head = data;
 	my_buf->mmaped = true;
 
-	debug3("%s: loaded file `%s` as Buf", __func__, file);
+	debug3("%s: loaded file `%s` as buf_t", __func__, file);
 
 	return my_buf;
 }
 
 
 /* free_buf - release memory associated with a given buffer */
-void free_buf(Buf my_buf)
+void free_buf(buf_t *my_buf)
 {
 	if (!my_buf)
 		return;
@@ -189,7 +190,7 @@ void free_buf(Buf my_buf)
 }
 
 /* Grow a buffer by the specified amount */
-void grow_buf (Buf buffer, uint32_t size)
+void grow_buf(buf_t *buffer, uint32_t size)
 {
 	if (buffer->mmaped)
 		fatal_abort("attempt to grow mmap()'d buffer not supported");
@@ -204,9 +205,9 @@ void grow_buf (Buf buffer, uint32_t size)
 }
 
 /* init_buf - create an empty buffer of the given size */
-Buf init_buf(uint32_t size)
+buf_t *init_buf(uint32_t size)
 {
-	Buf my_buf;
+	buf_t *my_buf;
 
 	if (size > MAX_BUF_SIZE) {
 		error("%s: Buffer size limit exceeded (%u > %u)",
@@ -215,7 +216,7 @@ Buf init_buf(uint32_t size)
 	}
 	if (size <= 0)
 		size = BUF_SIZE;
-	my_buf = xmalloc_nz(sizeof(struct slurm_buf));
+	my_buf = xmalloc_nz(sizeof(*my_buf));
 	my_buf->magic = BUF_MAGIC;
 	my_buf->size = size;
 	my_buf->processed = 0;
@@ -226,7 +227,7 @@ Buf init_buf(uint32_t size)
 
 /* xfer_buf_data - return a pointer to the buffer's data and release the
  * buffer's structure */
-void *xfer_buf_data(Buf my_buf)
+void *xfer_buf_data(buf_t *my_buf)
 {
 	void *data_ptr;
 
@@ -244,7 +245,7 @@ void *xfer_buf_data(Buf my_buf)
  * Given a time_t in host byte order, promote it to int64_t, convert to
  * network byte order, store in buffer and adjust buffer acc'd'ngly
  */
-void pack_time(time_t val, Buf buffer)
+void pack_time(time_t val, buf_t *buffer)
 {
 	int64_t n64 = HTON_int64((int64_t) val);
 
@@ -263,7 +264,7 @@ void pack_time(time_t val, Buf buffer)
 	buffer->processed += sizeof(n64);
 }
 
-int unpack_time(time_t * valp, Buf buffer)
+int unpack_time(time_t *valp, buf_t *buffer)
 {
 	int64_t n64;
 
@@ -283,7 +284,7 @@ int unpack_time(time_t * valp, Buf buffer)
  * store in buffer, and adjust buffer counters.
  * NOTE: There is an IEEE standard format for double.
  */
-void 	packdouble(double val, Buf buffer)
+void packdouble(double val, buf_t *buffer)
 {
 	uint64_t nl;
 	union {
@@ -318,7 +319,7 @@ void 	packdouble(double val, Buf buffer)
  * store a host float at 'valp', and adjust buffer counters.
  * NOTE: There is an IEEE standard format for float.
  */
-int	unpackfloat(float *valp, Buf buffer)
+int unpackfloat(float *valp, buf_t *buffer)
 {
 	uint32_t nl;
 	union {
@@ -341,7 +342,7 @@ int	unpackfloat(float *valp, Buf buffer)
  * store in buffer, and adjust buffer counters.
  * NOTE: There is an IEEE standard format for float.
  */
-void 	packfloat(float val, Buf buffer)
+void packfloat(float val, buf_t *buffer)
 {
 	union {
 		float f;
@@ -363,7 +364,7 @@ void 	packfloat(float val, Buf buffer)
  * store a host double at 'valp', and adjust buffer counters.
  * NOTE: There is an IEEE standard format for double.
  */
-int	unpackdouble(double *valp, Buf buffer)
+int unpackdouble(double *valp, buf_t *buffer)
 {
 	uint64_t nl;
 	union {
@@ -386,7 +387,7 @@ int	unpackdouble(double *valp, Buf buffer)
 /*
  * long double has no standard format, so pass the data as a string
  */
-void 	packlongdouble(long double val, Buf buffer)
+void packlongdouble(long double val, buf_t *buffer)
 {
 	char val_str[256];
 
@@ -397,7 +398,7 @@ void 	packlongdouble(long double val, Buf buffer)
 /*
  * long double has no standard format, so pass the data as a string
  */
-int	unpacklongdouble(long double *valp, Buf buffer)
+int unpacklongdouble(long double *valp, buf_t *buffer)
 {
 	long double nl;
 	char *val_str = NULL;
@@ -411,6 +412,16 @@ int	unpacklongdouble(long double *valp, Buf buffer)
 	if (sscanf(val_str, "%Lf", &nl) != 1)
 		return SLURM_ERROR;
 
+	/*
+	 * Workaround for a flawed glibc version which printed "nan" instead of
+	 * "0.000000" for long doubles. Restoring these as NaN will corrupt
+	 * the association state, so ensure they're 0.
+	 *
+	 * https://bugzilla.redhat.com/show_bug.cgi?id=1925204
+	 */
+	if (isnan(nl))
+		nl = 0L;
+
 	*valp = nl;
 	return SLURM_SUCCESS;
 }
@@ -419,7 +430,7 @@ int	unpacklongdouble(long double *valp, Buf buffer)
  * Given a 64-bit integer in host byte order, convert to network byte order
  * store in buffer, and adjust buffer counters.
  */
-void pack64(uint64_t val, Buf buffer)
+void pack64(uint64_t val, buf_t *buffer)
 {
 	uint64_t nl =  HTON_uint64(val);
 
@@ -442,7 +453,7 @@ void pack64(uint64_t val, Buf buffer)
  * Given a buffer containing a network byte order 64-bit integer,
  * store a host integer at 'valp', and adjust buffer counters.
  */
-int unpack64(uint64_t * valp, Buf buffer)
+int unpack64(uint64_t *valp, buf_t *buffer)
 {
 	uint64_t nl;
 	if (remaining_buf(buffer) < sizeof(nl))
@@ -458,7 +469,7 @@ int unpack64(uint64_t * valp, Buf buffer)
  * Given a 32-bit integer in host byte order, convert to network byte order
  * store in buffer, and adjust buffer counters.
  */
-void pack32(uint32_t val, Buf buffer)
+void pack32(uint32_t val, buf_t *buffer)
 {
 	uint32_t nl = htonl(val);
 
@@ -481,7 +492,7 @@ void pack32(uint32_t val, Buf buffer)
  * Given a buffer containing a network byte order 32-bit integer,
  * store a host integer at 'valp', and adjust buffer counters.
  */
-int unpack32(uint32_t * valp, Buf buffer)
+int unpack32(uint32_t *valp, buf_t *buffer)
 {
 	uint32_t nl;
 	if (remaining_buf(buffer) < sizeof(nl))
@@ -496,7 +507,7 @@ int unpack32(uint32_t * valp, Buf buffer)
 /*
  * Given a *uint16_t, it will pack an array of size_val
  */
-void pack16_array(uint16_t * valp, uint32_t size_val, Buf buffer)
+void pack16_array(uint16_t *valp, uint32_t size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -512,7 +523,7 @@ void pack16_array(uint16_t * valp, uint32_t size_val, Buf buffer)
 /*
  * Given a int ptr, it will unpack an array of size_val
  */
-int unpack16_array(uint16_t ** valp, uint32_t * size_val, Buf buffer)
+int unpack16_array(uint16_t **valp, uint32_t *size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -532,7 +543,7 @@ int unpack16_array(uint16_t ** valp, uint32_t * size_val, Buf buffer)
 /*
  * Given a *uint32_t, it will pack an array of size_val
  */
-void pack32_array(uint32_t * valp, uint32_t size_val, Buf buffer)
+void pack32_array(uint32_t *valp, uint32_t size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -548,7 +559,7 @@ void pack32_array(uint32_t * valp, uint32_t size_val, Buf buffer)
 /*
  * Given a int ptr, it will unpack an array of size_val
  */
-int unpack32_array(uint32_t ** valp, uint32_t * size_val, Buf buffer)
+int unpack32_array(uint32_t **valp, uint32_t *size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -568,7 +579,7 @@ int unpack32_array(uint32_t ** valp, uint32_t * size_val, Buf buffer)
 /*
  * Given a *uint64_t, it will pack an array of size_val
  */
-void pack64_array(uint64_t * valp, uint32_t size_val, Buf buffer)
+void pack64_array(uint64_t *valp, uint32_t size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -581,26 +592,9 @@ void pack64_array(uint64_t * valp, uint32_t size_val, Buf buffer)
 	}
 }
 
-/*
- * Pack an array of 64b-it values as if they were 32-bit
- * Used for backwards compatibility
- */
-void pack64_array_as_32(uint64_t * valp, uint32_t size_val, Buf buffer)
-{
-	uint32_t i = 0;
-
-	xassert(valp || !size_val);
-
-	pack32(size_val, buffer);
-
-	for (i = 0; i < size_val; i++) {
-		pack32((uint32_t) *(valp + i), buffer);
-	}
-}
-
 /* Given a int ptr, it will unpack an array of size_val
  */
-int unpack64_array(uint64_t ** valp, uint32_t * size_val, Buf buffer)
+int unpack64_array(uint64_t **valp, uint32_t *size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -617,29 +611,7 @@ int unpack64_array(uint64_t ** valp, uint32_t * size_val, Buf buffer)
 	return SLURM_SUCCESS;
 }
 
-/*
- * Unpack an array of 64bit values as if they were 32bit
- * Used for backwards compatibility
- */
-int unpack64_array_from_32(uint64_t ** valp, uint32_t * size_val, Buf buffer)
-{
-	uint32_t i = 0, val32;
-
-	if (unpack32(size_val, buffer))
-		return SLURM_ERROR;
-	if ((*size_val) > MAX_ARRAY_LEN_MEDIUM)
-		return SLURM_ERROR;
-
-	*valp = xmalloc_nz((*size_val) * sizeof(uint64_t));
-	for (i = 0; i < *size_val; i++) {
-		if (unpack32(&val32, buffer))
-			return SLURM_ERROR;
-		*(*valp + i) = val32;
-	}
-	return SLURM_SUCCESS;
-}
-
-void packdouble_array(double *valp, uint32_t size_val, Buf buffer)
+void packdouble_array(double *valp, uint32_t size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -652,7 +624,7 @@ void packdouble_array(double *valp, uint32_t size_val, Buf buffer)
 	}
 }
 
-int unpackdouble_array(double **valp, uint32_t* size_val, Buf buffer)
+int unpackdouble_array(double **valp, uint32_t* size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -669,7 +641,7 @@ int unpackdouble_array(double **valp, uint32_t* size_val, Buf buffer)
 	return SLURM_SUCCESS;
 }
 
-void packlongdouble_array(long double *valp, uint32_t size_val, Buf buffer)
+void packlongdouble_array(long double *valp, uint32_t size_val, buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -682,7 +654,8 @@ void packlongdouble_array(long double *valp, uint32_t size_val, Buf buffer)
 	}
 }
 
-int unpacklongdouble_array(long double **valp, uint32_t* size_val, Buf buffer)
+int unpacklongdouble_array(long double **valp, uint32_t *size_val,
+			   buf_t *buffer)
 {
 	uint32_t i = 0;
 
@@ -705,7 +678,7 @@ int unpacklongdouble_array(long double **valp, uint32_t* size_val, Buf buffer)
  * Given a 16-bit integer in host byte order, convert to network byte order,
  * store in buffer and adjust buffer counters.
  */
-void pack16(uint16_t val, Buf buffer)
+void pack16(uint16_t val, buf_t *buffer)
 {
 	uint16_t ns = htons(val);
 
@@ -728,7 +701,7 @@ void pack16(uint16_t val, Buf buffer)
  * Given a buffer containing a network byte order 16-bit integer,
  * store a host integer at 'valp', and adjust buffer counters.
  */
-int unpack16(uint16_t * valp, Buf buffer)
+int unpack16(uint16_t *valp, buf_t *buffer)
 {
 	uint16_t ns;
 
@@ -745,7 +718,7 @@ int unpack16(uint16_t * valp, Buf buffer)
  * Given a 8-bit integer in host byte order, convert to network byte order
  * store in buffer, and adjust buffer counters.
  */
-void pack8(uint8_t val, Buf buffer)
+void pack8(uint8_t val, buf_t *buffer)
 {
 	if (remaining_buf(buffer) < sizeof(uint8_t)) {
 		if ((buffer->size + BUF_SIZE) > MAX_BUF_SIZE) {
@@ -766,7 +739,7 @@ void pack8(uint8_t val, Buf buffer)
  * Given a buffer containing a network byte order 8-bit integer,
  * store a host integer at 'valp', and adjust buffer counters.
  */
-int unpack8(uint8_t * valp, Buf buffer)
+int unpack8(uint8_t *valp, buf_t *buffer)
 {
 	if (remaining_buf(buffer) < sizeof(uint8_t))
 		return SLURM_ERROR;
@@ -780,7 +753,7 @@ int unpack8(uint8_t * valp, Buf buffer)
  * Given a boolean in host byte order, convert to network byte order
  * store in buffer, and adjust buffer counters.
  */
-void packbool(bool val, Buf buffer)
+void packbool(bool val, buf_t *buffer)
 {
 	uint8_t tmp8 = val;
 	pack8(tmp8, buffer);
@@ -790,7 +763,7 @@ void packbool(bool val, Buf buffer)
  * Given a buffer containing a network byte order 8-bit integer,
  * store a host integer at 'valp', and adjust buffer counters.
  */
-int unpackbool(bool * valp, Buf buffer)
+int unpackbool(bool *valp, buf_t *buffer)
 {
 	uint8_t tmp8 = 0;
 
@@ -848,7 +821,7 @@ extern void packmem(void *valp, uint32_t size_val, buf_t *buffer)
  * NOTE: valp is set to point into the buffer bufp, a copy of
  *	the data is not made
  */
-int unpackmem_ptr(char **valp, uint32_t * size_valp, Buf buffer)
+int unpackmem_ptr(char **valp, uint32_t *size_valp, buf_t *buffer)
 {
 	uint32_t ns;
 
@@ -883,7 +856,7 @@ int unpackmem_ptr(char **valp, uint32_t * size_valp, Buf buffer)
  *	the caller is responsible for calling xfree() on *valp
  *	if non-NULL (set to NULL on zero size buffer value)
  */
-int unpackmem_xmalloc(char **valp, uint32_t * size_valp, Buf buffer)
+int unpackmem_xmalloc(char **valp, uint32_t *size_valp, buf_t *buffer)
 {
 	uint32_t ns;
 
@@ -920,7 +893,7 @@ int unpackmem_xmalloc(char **valp, uint32_t * size_valp, Buf buffer)
  *	the caller is responsible for calling free() on *valp
  *	if non-NULL (set to NULL on zero size buffer value)
  */
-int unpackmem_malloc(char **valp, uint32_t * size_valp, Buf buffer)
+int unpackmem_malloc(char **valp, uint32_t *size_valp, buf_t *buffer)
 {
 	uint32_t ns;
 
@@ -965,7 +938,7 @@ int unpackmem_malloc(char **valp, uint32_t * size_valp, Buf buffer)
  * WARNING: These escapes are sufficient to protect MariaDB/MySQL, but
  *          may not be sufficient if databases are added in the future.
  */
-int unpackstr_xmalloc_escaped(char **valp, uint32_t *size_valp, Buf buffer)
+int unpackstr_xmalloc_escaped(char **valp, uint32_t *size_valp, buf_t *buffer)
 {
 	uint32_t ns;
 
@@ -1015,7 +988,7 @@ int unpackstr_xmalloc_escaped(char **valp, uint32_t *size_valp, Buf buffer)
 	return SLURM_SUCCESS;
 }
 
-int unpackstr_xmalloc_chooser(char **valp, uint32_t *size_valp, Buf buf)
+int unpackstr_xmalloc_chooser(char **valp, uint32_t *size_valp, buf_t *buf)
 {
 	if (slurmdbd_conf)
 		return unpackstr_xmalloc_escaped(valp, size_valp, buf);
@@ -1029,7 +1002,7 @@ int unpackstr_xmalloc_chooser(char **valp, uint32_t *size_valp, Buf buf)
  * (size_val), convert size_val to network byte order and store in the
  * buffer followed by the data at valp. Adjust buffer counters.
  */
-void packstr_array(char **valp, uint32_t size_val, Buf buffer)
+void packstr_array(char **valp, uint32_t size_val, buf_t *buffer)
 {
 	int i;
 	uint32_t ns = htonl(size_val);
@@ -1062,7 +1035,7 @@ void packstr_array(char **valp, uint32_t size_val, Buf buffer)
  *	the caller is responsible for calling xfree on *valp
  *	if non-NULL (set to NULL on zero size buffer value)
  */
-int unpackstr_array(char ***valp, uint32_t * size_valp, Buf buffer)
+int unpackstr_array(char ***valp, uint32_t *size_valp, buf_t *buffer)
 {
 	int i;
 	uint32_t ns;
@@ -1099,7 +1072,7 @@ int unpackstr_array(char ***valp, uint32_t * size_valp, Buf buffer)
  * Given a pointer to memory (valp), size (size_val), and buffer,
  * store the memory contents into the buffer
  */
-void packmem_array(char *valp, uint32_t size_val, Buf buffer)
+void packmem_array(char *valp, uint32_t size_val, buf_t *buffer)
 {
 	if (remaining_buf(buffer) < size_val) {
 		if ((buffer->size + size_val + BUF_SIZE) > MAX_BUF_SIZE) {
@@ -1120,7 +1093,7 @@ void packmem_array(char *valp, uint32_t size_val, Buf buffer)
  * Given a pointer to memory (valp), size (size_val), and buffer,
  * store the buffer contents into memory
  */
-int unpackmem_array(char *valp, uint32_t size_valp, Buf buffer)
+int unpackmem_array(char *valp, uint32_t size_valp, buf_t *buffer)
 {
 	if (remaining_buf(buffer) >= size_valp) {
 		memcpy(valp, &buffer->head[buffer->processed], size_valp);
