@@ -41,12 +41,12 @@
 
 #include <ctype.h>
 #include <regex.h>
-#include <string.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "src/common/hostlist.h"
@@ -57,9 +57,9 @@
 #include "src/common/parse_value.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_protocol_interface.h"
+#include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
-#include "src/common/xassert.h"
 
 #include "slurm/slurm.h"
 
@@ -84,8 +84,6 @@ strong_alias(s_p_get_boolean,		slurm_s_p_get_boolean);
 strong_alias(s_p_dump_values,		slurm_s_p_dump_values);
 strong_alias(transfer_s_p_options,	slurm_transfer_s_p_options);
 
-#define BUFFER_SIZE 4096
-
 #define CONF_HASH_LEN 173
 
 static char *keyvalue_pattern =
@@ -102,7 +100,6 @@ struct s_p_values {
 	slurm_parser_operator_t operator;
 	int data_count;
 	void *data;
-	regex_t *keyvalue_re;
 	int (*handler)(void **data, slurm_parser_enum_t type,
 		       const char *key, const char *value,
 		       const char *line, char **leftover);
@@ -110,6 +107,11 @@ struct s_p_values {
 	s_p_values_t *next;
 };
 
+struct s_p_hashtbl {
+	regex_t keyvalue_re;
+	s_p_values_t *hash[CONF_HASH_LEN];
+};
+
 typedef struct _expline_values_st {
 	s_p_hashtbl_t*	template;
 	s_p_hashtbl_t*	index;
@@ -135,53 +137,44 @@ static int _conf_hashtbl_index(const char *key)
 	return hashval % CONF_HASH_LEN;
 }
 
-static void _conf_hashtbl_insert(s_p_hashtbl_t *hashtbl,
-				 s_p_values_t *value)
+static void _conf_hashtbl_insert(s_p_hashtbl_t *tbl, s_p_values_t *value)
 {
 	int idx;
 
 	xassert(value);
-	if (!value->key && value->keyvalue_re)
-		/* Inserting a holder for the regex_t of the structure. */
-		idx = 0;
-	else
-		idx = _conf_hashtbl_index(value->key);
-	value->next = hashtbl[idx];
-	hashtbl[idx] = value;
+	idx = _conf_hashtbl_index(value->key);
+	value->next = tbl->hash[idx];
+	tbl->hash[idx] = value;
 }
 
 /*
  * NOTE - "key" is case insensitive.
  */
-static s_p_values_t *_conf_hashtbl_lookup(
-	const s_p_hashtbl_t *hashtbl, const char *key)
+static s_p_values_t *_conf_hashtbl_lookup(const s_p_hashtbl_t *tbl,
+					  const char *key)
 {
 	int idx;
 	s_p_values_t *p;
 
 	xassert(key);
-	if (hashtbl == NULL)
+	if (!tbl)
 		return NULL;
 
 	idx = _conf_hashtbl_index(key);
-	for (p = hashtbl[idx]; p != NULL; p = p->next) {
+	for (p = tbl->hash[idx]; p; p = p->next) {
 		if (xstrcasecmp(p->key, key) == 0)
 			return p;
 	}
+
 	return NULL;
 }
 
 s_p_hashtbl_t *s_p_hashtbl_create(const s_p_options_t options[])
 {
-	const s_p_options_t *op = NULL;
-	s_p_values_t *value = NULL;
-	s_p_hashtbl_t *hashtbl = NULL;
-	_expline_values_t* expdata;
+	s_p_hashtbl_t *tbl = xmalloc(sizeof(*tbl));
 
-	hashtbl = xcalloc(CONF_HASH_LEN, sizeof(s_p_values_t *));
-
-	for (op = options; op->key != NULL; op++) {
-		value = xmalloc(sizeof(s_p_values_t));
+	for (const s_p_options_t *op = options; op->key; op++) {
+		s_p_values_t *value = xmalloc(sizeof(*value));
 		value->key = xstrdup(op->key);
 		value->operator = S_P_OPERATOR_SET;
 		value->type = op->type;
@@ -192,36 +185,21 @@ s_p_hashtbl_t *s_p_hashtbl_create(const s_p_options_t options[])
 		value->destroy = op->destroy;
 		if (op->type == S_P_LINE || op->type == S_P_EXPLINE) {
 			/* line_options mandatory for S_P_*LINE */
+			_expline_values_t *expdata = xmalloc(sizeof(*expdata));
 			xassert(op->line_options);
-			expdata = xmalloc(sizeof(_expline_values_t));
 			expdata->template =
 				s_p_hashtbl_create(op->line_options);
-			expdata->index = xcalloc(CONF_HASH_LEN,
-						 sizeof(s_p_values_t *));
+			expdata->index = xmalloc(sizeof(*expdata->index));
 			expdata->values = NULL;
 			value->data = expdata;
 		}
-		_conf_hashtbl_insert(hashtbl, value);
+		_conf_hashtbl_insert(tbl, value);
 	}
 
-	/*
-	 * Here we use hashtbl[0] as a place holder for the regex_t of the
-	 * structure.  If it doesn't exist put it in.  It will be initialized
-	 * later.
-	 */
-	if (!hashtbl[0]) {
-		value = xmalloc(sizeof(s_p_values_t));
-		value->next = hashtbl[0];
-		hashtbl[0] = value;
-	}
+	if (regcomp(&tbl->keyvalue_re, keyvalue_pattern, REG_EXTENDED))
+		fatal("keyvalue regex compilation failed");
 
-	hashtbl[0]->keyvalue_re = xmalloc(sizeof(regex_t));
-	if (regcomp(hashtbl[0]->keyvalue_re, keyvalue_pattern, REG_EXTENDED)) {
-		/* FIXME - should be fatal? */
-		error("keyvalue regex compilation failed");
-	}
-
-	return hashtbl;
+	return tbl;
 }
 
 /* Swap the data in two data structures without changing the linked list
@@ -285,29 +263,27 @@ static void _conf_file_values_free(s_p_values_t *p)
 	xfree(p);
 }
 
-void s_p_hashtbl_destroy(s_p_hashtbl_t *hashtbl) {
-	int i;
+void s_p_hashtbl_destroy(s_p_hashtbl_t *tbl)
+{
 	s_p_values_t *p, *next;
 
-	if (!hashtbl)
+	if (!tbl)
 		return;
 
-	if (hashtbl[0] && hashtbl[0]->keyvalue_re) {
-		regfree(hashtbl[0]->keyvalue_re);
-		xfree(hashtbl[0]->keyvalue_re);
-	}
-
-	for (i = 0; i < CONF_HASH_LEN; i++) {
-		for (p = hashtbl[i]; p != NULL; p = next) {
+	for (int i = 0; i < CONF_HASH_LEN; i++) {
+		for (p = tbl->hash[i]; p; p = next) {
 			next = p->next;
 			_conf_file_values_free(p);
 		}
 	}
-	xfree(hashtbl);
+
+	regfree(&tbl->keyvalue_re);
+
+	xfree(tbl);
 }
 
 /*
- * IN hashtbl - table to work off
+ * IN tbl - table to work off
  * IN line - string to be search for a key=value pair
  * OUT key - pointer to the key string (caller must free with xfree())
  * OUT value - pointer to the value string (caller must free with xfree())
@@ -315,7 +291,7 @@ void s_p_hashtbl_destroy(s_p_hashtbl_t *hashtbl) {
  *                 of the unsearched portion of the string
  * Return 0 when a key-value pair is found, and -1 otherwise.
  */
-static int _keyvalue_regex(s_p_hashtbl_t *hashtbl, const char *line,
+static int _keyvalue_regex(s_p_hashtbl_t *tbl, const char *line,
 			   char **key, char **value, char **remaining,
 			   slurm_parser_operator_t *operator)
 {
@@ -329,12 +305,8 @@ static int _keyvalue_regex(s_p_hashtbl_t *hashtbl, const char *line,
 	*operator = S_P_OPERATOR_SET;
 	memset(pmatch, 0, sizeof(regmatch_t)*nmatch);
 
-	xassert(hashtbl[0]->keyvalue_re);
-
-	if (regexec(hashtbl[0]->keyvalue_re, line, nmatch, pmatch, 0)
-	    == REG_NOMATCH) {
+	if (regexec(&tbl->keyvalue_re, line, nmatch, pmatch, 0) == REG_NOMATCH)
 		return -1;
-	}
 
 	*key = (char *)(xstrndup(line + pmatch[1].rm_so,
 				 pmatch[1].rm_eo - pmatch[1].rm_so));
@@ -507,67 +479,38 @@ static int _get_next_line(char *buf, int buf_size,
 	return lines;
 }
 
-/* copy all the keys from 'from_hashtbl' along with their types, handler, and
+/*
+ * Copy all the keys from 'from_hashtbl' along with their types, handler, and
  * destroy fields. Omit values in the copy and initialize them to NULL/0.
- *
- * if change_* is true, corresponding field will be updated with the next
- * corresponding parameter. */
-s_p_hashtbl_t* _hashtbl_copy_keys(const s_p_hashtbl_t* from_hashtbl,
-				  bool change_type,
-				  slurm_parser_enum_t new_type,
-				  bool change_handler,
-				  int (*handler)(void **data,
-					  slurm_parser_enum_t type, const char
-					  *key, const char *value, const char
-					  *line, char **leftover),
-				  bool change_destroyer,
-				  void (*destroy)(void *data)
-				  )
+ */
+s_p_hashtbl_t *_hashtbl_copy_keys(const s_p_hashtbl_t *from_tbl)
 {
-	s_p_hashtbl_t* to_hashtbl = NULL;
-	s_p_values_t *val_ptr,* val_copy;
-	int len, i;
+	s_p_hashtbl_t *to_tbl = xmalloc(sizeof(*to_tbl));
 
-	xassert(from_hashtbl);
+	xassert(from_tbl);
 
-	len = CONF_HASH_LEN * sizeof(s_p_values_t *);
-	to_hashtbl = xmalloc(len);
+	for (int i = 0; i < CONF_HASH_LEN; ++i) {
+		for (s_p_values_t *val_ptr = from_tbl->hash[i];
+		     val_ptr; val_ptr = val_ptr->next) {
+			s_p_values_t *val_copy = xmalloc(sizeof(*val_copy));
 
-	for (i = 0; i < CONF_HASH_LEN; ++i) {
-		for (val_ptr = from_hashtbl[i]; val_ptr;
-		     val_ptr = val_ptr->next) {
-			val_copy = xmalloc(sizeof(s_p_values_t));
 			val_copy->key = xstrdup(val_ptr->key);
 			val_copy->operator = val_ptr->operator;
 			val_copy->type = val_ptr->type;
 			val_copy->handler = val_ptr->handler;
 			val_copy->destroy = val_ptr->destroy;
-			if (change_type) {
-				val_copy->type = new_type;
-			}
-			if (change_handler) {
-				val_copy->handler = handler;
-			}
-			if (change_destroyer) {
-				val_copy->destroy = destroy;
-			}
-			/*
-			 * We cannot copy a regex since a regfree() on either
-			 * the original or the copy can affect the other one.
-			 */
-			if (val_ptr->keyvalue_re != NULL) {
-				val_copy->keyvalue_re = xmalloc(
-					sizeof(regex_t));
-				if (regcomp(val_copy->keyvalue_re,
-					    keyvalue_pattern, REG_EXTENDED)) {
-					error("keyvalue regex compilation failed");
-				}
-			}
-			_conf_hashtbl_insert(to_hashtbl, val_copy);
+			_conf_hashtbl_insert(to_tbl, val_copy);
 		}
 	}
 
-	return to_hashtbl;
+	/*
+	 * We cannot copy a regex since a regfree() on either
+	 * the original or the copy can affect the other one.
+	 */
+	if (regcomp(&to_tbl->keyvalue_re, keyvalue_pattern, REG_EXTENDED))
+		fatal("keyvalue regex compilation failed");
+
+	return to_tbl;
 }
 
 
@@ -609,64 +552,80 @@ static void *_handle_string(const char *key, const char *value)
 static void *_handle_long(const char *key, const char *value)
 {
 	long *data = xmalloc(sizeof(*data));
-	if (s_p_handle_long(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_long(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_uint16(const char *key, const char *value)
 {
 	uint16_t *data = xmalloc(sizeof(*data));
-	if (s_p_handle_uint16(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_uint16(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_uint32(const char *key, const char *value)
 {
 	uint32_t *data = xmalloc(sizeof(*data));
-	if (s_p_handle_uint32(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_uint32(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_uint64(const char *key, const char *value)
 {
 	uint64_t *data = xmalloc(sizeof(*data));
-	if (s_p_handle_uint64(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_uint64(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_boolean(const char *key, const char *value)
 {
 	bool *data = xmalloc(sizeof(*data));
-	if (s_p_handle_boolean(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_boolean(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_float(const char *key, const char *value)
 {
 	float *data = xmalloc(sizeof(*data));
-	if (s_p_handle_float(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_float(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_double(const char *key, const char *value)
 {
 	double *data = xmalloc(sizeof(*data));
-	if (s_p_handle_double(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_double(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
 static void *_handle_ldouble(const char *key, const char *value)
 {
 	long double *data = xmalloc(sizeof(*data));
-	if (s_p_handle_long_double(data, key, value) == SLURM_ERROR)
+	if (s_p_handle_long_double(data, key, value) == SLURM_ERROR) {
+		xfree(data);
 		return NULL;
+	}
 	return data;
 }
 
@@ -883,8 +842,7 @@ static int _handle_line(s_p_values_t* v, const char* value,
 	_expline_values_t* v_data = (_expline_values_t*)v->data;
 	s_p_hashtbl_t* newtable;
 
-	newtable = _hashtbl_copy_keys(v_data->template, false, S_P_IGNORE,
-				      false, NULL, false, NULL);
+	newtable = _hashtbl_copy_keys(v_data->template);
 	if (s_p_parse_line_complete(newtable, v->key, value, line,
 				    leftover) == SLURM_ERROR) {
 		s_p_hashtbl_destroy(newtable);
@@ -927,55 +885,64 @@ static int _handle_expline(s_p_values_t* v, const char* value,
  *                  If the handler for that key parses more of the line,
  *                  it will move the leftover pointer to point to the character
  *                  after it has finished parsing in the line.
+ * RET:
+ *	-1 if the value is invalid.
+ *	0 if the value is validbut no value will be set for "data".
+ *	1 if "data" is set.
  */
-static void _handle_keyvalue_match(s_p_values_t *v,
-				   const char *value, const char *line,
-				   char **leftover)
+static int _handle_keyvalue_match(s_p_values_t *v,
+				  const char *value, const char *line,
+				  char **leftover)
 {
+	int rc = 1;
+
 	switch (v->type) {
 	case S_P_IGNORE:
 		/* do nothing */
 		break;
 	case S_P_STRING:
-		_handle_common(v, value, line, leftover, _handle_string);
+		rc = _handle_common(v, value, line, leftover, _handle_string);
 		break;
 	case S_P_LONG:
-		_handle_common(v, value, line, leftover, _handle_long);
+		rc = _handle_common(v, value, line, leftover, _handle_long);
 		break;
 	case S_P_UINT16:
-		_handle_common(v, value, line, leftover, _handle_uint16);
+		rc = _handle_common(v, value, line, leftover, _handle_uint16);
 		break;
 	case S_P_UINT32:
-		_handle_common(v, value, line, leftover, _handle_uint32);
+		rc = _handle_common(v, value, line, leftover, _handle_uint32);
 		break;
 	case S_P_UINT64:
-		_handle_common(v, value, line, leftover, _handle_uint64);
+		rc = _handle_common(v, value, line, leftover, _handle_uint64);
 		break;
 	case S_P_POINTER:
-		_handle_pointer(v, value, line, leftover);
+		rc = _handle_pointer(v, value, line, leftover);
 		break;
 	case S_P_ARRAY:
-		_handle_array(v, value, line, leftover);
+		rc = _handle_array(v, value, line, leftover);
 		break;
 	case S_P_BOOLEAN:
-		_handle_common(v, value, line, leftover, _handle_boolean);
+		rc = _handle_common(v, value, line, leftover, _handle_boolean);
 		break;
 	case S_P_LINE:
-		_handle_line(v, value, line, leftover);
+		rc = _handle_line(v, value, line, leftover);
 		break;
 	case S_P_EXPLINE:
-		_handle_expline(v, value, line, leftover);
+		rc = _handle_expline(v, value, line, leftover);
 		break;
 	case S_P_FLOAT:
-		_handle_common(v, value, line, leftover, _handle_float);
+		rc = _handle_common(v, value, line, leftover, _handle_float);
 		break;
 	case S_P_DOUBLE:
-		_handle_common(v, value, line, leftover, _handle_double);
+		rc = _handle_common(v, value, line, leftover, _handle_double);
 		break;
 	case S_P_LONG_DOUBLE:
-		_handle_common(v, value, line, leftover, _handle_ldouble);
+		rc = _handle_common(v, value, line, leftover, _handle_ldouble);
 		break;
+	default:
+		fatal("%s: unsupported s_p_value_t type %d", __func__, v->type);
 	}
+	return rc;
 }
 
 /*
@@ -1014,8 +981,13 @@ int s_p_parse_line(s_p_hashtbl_t *hashtbl, const char *line, char **leftover)
 	while (_keyvalue_regex(hashtbl, ptr, &key, &value, &new_leftover, &op) == 0) {
 		if ((p = _conf_hashtbl_lookup(hashtbl, key))) {
 			p->operator = op;
-			_handle_keyvalue_match(p, value,
-					       new_leftover, &new_leftover);
+			if (_handle_keyvalue_match(p, value, new_leftover,
+						   &new_leftover) == -1) {
+				xfree(key);
+				xfree(value);
+				slurm_seterrno(EINVAL);
+				return 0;
+			}
 			*leftover = ptr = new_leftover;
 		} else {
 			error("Parsing error at unrecognized key: %s", key);
@@ -1046,8 +1018,14 @@ static int _parse_next_key(s_p_hashtbl_t *hashtbl,
 	if (_keyvalue_regex(hashtbl, line, &key, &value, &new_leftover, &op) == 0) {
 		if ((p = _conf_hashtbl_lookup(hashtbl, key))) {
 			p->operator = op;
-			_handle_keyvalue_match(p, value,
-					       new_leftover, &new_leftover);
+			if (_handle_keyvalue_match(p, value, new_leftover,
+						   &new_leftover) == -1) {
+				xfree(key);
+				xfree(value);
+				*leftover = (char *)line;
+				slurm_seterrno(EINVAL);
+				return 0;
+			}
 			*leftover = new_leftover;
 		} else if (ignore_new) {
 			debug("%s: Parsing error at unrecognized key: %s",
@@ -1228,7 +1206,12 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename,
 						  line, &leftover, ignore_new,
 						  filename);
 		if (inc_rc == 0) {
-			_parse_next_key(hashtbl, line, &leftover, ignore_new);
+			if (!_parse_next_key(hashtbl, line, &leftover,
+					     ignore_new)) {
+				rc = SLURM_ERROR;
+				line_number += merged_lines;
+				continue;
+			}
 		} else if (inc_rc < 0) {
 			error("\"Include\" failed in file %s line %d",
 			      filename, line_number);
@@ -1260,7 +1243,7 @@ int s_p_parse_file(s_p_hashtbl_t *hashtbl, uint32_t *hash_val, char *filename,
 }
 
 int s_p_parse_buffer(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
-		     Buf buffer, bool ignore_new)
+		     buf_t *buffer, bool ignore_new)
 {
 	char *leftover = NULL;
 	int rc = SLURM_SUCCESS;
@@ -1282,7 +1265,12 @@ int s_p_parse_buffer(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
 				xfree(tmp_str);
 				continue;
 			}
-			_parse_next_key(hashtbl, tmp_str, &leftover, ignore_new);
+			if (!_parse_next_key(hashtbl, tmp_str, &leftover,
+					     ignore_new)) {
+				rc = SLURM_ERROR;
+				xfree(tmp_str);
+				continue;
+			}
 			/* Make sure that after parsing only whitespace
 			   is left over */
 			if (!_line_is_space(leftover)) {
@@ -1322,27 +1310,26 @@ int s_p_parse_buffer(s_p_hashtbl_t *hashtbl, uint32_t *hash_val,
  * IN from_hashtbl - Source of old data
  * IN to_hashtbl - Destination for old data
  */
-void s_p_hashtbl_merge(s_p_hashtbl_t *to_hashtbl, s_p_hashtbl_t *from_hashtbl)
+void s_p_hashtbl_merge(s_p_hashtbl_t *to_tbl, s_p_hashtbl_t *from_tbl)
 {
 	int i;
 	s_p_values_t **val_pptr, *val_ptr, *match_ptr;
 
-	if (!to_hashtbl || !from_hashtbl)
+	if (!to_tbl || !from_tbl)
 		return;
 
 	for (i = 0; i < CONF_HASH_LEN; i++) {
-		val_pptr = &from_hashtbl[i];
-		val_ptr = from_hashtbl[i];
+		val_pptr = &from_tbl->hash[i];
+		val_ptr = from_tbl->hash[i];
 		while (val_ptr) {
 			if (val_ptr->data_count == 0) {
-				/* No data in from_hashtbl record to move.
+				/* No data in from_tbl record to move.
 				 * Skip record */
 				val_pptr = &val_ptr->next;
 				val_ptr = val_ptr->next;
 				continue;
 			}
-			match_ptr = _conf_hashtbl_lookup(to_hashtbl,
-							 val_ptr->key);
+			match_ptr = _conf_hashtbl_lookup(to_tbl, val_ptr->key);
 			if (match_ptr) {	/* Found matching key */
 				if (match_ptr->data_count == 0) {
 					_conf_hashtbl_swap_data(val_ptr,
@@ -1353,25 +1340,25 @@ void s_p_hashtbl_merge(s_p_hashtbl_t *to_hashtbl, s_p_hashtbl_t *from_hashtbl)
 			} else {	/* No match, move record */
 				*val_pptr = val_ptr->next;
 				val_ptr->next = NULL;
-				_conf_hashtbl_insert(to_hashtbl, val_ptr);
+				_conf_hashtbl_insert(to_tbl, val_ptr);
 				val_ptr = *val_pptr;
 			}
 		}
 	}
 }
 
-void s_p_hashtbl_merge_override(s_p_hashtbl_t *to_hashtbl,
-				s_p_hashtbl_t *from_hashtbl)
+void s_p_hashtbl_merge_override(s_p_hashtbl_t *to_tbl,
+				s_p_hashtbl_t *from_tbl)
 {
 	int i;
 	s_p_values_t **val_pptr, *val_ptr, *match_ptr;
 
-	if (!to_hashtbl || !from_hashtbl)
+	if (!to_tbl || !from_tbl)
 		return;
 
 	for (i = 0; i < CONF_HASH_LEN; i++) {
-		val_pptr = &from_hashtbl[i];
-		val_ptr = from_hashtbl[i];
+		val_pptr = &from_tbl->hash[i];
+		val_ptr = from_tbl->hash[i];
 		while (val_ptr) {
 			if (val_ptr->data_count == 0) {
 				/* No data in from_hashtbl record to move.
@@ -1380,8 +1367,7 @@ void s_p_hashtbl_merge_override(s_p_hashtbl_t *to_hashtbl,
 				val_ptr = val_ptr->next;
 				continue;
 			}
-			match_ptr = _conf_hashtbl_lookup(to_hashtbl,
-							 val_ptr->key);
+			match_ptr = _conf_hashtbl_lookup(to_tbl, val_ptr->key);
 			if (match_ptr) {	/* Found matching key */
 				_conf_hashtbl_swap_data(val_ptr, match_ptr);
 				val_pptr = &val_ptr->next;
@@ -1389,29 +1375,29 @@ void s_p_hashtbl_merge_override(s_p_hashtbl_t *to_hashtbl,
 			} else {	/* No match, move record */
 				*val_pptr = val_ptr->next;
 				val_ptr->next = NULL;
-				_conf_hashtbl_insert(to_hashtbl, val_ptr);
+				_conf_hashtbl_insert(to_tbl, val_ptr);
 				val_ptr = *val_pptr;
 			}
 		}
 	}
 }
 
-void s_p_hashtbl_merge_keys(s_p_hashtbl_t *to_hashtbl,
-			    s_p_hashtbl_t *from_hashtbl)
+void s_p_hashtbl_merge_keys(s_p_hashtbl_t *to_tbl,
+			    s_p_hashtbl_t *from_tbl)
 {
 	int i;
 	_expline_values_t* f_expline;
 	_expline_values_t* t_expline;
 	s_p_values_t **pp, *p, *match_ptr;
 
-	if (!to_hashtbl || !from_hashtbl)
+	if (!to_tbl || !from_tbl)
 		return;
 
 	for (i = 0; i < CONF_HASH_LEN; i++) {
-		pp = &from_hashtbl[i];
-		p = from_hashtbl[i];
+		pp = &from_tbl->hash[i];
+		p = from_tbl->hash[i];
 		while (p) {
-			match_ptr = _conf_hashtbl_lookup(to_hashtbl, p->key);
+			match_ptr = _conf_hashtbl_lookup(to_tbl, p->key);
 			if (match_ptr) {	/* Found matching key */
 				if (match_ptr->type == p->type &&
 				    (p->type == S_P_LINE ||
@@ -1434,7 +1420,7 @@ void s_p_hashtbl_merge_keys(s_p_hashtbl_t *to_hashtbl,
 			} else {	/* No match, move record */
 				*pp = p->next;
 				p->next = NULL;
-				_conf_hashtbl_insert(to_hashtbl, p);
+				_conf_hashtbl_insert(to_tbl, p);
 				p = *pp;
 			}
 		}
@@ -1493,19 +1479,16 @@ static void _parse_line_expanded_destroyer(void* data)
  * S_P_PLAIN_STRING specifying not be considered as an expandable string
  * is thus just converted to a real S_P_STRING and not an hostlist.
  */
-static s_p_hashtbl_t* _parse_expline_adapt_table(const s_p_hashtbl_t* hashtbl)
+static s_p_hashtbl_t *_parse_expline_adapt_table(const s_p_hashtbl_t *tbl)
 {
-	s_p_hashtbl_t* to_hashtbl = NULL;
-	s_p_values_t *val_ptr,* val_copy;
-	int i;
+	s_p_hashtbl_t *to_tbl = xmalloc(sizeof(*to_tbl));
 
-	xassert(hashtbl);
+	xassert(tbl);
 
-	to_hashtbl = xcalloc(CONF_HASH_LEN, sizeof(s_p_values_t *));
-
-	for (i = 0; i < CONF_HASH_LEN; ++i) {
-		for (val_ptr = hashtbl[i]; val_ptr; val_ptr = val_ptr->next) {
-			val_copy = xmalloc(sizeof(s_p_values_t));
+	for (int i = 0; i < CONF_HASH_LEN; ++i) {
+		for (s_p_values_t *val_ptr = tbl->hash[i];
+		     val_ptr; val_ptr = val_ptr->next) {
+			s_p_values_t *val_copy = xmalloc(sizeof(*val_copy));
 			val_copy->key = xstrdup(val_ptr->key);
 			val_copy->operator = val_ptr->operator;
 			if (val_ptr->type == S_P_PLAIN_STRING) {
@@ -1517,39 +1500,34 @@ static s_p_hashtbl_t* _parse_expline_adapt_table(const s_p_hashtbl_t* hashtbl)
 				val_copy->destroy =
 					_parse_line_expanded_destroyer;
 			}
-			/*
-			 * We cannot copy a regex since a regfree() on either
-			 * the original or the copy can affect the other one.
-			 */
-			if (val_ptr->keyvalue_re != NULL) {
-				val_copy->keyvalue_re = xmalloc(
-					sizeof(regex_t));
-				if (regcomp(val_copy->keyvalue_re,
-					    keyvalue_pattern, REG_EXTENDED)) {
-					error("keyvalue regex compilation failed");
-				}
-			}
-			_conf_hashtbl_insert(to_hashtbl, val_copy);
+			_conf_hashtbl_insert(to_tbl, val_copy);
 		}
 	}
 
-	return to_hashtbl;
+	/*
+	 * We cannot copy a regex since a regfree() on either
+	 * the original or the copy can affect the other one.
+	 */
+	if (regcomp(&to_tbl->keyvalue_re, keyvalue_pattern, REG_EXTENDED))
+		fatal("keyvalue regex compilation failed");
+
+	return to_tbl;
 }
 
 /*
  * walk down a tree of s_p_values_t converting every S_P_PLAIN_STRING
  * element to an S_P_STRING element.
  */
-static void _hashtbl_plain_to_string(s_p_hashtbl_t* hashtbl)
+static void _hashtbl_plain_to_string(s_p_hashtbl_t *tbl)
 {
 	_expline_values_t* v_data;
 	s_p_values_t *p;
 	int i, j;
 
-	xassert(hashtbl);
+	xassert(tbl);
 
 	for (i = 0; i < CONF_HASH_LEN; ++i) {
-		for (p = hashtbl[i]; p; p = p->next) {
+		for (p = tbl->hash[i]; p; p = p->next) {
 			if (p->type == S_P_PLAIN_STRING) {
 				p->type = S_P_STRING;
 			} else if (p->type == S_P_LINE
@@ -1697,7 +1675,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 			    const char *line, char **leftover)
 {
 	int i, status;
-	s_p_hashtbl_t* strhashtbl = NULL;
+	s_p_hashtbl_t* strtbl = NULL;
 	s_p_hashtbl_t** tables = NULL;
 	int tables_count = 0;
 	hostlist_t value_hl = NULL;
@@ -1707,7 +1685,8 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 	status = SLURM_ERROR;
 
 	/* create the adapted temporary hash table used for expansion */
-	strhashtbl = _parse_expline_adapt_table(hashtbl);
+	strtbl = _parse_expline_adapt_table(hashtbl);
+	xassert(strtbl);
 
 	/* create hostlist and one iterator over it, since we will walk
 	 * through the list for each new attribute to create final expanded
@@ -1720,7 +1699,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 	/* populate the temporary expansion hash table, it will map the
 	 * different config elements to either an hostlist (through S_P_POINTER) or
 	 * to an S_P_STRING (for original element of type S_P_PLAIN_STRING) */
-	if (!s_p_parse_line(strhashtbl, *leftover, leftover)) {
+	if (!s_p_parse_line(strtbl, *leftover, leftover)) {
 		error("Unable to parse line %s", *leftover);
 		goto cleanup;
 	}
@@ -1738,10 +1717,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 	for (i = 0; i < tables_count; i++) {
 		free(value_str);
 		value_str = hostlist_shift(value_hl);
-		tables[i] = _hashtbl_copy_keys(hashtbl,
-					       false, S_P_IGNORE,
-					       false, NULL,
-					       false, NULL);
+		tables[i] = _hashtbl_copy_keys(hashtbl);
 		_hashtbl_plain_to_string(tables[i]);
 		if (!s_p_parse_pair(tables[i], key, value_str)) {
 			error("Error parsing '%s = %s', most left part of the"
@@ -1754,7 +1730,7 @@ int s_p_parse_line_expanded(const s_p_hashtbl_t *hashtbl,
 	 * conversions and handlers. This is done at the same time as storing
 	 * the parsed attribute values with s_p_parse_pair */
 	for (i = 0; i < CONF_HASH_LEN; ++i) {
-		for (attr = strhashtbl[i]; attr; attr = attr->next) {
+		for (attr = strtbl->hash[i]; attr; attr = attr->next) {
 			if (!_parse_expline_doexpand(tables,
 						     tables_count,
 						     attr)) {
@@ -1770,8 +1746,7 @@ cleanup:
 		free(value_str);
 	if (value_hl)
 		hostlist_destroy(value_hl);
-	if (strhashtbl)
-		s_p_hashtbl_destroy(strhashtbl);
+	s_p_hashtbl_destroy(strtbl);
 
 	if (status == SLURM_ERROR && tables) {
 		for (i = 0; i < tables_count; i++)
@@ -1829,7 +1804,11 @@ int s_p_parse_pair_with_op(s_p_hashtbl_t *hashtbl, const char *key,
 		leftover++;
 	while (*leftover != '\0' && isspace(*leftover))
 		leftover++; /* skip trailing spaces */
-	_handle_keyvalue_match(p, value, leftover, &leftover);
+	if (_handle_keyvalue_match(p, value, leftover, &leftover) == -1) {
+		xfree(value);
+		slurm_seterrno(EINVAL);
+		return 0;
+	}
 	xfree(value);
 
 	return 1;
@@ -2179,11 +2158,11 @@ void s_p_dump_values(const s_p_hashtbl_t *hashtbl,
  * Primarily for sending a table across the network so you don't have to read a
  * file in.
  */
-extern Buf s_p_pack_hashtbl(const s_p_hashtbl_t *hashtbl,
-			   const s_p_options_t options[],
-			   const uint32_t cnt)
+extern buf_t *s_p_pack_hashtbl(const s_p_hashtbl_t *hashtbl,
+			       const s_p_options_t options[],
+			       const uint32_t cnt)
 {
-	Buf buffer = init_buf(0);
+	buf_t *buffer = init_buf(0);
 	s_p_values_t *p;
 	int i;
 
@@ -2247,7 +2226,7 @@ extern Buf s_p_pack_hashtbl(const s_p_hashtbl_t *hashtbl,
 /*
  * Given a buffer, unpack key, type, op and value into a hashtbl.
  */
-extern s_p_hashtbl_t *s_p_unpack_hashtbl(Buf buffer)
+extern s_p_hashtbl_t *s_p_unpack_hashtbl(buf_t *buffer)
 {
 	s_p_values_t *value = NULL;
 	s_p_hashtbl_t *hashtbl = NULL;
@@ -2263,7 +2242,7 @@ extern s_p_hashtbl_t *s_p_unpack_hashtbl(Buf buffer)
 
 	safe_unpack32(&cnt, buffer);
 
-	hashtbl = xcalloc(CONF_HASH_LEN, sizeof(s_p_values_t *));
+	hashtbl = xmalloc(sizeof(*hashtbl));
 
 	for (i = 0; i < cnt; i++) {
 		value = xmalloc(sizeof(s_p_values_t));
@@ -2347,14 +2326,13 @@ extern void transfer_s_p_options(s_p_options_t **full_options,
 				 s_p_options_t *options,
 				 int *full_options_cnt)
 {
-	s_p_options_t *op = NULL;
 	s_p_options_t *full_options_ptr;
 	int cnt = *full_options_cnt;
 
 	xassert(full_options);
 
-	for (op = options; op->key != NULL; op++, cnt++) {
-		xrealloc(*full_options, ((cnt + 1) * sizeof(s_p_options_t)));
+	for (s_p_options_t *op = options; op->key; op++, cnt++) {
+		xrecalloc(*full_options, cnt + 1, sizeof(s_p_options_t));
 		full_options_ptr = &(*full_options)[cnt];
 		memcpy(full_options_ptr, op, sizeof(s_p_options_t));
 		full_options_ptr->key = xstrdup(op->key);
