@@ -50,7 +50,7 @@
 #include "src/common/parse_config.h"
 #include "src/common/run_in_daemon.h"
 
-extern slurm_ctl_conf_t slurmctld_conf;
+extern slurm_conf_t slurm_conf;
 extern char *default_slurm_config_file;
 extern char *default_plugin_path;
 
@@ -74,6 +74,7 @@ extern uint16_t drop_priv_flag;
 #define DEFAULT_AUTH_TYPE          "auth/munge"
 #define DEFAULT_AUTH_TOKEN_LIFESPAN 1800
 #define DEFAULT_BATCH_START_TIMEOUT 10
+#define DEFAULT_BCAST_EXCLUDE       "/lib,/usr/lib,/lib64,/usr/lib64"
 #define DEFAULT_COMPLETE_WAIT       0
 #define DEFAULT_CRED_TYPE           "cred/munge"
 #define DEFAULT_EPILOG_MSG_TIME     2000
@@ -86,6 +87,7 @@ extern uint16_t drop_priv_flag;
 					 * /etc/group file */
 /* NOTE: DEFAULT_INACTIVE_LIMIT must be 0 for Blue Gene/L systems */
 #define DEFAULT_INACTIVE_LIMIT      0
+#define DEFAULT_INTERACTIVE_STEP_OPTS "--interactive --preserve-env --pty $SHELL"
 #define DEFAULT_JOB_ACCT_GATHER_TYPE  "jobacct_gather/none"
 #define JOB_ACCT_GATHER_TYPE_NONE "jobacct_gather/none"
 #define DEFAULT_JOB_ACCT_GATHER_FREQ  "30"
@@ -212,7 +214,7 @@ typedef struct slurm_conf_node {
 	uint16_t cpus;		/* count of cpus running on the node */
 	char *cpu_spec_list;	/* arbitrary list of specialized cpus */
 	uint16_t boards; 	/* number of boards per node */
-	uint16_t sockets;       /* number of sockets per node */
+	uint16_t tot_sockets;   /* number of sockets per node */
 	uint16_t cores;         /* number of cores per CPU */
 	uint16_t core_spec_cnt;	/* number of specialized cores */
 	uint16_t threads;       /* number of threads per core */
@@ -272,9 +274,15 @@ typedef struct slurm_conf_partition {
 	char    *qos_char;      /* Name of QOS associated with partition */
 	bool     req_resv_flag; /* 1 if partition can only be used in a
 				 * reservation */
+	uint16_t resume_timeout; /* time required in order to perform a node
+				  * resume operation */
 	bool     root_only_flag;/* 1 if allocate/submit RPC can only be
 				   issued by user root */
 	uint16_t state_up;	/* for states see PARTITION_* in slurm.h */
+	uint32_t suspend_time;  /* node idle for this long before power save
+				 * mode */
+	uint16_t suspend_timeout; /* time required in order to perform a node
+				   * suspend operation */
 	uint32_t total_nodes;	/* total number of nodes in the partition */
 	uint32_t total_cpus;	/* total number of cpus in the partition */
 } slurm_conf_partition_t;
@@ -336,11 +344,12 @@ extern int job_defaults_list(char *in_str, List *out_list);
 extern char *job_defaults_str(List in_list);
 
 /* Pack a job_defaults_t element. Used by slurm_pack_list() */
-extern void job_defaults_pack(void *in, uint16_t protocol_version, Buf buffer);
+extern void job_defaults_pack(void *in, uint16_t protocol_version,
+			      buf_t *buffer);
 
 /* Unpack a job_defaults_t element. Used by slurm_pack_list() */
 extern int job_defaults_unpack(void **out, uint16_t protocol_version,
-			       Buf buffer);
+			       buf_t *buffer);
 
 /*
  * list_find_frontend - find an entry in the front_end list, see list.h for
@@ -393,7 +402,7 @@ void slurm_conf_install_fork_handlers(void);
  */
 extern int slurm_conf_destroy(void);
 
-extern slurm_ctl_conf_t *slurm_conf_lock(void);
+extern slurm_conf_t *slurm_conf_lock(void);
 
 extern void slurm_conf_unlock(void);
 
@@ -473,20 +482,14 @@ extern char *slurm_conf_get_nodename(const char *node_hostname);
 extern char *slurm_conf_get_aliases(const char *node_hostname);
 
 /*
- * slurm_conf_get_nodeaddr - Return the NodeAddr for given NodeHostname
+ * slurm_conf_get_nodeaddr - Return the NodeAddr for given
+ * NodeHostname or NodeName
  *
  * NOTE: Call xfree() to release returned value's memory.
  * NOTE: Caller must NOT be holding slurm_conf_lock().
  */
-extern char *slurm_conf_get_nodeaddr(const char *node_hostname);
-
-/*
- * slurm_conf_get_nodename_from_addr - Return the NodeName for given NodeAddr
- *
- * NOTE: Call xfree() to release returned value's memory.
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern char *slurm_conf_get_nodename_from_addr(const char *node_addr);
+extern char *slurm_conf_get_nodeaddr(const char *node_hostname,
+				     const char *node_name);
 
 /*
  * slurm_conf_get_aliased_nodename - Return the NodeName matching an alias
@@ -557,16 +560,15 @@ extern int slurm_conf_get_res_spec_info(const char *node_name,
  *	file pathname (slurm_conf) is not changed.
  * IN/OUT ctl_conf_ptr - pointer to data structure to be initialized
  */
-extern void init_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr);
+extern void init_slurm_conf(slurm_conf_t *ctl_conf_ptr);
 
 /*
- * free_slurm_conf - free all storage associated with a slurm_ctl_conf_t.
+ * free_slurm_conf - free all storage associated with a slurm_conf_t.
  * IN/OUT ctl_conf_ptr - pointer to data structure to be freed
  * IN purge_node_hash - purge system-wide node hash table if set,
  *			set to zero if clearing private copy of config data
  */
-extern void free_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr,
-			     bool purge_node_hash);
+extern void free_slurm_conf(slurm_conf_t *ctl_conf_ptr, bool purge_node_hash);
 
 /*
  * gethostname_short - equivalent to gethostname(), but return only the first
@@ -626,24 +628,24 @@ extern uint16_t reconfig_str2flags(char *reconfig_flags);
 
 extern void destroy_config_plugin_params(void *object);
 extern void pack_config_plugin_params(void *in, uint16_t protocol_version,
-				      Buf buff);
+				      buf_t *buff);
 extern int unpack_config_plugin_params(void **object, uint16_t protocol_version,
-				       Buf buff);
+				       buf_t *buff);
 extern void pack_config_plugin_params_list(void *in, uint16_t protocol_version,
-					   Buf buff);
+					   buf_t *buff);
 extern int unpack_config_plugin_params_list(void **object,
 					    uint16_t protocol_version,
-					    Buf buff);
+					    buf_t *buff);
 
 extern void destroy_config_key_pair(void *object);
 extern void pack_key_pair_list(void *key_pairs, uint16_t protocol_version,
-			       Buf buffer);
+			       buf_t *buffer);
 extern int unpack_key_pair_list(void **key_pairs, uint16_t protocol_version,
-				Buf buffer);
+				buf_t *buffer);
 extern void pack_config_key_pair(void *in, uint16_t protocol_version,
-				 Buf buffer);
+				 buf_t *buffer);
 extern int unpack_config_key_pair(void **object, uint16_t protocol_version,
-				  Buf buffer);
+				  buf_t *buffer);
 
 extern int sort_key_pairs(void *v1, void *v2);
 /*
