@@ -52,7 +52,9 @@ strong_alias(pack_slurmdbd_msg, slurm_pack_slurmdbd_msg);
 strong_alias(unpack_slurmdbd_msg, slurm_unpack_slurmdbd_msg);
 strong_alias(slurmdbd_pack_fini_msg, slurm_slurmdbd_pack_fini_msg);
 
-static int _unpack_config_name(char **object, uint16_t rpc_version, Buf buffer)
+
+static int _unpack_config_name(char **object, uint16_t rpc_version,
+			       buf_t *buffer)
 {
 	char *config_name;
 	uint32_t uint32_tmp;
@@ -66,12 +68,12 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _slurmdbd_packstr(void *str, uint16_t rpc_version, Buf buffer)
+static void _slurmdbd_packstr(void *str, uint16_t rpc_version, buf_t *buffer)
 {
 	packstr((char *)str, buffer);
 }
 
-static int _slurmdbd_unpackstr(void **str, uint16_t rpc_version, Buf buffer)
+static int _slurmdbd_unpackstr(void **str, uint16_t rpc_version, buf_t *buffer)
 {
 	uint32_t uint32_tmp;
 	safe_unpackstr_xmalloc((char **)str, &uint32_tmp, buffer);
@@ -84,7 +86,7 @@ unpack_error:
  * Pack and unpack data structures
 \****************************************************************************/
 static void _pack_acct_coord_msg(dbd_acct_coord_msg_t *msg,
-				 uint16_t rpc_version, Buf buffer)
+				 uint16_t rpc_version, buf_t *buffer)
 {
 	char *acct = NULL;
 	ListIterator itr = NULL;
@@ -106,7 +108,7 @@ static void _pack_acct_coord_msg(dbd_acct_coord_msg_t *msg,
 }
 
 static int _unpack_acct_coord_msg(dbd_acct_coord_msg_t **msg,
-				  uint16_t rpc_version, Buf buffer)
+				  uint16_t rpc_version, buf_t *buffer)
 {
 	uint32_t uint32_tmp;
 	int i;
@@ -137,7 +139,7 @@ unpack_error:
 }
 
 static void _pack_cluster_tres_msg(dbd_cluster_tres_msg_t *msg,
-				   uint16_t rpc_version, Buf buffer)
+				   uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(msg->cluster_nodes, buffer);
@@ -147,7 +149,7 @@ static void _pack_cluster_tres_msg(dbd_cluster_tres_msg_t *msg,
 }
 
 static int _unpack_cluster_tres_msg(dbd_cluster_tres_msg_t **msg,
-				    uint16_t rpc_version, Buf buffer)
+				    uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_cluster_tres_msg_t *msg_ptr;
 	uint32_t uint32_tmp;
@@ -171,11 +173,10 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _pack_rec_msg(dbd_rec_msg_t *msg,
-			  uint16_t rpc_version,
-			  slurmdbd_msg_type_t type, Buf buffer)
+static void _pack_rec_msg(dbd_rec_msg_t *msg, uint16_t rpc_version,
+			  slurmdbd_msg_type_t type, buf_t *buffer)
 {
-	void (*my_function) (void *object, uint16_t rpc_version, Buf buffer);
+	void (*my_function) (void *object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_ADD_RESV:
@@ -191,12 +192,11 @@ static void _pack_rec_msg(dbd_rec_msg_t *msg,
 	(*(my_function))(msg->rec, rpc_version, buffer);
 }
 
-static int _unpack_rec_msg(dbd_rec_msg_t **msg,
-			   uint16_t rpc_version,
-			   slurmdbd_msg_type_t type, Buf buffer)
+static int _unpack_rec_msg(dbd_rec_msg_t **msg, uint16_t rpc_version,
+			   slurmdbd_msg_type_t type, buf_t *buffer)
 {
 	dbd_rec_msg_t *msg_ptr = NULL;
-	int (*my_function) (void **object, uint16_t rpc_version, Buf buffer);
+	int (*my_function) (void **object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_ADD_RESV:
@@ -223,11 +223,10 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _pack_cond_msg(dbd_cond_msg_t *msg,
-			   uint16_t rpc_version,
-			   slurmdbd_msg_type_t type, Buf buffer)
+static void _pack_cond_msg(dbd_cond_msg_t *msg, uint16_t rpc_version,
+			   slurmdbd_msg_type_t type, buf_t *buffer)
 {
-	void (*my_function) (void *object, uint16_t rpc_version, Buf buffer);
+	void (*my_function) (void *object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_GET_ACCOUNTS:
@@ -289,12 +288,11 @@ static void _pack_cond_msg(dbd_cond_msg_t *msg,
 	(*(my_function))(msg->cond, rpc_version, buffer);
 }
 
-static int _unpack_cond_msg(dbd_cond_msg_t **msg,
-			    uint16_t rpc_version,
-			    slurmdbd_msg_type_t type, Buf buffer)
+static int _unpack_cond_msg(dbd_cond_msg_t **msg, uint16_t rpc_version,
+			    slurmdbd_msg_type_t type, buf_t *buffer)
 {
 	dbd_cond_msg_t *msg_ptr = NULL;
-	int (*my_function) (void **object, uint16_t rpc_version, Buf buffer);
+	int (*my_function) (void **object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_GET_ACCOUNTS:
@@ -369,7 +367,7 @@ unpack_error:
 }
 
 static void _pack_job_complete_msg(dbd_job_comp_msg_t *msg,
-				   uint16_t rpc_version, Buf buffer)
+				   uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(msg->admin_comment, buffer);
@@ -391,7 +389,7 @@ static void _pack_job_complete_msg(dbd_job_comp_msg_t *msg,
 }
 
 static int _unpack_job_complete_msg(dbd_job_comp_msg_t **msg,
-				    uint16_t rpc_version, Buf buffer)
+				    uint16_t rpc_version, buf_t *buffer)
 {
 	uint32_t uint32_tmp;
 	dbd_job_comp_msg_t *msg_ptr = xmalloc(sizeof(dbd_job_comp_msg_t));
@@ -427,12 +425,56 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _pack_job_start_msg(void *in,
-				uint16_t rpc_version, Buf buffer)
+static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_job_start_msg_t *msg = (dbd_job_start_msg_t *)in;
 
-	if (rpc_version >= SLURM_19_05_PROTOCOL_VERSION) {
+	if (msg->script_buf)
+		msg->script = msg->script_buf->head;
+
+	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
+		packstr(msg->account, buffer);
+		pack32(msg->alloc_nodes, buffer);
+		pack32(msg->array_job_id, buffer);
+		pack32(msg->array_max_tasks, buffer);
+		pack32(msg->array_task_id, buffer);
+		packstr(msg->array_task_str, buffer);
+		pack32(msg->array_task_pending, buffer);
+		pack32(msg->assoc_id, buffer);
+		packstr(msg->constraints, buffer);
+		packstr(msg->container, buffer);
+		pack32(msg->db_flags, buffer);
+		pack64(msg->db_index, buffer);
+		pack_time(msg->eligible_time, buffer);
+		packstr(msg->env, buffer);
+		pack32(msg->gid, buffer);
+		packstr(msg->gres_used, buffer);
+		pack32(msg->job_id, buffer);
+		pack32(msg->job_state, buffer);
+		pack32(msg->state_reason_prev, buffer);
+		packstr(msg->mcs_label, buffer);
+		packstr(msg->name, buffer);
+		packstr(msg->nodes, buffer);
+		packstr(msg->node_inx, buffer);
+		pack32(msg->het_job_id, buffer);
+		pack32(msg->het_job_offset, buffer);
+		packstr(msg->partition, buffer);
+		pack32(msg->priority, buffer);
+		pack32(msg->qos_id, buffer);
+		pack32(msg->req_cpus, buffer);
+		pack64(msg->req_mem, buffer);
+		pack32(msg->resv_id, buffer);
+		packstr(msg->script, buffer);
+		pack_time(msg->start_time, buffer);
+		packstr(msg->submit_line, buffer);
+		pack_time(msg->submit_time, buffer);
+		pack32(msg->timelimit, buffer);
+		packstr(msg->tres_alloc_str, buffer);
+		packstr(msg->tres_req_str, buffer);
+		pack32(msg->uid, buffer);
+		packstr(msg->wckey, buffer);
+		packstr(msg->work_dir, buffer);
+	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
 		packstr(msg->account, buffer);
 		pack32(msg->alloc_nodes, buffer);
 		pack32(msg->array_job_id, buffer);
@@ -446,8 +488,6 @@ static void _pack_job_start_msg(void *in,
 		pack64(msg->db_index, buffer);
 		pack_time(msg->eligible_time, buffer);
 		pack32(msg->gid, buffer);
-		packstr(msg->gres_alloc, buffer);
-		packstr(msg->gres_req, buffer);
 		packstr(msg->gres_used, buffer);
 		pack32(msg->job_id, buffer);
 		pack32(msg->job_state, buffer);
@@ -481,15 +521,17 @@ static void _pack_job_start_msg(void *in,
 		packstr(msg->array_task_str, buffer);
 		pack32(msg->array_task_pending, buffer);
 		pack32(msg->assoc_id, buffer);
-		packnull(buffer); /* block_id */
+		packstr(msg->constraints, buffer);
+		pack32(msg->db_flags, buffer);
 		pack64(msg->db_index, buffer);
 		pack_time(msg->eligible_time, buffer);
 		pack32(msg->gid, buffer);
-		packstr(msg->gres_alloc, buffer);
-		packstr(msg->gres_req, buffer);
+		packnull(buffer);
+		packnull(buffer);
 		packstr(msg->gres_used, buffer);
 		pack32(msg->job_id, buffer);
 		pack32(msg->job_state, buffer);
+		pack32(msg->state_reason_prev, buffer);
 		packstr(msg->mcs_label, buffer);
 		packstr(msg->name, buffer);
 		packstr(msg->nodes, buffer);
@@ -511,10 +553,13 @@ static void _pack_job_start_msg(void *in,
 		packstr(msg->wckey, buffer);
 		packstr(msg->work_dir, buffer);
 	}
+
+	if (msg->script_buf)
+		msg->script = NULL;
 }
 
-static int _unpack_job_start_msg(void **msg,
-				 uint16_t rpc_version, Buf buffer)
+static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
+				 buf_t *buffer)
 {
 	uint32_t uint32_tmp;
 	dbd_job_start_msg_t *msg_ptr = xmalloc(sizeof(dbd_job_start_msg_t));
@@ -523,7 +568,58 @@ static int _unpack_job_start_msg(void **msg,
 	msg_ptr->array_job_id = 0;
 	msg_ptr->array_task_id = NO_VAL;
 
-	if (rpc_version >= SLURM_19_05_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
+		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
+		safe_unpack32(&msg_ptr->array_job_id, buffer);
+		safe_unpack32(&msg_ptr->array_max_tasks, buffer);
+		safe_unpack32(&msg_ptr->array_task_id, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->array_task_str,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->array_task_pending, buffer);
+		safe_unpack32(&msg_ptr->assoc_id, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->constraints,
+				       &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->container,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->db_flags, buffer);
+		safe_unpack64(&msg_ptr->db_index, buffer);
+		safe_unpack_time(&msg_ptr->eligible_time, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->env, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->gid, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
+				       buffer);
+		safe_unpack32(&msg_ptr->job_id, buffer);
+		safe_unpack32(&msg_ptr->job_state, buffer);
+		safe_unpack32(&msg_ptr->state_reason_prev, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->mcs_label,
+				       &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->nodes, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->node_inx, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->het_job_id, buffer);
+		safe_unpack32(&msg_ptr->het_job_offset, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->partition,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->priority, buffer);
+		safe_unpack32(&msg_ptr->qos_id, buffer);
+		safe_unpack32(&msg_ptr->req_cpus, buffer);
+		safe_unpack64(&msg_ptr->req_mem, buffer);
+		safe_unpack32(&msg_ptr->resv_id, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->script, &uint32_tmp, buffer);
+		safe_unpack_time(&msg_ptr->start_time, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->submit_line,
+				       &uint32_tmp, buffer);
+		safe_unpack_time(&msg_ptr->submit_time, buffer);
+		safe_unpack32(&msg_ptr->timelimit, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
+				       &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->tres_req_str,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->uid, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->wckey, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->work_dir, &uint32_tmp, buffer);
+	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
 		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
 		safe_unpack32(&msg_ptr->array_job_id, buffer);
@@ -539,10 +635,6 @@ static int _unpack_job_start_msg(void **msg,
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->eligible_time, buffer);
 		safe_unpack32(&msg_ptr->gid, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->gres_alloc, &uint32_tmp,
-				       buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->gres_req, &uint32_tmp,
-				       buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
 				       buffer);
 		safe_unpack32(&msg_ptr->job_id, buffer);
@@ -573,7 +665,7 @@ static int _unpack_job_start_msg(void **msg,
 		safe_unpackstr_xmalloc(&msg_ptr->wckey, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->work_dir, &uint32_tmp, buffer);
 	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		char *tmp_char = NULL;
+		char *tmp_ptr = NULL;
 		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
 		safe_unpack32(&msg_ptr->array_job_id, buffer);
@@ -583,19 +675,21 @@ static int _unpack_job_start_msg(void **msg,
 				       &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->array_task_pending, buffer);
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
-		safe_unpackstr_xmalloc(&tmp_char, &uint32_tmp, buffer);
-		xfree(tmp_char); /* block_id */
+		safe_unpackstr_xmalloc(&msg_ptr->constraints,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->db_flags, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->eligible_time, buffer);
 		safe_unpack32(&msg_ptr->gid, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->gres_alloc, &uint32_tmp,
-				       buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->gres_req, &uint32_tmp,
-				       buffer);
+		safe_unpackstr_xmalloc(&tmp_ptr, &uint32_tmp, buffer);
+		xfree(tmp_ptr);
+		safe_unpackstr_xmalloc(&tmp_ptr, &uint32_tmp, buffer);
+		xfree(tmp_ptr);
 		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
 				       buffer);
 		safe_unpack32(&msg_ptr->job_id, buffer);
 		safe_unpack32(&msg_ptr->job_state, buffer);
+		safe_unpack32(&msg_ptr->state_reason_prev, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->mcs_label,
 				       &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
@@ -632,7 +726,7 @@ unpack_error:
 }
 
 static void _pack_job_suspend_msg(dbd_job_suspend_msg_t *msg,
-				  uint16_t rpc_version, Buf buffer)
+				  uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
@@ -645,7 +739,7 @@ static void _pack_job_suspend_msg(dbd_job_suspend_msg_t *msg,
 }
 
 static int _unpack_job_suspend_msg(dbd_job_suspend_msg_t **msg,
-				   uint16_t rpc_version, Buf buffer)
+				   uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_job_suspend_msg_t *msg_ptr = xmalloc(sizeof(dbd_job_suspend_msg_t));
 	*msg = msg_ptr;
@@ -667,13 +761,11 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _pack_modify_msg(dbd_modify_msg_t *msg,
-			     uint16_t rpc_version,
-			     slurmdbd_msg_type_t type,
-			     Buf buffer)
+static void _pack_modify_msg(dbd_modify_msg_t *msg, uint16_t rpc_version,
+			     slurmdbd_msg_type_t type, buf_t *buffer)
 {
-	void (*my_cond) (void *object, uint16_t rpc_version, Buf buffer);
-	void (*my_rec) (void *object, uint16_t rpc_version, Buf buffer);
+	void (*my_cond) (void *object, uint16_t rpc_version, buf_t *buffer);
+	void (*my_rec) (void *object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_MODIFY_ACCOUNTS:
@@ -719,14 +811,12 @@ static void _pack_modify_msg(dbd_modify_msg_t *msg,
 	(*(my_rec))(msg->rec, rpc_version, buffer);
 }
 
-static int _unpack_modify_msg(dbd_modify_msg_t **msg,
-			      uint16_t rpc_version,
-			      slurmdbd_msg_type_t type,
-			      Buf buffer)
+static int _unpack_modify_msg(dbd_modify_msg_t **msg, uint16_t rpc_version,
+			      slurmdbd_msg_type_t type, buf_t *buffer)
 {
 	dbd_modify_msg_t *msg_ptr = NULL;
-	int (*my_cond) (void **object, uint16_t rpc_version, Buf buffer);
-	int (*my_rec) (void **object, uint16_t rpc_version, Buf buffer);
+	int (*my_cond) (void **object, uint16_t rpc_version, buf_t *buffer);
+	int (*my_rec) (void **object, uint16_t rpc_version, buf_t *buffer);
 
 	msg_ptr = xmalloc(sizeof(dbd_modify_msg_t));
 	*msg = msg_ptr;
@@ -786,7 +876,7 @@ unpack_error:
 }
 
 static void _pack_node_state_msg(dbd_node_state_msg_t *msg,
-				 uint16_t rpc_version, Buf buffer)
+				 uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(msg->hostlist, buffer);
@@ -800,7 +890,7 @@ static void _pack_node_state_msg(dbd_node_state_msg_t *msg,
 }
 
 static int _unpack_node_state_msg(dbd_node_state_msg_t **msg,
-				  uint16_t rpc_version, Buf buffer)
+				  uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_node_state_msg_t *msg_ptr;
 	uint32_t uint32_tmp;
@@ -830,7 +920,7 @@ unpack_error:
 }
 
 static void _pack_register_ctld_msg(dbd_register_ctld_msg_t *msg,
-				    uint16_t rpc_version, Buf buffer)
+				    uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack16(msg->dimensions, buffer);
@@ -841,7 +931,7 @@ static void _pack_register_ctld_msg(dbd_register_ctld_msg_t *msg,
 }
 
 static int _unpack_register_ctld_msg(dbd_register_ctld_msg_t **msg,
-				     uint16_t rpc_version, Buf buffer)
+				     uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_register_ctld_msg_t *msg_ptr = xmalloc(
 		sizeof(dbd_register_ctld_msg_t));
@@ -861,7 +951,7 @@ unpack_error:
 }
 
 static void _pack_roll_usage_msg(dbd_roll_usage_msg_t *msg,
-				 uint16_t rpc_version, Buf buffer)
+				 uint16_t rpc_version, buf_t *buffer)
 {
 	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack16(msg->archive_data, buffer);
@@ -871,7 +961,7 @@ static void _pack_roll_usage_msg(dbd_roll_usage_msg_t *msg,
 }
 
 static int _unpack_roll_usage_msg(dbd_roll_usage_msg_t **msg,
-				  uint16_t rpc_version, Buf buffer)
+				  uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_roll_usage_msg_t *msg_ptr = xmalloc(sizeof(dbd_roll_usage_msg_t));
 
@@ -891,48 +981,81 @@ unpack_error:
 }
 
 static void _pack_step_complete_msg(dbd_step_comp_msg_t *msg,
-				    uint16_t rpc_version, Buf buffer)
+				    uint16_t rpc_version, buf_t *buffer)
 {
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
 		pack64(msg->db_index, buffer);
 		pack_time(msg->end_time, buffer);
 		pack32(msg->exit_code, buffer);
 		jobacctinfo_pack((struct jobacctinfo *)msg->jobacct,
 				 rpc_version, PROTOCOL_TYPE_DBD, buffer);
-		pack32(msg->job_id, buffer);
 		pack_time(msg->job_submit_time, buffer);
 		packstr(msg->job_tres_alloc_str, buffer);
 		pack32(msg->req_uid, buffer);
 		pack_time(msg->start_time, buffer);
 		pack16(msg->state, buffer);
-		pack32(msg->step_id, buffer);
+		pack_step_id(&msg->step_id, buffer, rpc_version);
+		pack32(msg->total_tasks, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		pack32(msg->assoc_id, buffer);
+		pack64(msg->db_index, buffer);
+		pack_time(msg->end_time, buffer);
+		pack32(msg->exit_code, buffer);
+		jobacctinfo_pack((struct jobacctinfo *)msg->jobacct,
+				 rpc_version, PROTOCOL_TYPE_DBD, buffer);
+		pack32(msg->step_id.job_id, buffer);
+		pack_time(msg->job_submit_time, buffer);
+		packstr(msg->job_tres_alloc_str, buffer);
+		pack32(msg->req_uid, buffer);
+		pack_time(msg->start_time, buffer);
+		pack16(msg->state, buffer);
+		pack_old_step_id(msg->step_id.step_id, buffer);
 		pack32(msg->total_tasks, buffer);
 	}
 }
 
 static int _unpack_step_complete_msg(dbd_step_comp_msg_t **msg,
-				     uint16_t rpc_version, Buf buffer)
+				     uint16_t rpc_version, buf_t *buffer)
 {
 	uint32_t uint32_tmp;
 	dbd_step_comp_msg_t *msg_ptr = xmalloc(sizeof(dbd_step_comp_msg_t));
 	*msg = msg_ptr;
 
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->end_time, buffer);
 		safe_unpack32(&msg_ptr->exit_code, buffer);
 		jobacctinfo_unpack((struct jobacctinfo **)&msg_ptr->jobacct,
 				   rpc_version, PROTOCOL_TYPE_DBD, buffer, 1);
-		safe_unpack32(&msg_ptr->job_id, buffer);
 		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->job_tres_alloc_str,
 				       &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->req_uid, buffer);
 		safe_unpack_time(&msg_ptr->start_time, buffer);
 		safe_unpack16(&msg_ptr->state, buffer);
-		safe_unpack32(&msg_ptr->step_id, buffer);
+		if (unpack_step_id_members(&msg_ptr->step_id, buffer,
+					   rpc_version) != SLURM_SUCCESS)
+			goto unpack_error;
+		safe_unpack32(&msg_ptr->total_tasks, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		safe_unpack32(&msg_ptr->assoc_id, buffer);
+		safe_unpack64(&msg_ptr->db_index, buffer);
+		safe_unpack_time(&msg_ptr->end_time, buffer);
+		safe_unpack32(&msg_ptr->exit_code, buffer);
+		jobacctinfo_unpack((struct jobacctinfo **)&msg_ptr->jobacct,
+				   rpc_version, PROTOCOL_TYPE_DBD, buffer, 1);
+		safe_unpack32(&msg_ptr->step_id.job_id, buffer);
+		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->job_tres_alloc_str,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->req_uid, buffer);
+		safe_unpack_time(&msg_ptr->start_time, buffer);
+		safe_unpack16(&msg_ptr->state, buffer);
+		safe_unpack32(&msg_ptr->step_id.step_id, buffer);
+		convert_old_step_id(&msg_ptr->step_id.step_id);
+		msg_ptr->step_id.step_het_comp = NO_VAL;
 		safe_unpack32(&msg_ptr->total_tasks, buffer);
 	} else
 		goto unpack_error;
@@ -950,12 +1073,12 @@ unpack_error:
 
 static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 				 uint16_t rpc_version,
-				 Buf buffer)
+				 buf_t *buffer)
 {
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
 		pack64(msg->db_index, buffer);
-		pack32(msg->job_id, buffer);
+		packstr(msg->container, buffer);
 		packstr(msg->name, buffer);
 		packstr(msg->nodes, buffer);
 		packstr(msg->node_inx, buffer);
@@ -965,7 +1088,41 @@ static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 		pack32(msg->req_cpufreq_min, buffer);
 		pack32(msg->req_cpufreq_max, buffer);
 		pack32(msg->req_cpufreq_gov, buffer);
-		pack32(msg->step_id, buffer);
+		pack_step_id(&msg->step_id, buffer, rpc_version);
+		packstr(msg->submit_line, buffer);
+		pack32(msg->task_dist, buffer);
+		pack32(msg->total_tasks, buffer);
+		packstr(msg->tres_alloc_str, buffer);
+	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+		pack32(msg->assoc_id, buffer);
+		pack64(msg->db_index, buffer);
+		packstr(msg->name, buffer);
+		packstr(msg->nodes, buffer);
+		packstr(msg->node_inx, buffer);
+		pack32(msg->node_cnt, buffer);
+		pack_time(msg->start_time, buffer);
+		pack_time(msg->job_submit_time, buffer);
+		pack32(msg->req_cpufreq_min, buffer);
+		pack32(msg->req_cpufreq_max, buffer);
+		pack32(msg->req_cpufreq_gov, buffer);
+		pack_step_id(&msg->step_id, buffer, rpc_version);
+		pack32(msg->task_dist, buffer);
+		pack32(msg->total_tasks, buffer);
+		packstr(msg->tres_alloc_str, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		pack32(msg->assoc_id, buffer);
+		pack64(msg->db_index, buffer);
+		pack32(msg->step_id.job_id, buffer);
+		packstr(msg->name, buffer);
+		packstr(msg->nodes, buffer);
+		packstr(msg->node_inx, buffer);
+		pack32(msg->node_cnt, buffer);
+		pack_time(msg->start_time, buffer);
+		pack_time(msg->job_submit_time, buffer);
+		pack32(msg->req_cpufreq_min, buffer);
+		pack32(msg->req_cpufreq_max, buffer);
+		pack32(msg->req_cpufreq_gov, buffer);
+		pack_old_step_id(msg->step_id.step_id, buffer);
 		pack32(msg->task_dist, buffer);
 		pack32(msg->total_tasks, buffer);
 		packstr(msg->tres_alloc_str, buffer);
@@ -973,16 +1130,16 @@ static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 }
 
 static int _unpack_step_start_msg(dbd_step_start_msg_t **msg,
-				  uint16_t rpc_version, Buf buffer)
+				  uint16_t rpc_version, buf_t *buffer)
 {
 	uint32_t uint32_tmp = 0;
 	dbd_step_start_msg_t *msg_ptr = xmalloc(sizeof(dbd_step_start_msg_t));
 	*msg = msg_ptr;
 
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
-		safe_unpack32(&msg_ptr->job_id, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->container, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->nodes, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->node_inx, &uint32_tmp, buffer);
@@ -992,7 +1149,50 @@ static int _unpack_step_start_msg(dbd_step_start_msg_t **msg,
 		safe_unpack32(&msg_ptr->req_cpufreq_min, buffer);
 		safe_unpack32(&msg_ptr->req_cpufreq_max, buffer);
 		safe_unpack32(&msg_ptr->req_cpufreq_gov, buffer);
-		safe_unpack32(&msg_ptr->step_id, buffer);
+		if (unpack_step_id_members(&msg_ptr->step_id, buffer,
+					   rpc_version) != SLURM_SUCCESS)
+			goto unpack_error;
+		safe_unpackstr_xmalloc(&msg_ptr->submit_line,
+				       &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->task_dist, buffer);
+		safe_unpack32(&msg_ptr->total_tasks, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
+				       &uint32_tmp, buffer);
+	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+		safe_unpack32(&msg_ptr->assoc_id, buffer);
+		safe_unpack64(&msg_ptr->db_index, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->nodes, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->node_inx, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->node_cnt, buffer);
+		safe_unpack_time(&msg_ptr->start_time, buffer);
+		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_min, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_max, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_gov, buffer);
+		if (unpack_step_id_members(&msg_ptr->step_id, buffer,
+					   rpc_version) != SLURM_SUCCESS)
+			goto unpack_error;
+		safe_unpack32(&msg_ptr->task_dist, buffer);
+		safe_unpack32(&msg_ptr->total_tasks, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
+				       &uint32_tmp, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		safe_unpack32(&msg_ptr->assoc_id, buffer);
+		safe_unpack64(&msg_ptr->db_index, buffer);
+		safe_unpack32(&msg_ptr->step_id.job_id, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->nodes, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->node_inx, &uint32_tmp, buffer);
+		safe_unpack32(&msg_ptr->node_cnt, buffer);
+		safe_unpack_time(&msg_ptr->start_time, buffer);
+		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_min, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_max, buffer);
+		safe_unpack32(&msg_ptr->req_cpufreq_gov, buffer);
+		safe_unpack32(&msg_ptr->step_id.step_id, buffer);
+		convert_old_step_id(&msg_ptr->step_id.step_id);
+		msg_ptr->step_id.step_het_comp = NO_VAL;
 		safe_unpack32(&msg_ptr->task_dist, buffer);
 		safe_unpack32(&msg_ptr->total_tasks, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
@@ -1011,20 +1211,16 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-static void _pack_buffer(void *in,
-			 uint16_t rpc_version,
-			 Buf buffer)
+static void _pack_buffer(void *in, uint16_t rpc_version, buf_t *buffer)
 {
-	Buf object = (Buf)in;
+	buf_t *object = (buf_t *) in;
 
 	packmem(get_buf_data(object), get_buf_offset(object), buffer);
 }
 
-static int _unpack_buffer(void **out,
-			  uint16_t rpc_version,
-			  Buf buffer)
+static int _unpack_buffer(void **out, uint16_t rpc_version, buf_t *buffer)
 {
-	Buf out_ptr = NULL;
+	buf_t *out_ptr = NULL;
 	char *msg = NULL;
 	uint32_t uint32_tmp;
 
@@ -1043,8 +1239,8 @@ unpack_error:
 
 }
 
-extern void slurmdbd_pack_id_rc_msg(void *in,
-				    uint16_t rpc_version, Buf buffer)
+extern void slurmdbd_pack_id_rc_msg(void *in, uint16_t rpc_version,
+				    buf_t *buffer)
 {
 	dbd_id_rc_msg_t *msg = (dbd_id_rc_msg_t *)in;
 
@@ -1055,8 +1251,8 @@ extern void slurmdbd_pack_id_rc_msg(void *in,
 	}
 }
 
-extern int slurmdbd_unpack_id_rc_msg(void **msg,
-				     uint16_t rpc_version, Buf buffer)
+extern int slurmdbd_unpack_id_rc_msg(void **msg, uint16_t rpc_version,
+				     buf_t *buffer)
 {
 	dbd_id_rc_msg_t *msg_ptr = xmalloc(sizeof(dbd_id_rc_msg_t));
 
@@ -1076,12 +1272,10 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void slurmdbd_pack_usage_msg(dbd_usage_msg_t *msg,
-				    uint16_t rpc_version,
-				    slurmdbd_msg_type_t type,
-				    Buf buffer)
+extern void slurmdbd_pack_usage_msg(dbd_usage_msg_t *msg, uint16_t rpc_version,
+				    slurmdbd_msg_type_t type, buf_t *buffer)
 {
-	void (*my_rec) (void *object, uint16_t rpc_version, Buf buffer);
+	void (*my_rec) (void *object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_GET_ASSOC_USAGE:
@@ -1109,10 +1303,10 @@ extern void slurmdbd_pack_usage_msg(dbd_usage_msg_t *msg,
 extern int slurmdbd_unpack_usage_msg(dbd_usage_msg_t **msg,
 				     uint16_t rpc_version,
 				     slurmdbd_msg_type_t type,
-				     Buf buffer)
+				     buf_t *buffer)
 {
 	dbd_usage_msg_t *msg_ptr = NULL;
-	int (*my_rec) (void **object, uint16_t rpc_version, Buf buffer);
+	int (*my_rec) (void **object, uint16_t rpc_version, buf_t *buffer);
 
 	msg_ptr = xmalloc(sizeof(dbd_usage_msg_t));
 	*msg = msg_ptr;
@@ -1150,15 +1344,15 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void slurmdbd_pack_fini_msg(dbd_fini_msg_t *msg,
-				   uint16_t rpc_version, Buf buffer)
+extern void slurmdbd_pack_fini_msg(dbd_fini_msg_t *msg, uint16_t rpc_version,
+				   buf_t *buffer)
 {
 	pack16(msg->close_conn, buffer);
 	pack16(msg->commit, buffer);
 }
 
-extern int slurmdbd_unpack_fini_msg(dbd_fini_msg_t **msg,
-				    uint16_t rpc_version, Buf buffer)
+extern int slurmdbd_unpack_fini_msg(dbd_fini_msg_t **msg, uint16_t rpc_version,
+				    buf_t *buffer)
 {
 	dbd_fini_msg_t *msg_ptr = xmalloc(sizeof(dbd_fini_msg_t));
 	*msg = msg_ptr;
@@ -1174,13 +1368,11 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void slurmdbd_pack_list_msg(dbd_list_msg_t *msg,
-				   uint16_t rpc_version,
-				   slurmdbd_msg_type_t type,
-				   Buf buffer)
+extern void slurmdbd_pack_list_msg(dbd_list_msg_t *msg, uint16_t rpc_version,
+				   slurmdbd_msg_type_t type, buf_t *buffer)
 {
 	int rc;
-	void (*my_function) (void *object, uint16_t rpc_version, Buf buffer);
+	void (*my_function) (void *object, uint16_t rpc_version, buf_t *buffer);
 
 	switch (type) {
 	case DBD_ADD_ACCOUNTS:
@@ -1240,8 +1432,10 @@ extern void slurmdbd_pack_list_msg(dbd_list_msg_t *msg,
 		my_function = slurmdb_pack_event_rec;
 		break;
 	case DBD_SEND_MULT_JOB_START:
-		my_function = _pack_job_start_msg;
-		break;
+		slurm_pack_list_until(msg->my_list, _pack_job_start_msg,
+				      buffer, MAX_MSG_SIZE, rpc_version);
+		pack32(msg->return_code, buffer);
+		return;
 	case DBD_GOT_MULT_JOB_START:
 		my_function = slurmdbd_pack_id_rc_msg;
 		break;
@@ -1262,10 +1456,10 @@ extern void slurmdbd_pack_list_msg(dbd_list_msg_t *msg,
 }
 
 extern int slurmdbd_unpack_list_msg(dbd_list_msg_t **msg, uint16_t rpc_version,
-				    slurmdbd_msg_type_t type, Buf buffer)
+				    slurmdbd_msg_type_t type, buf_t *buffer)
 {
 	dbd_list_msg_t *msg_ptr = NULL;
-	int (*my_function) (void **object, uint16_t rpc_version, Buf buffer);
+	int (*my_function) (void **object, uint16_t rpc_version, buf_t *buffer);
 	void (*my_destroy) (void *object);
 
 	switch (type) {
@@ -1375,9 +1569,9 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern Buf pack_slurmdbd_msg(persist_msg_t *req, uint16_t rpc_version)
+extern buf_t *pack_slurmdbd_msg(persist_msg_t *req, uint16_t rpc_version)
 {
-	Buf buffer;
+	buf_t *buffer;
 
 	if (rpc_version < SLURM_MIN_PROTOCOL_VERSION) {
 		error("slurmdbd: Invalid message version=%hu, type:%hu",
@@ -1564,8 +1758,8 @@ extern Buf pack_slurmdbd_msg(persist_msg_t *req, uint16_t rpc_version)
 	return buffer;
 }
 
-extern int unpack_slurmdbd_msg(persist_msg_t *resp,
-			       uint16_t rpc_version, Buf buffer)
+extern int unpack_slurmdbd_msg(persist_msg_t *resp, uint16_t rpc_version,
+			       buf_t *buffer)
 {
 	int rc = SLURM_SUCCESS;
 	slurm_msg_t msg;
