@@ -129,7 +129,7 @@ static int dataset_id = -1; /* id of the dataset for profile data */
 /* one cpu in the package */
 static int pkg2cpu[MAX_PKGS] = {[0 ... MAX_PKGS-1] = -1};
 static int pkg_fd[MAX_PKGS] = {[0 ... MAX_PKGS-1] = -1};
-static char hostname[MAXHOSTNAMELEN];
+static char hostname[HOST_NAME_MAX];
 
 static int nb_pkg = 0;
 
@@ -360,12 +360,13 @@ static void _get_joules_task(acct_gather_energy_t *energy)
 			(uint64_t)ret - energy->base_consumed_energy;
 		energy->current_watts =
 			(uint32_t)ret - energy->previous_consumed_energy;
-		energy->ave_watts =  ((energy->ave_watts * readings) +
-				       energy->current_watts) / (readings + 1);
 
 		interval = time(NULL) - energy->poll_time;
 		if (interval)	/* Prevent divide by zero */
 			energy->current_watts /= (float)interval;
+
+		energy->ave_watts =  ((energy->ave_watts * readings) +
+				      energy->current_watts) / (readings + 1);
 	} else {
 		energy->consumed_energy = 1;
 		energy->base_consumed_energy = (uint64_t)ret;
@@ -375,8 +376,11 @@ static void _get_joules_task(acct_gather_energy_t *energy)
 	energy->previous_consumed_energy = (uint64_t)ret;
 	energy->poll_time = time(NULL);
 
-	log_flag(ENERGY, "%s: current %.6f Joules, consumed %"PRIu64"",
-		 __func__, ret, energy->consumed_energy);
+	log_flag(ENERGY, "PollTime = %ld, ConsumedEnergy = %"PRIu64"J, CurrentWatts = %uW, AveWatts = %uW",
+		 energy->poll_time,
+		 energy->consumed_energy,
+		 energy->current_watts,
+		 energy->ave_watts);
 }
 
 static int _running_profile(void)
@@ -454,7 +458,7 @@ extern int acct_gather_energy_p_update_node_energy(void)
  */
 extern int init(void)
 {
-	gethostname(hostname, MAXHOSTNAMELEN);
+	gethostname(hostname, HOST_NAME_MAX);
 
 	/* put anything that requires the .conf being read in
 	   acct_gather_energy_p_conf_parse
