@@ -49,6 +49,7 @@
 #include <dlfcn.h>
 
 #include "src/common/slurm_xlator.h"
+#include "src/common/cgroup.h"
 #include "src/common/slurm_acct_gather_energy.h"
 #include "src/common/slurm_acct_gather_profile.h"
 #include "src/common/gres.h"
@@ -106,7 +107,6 @@ static uint64_t *start_current_energies = NULL;
 
 static int dataset_id = -1; // id of the dataset for profile data
 
-static uint64_t debug_flags = 0;
 static bool flag_energy_accounting_shutdown = false;
 static bool flag_thread_started = false;
 static pthread_mutex_t rsmi_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -162,8 +162,8 @@ static int _send_profile(void)
 			"Energy", NO_PARENT, dataset);
 		for (i = 0; i < gpus_len; i++)
 			xfree(dataset[i].name);
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			debug("Energy: dataset created (id = %d)", dataset_id);
+		log_flag(ENERGY, "Energy: dataset created (id = %d)",
+			 dataset_id);
 		if (dataset_id == SLURM_ERROR) {
 			error("Energy: Failed to create the dataset");
 			return SLURM_ERROR;
@@ -177,7 +177,7 @@ static int _send_profile(void)
 		last_time = gpus[i].energy.poll_time;
 	}
 
-	if (debug_flags & DEBUG_FLAG_PROFILE) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PROFILE) {
 		for (i = 0; i < gpus_len; i++) {
 			info("PROFILE-Energy: GPU%dPower=%"PRIu64"",
 			     i, data[i]);
@@ -278,7 +278,7 @@ static int _thread_update_node_energy(void)
 	}
 	readings++;
 
-	if (debug_flags & DEBUG_FLAG_ENERGY) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_ENERGY) {
 		for (i = 0; i < gpus_len; i++)
 			info("rsmi-thread: gpu %u current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
 			     i,
@@ -314,8 +314,7 @@ static int _thread_init(void)
 {
 
 	if (gpus_len && gpus) {
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("%s thread init", plugin_name);
+		log_flag(ENERGY, "%s thread init", plugin_name);
 		return SLURM_SUCCESS;
 	} else {
 		error("%s thread init failed, no GPU available", plugin_name);
@@ -333,16 +332,14 @@ static void *_thread_rsmi_run(void *no_data)
 	struct timespec abs;
 
 	flag_energy_accounting_shutdown = false;
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("rsmi-thread: launched");
+	log_flag(ENERGY, "rsmi-thread: launched");
 
 	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
 	slurm_mutex_lock(&rsmi_mutex);
 	if (_thread_init() != SLURM_SUCCESS) {
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("rsmi-thread: aborted");
+		log_flag(ENERGY, "rsmi-thread: aborted");
 		slurm_mutex_unlock(&rsmi_mutex);
 
 		slurm_mutex_lock(&launch_mutex);
@@ -379,8 +376,7 @@ static void *_thread_rsmi_run(void *no_data)
 		slurm_mutex_unlock(&rsmi_mutex);
 	}
 
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("rsmi-thread: ended");
+	log_flag(ENERGY, "rsmi-thread: ended");
 
 	return NULL;
 }
@@ -446,14 +442,10 @@ static void _add_energy(acct_gather_energy_t *energy_tot,
 	if (!energy_tot->poll_time ||
 	    (energy_tot->poll_time > energy_new->poll_time))
 		energy_tot->poll_time = energy_new->poll_time;
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("%s: gpu: %d, current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
-		     __func__,
-		     gpu_num,
-		     energy_new->current_watts,
-		     energy_new->consumed_energy,
-		     energy_new->base_consumed_energy,
-		     energy_new->ave_watts);
+	log_flag(ENERGY, "%s: gpu: %d, current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
+		 __func__, gpu_num, energy_new->current_watts,
+		 energy_new->consumed_energy, energy_new->base_consumed_energy,
+		 energy_new->ave_watts);
 }
 
 /* Get the energy for a job
@@ -461,35 +453,26 @@ static void _add_energy(acct_gather_energy_t *energy_tot,
  */
 static void _get_node_energy_up(acct_gather_energy_t *energy)
 {
-	slurm_cgroup_conf_t *cg_conf;
 	bool task_cgroup = false;
 	bool constrained_devices = false;
 	bool cgroups_active = false;
-	char *task_plugin_type = NULL;
 
 	uint16_t i;
 
 	// Check if GPUs are constrained by cgroups
-	slurm_mutex_lock(&xcgroup_config_read_mutex);
-	cg_conf = xcgroup_get_slurm_cgroup_conf();
-	if (cg_conf && cg_conf->constrain_devices)
-		constrained_devices = true;
-	slurm_mutex_unlock(&xcgroup_config_read_mutex);
+	cgroup_conf_init();
+	constrained_devices = slurm_cgroup_conf.constrain_devices;
 
 	// Check if task/cgroup plugin is loaded
-	task_plugin_type = slurm_get_task_plugin();
-	if (xstrstr(task_plugin_type, "cgroup"))
+	if (xstrstr(slurm_conf.task_plugin, "cgroup"))
 		task_cgroup = true;
-	xfree(task_plugin_type);
 
 	// If both of these are true, then GPUs will be constrained
 	if (constrained_devices && task_cgroup) {
 		cgroups_active = true;
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			debug2("%s: cgroups are configured.", __func__);
+		log_flag(ENERGY, "%s: cgroups are configured.", __func__);
 	} else {
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			debug2("%s: cgroups are NOT configured.", __func__);
+		log_flag(ENERGY, "%s: cgroups are NOT configured.", __func__);
 	}
 
 	// sum the energy of all gpus for this job
@@ -497,19 +480,14 @@ static void _get_node_energy_up(acct_gather_energy_t *energy)
 	for (i = 0; i < gpus_len; i++) {
 		// Skip if not using cgroups, or bit is not set
 		if (cgroups_active && !bit_test(saved_usable_gpus, i)) {
-			if (debug_flags & DEBUG_FLAG_ENERGY)
-				debug2("Passing over gpu %u", i);
+			log_flag(ENERGY, "Passing over gpu %u", i);
 			continue;
 		}
 		_add_energy(energy, &gpus[i].energy, i);
 	}
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("%s: current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
-		     __func__,
-		     energy->current_watts,
-		     energy->consumed_energy,
-		     energy->base_consumed_energy,
-		     energy->ave_watts);
+	log_flag(ENERGY, "%s: current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
+		 __func__, energy->current_watts, energy->consumed_energy,
+		 energy->base_consumed_energy, energy->ave_watts);
 }
 
 /* Get the energy for a node
@@ -523,13 +501,9 @@ static void _get_node_energy(acct_gather_energy_t *energy)
 	memset(energy, 0, sizeof(acct_gather_energy_t));
 	for (i = 0; i < gpus_len; i++)
 		_add_energy(energy, &gpus[i].energy, i);
-	if (debug_flags & DEBUG_FLAG_ENERGY)
-		info("%s: current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
-		     __func__,
-		     energy->current_watts,
-		     energy->consumed_energy,
-		     energy->base_consumed_energy,
-		     energy->ave_watts);
+	log_flag(ENERGY, "%s: current_watts: %u, consumed %"PRIu64" Joules %"PRIu64" new, ave watts %u",
+		 __func__, energy->current_watts, energy->consumed_energy,
+		 energy->base_consumed_energy, energy->ave_watts);
 }
 
 /* Get the energy in joules for a job
@@ -595,12 +569,9 @@ static int _get_joules_task(uint16_t delta)
 			+ new->base_consumed_energy;
 		memcpy(old, new, sizeof(acct_gather_energy_t));
 
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("%s: consumed %"PRIu64" Joules (received %"PRIu64"(%u watts) from slurmd)",
-			     __func__,
-			     new->consumed_energy,
-			     new->base_consumed_energy,
-			     new->current_watts);
+		log_flag(ENERGY, "%s: consumed %"PRIu64" Joules (received %"PRIu64"(%u watts) from slurmd)",
+			 __func__, new->consumed_energy,
+			 new->base_consumed_energy, new->current_watts);
 	}
 
 	acct_gather_energy_destroy(energies);
@@ -619,8 +590,6 @@ extern int init(void)
 	if (!dlopen("librocm_smi64.so", RTLD_NOW | RTLD_GLOBAL))
 		fatal("RSMI configured, but wasn't found.");
 
-	debug_flags = slurm_get_debug_flags();
-
 	/* put anything that requires the .conf being read in
 	   acct_gather_energy_p_conf_parse
 	*/
@@ -635,7 +604,7 @@ extern int init(void)
  */
 extern int fini(void)
 {
-	if (!running_in_slurmdstepd())
+	if (!running_in_slurmd_stepd())
 		return SLURM_SUCCESS;
 
 	flag_energy_accounting_shutdown = true;
@@ -667,7 +636,7 @@ extern int fini(void)
 extern int acct_gather_energy_p_update_node_energy(void)
 {
 	int rc = SLURM_SUCCESS;
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	return rc;
 }
@@ -681,7 +650,7 @@ extern int acct_gather_energy_p_get_data(enum acct_energy_type data_type,
 	time_t *last_poll = (time_t *)data;
 	uint16_t *gpu_cnt = (uint16_t *)data;
 
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	switch (data_type) {
 	case ENERGY_DATA_NODE_ENERGY_UP:
@@ -747,11 +716,10 @@ extern int acct_gather_energy_p_set_data(enum acct_energy_type data_type,
 	int rc = SLURM_SUCCESS;
 	int *delta = (int *)data;
 
-	xassert(running_in_slurmdstepd());
+	xassert(running_in_slurmd_stepd());
 
 	switch (data_type) {
 	case ENERGY_DATA_RECONFIG:
-		debug_flags = slurm_get_debug_flags();
 		break;
 	case ENERGY_DATA_PROFILE:
 		slurm_mutex_lock(&rsmi_mutex);
@@ -778,10 +746,9 @@ extern int acct_gather_energy_p_set_data(enum acct_energy_type data_type,
 			saved_usable_gpus = usable_gpus;
 			usable_gpus = NULL;
 		}
-		if (debug_flags & DEBUG_FLAG_ENERGY)
-			info("usable_gpus = %d of %ld",
-			     bit_set_count(saved_usable_gpus),
-			     bit_size(saved_usable_gpus));
+		log_flag(ENERGY, "usable_gpus = %d of %ld",
+			 bit_set_count(saved_usable_gpus),
+			 bit_size(saved_usable_gpus));
 		break;
 	}
 	default:
@@ -806,7 +773,7 @@ extern void acct_gather_energy_p_conf_set(int context_id_in,
 
 	context_id = context_id_in;
 
-	if (!running_in_slurmdstepd())
+	if (!running_in_slurmd_stepd())
 		return;
 
 	if (!flag_init) {
@@ -818,8 +785,7 @@ extern void acct_gather_energy_p_conf_set(int context_id_in,
 				slurm_thread_create(&thread_rsmi_id_launcher,
 						    _thread_launcher, NULL);
 			}
-			if (debug_flags & DEBUG_FLAG_ENERGY)
-				info("%s thread launched", plugin_name);
+			log_flag(ENERGY, "%s thread launched", plugin_name);
 		} else
 			_get_joules_task(0);
 
