@@ -122,7 +122,6 @@ static hid_t     gid_samples = -1;
 static hid_t     gid_totals = -1;
 static char      group_node[MAX_GROUP_NAME+1];
 static slurm_hdf5_conf_t hdf5_conf;
-static uint64_t debug_flags = 0;
 static uint32_t g_profile_running = ACCT_GATHER_PROFILE_NOT_SET;
 static stepd_step_rec_t *g_job = NULL;
 static time_t step_start_time;
@@ -154,50 +153,32 @@ static uint32_t _determine_profile(void)
 	return profile;
 }
 
-static int _create_directories(void)
+static void _create_directories(void)
 {
-	int rc;
-	struct stat st;
-	char   *user_dir = NULL;
+	char *user_dir = NULL;
 
 	xassert(g_job);
 	xassert(hdf5_conf.dir);
+
+	xstrfmtcat(user_dir, "%s/%s", hdf5_conf.dir, g_job->user_name);
+
 	/*
-	 * If profile director does not exist, try to create it.
-	 *  Otherwise, ensure path is a directory as expected, and that
-	 *  we have permission to write to it.
-	 *  also make sure the subdirectory tmp exists.
+	 * To avoid race conditions (TOCTOU) with stat() calls, always
+	 * attempt to create the ProfileHDF5Dir and the user directory within.
 	 */
+	if (((mkdir(hdf5_conf.dir, 0755)) < 0) && (errno != EEXIST))
+		fatal("mkdir(%s): %m", hdf5_conf.dir);
+	if (chmod(hdf5_conf.dir, 0755) < 0)
+		fatal("chmod(%s): %m", hdf5_conf.dir);
 
-	if (((rc = stat(hdf5_conf.dir, &st)) < 0) && (errno == ENOENT)) {
-		if (mkdir(hdf5_conf.dir, 0755) < 0)
-			fatal("mkdir(%s): %m", hdf5_conf.dir);
-	} else if (rc < 0)
-		fatal("Unable to stat acct_gather_profile_dir: %s: %m",
-		      hdf5_conf.dir);
-	else if (!S_ISDIR(st.st_mode))
-		fatal("acct_gather_profile_dir: %s: Not a directory!",
-		      hdf5_conf.dir);
-	else if (access(hdf5_conf.dir, R_OK|W_OK|X_OK) < 0)
-		fatal("Incorrect permissions on acct_gather_profile_dir: %s",
-		      hdf5_conf.dir);
-	if (chmod(hdf5_conf.dir, 0755) == -1)
-		error("%s: chmod(%s): %m", __func__, hdf5_conf.dir);
-
-	user_dir = xstrdup_printf("%s/%s", hdf5_conf.dir, g_job->user_name);
-	if (((rc = stat(user_dir, &st)) < 0) && (errno == ENOENT)) {
-		if (mkdir(user_dir, 0700) < 0)
-			fatal("mkdir(%s): %m", user_dir);
-	}
-	if (chmod(user_dir, 0700) == -1)
-		error("%s: chmod(%s): %m", __func__, user_dir);
-	if (chown(user_dir, (uid_t)g_job->uid,
-		  (gid_t)g_job->gid) < 0)
-		error("chown(%s): %m", user_dir);
+	if (((mkdir(user_dir, 0700)) < 0) && (errno != EEXIST))
+		fatal("mkdir(%s): %m", user_dir);
+	if (chmod(user_dir, 0700) < 0)
+		fatal("chmod(%s): %m", user_dir);
+	if (chown(user_dir, g_job->uid, g_job->gid) < 0)
+		fatal("chown(%s): %m", user_dir);
 
 	xfree(user_dir);
-
-	return SLURM_SUCCESS;
 }
 
 /*
@@ -209,8 +190,6 @@ extern int init(void)
 	if (!running_in_slurmstepd())
 		return SLURM_SUCCESS;
 
-	debug_flags = slurm_get_debug_flags();
-
 	/* Move HDF5 trace printing to log file instead of stderr */
 	H5Eset_auto(H5E_DEFAULT, (herr_t (*)(hid_t, void *))H5Eprint,
 	            log_fp());
@@ -290,7 +269,6 @@ extern int acct_gather_profile_p_node_step_start(stepd_step_rec_t* job)
 	int rc = SLURM_SUCCESS;
 
 	char *profile_file_name;
-	char *profile_str;
 
 	xassert(running_in_slurmstepd());
 
@@ -298,10 +276,8 @@ extern int acct_gather_profile_p_node_step_start(stepd_step_rec_t* job)
 
 	xassert(hdf5_conf.dir);
 
-	if (debug_flags & DEBUG_FLAG_PROFILE) {
-		profile_str = acct_gather_profile_to_string(g_job->profile);
-		info("PROFILE: option --profile=%s", profile_str);
-	}
+	log_flag(PROFILE, "PROFILE: option --profile=%s",
+		 acct_gather_profile_to_string(g_job->profile));
 
 	if (g_profile_running == ACCT_GATHER_PROFILE_NOT_SET)
 		g_profile_running = _determine_profile();
@@ -315,25 +291,24 @@ extern int acct_gather_profile_p_node_step_start(stepd_step_rec_t* job)
 	 * Use a more user friendly string "batch" rather
 	 * then 4294967294.
 	 */
-	if (g_job->stepid == NO_VAL) {
+	if (g_job->step_id.step_id == SLURM_BATCH_SCRIPT) {
 		profile_file_name = xstrdup_printf("%s/%s/%u_%s_%s.h5",
 						   hdf5_conf.dir,
 						   g_job->user_name,
-						   g_job->jobid,
+						   g_job->step_id.job_id,
 						   "batch",
 						   g_job->node_name);
 	} else {
 		profile_file_name = xstrdup_printf(
 			"%s/%s/%u_%u_%s.h5",
 			hdf5_conf.dir, g_job->user_name,
-			g_job->jobid, g_job->stepid, g_job->node_name);
+			g_job->step_id.job_id, g_job->step_id.step_id,
+			g_job->node_name);
 	}
 
-	if (debug_flags & DEBUG_FLAG_PROFILE) {
-		profile_str = acct_gather_profile_to_string(g_profile_running);
-		info("PROFILE: node_step_start, opt=%s file=%s",
-		     profile_str, profile_file_name);
-	}
+	log_flag(PROFILE, "PROFILE: node_step_start, opt=%s file=%s",
+		 acct_gather_profile_to_string(g_profile_running),
+		 profile_file_name);
 
 	/*
 	 * Create a new file using the default properties
@@ -406,8 +381,7 @@ extern int acct_gather_profile_p_node_step_end(void)
 	if (g_profile_running <= ACCT_GATHER_PROFILE_NONE)
 		return rc;
 
-	if (debug_flags & DEBUG_FLAG_PROFILE)
-		info("PROFILE: node_step_end (shutdown)");
+	log_flag(PROFILE, "PROFILE: node_step_end (shutdown)");
 
 	/* close tables */
 	for (i = 0; i < tables_cur_len; ++i) {
@@ -446,16 +420,14 @@ extern int acct_gather_profile_p_task_start(uint32_t taskid)
 	if (g_profile_running <= ACCT_GATHER_PROFILE_NONE)
 		return rc;
 
-	if (debug_flags & DEBUG_FLAG_PROFILE)
-		info("PROFILE: task_start");
+	log_flag(PROFILE, "PROFILE: task_start");
 
 	return rc;
 }
 
 extern int acct_gather_profile_p_task_end(pid_t taskpid)
 {
-	if (debug_flags & DEBUG_FLAG_PROFILE)
-		info("PROFILE: task_end");
+	log_flag(PROFILE, "PROFILE: task_end");
 	return SLURM_SUCCESS;
 }
 
