@@ -140,9 +140,12 @@ static void _free_tables(void)
 
 	debug3("%s %s called", plugin_type, __func__);
 
+	if (!tables)
+		return;
+
 	for (i = 0; i < tables_cur_len; i++) {
 		table_t *table = &(tables[i]);
-		for (j = 0; j < tables->size; j++)
+		for (j = 0; j < table->size; j++)
 			xfree(table->names[j]);
 		xfree(table->name);
 		xfree(table->names);
@@ -212,9 +215,8 @@ static int _send_data(const char *data)
 		xstrcat(datastr, data);
 		length = strlen(data);
 		datastrlen += length;
-		if (slurm_get_debug_flags() & DEBUG_FLAG_PROFILE)
-			info("%s %s: %zu bytes of data added to buffer. New buffer size: %d",
-			     plugin_type, __func__, length, datastrlen);
+		log_flag(PROFILE, "%s %s: %zu bytes of data added to buffer. New buffer size: %d",
+			 plugin_type, __func__, length, datastrlen);
 		return rc;
 	}
 
@@ -280,7 +282,7 @@ static int _send_data(const char *data)
 		rc = SLURM_ERROR;
 		debug2("%s %s: data write failed, response code: %ld",
 		       plugin_type, __func__, response_code);
-		if (slurm_get_debug_flags() & DEBUG_FLAG_PROFILE) {
+		if (slurm_conf.debug_flags & DEBUG_FLAG_PROFILE) {
 			/* Strip any trailing newlines. */
 			while (chunk.message[strlen(chunk.message) - 1] == '\n')
 				chunk.message[strlen(chunk.message) - 1] = '\0';
@@ -298,9 +300,8 @@ cleanup_global_init:
 	curl_global_cleanup();
 
 	END_TIMER;
-	if (slurm_get_debug_flags() & DEBUG_FLAG_PROFILE)
-		debug("%s %s: took %s to send data", plugin_type, __func__,
-		      TIME_STR);
+	log_flag(PROFILE, "%s %s: took %s to send data",
+		 plugin_type, __func__, TIME_STR);
 
 	if (data) {
 		datastr = xstrdup(data);
@@ -561,7 +562,8 @@ extern int acct_gather_profile_p_add_sample_data(int table_id, void *data,
 			xstrfmtcat(str, "%s,job=%d,step=%d,task=%s,"
 				   "host=%s value=%"PRIu64" "
 				   "%"PRIu64"\n", table->names[i],
-				   g_job->jobid, g_job->stepid,
+				   g_job->step_id.job_id,
+				   g_job->step_id.step_id,
 				   table->name, g_job->node_name,
 				   ((union data_t*)data)[i].u,
 				   (uint64_t)sample_time);
@@ -570,7 +572,8 @@ extern int acct_gather_profile_p_add_sample_data(int table_id, void *data,
 			xstrfmtcat(str, "%s,job=%d,step=%d,task=%s,"
 				   "host=%s value=%.2f %"PRIu64""
 				   "\n", table->names[i],
-				   g_job->jobid, g_job->stepid,
+				   g_job->step_id.job_id,
+				   g_job->step_id.step_id,
 				   table->name, g_job->node_name,
 				   ((union data_t*)data)[i].d,
 				   (uint64_t)sample_time);
