@@ -39,8 +39,10 @@
 #define _GNU_SOURCE
 
 #include "src/common/slurm_xlator.h"
+#include "src/common/cgroup.h"
 #include "src/common/gres.h"
 #include "src/common/log.h"
+#include "src/common/read_config.h"
 #include <rocm_smi/rocm_smi.h>
 
 /*
@@ -67,17 +69,17 @@ typedef struct rsmiPciInfo_st {
 	union {
 		struct {
 #ifdef SLURM_BIGENDIAN
-			uint64_t reserved : 35;
-			uint64_t domain : 16;
-			uint64_t bus : 5;
+			uint64_t domain : 32;
+			uint64_t reserved : 16;
+			uint64_t bus : 8;
 			uint64_t device : 5;
 			uint64_t function : 3;
 #else
 			uint64_t function : 3;
 			uint64_t device : 5;
-			uint64_t bus : 5;
-			uint64_t domain : 16;
-			uint64_t reserved : 35;
+			uint64_t bus : 8;
+			uint64_t reserved : 16;
+			uint64_t domain : 32;
 #endif
 		};
 		uint64_t bdfid;
@@ -110,17 +112,13 @@ typedef struct rsmiPciInfo_st {
  * (major.minor.micro combined into a single number).
  */
 const char	*plugin_name		= "GPU RSMI plugin";
-const char	*plugin_type		= "gpu/rsmi";
+const char	plugin_type[]		= "gpu/rsmi";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
-static log_level_t log_lvl              = LOG_LEVEL_DEBUG5;
 
 extern int init(void)
 {
 	debug("%s: %s loaded", __func__, plugin_name);
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-
 	return SLURM_SUCCESS;
 }
 
@@ -428,15 +426,15 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 	unsigned int i;
 
 	if (!freq || !(*freq)) {
-		log_var(log_lvl, "%s: No frequency supplied", __func__);
+		log_flag(GRES, "%s: No frequency supplied", __func__);
 		return;
 	}
 	if (!freqs || !(*freqs)) {
-		log_var(log_lvl, "%s: No frequency list supplied", __func__);
+		log_flag(GRES, "%s: No frequency list supplied", __func__);
 		return;
 	}
 	if (freqs_size <= 0) {
-		log_var(log_lvl, "%s: Frequency list is empty", __func__);
+		log_flag(GRES, "%s: Frequency list is empty", __func__);
 		return;
 	}
 
@@ -472,31 +470,32 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 
 	/* check if freq is out of bounds of freqs */
 	if (*freq > freqs[0]) {
-		log_var(log_lvl, "Rounding frequency %u MHz down to %u MHz",
-			*freq, freqs[0]);
+		log_flag(GRES, "Rounding frequency %u MHz down to %u MHz",
+		         *freq, freqs[0]);
 		*freq = freqs[0];
 		return;
 	} else if (*freq < freqs[freqs_size - 1]) {
-		log_var(log_lvl, "Rounding frequency %u MHz up to %u MHz",
-			*freq, freqs[freqs_size - 1]);
+		log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
+		         *freq, freqs[freqs_size - 1]);
 		*freq = freqs[freqs_size - 1];
 		return;
 	}
 
 	/* check for frequency, and round up if no exact match */
 	for (i = 0; i < freqs_size - 1;) {
-		if (*freq == freqs[i])
+		if (*freq == freqs[i]) {
 			// No change necessary
 			debug2("No change necessary. Freq: %u MHz", *freq);
-		return;
+			return;
+		}
 		i++;
 		/*
 		 * Step down to next element to round up.
 		 * Safe to advance due to bounds checks above here
 		 */
 		if (*freq > freqs[i]) {
-			log_var(log_lvl, "Rounding frequency %u MHz up to %u MHz",
-				*freq, freqs[i - 1]);
+			log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
+			         *freq, freqs[i - 1]);
 			*freq = freqs[i - 1];
 			return;
 		}
@@ -759,17 +758,16 @@ static void _reset_freq(bitstr_t *gpus)
 		// TODO: Check to make sure that the frequency reset
 
 		if (freq_reset) {
-			log_var(log_lvl, "Successfully reset GPU[%d]", i);
+			log_flag(GRES, "Successfully reset GPU[%d]", i);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to reset GPU[%d]", i);
+			log_flag(GRES, "Failed to reset GPU[%d]", i);
 		}
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not reset frequencies for all GPUs %d/%d total GPUs",
-			__func__, count_set, count);
+		log_flag(GRES, "%s: Could not reset frequencies for all GPUs %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not reset frequencies for all GPUs %d/%d total GPUs\n",
 			count_set, count);
 	}
@@ -794,11 +792,9 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	uint64_t mem_bitmask = 0, gpu_bitmask = 0;
 	bool freq_set = false, freq_logged = false;
 	char *tmp = NULL;
-	slurm_cgroup_conf_t *cg_conf;
 	bool task_cgroup = false;
 	bool constrained_devices = false;
 	bool cgroups_active = false;
-	char *task_plugin_type = NULL;
 
 	// Parse frequency information
 	debug2("_parse_gpu_freq(%s)", gpu_freq);
@@ -819,17 +815,13 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	}
 
 	// Check if GPUs are constrained by cgroups
-	slurm_mutex_lock(&xcgroup_config_read_mutex);
-	cg_conf = xcgroup_get_slurm_cgroup_conf();
-	if (cg_conf && cg_conf->constrain_devices)
+	cgroup_conf_init();
+	if (slurm_cgroup_conf.constrain_devices)
 		constrained_devices = true;
-	slurm_mutex_unlock(&xcgroup_config_read_mutex);
 
 	// Check if task/cgroup plugin is loaded
-	task_plugin_type = slurm_get_task_plugin();
-	if (strstr(task_plugin_type, "cgroup"))
+	if (xstrstr(slurm_conf.task_plugin, "cgroup"))
 		task_cgroup = true;
-	xfree(task_plugin_type);
 
 	// If both of these are true, then GPUs will be constrained
 	if (constrained_devices && task_cgroup) {
@@ -878,10 +870,10 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 		}
 
 		if (freq_set) {
-			log_var(log_lvl, "Successfully set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Successfully set GPU[%d] %s", i, tmp);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Failed to set GPU[%d] %s", i, tmp);
 		}
 
 		if (verbose_flag && !freq_logged) {
@@ -892,9 +884,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not set frequencies for all GPUs %d/%d total GPUs",
-			__func__, count_set, count);
+		log_flag(GRES, "%s: Could not set frequencies for all GPUs %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not set frequencies for all GPUs %d/%d total GPUs\n",
 			count_set, count);
 	}
@@ -1075,7 +1066,7 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 	// Loop through all the GPUs on the system and add to gres_list_system
 	for (i = 0; i < device_count; ++i) {
 		unsigned int minor_number = 0;
-		char *device_file = NULL;
+		char *device_file = NULL, *links = NULL;
 		char device_name[RSMI_STRING_BUFFER_SIZE] = {0};
 		char device_brand[RSMI_STRING_BUFFER_SIZE] = {0};
 		rsmiPciInfo_t pci_info;
@@ -1089,6 +1080,9 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 		_rsmi_get_device_pci_info(i, &pci_info);
 		_rsmi_get_device_unique_id(i, &uuid);
 
+		/* Use links to record PCI bus ID order */
+		links = gres_links_create_empty(i, device_count);
+
 		xstrfmtcat(device_file, "/dev/dri/renderD%u", minor_number);
 
 		debug2("GPU index %u:", i);
@@ -1098,6 +1092,7 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 		debug2("    PCI Domain/Bus/Device/Function: %u:%u:%u.%u",
 		       pci_info.domain,
 		       pci_info.bus, pci_info.device, pci_info.function);
+		debug2("    Links: %s", links);
 		debug2("    Device File (minor number): %s", device_file);
 		if (minor_number != i+128)
 			debug("Note: GPU index %u is different from minor # %u",
@@ -1108,9 +1103,11 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 
 		add_gres_to_list(gres_list_system, "gpu", 1,
 				 node_config->cpu_cnt, NULL, NULL,
-				 device_file, device_brand, NULL);
+				 device_file, device_brand, links, NULL,
+				 GRES_CONF_ENV_RSMI);
 
 		xfree(device_file);
+		xfree(links);
 	}
 
 	rsmi_shut_down();
@@ -1121,15 +1118,9 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 
 extern int gpu_p_reconfig(void)
 {
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-	else
-		log_lvl = LOG_LEVEL_DEBUG5;
-
 	return SLURM_SUCCESS;
 }
 
-
 extern List gpu_p_get_system_gpu_list(node_config_load_t *node_config)
 {
 	List gres_list_system = _get_system_gpu_list_rsmi(node_config);
