@@ -72,18 +72,18 @@ char **assoc_mgr_tres_name_array;
 
 static int cpunfo_frequency = 0;
 static long hertz = 0;
+List prec_list = NULL;
 
 static int my_pagesize = 0;
 static DIR  *slash_proc = NULL;
 static int energy_profile = ENERGY_DATA_NODE_ENERGY_UP;
-static uint64_t debug_flags = 0;
 
 static int _find_prec(void *x, void *key)
 {
 	jag_prec_t *prec = (jag_prec_t *) x;
-	struct jobacctinfo *jobacct = (struct jobacctinfo *) key;
+	pid_t pid = *(pid_t *) key;
 
-	if (prec->pid == jobacct->pid)
+	if (prec->pid == pid)
 		return 1;
 
 	return 0;
@@ -136,7 +136,7 @@ inline static bool _get_freq(char *str)
 		return false;
 
 	cpunfo_frequency = cpufreq_value * cpu_mult;
-	debug2("cpunfo_frequency=%d", cpunfo_frequency);
+	log_flag(JAG, "cpuinfo_frequency=%d", cpunfo_frequency);
 
 	return true;
 }
@@ -189,13 +189,14 @@ static int _get_pss(char *proc_smaps_file, jag_prec_t *prec)
         fclose(fp);
         /* Sanity checks */
 
-        if (pss > 0 && prec->tres_data[TRES_ARRAY_MEM].size_read > pss) {
+        if (pss > 0) {
 		pss *= 1024; /* Scale KB to B */
-                prec->tres_data[TRES_ARRAY_MEM].size_read = pss;
+		if (prec->tres_data[TRES_ARRAY_MEM].size_read > pss)
+			prec->tres_data[TRES_ARRAY_MEM].size_read = pss;
         }
 
-	debug3("%s: read pss %"PRIu64" for process %s",
-	       __func__, pss, proc_smaps_file);
+	log_flag(JAG, "%s read pss %"PRIu64" for process %s",
+		 __func__, pss, proc_smaps_file);
 
         return 0;
 }
@@ -215,7 +216,7 @@ static int _get_sys_interface_freq_line(uint32_t cpu, char *filename,
 	snprintf(freq_file, 79,
 		 "/sys/devices/system/cpu/cpu%d/cpufreq/%s",
 		 cpu, filename);
-	debug2("_get_sys_interface_freq_line: filename = %s ", freq_file);
+	log_flag(JAG, "filename = %s", freq_file);
 	if ((sys_fp = fopen(freq_file, "r"))!= NULL) {
 		/* frequency scaling enabled */
 		fd = fileno(sys_fp);
@@ -224,14 +225,15 @@ static int _get_sys_interface_freq_line(uint32_t cpu, char *filename,
 		num_read = read(fd, sbuf, (sizeof(sbuf) - 1));
 		if (num_read > 0) {
 			sbuf[num_read] = '\0';
-			debug2(" cpu %d freq= %s", cpu, sbuf);
+			log_flag(JAG, "scaling enabled on cpu %d freq= %s",
+				 cpu, sbuf);
 		}
 		fclose(sys_fp);
 	} else {
 		/* frequency scaling not enabled */
 		if (!cpunfo_frequency) {
 			snprintf(freq_file, 14, "/proc/cpuinfo");
-			debug2("_get_sys_interface_freq_line: filename = %s ",
+			log_flag(JAG, "filename = %s (cpu scaling not enabled)",
 			       freq_file);
 			if ((sys_fp = fopen(freq_file, "r")) != NULL) {
 				while (fgets(cpunfo_line, sizeof(cpunfo_line),
@@ -288,12 +290,12 @@ again:
 		error("%s: Tgid: string not found for pid=%u", __func__, pid);
 
 	if (pid != (uint32_t)tgid) {
-		debug3("%s: pid=%u != tgid=%u is a lightweight process",
-		       __func__, pid, tgid);
+		log_flag(JAG, "pid=%u != tgid=%u is a lightweight process",
+			 pid, tgid);
 		return 1;
 	} else {
-		debug3("%s: pid=%u == tgid=%u is the leader LWP",
-		       __func__, pid, tgid);
+		log_flag(JAG, "pid=%u == tgid=%u is the leader LWP",
+			 pid, tgid);
 		return 0;
 	}
 }
@@ -416,7 +418,7 @@ static int _get_process_memory_line(int in, jag_prec_t *prec)
 
 	/* If shared > rss then there is a problem, give up... */
 	if (share > rss) {
-		debug("jobacct_gather_linux: share > rss - bail!");
+		log_flag(JAG, "share > rss - bail!");
 		return 0;
 	}
 
@@ -484,30 +486,40 @@ static int _get_process_io_data_line(int in, jag_prec_t *prec) {
 	return 1;
 }
 
-static void _handle_stats(List prec_list, char *proc_stat_file,
-			  char *proc_io_file, char *proc_smaps_file,
-			  jag_callbacks_t *callbacks,
+static int _init_tres(jag_prec_t *prec, void *empty)
+{
+	/* Initialize read/writes */
+	for (int i = 0; i < prec->tres_count; i++) {
+		prec->tres_data[i].num_reads = INFINITE64;
+		prec->tres_data[i].num_writes = INFINITE64;
+		prec->tres_data[i].size_read = INFINITE64;
+		prec->tres_data[i].size_write = INFINITE64;
+	}
+
+	return SLURM_SUCCESS;
+}
+
+static void _handle_stats(char *proc_stat_file, char *proc_io_file,
+			  char *proc_smaps_file, jag_callbacks_t *callbacks,
 			  int tres_count)
 {
 	static int no_share_data = -1;
 	static int use_pss = -1;
 	FILE *stat_fp = NULL;
 	FILE *io_fp = NULL;
-	int fd, fd2, i;
+	int fd, fd2;
 	jag_prec_t *prec = NULL;
 
 	if (no_share_data == -1) {
-		char *acct_params = slurm_get_jobacct_gather_params();
-		if (acct_params && xstrcasestr(acct_params, "NoShare"))
+		if (xstrcasestr(slurm_conf.job_acct_gather_params, "NoShare"))
 			no_share_data = 1;
 		else
 			no_share_data = 0;
 
-		if (acct_params && xstrcasestr(acct_params, "UsePss"))
+		if (xstrcasestr(slurm_conf.job_acct_gather_params, "UsePss"))
 			use_pss = 1;
 		else
 			use_pss = 0;
-		xfree(acct_params);
 	}
 
 	if (!(stat_fp = fopen(proc_stat_file, "r")))
@@ -541,13 +553,7 @@ static void _handle_stats(List prec_list, char *proc_stat_file,
 	prec->tres_data = xmalloc(prec->tres_count *
 				  sizeof(acct_gather_data_t));
 
-	/* Initialize read/writes */
-	for (i = 0; i < prec->tres_count; i++) {
-		prec->tres_data[i].num_reads = INFINITE64;
-		prec->tres_data[i].num_writes = INFINITE64;
-		prec->tres_data[i].size_read = INFINITE64;
-		prec->tres_data[i].size_write = INFINITE64;
-	}
+	(void)_init_tres(prec, NULL);
 
 	if (!_get_process_data_line(fd, prec)) {
 		fclose(stat_fp);
@@ -557,11 +563,11 @@ static void _handle_stats(List prec_list, char *proc_stat_file,
 	fclose(stat_fp);
 
 	if (acct_gather_filesystem_g_get_data(prec->tres_data) < 0) {
-		debug2("problem retrieving filesystem data");
+		log_flag(JAG, "problem retrieving filesystem data");
 	}
 
 	if (acct_gather_interconnect_g_get_data(prec->tres_data) < 0) {
-		debug2("problem retrieving interconnect data");
+		log_flag(JAG, "problem retrieving interconnect data");
 	}
 
 	/* Remove shared data from rss */
@@ -583,6 +589,7 @@ static void _handle_stats(List prec_list, char *proc_stat_file,
 		fclose(io_fp);
 	}
 
+	destroy_jag_prec(list_remove_first(prec_list, _find_prec, &prec->pid));
 	list_append(prec_list, prec);
 	return;
 
@@ -595,7 +602,6 @@ bail_out:
 static List _get_precs(List task_list, bool pgid_plugin, uint64_t cont_id,
 		       jag_callbacks_t *callbacks)
 {
-	List prec_list = list_create(destroy_jag_prec);
 	char	proc_stat_file[256];	/* Allow ~20x extra length */
 	char	proc_io_file[256];	/* Allow ~20x extra length */
 	char	proc_smaps_file[256];	/* Allow ~20x extra length */
@@ -622,22 +628,22 @@ static List _get_precs(List task_list, bool pgid_plugin, uint64_t cont_id,
 					jobacct->energy.consumed_energy;
 				jobacct->tres_usage_out_tot[TRES_ARRAY_ENERGY] =
 					jobacct->energy.current_watts;
-				debug2("%s: energy = %"PRIu64" watts = %"PRIu64,
-				       __func__,
-				       jobacct->tres_usage_in_tot[
-					       TRES_ARRAY_ENERGY],
-				       jobacct->tres_usage_out_tot[
-					       TRES_ARRAY_ENERGY]);
+				log_flag(JAG, "energy = %"PRIu64" watts = %"PRIu64,
+					 jobacct->tres_usage_in_tot[
+						TRES_ARRAY_ENERGY],
+					 jobacct->tres_usage_out_tot[
+						TRES_ARRAY_ENERGY]);
 			}
 
-			debug4("no pids in this container %"PRIu64"", cont_id);
+			log_flag(JAG, "no pids in this container %"PRIu64"",
+				 cont_id);
 			goto finished;
 		}
 		for (i = 0; i < npids; i++) {
 			snprintf(proc_stat_file, 256, "/proc/%d/stat", pids[i]);
 			snprintf(proc_io_file, 256, "/proc/%d/io", pids[i]);
 			snprintf(proc_smaps_file, 256, "/proc/%d/smaps", pids[i]);
-			_handle_stats(prec_list, proc_stat_file, proc_io_file,
+			_handle_stats(proc_stat_file, proc_io_file,
 				      proc_smaps_file, callbacks,
 				      jobacct ? jobacct->tres_count : 0);
 		}
@@ -727,8 +733,8 @@ static List _get_precs(List task_list, bool pgid_plugin, uint64_t cont_id,
 			} while (*iptr);
 			*optr2 = 0;
 
-			_handle_stats(prec_list, proc_stat_file, proc_io_file,
-				      proc_smaps_file,callbacks,
+			_handle_stats(proc_stat_file, proc_io_file,
+				      proc_smaps_file, callbacks,
 				      jobacct ? jobacct->tres_count : 0);
 		}
 	}
@@ -770,6 +776,7 @@ static void _record_profile(struct jobacctinfo *jobacct)
 		double d;
 		uint64_t u64;
 	} data[FIELD_CNT];
+	char str[256];
 
 	if (profile_gid == -1)
 		profile_gid = acct_gather_profile_g_create_group("Tasks");
@@ -845,11 +852,9 @@ static void _record_profile(struct jobacctinfo *jobacct)
 		data[FIELD_WRITE].d /= 1048576.0;
 	}
 
-	if (debug_flags & DEBUG_FLAG_PROFILE) {
-		char str[256];
-		info("PROFILE-Task: %s", acct_gather_profile_dataset_str(
-			     dataset, data, str, sizeof(str)));
-	}
+	log_flag(PROFILE, "PROFILE-Task: %s",
+		 acct_gather_profile_dataset_str(dataset, data, str,
+						 sizeof(str)));
 	acct_gather_profile_g_add_sample_data(jobacct->dataset_id,
 	                                      (void *)data, jobacct->cur_time);
 }
@@ -858,7 +863,7 @@ extern void jag_common_init(long in_hertz)
 {
 	uint32_t profile_opt;
 
-	debug_flags = slurm_get_debug_flags();
+	prec_list = list_create(destroy_jag_prec);
 
 	acct_gather_profile_g_get(ACCT_GATHER_PROFILE_RUNNING,
 				  &profile_opt);
@@ -885,6 +890,8 @@ extern void jag_common_init(long in_hertz)
 
 extern void jag_common_fini(void)
 {
+	FREE_NULL_LIST(prec_list);
+
 	if (slash_proc)
 		(void) closedir(slash_proc);
 }
@@ -892,34 +899,41 @@ extern void jag_common_fini(void)
 extern void destroy_jag_prec(void *object)
 {
 	jag_prec_t *prec = (jag_prec_t *)object;
+
+	if (!prec)
+		return;
+
 	xfree(prec->tres_data);
 	xfree(prec);
 	return;
 }
 
-extern void print_jag_prec(jag_prec_t *prec)
+static void _print_jag_prec(jag_prec_t *prec)
 {
 	int i;
 	assoc_mgr_lock_t locks = {
 		NO_LOCK, NO_LOCK, NO_LOCK, NO_LOCK,
 		READ_LOCK, NO_LOCK, NO_LOCK };
 
-	info("pid %d (ppid %d)", prec->pid, prec->ppid);
-	info("act_cpufreq\t%d", prec->act_cpufreq);
-	info("ssec \t%f", prec->ssec);
+	if (!(slurm_conf.debug_flags & DEBUG_FLAG_JAG))
+		return;
+
+	log_flag(JAG, "pid %d (ppid %d)", prec->pid, prec->ppid);
+	log_flag(JAG, "act_cpufreq\t%d", prec->act_cpufreq);
+	log_flag(JAG, "ssec \t%f", prec->ssec);
 	assoc_mgr_lock(&locks);
 	for (i = 0; i < prec->tres_count; i++) {
 		if (prec->tres_data[i].size_read == INFINITE64)
 			continue;
-		info("%s in/read \t%"PRIu64"",
-		     assoc_mgr_tres_name_array[i],
-		     prec->tres_data[i].size_read);
-		info("%s out/write \t%"PRIu64"",
-		     assoc_mgr_tres_name_array[i],
-		     prec->tres_data[i].size_write);
+		log_flag(JAG, "%s in/read \t%" PRIu64 "",
+			 assoc_mgr_tres_name_array[i],
+			 prec->tres_data[i].size_read);
+		log_flag(JAG, "%s out/write \t%" PRIu64 "",
+			 assoc_mgr_tres_name_array[i],
+			 prec->tres_data[i].size_write);
 	}
 	assoc_mgr_unlock(&locks);
-	info("usec \t%f", prec->usec);
+	log_flag(JAG, "usec \t%f", prec->usec);
 }
 
 extern void jag_common_poll_data(
@@ -927,27 +941,26 @@ extern void jag_common_poll_data(
 	jag_callbacks_t *callbacks, bool profile)
 {
 	/* Update the data */
-	List prec_list = NULL;
 	uint64_t total_job_mem = 0, total_job_vsize = 0;
+	uint32_t last_taskid = NO_VAL;
 	ListIterator itr;
-	jag_prec_t *prec = NULL;
+	jag_prec_t *prec = NULL, tmp_prec;
 	struct jobacctinfo *jobacct = NULL;
 	static int processing = 0;
 	char sbuf[72];
 	int energy_counted = 0;
 	time_t ct;
-	static int over_memory_kill = -1;
 	int i = 0;
 
 	xassert(callbacks);
 
 	if (!pgid_plugin && (cont_id == NO_VAL64)) {
-		debug("cont_id hasn't been set yet not running poll");
+		log_flag(JAG, "cont_id hasn't been set yet not running poll");
 		return;
 	}
 
 	if (processing) {
-		debug("already running, returning");
+		log_flag(JAG, "already running, returning");
 		return;
 	}
 	processing = 1;
@@ -956,8 +969,9 @@ extern void jag_common_poll_data(
 		callbacks->get_precs = _get_precs;
 
 	ct = time(NULL);
-	prec_list = (*(callbacks->get_precs))(task_list, pgid_plugin, cont_id,
-					      callbacks);
+
+	(void)list_for_each(prec_list, (ListForF)_init_tres, NULL);
+	(*(callbacks->get_precs))(task_list, pgid_plugin, cont_id, callbacks);
 
 	if (!list_count(prec_list) || !task_list || !list_count(task_list))
 		goto finished;	/* We have no business being here! */
@@ -966,22 +980,45 @@ extern void jag_common_poll_data(
 	while ((jobacct = list_next(itr))) {
 		double cpu_calc;
 		double last_total_cputime;
-		if (!(prec = list_find_first(prec_list, _find_prec, jobacct)))
+		if (!(prec = list_find_first(prec_list, _find_prec,
+					     &jobacct->pid)))
 			continue;
+		/*
+		 * We can't use the prec from the list as we need to keep it in
+		 * the original state without offspring since we reuse this list
+		 * keeping around precs after they end.
+		 */
+		memcpy(&tmp_prec, prec, sizeof(*prec));
+		prec = &tmp_prec;
 
 		/*
 		 * Only jobacct_gather/cgroup uses prec_extra, and we want to
 		 * make sure we call it once per task, so call it here as we
 		 * iterate through the tasks instead of in get_precs.
 		 */
-		if (callbacks->prec_extra)
+		if (callbacks->prec_extra) {
+			if (last_taskid == jobacct->id.taskid) {
+				log_flag(JAG, "skipping prec_extra() call against nodeid:%u taskid:%u",
+					 jobacct->id.nodeid,
+					 jobacct->id.taskid);
+				continue;
+			} else {
+				log_flag(JAG, "calling prec_extra() call against nodeid:%u taskid:%u",
+					 jobacct->id.nodeid,
+					 jobacct->id.taskid);
+			}
+
+			last_taskid = jobacct->id.taskid;
 			(*(callbacks->prec_extra))(prec, jobacct->id.taskid);
+			_print_jag_prec(prec);
+		}
+
+		log_flag(JAG, "pid:%u ppid:%u %s:%" PRIu64 " B",
+			 prec->pid, prec->ppid,
+			 (xstrcasestr(slurm_conf.job_acct_gather_params,
+				      "UsePss") ?  "pss" : "rss"),
+			 prec->tres_data[TRES_ARRAY_MEM].size_read);
 
-#if _DEBUG
-		info("pid:%u ppid:%u rss:%"PRIu64" B",
-		     prec->pid, prec->ppid,
-		     prec->tres_data[TRES_ARRAY_MEM].size_read);
-#endif
 		/* find all my descendents */
 		if (callbacks->get_offspring_data)
 			(*(callbacks->get_offspring_data))
@@ -1005,7 +1042,7 @@ extern void jag_common_poll_data(
 		 * report per node energy consumption.
 		 * Energy is stored in read fields, while power is stored
 		 * in write fields.*/
-		debug2("energycounted = %d", energy_counted);
+		log_flag(JAG, "energycounted = %d", energy_counted);
 		if (energy_counted == 0) {
 			acct_gather_energy_g_get_sum(
 				energy_profile,
@@ -1014,11 +1051,10 @@ extern void jag_common_poll_data(
 				jobacct->energy.consumed_energy;
 			prec->tres_data[TRES_ARRAY_ENERGY].size_write =
 				jobacct->energy.current_watts;
-			debug2("%s: energy = %"PRIu64" watts = %"PRIu64" ave_watts = %u",
-			       __func__,
-			       prec->tres_data[TRES_ARRAY_ENERGY].size_read,
-			       prec->tres_data[TRES_ARRAY_ENERGY].size_write,
-			       jobacct->energy.ave_watts);
+			log_flag(JAG, "energy = %"PRIu64" watts = %"PRIu64" ave_watts = %u",
+				 prec->tres_data[TRES_ARRAY_ENERGY].size_read,
+				 prec->tres_data[TRES_ARRAY_ENERGY].size_write,
+				 jobacct->energy.ave_watts);
 			energy_counted = 1;
 		}
 
@@ -1035,7 +1071,7 @@ extern void jag_common_poll_data(
 					    prec->tres_data[i].size_read);
 			/*
 			 * Even with min we want to get the max as we are
-			 * looking at a specific task aso we are always looking
+			 * looking at a specific task.  We are always looking
 			 * at the max that task had, not the min (or lots of
 			 * things will be zero).  The min is from comparing
 			 * ranks later when combining.  So here it will be the
@@ -1064,8 +1100,8 @@ extern void jag_common_poll_data(
 		total_job_vsize += jobacct->tres_usage_in_tot[TRES_ARRAY_VMEM];
 
 		/* Update the cpu times */
-		jobacct->user_cpu_sec = (uint32_t)(prec->usec / (double)hertz);
-		jobacct->sys_cpu_sec = (uint32_t)(prec->ssec / (double)hertz);
+		jobacct->user_cpu_sec = (uint64_t)(prec->usec / (double)hertz);
+		jobacct->sys_cpu_sec = (uint64_t)(prec->ssec / (double)hertz);
 
 		/* compute frequency */
 		jobacct->this_sampled_cputime =
@@ -1076,28 +1112,27 @@ extern void jag_common_poll_data(
 		jobacct->act_cpufreq =
 			_update_weighted_freq(jobacct, sbuf);
 
-		debug("%s: Task %u pid %d ave_freq = %u mem size/max %"PRIu64"/%"PRIu64" vmem size/max %"PRIu64"/%"PRIu64", disk read size/max (%"PRIu64"/%"PRIu64"), disk write size/max (%"PRIu64"/%"PRIu64"), time %f(%u+%u) Energy tot/max %"PRIu64"/%"PRIu64" TotPower %"PRIu64" MaxPower %"PRIu64" MinPower %"PRIu64,
-		      __func__,
-		      jobacct->id.taskid,
-		      jobacct->pid,
-		      jobacct->act_cpufreq,
-		      jobacct->tres_usage_in_tot[TRES_ARRAY_MEM],
-		      jobacct->tres_usage_in_max[TRES_ARRAY_MEM],
-		      jobacct->tres_usage_in_tot[TRES_ARRAY_VMEM],
-		      jobacct->tres_usage_in_max[TRES_ARRAY_VMEM],
-		      jobacct->tres_usage_in_tot[TRES_ARRAY_FS_DISK],
-		      jobacct->tres_usage_in_max[TRES_ARRAY_FS_DISK],
-		      jobacct->tres_usage_out_tot[TRES_ARRAY_FS_DISK],
-		      jobacct->tres_usage_out_max[TRES_ARRAY_FS_DISK],
-		      (double)(jobacct->tres_usage_in_tot[TRES_ARRAY_CPU] /
-			       CPU_TIME_ADJ),
-		      jobacct->user_cpu_sec,
-		      jobacct->sys_cpu_sec,
-		      jobacct->tres_usage_in_tot[TRES_ARRAY_ENERGY],
-		      jobacct->tres_usage_in_max[TRES_ARRAY_ENERGY],
-		      jobacct->tres_usage_out_tot[TRES_ARRAY_ENERGY],
-		      jobacct->tres_usage_out_max[TRES_ARRAY_ENERGY],
-		      jobacct->tres_usage_out_min[TRES_ARRAY_ENERGY]);
+		log_flag(JAG, "Task %u pid %d ave_freq = %u mem size/max %"PRIu64"/%"PRIu64" vmem size/max %"PRIu64"/%"PRIu64", disk read size/max (%"PRIu64"/%"PRIu64"), disk write size/max (%"PRIu64"/%"PRIu64"), time %f(%"PRIu64"+%"PRIu64") Energy tot/max %"PRIu64"/%"PRIu64" TotPower %"PRIu64" MaxPower %"PRIu64" MinPower %"PRIu64,
+			 jobacct->id.taskid,
+			 jobacct->pid,
+			 jobacct->act_cpufreq,
+			 jobacct->tres_usage_in_tot[TRES_ARRAY_MEM],
+			 jobacct->tres_usage_in_max[TRES_ARRAY_MEM],
+			 jobacct->tres_usage_in_tot[TRES_ARRAY_VMEM],
+			 jobacct->tres_usage_in_max[TRES_ARRAY_VMEM],
+			 jobacct->tres_usage_in_tot[TRES_ARRAY_FS_DISK],
+			 jobacct->tres_usage_in_max[TRES_ARRAY_FS_DISK],
+			 jobacct->tres_usage_out_tot[TRES_ARRAY_FS_DISK],
+			 jobacct->tres_usage_out_max[TRES_ARRAY_FS_DISK],
+			 (double)(jobacct->tres_usage_in_tot[TRES_ARRAY_CPU] /
+			          CPU_TIME_ADJ),
+			 jobacct->user_cpu_sec,
+			 jobacct->sys_cpu_sec,
+			 jobacct->tres_usage_in_tot[TRES_ARRAY_ENERGY],
+			 jobacct->tres_usage_in_max[TRES_ARRAY_ENERGY],
+			 jobacct->tres_usage_out_tot[TRES_ARRAY_ENERGY],
+			 jobacct->tres_usage_out_max[TRES_ARRAY_ENERGY],
+			 jobacct->tres_usage_out_min[TRES_ARRAY_ENERGY]);
 
 		if (profile &&
 		    acct_gather_profile_g_is_active(ACCT_GATHER_PROFILE_TASK)) {
@@ -1117,14 +1152,10 @@ extern void jag_common_poll_data(
 	}
 	list_iterator_destroy(itr);
 
-	if (over_memory_kill == -1)
-		over_memory_kill = slurm_get_job_acct_oom_kill();
-
-	if (over_memory_kill)
+	if (slurm_conf.job_acct_oom_kill)
 		jobacct_gather_handle_mem_limit(total_job_mem,
 						total_job_vsize);
 
 finished:
-	FREE_NULL_LIST(prec_list);
 	processing = 0;
 }
