@@ -208,7 +208,7 @@ static void _set_procdatas(List lresp)
 		pmix_info_t *info;
 		int tmp;
 
-		rankinfo = list_create(pmixp_xfree_xmalloced);
+		rankinfo = list_create(xfree_ptr);
 
 		PMIXP_KVP_ALLOC(kvp, PMIX_RANK);
 		PMIXP_VAL_SET_RANK(&kvp->value, i);
@@ -254,6 +254,10 @@ static void _set_procdatas(List lresp)
 		list_append(rankinfo, kvp);
 		free(nodename);
 
+		PMIXP_KVP_CREATE(kvp, PMIX_NODEID, &nsptr->node_id,
+				 PMIX_UINT32);
+		list_append(rankinfo, kvp);
+
 		/* merge rankinfo into one PMIX_PROC_DATA key */
 		count = list_count(rankinfo);
 		PMIX_INFO_CREATE(info, count);
@@ -652,7 +656,7 @@ extern int pmixp_libpmix_job_set(void)
 	pmixp_debug_hang(0);
 
 	/* Use list to safely expand/reduce key-value pairs. */
-	lresp = list_create(pmixp_xfree_xmalloced);
+	lresp = list_create(xfree_ptr);
 
 	_general_proc_info(lresp);
 
