@@ -37,7 +37,7 @@
  \*****************************************************************************/
 
 #include "pmixp_common.h"
-#include "src/slurmd/common/reverse_tree_math.h"
+#include "src/common/reverse_tree.h"
 #include "src/common/slurm_protocol_api.h"
 #include "pmixp_coll.h"
 #include "pmixp_nspaces.h"
@@ -47,7 +47,7 @@
 static void _progress_coll_tree(pmixp_coll_t *coll);
 static void _reset_coll(pmixp_coll_t *coll);
 
-static int _pack_coll_info(pmixp_coll_t *coll, Buf buf)
+static int _pack_coll_info(pmixp_coll_t *coll, buf_t *buf)
 {
 	pmixp_proc_t *procs = coll->pset.procs;
 	size_t nprocs = coll->pset.nprocs;
@@ -69,7 +69,7 @@ static int _pack_coll_info(pmixp_coll_t *coll, Buf buf)
 	return SLURM_SUCCESS;
 }
 
-int pmixp_coll_tree_unpack(Buf buf, pmixp_coll_type_t *type,
+int pmixp_coll_tree_unpack(buf_t *buf, pmixp_coll_type_t *type,
 			   int *nodeid, pmixp_proc_t **r, size_t *nr)
 {
 	pmixp_proc_t *procs = NULL;
@@ -206,7 +206,7 @@ int pmixp_coll_tree_init(pmixp_coll_t *coll, hostlist_t *hl)
 	tree = &coll->state.tree;
 	tree->state = PMIXP_COLL_TREE_SYNC;
 
-	width = slurm_get_tree_width();
+	width = slurm_conf.tree_width;
 	reverse_tree_info(coll->my_peerid, coll->peers_cnt, width,
 			  &tree->prnt_peerid, &tree->chldrn_cnt, &depth,
 			  &max_depth);
@@ -1010,8 +1010,8 @@ static char *_chld_ids_str(pmixp_coll_tree_t *tree)
 }
 
 
-int pmixp_coll_tree_child(pmixp_coll_t *coll, uint32_t peerid,
-			  uint32_t seq, Buf buf)
+int pmixp_coll_tree_child(pmixp_coll_t *coll, uint32_t peerid, uint32_t seq,
+			  buf_t *buf)
 {
 	char *data_src = NULL, *data_dst = NULL;
 	uint32_t size;
@@ -1150,8 +1150,8 @@ error2:
 	return SLURM_ERROR;
 }
 
-int pmixp_coll_tree_parent(pmixp_coll_t *coll, uint32_t peerid,
-			   uint32_t seq, Buf buf)
+int pmixp_coll_tree_parent(pmixp_coll_t *coll, uint32_t peerid, uint32_t seq,
+			   buf_t *buf)
 {
 	pmixp_coll_tree_t *tree = NULL;
 	char *data_src = NULL, *data_dst = NULL;
