@@ -167,7 +167,6 @@ static int allowed_uid_cnt = 0;
 static uint32_t boot_time = (5 * 60);	/* 5 minute estimated boot time */
 static pthread_mutex_t config_mutex = PTHREAD_MUTEX_INITIALIZER;
 static uint32_t cpu_bind[KNL_NUMA_CNT];	/* Derived from numa_cpu_bind */
-static bool debug_flag = false;
 static uint16_t default_mcdram = KNL_CACHE;
 static uint16_t default_numa = KNL_ALL2ALL;
 static char *mc_path = NULL;
@@ -558,7 +557,7 @@ static void _update_cpu_bind(void)
 		      plugin_type, numa_cpu_bind);
 	}
 
-	if (debug_flag) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES) {
 		for (i = 0; i < KNL_NUMA_CNT; i++) {
 			char cpu_bind_str[128], *numa_str;
 			if (cpu_bind[i] == 0)
@@ -577,7 +576,7 @@ static void _log_script_argv(char **script_argv, char *resp_msg)
 	char *cmd_line = NULL;
 	int i;
 
-	if (!debug_flag)
+	if (!(slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES))
 		return;
 
 	for (i = 0; script_argv[i]; i++) {
@@ -807,7 +806,7 @@ extern int init(void)
 {
 	char *allow_mcdram_str, *allow_numa_str, *allow_user_str;
 	char *default_mcdram_str, *default_numa_str;
-	char *knl_conf_file, *tmp_str = NULL, *resume_program;
+	char *knl_conf_file, *tmp_str = NULL;
 	s_p_hashtbl_t *tbl;
 	struct stat stat_buf;
 	int i, rc = SLURM_SUCCESS;
@@ -825,7 +824,6 @@ extern int init(void)
 	for (i = 0; i < KNL_NUMA_CNT; i++)
 		cpu_bind[i] = 0;
 	syscfg_timeout = DEFAULT_SYSCFG_TIMEOUT;
-	debug_flag = false;
 	default_mcdram = KNL_CACHE;
 	default_numa = KNL_ALL2ALL;
 //FIXME: Need better mechanism to get MCDRAM percentages
@@ -838,9 +836,6 @@ extern int init(void)
 	mcdram_pct[4] = 0;	// KNL_AUTO
 	xfree(numa_cpu_bind);
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_NODE_FEATURES)
-		debug_flag = true;
-
 	knl_conf_file = get_extra_conf_path("knl_generic.conf");
 	if ((stat(knl_conf_file, &stat_buf) == 0) &&
 	    (tbl = _config_make_tbl(knl_conf_file))) {
@@ -928,14 +923,13 @@ extern int init(void)
 		fclose(cpu_info_file);
 	}
 
-	if ((resume_program = slurm_get_resume_program())) {
+	if (slurm_conf.resume_program) {
 		error("Use of ResumeProgram with %s not currently supported",
 		      plugin_name);
-		xfree(resume_program);
 		rc = SLURM_ERROR;
 	}
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_NODE_FEATURES) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES) {
 		allow_mcdram_str = _knl_mcdram_str(allow_mcdram);
 		allow_numa_str = _knl_numa_str(allow_numa);
 		allow_user_str = _make_uid_str(allowed_uid, allowed_uid_cnt);
@@ -961,7 +955,7 @@ extern int init(void)
 		xfree(default_mcdram_str);
 		xfree(default_numa_str);
 	}
-	gres_plugin_add("hbm");
+	gres_add("hbm");
 
 	if ((rc == SLURM_SUCCESS) &&
 	    ume_check_interval && running_in_slurmd()) {
@@ -985,7 +979,6 @@ extern int fini(void)
 	slurm_mutex_unlock(&ume_mutex);
 	xfree(allowed_uid);
 	allowed_uid_cnt = 0;
-	debug_flag = false;
 	xfree(mcdram_per_node);
 	xfree(mc_path);
 	xfree(numa_cpu_bind);
@@ -1725,9 +1718,9 @@ extern int node_features_p_node_update(char *active_features,
 			if (!node_ptr->gres)
 				node_ptr->gres =
 					xstrdup(node_ptr->config_ptr->gres);
-			gres_plugin_node_feature(node_ptr->name, "hbm",
-						 mcdram_size, &node_ptr->gres,
-						 &node_ptr->gres_list);
+			gres_node_feature(node_ptr->name, "hbm",
+					  mcdram_size, &node_ptr->gres,
+					  &node_ptr->gres_list);
 		}
 	}
 
@@ -2026,19 +2019,9 @@ extern void node_features_p_step_config(bool mem_sort, bitstr_t *numa_bitmap)
  * features */
 extern bool node_features_p_user_update(uid_t uid)
 {
-	static int reboot_allowed = -1;
 	int i;
 
-	if (reboot_allowed == -1) {
-		char *reboot_program = slurm_get_reboot_program();
-		if (reboot_program && reboot_program[0])
-			reboot_allowed = 1;
-		else
-			reboot_allowed = 0;
-		xfree(reboot_program);
-	}
-
-	if (reboot_allowed != 1) {
+	if (!slurm_conf.reboot_program || !slurm_conf.reboot_program[0]) {
 		info("Change in KNL mode not supported. No RebootProgram configured");
 		return false;
 	}
