@@ -84,7 +84,7 @@ extern void *acct_db_conn  __attribute__((weak_import));
 extern uint32_t cluster_cpus __attribute__((weak_import));
 extern List job_list  __attribute__((weak_import));
 extern time_t last_job_update __attribute__((weak_import));
-extern slurm_ctl_conf_t slurmctld_conf __attribute__((weak_import));
+extern slurm_conf_t slurm_conf __attribute__((weak_import));
 extern int slurmctld_tres_cnt __attribute__((weak_import));
 extern uint16_t accounting_enforce __attribute__((weak_import));
 #else
@@ -92,7 +92,7 @@ void *acct_db_conn = NULL;
 uint32_t cluster_cpus = NO_VAL;
 List job_list = NULL;
 time_t last_job_update = (time_t) 0;
-slurm_ctl_conf_t slurmctld_conf;
+slurm_conf_t slurm_conf;
 int slurmctld_tres_cnt = 0;
 uint16_t accounting_enforce = 0;
 #endif
@@ -133,11 +133,9 @@ static pthread_mutex_t decay_init_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t decay_init_cond = PTHREAD_COND_INITIALIZER;
 static bool running_decay = 0, reconfig = 0, calc_fairshare = 1;
 static time_t plugin_shutdown = 0;
-static bool favor_small; /* favor small jobs over large */
 static uint16_t damp_factor = 1;  /* weight for age factor */
 static uint32_t max_age; /* time when not to add any more
 			  * priority to a job if reached */
-static uint16_t enforce;     /* AccountingStorageEnforce */
 static uint32_t weight_age;  /* weight for age factor */
 static uint32_t weight_assoc;/* weight for assoc factor */
 static uint32_t weight_fs;   /* weight for Fairshare factor */
@@ -146,13 +144,10 @@ static uint32_t weight_part; /* weight for Partition factor */
 static uint32_t weight_qos;  /* weight for QOS factor */
 static double  *weight_tres; /* tres weights */
 static uint32_t flags;       /* Priority Flags */
-static uint32_t prevflags;    /* Priority Flags before _internal_setup() resets
-			       * flags after a reconfigure */
 static time_t g_last_ran = 0; /* when the last poll ran */
 static double decay_factor = 1; /* The decay factor when decaying time. */
 
 /* variables defined in priority_multifactor.h */
-bool priority_debug = 0;
 
 static void _priority_p_set_assoc_usage_debug(slurmdb_assoc_rec_t *assoc);
 static void _set_assoc_usage_efctv(slurmdb_assoc_rec_t *assoc);
@@ -262,7 +257,7 @@ static int _reset_usage(void)
 static void _read_last_decay_ran(time_t *last_ran, time_t *last_reset)
 {
 	char *state_file;
-	Buf buffer;
+	buf_t *buffer;
 
 	xassert(last_ran);
 	xassert(last_reset);
@@ -271,7 +266,7 @@ static void _read_last_decay_ran(time_t *last_ran, time_t *last_reset)
 	(*last_reset) = 0;
 
 	/* read the file */
-	state_file = xstrdup(slurmctld_conf.state_save_location);
+	state_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(state_file, "/priority_last_decay_ran");
 	lock_state_files();
 
@@ -287,8 +282,7 @@ static void _read_last_decay_ran(time_t *last_ran, time_t *last_reset)
 	safe_unpack_time(last_ran, buffer);
 	safe_unpack_time(last_reset, buffer);
 	free_buf(buffer);
-	if (priority_debug)
-		info("Last ran decay on jobs at %ld", (long)*last_ran);
+	log_flag(PRIO, "Last ran decay on jobs at %ld", (long) *last_ran);
 
 	return;
 
@@ -308,9 +302,9 @@ static int _write_last_decay_ran(time_t last_ran, time_t last_reset)
 	int error_code = SLURM_SUCCESS;
 	int state_fd;
 	char *old_file, *new_file, *state_file;
-	Buf buffer;
+	buf_t *buffer;
 
-	if (!xstrcmp(slurmctld_conf.state_save_location, "/dev/null")) {
+	if (!xstrcmp(slurm_conf.state_save_location, "/dev/null")) {
 		error("Can not save priority state information, "
 		      "StateSaveLocation is /dev/null");
 		return error_code;
@@ -321,11 +315,11 @@ static int _write_last_decay_ran(time_t last_ran, time_t last_reset)
 	pack_time(last_reset, buffer);
 
 	/* read the file */
-	old_file = xstrdup(slurmctld_conf.state_save_location);
+	old_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(old_file, "/priority_last_decay_ran.old");
-	state_file = xstrdup(slurmctld_conf.state_save_location);
+	state_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(state_file, "/priority_last_decay_ran");
-	new_file = xstrdup(slurmctld_conf.state_save_location);
+	new_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(new_file, "/priority_last_decay_ran.new");
 
 	lock_state_files();
@@ -445,22 +439,17 @@ static double _get_fairshare_priority(job_record_t *job_ptr)
 	/* Priority is 0 -> 1 */
 	if (flags & PRIORITY_FLAGS_FAIR_TREE) {
 		priority_fs = job_assoc->usage->fs_factor;
-		if (priority_debug) {
-			info("Fairshare priority of job %u for user %s in acct %s is %f",
-			     job_ptr->job_id, job_assoc->user, job_assoc->acct,
-			     priority_fs);
-		}
+		log_flag(PRIO, "Fairshare priority of job %u for user %s in acct %s is %f",
+			 job_ptr->job_id, job_assoc->user, job_assoc->acct,
+			 priority_fs);
 	} else {
 		priority_fs = priority_p_calc_fs_factor(
 			fs_assoc->usage->usage_efctv,
 			(long double)fs_assoc->usage->shares_norm);
-		if (priority_debug) {
-			info("Fairshare priority of job %u for user %s in acct"
-			     " %s is 2**(-%Lf/%f) = %f",
-			     job_ptr->job_id, job_assoc->user, job_assoc->acct,
-			     fs_assoc->usage->usage_efctv,
-			     fs_assoc->usage->shares_norm, priority_fs);
-		}
+		log_flag(PRIO, "Fairshare priority of job %u for user %s in acct %s is 2**(-%Lf/%f) = %f",
+			 job_ptr->job_id, job_assoc->user, job_assoc->acct,
+			 fs_assoc->usage->usage_efctv,
+			 fs_assoc->usage->shares_norm, priority_fs);
 	}
 	assoc_mgr_unlock(&locks);
 
@@ -548,7 +537,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 
 	set_priority_factors(start_time, job_ptr);
 
-	if (priority_debug) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO) {
 		memcpy(&pre_factors, job_ptr->prio_factors,
 		       sizeof(priority_factors_object_t));
 		if (job_ptr->prio_factors->priority_tres) {
@@ -592,7 +581,8 @@ static uint32_t _get_priority_internal(time_t start_time,
 
 	tmp_64 = (uint64_t) priority;
 	if (tmp_64 > 0xffffffff) {
-		error("Job %u priority exceeds 32 bits", job_ptr->job_id);
+		error("%pJ priority '%"PRIu64"' exceeds 32 bits. Reducing it to 4294967295 (2^32 - 1)",
+		      job_ptr, tmp_64);
 		tmp_64 = 0xffffffff;
 		priority = (double) tmp_64;
 	}
@@ -649,8 +639,8 @@ static uint32_t _get_priority_internal(time_t start_time,
 
 			tmp_64 = (uint64_t) priority_part;
 			if (tmp_64 > 0xffffffff) {
-				error("Job %u priority exceeds 32 bits",
-				      job_ptr->job_id);
+				error("%pJ priority '%"PRIu64"' exceeds 32 bits. Reducing it to 4294967295 (2^32 - 1)",
+				      job_ptr, tmp_64);
 				tmp_64 = 0xffffffff;
 				priority_part = (double) tmp_64;
 			}
@@ -660,21 +650,20 @@ static uint32_t _get_priority_internal(time_t start_time,
 				job_ptr->priority_array[i] =
 					(uint32_t) priority_part;
 			}
-			if (priority_debug) {
+			if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO) {
 				xstrfmtcat(multi_part_str, multi_part_str ?
 					   ", %s=%u" : "%s=%u", part_ptr->name,
 					   job_ptr->priority_array[i]);
 			}
 			i++;
 		}
-		if (priority_debug && multi_part_str)
-			info("%pJ multi-partition priorities: %s",
-			     job_ptr, multi_part_str);
+		log_flag(PRIO, "%pJ multi-partition priorities: %s",
+			 job_ptr, multi_part_str);
 		xfree(multi_part_str);
 		list_iterator_destroy(part_iterator);
 	}
 
-	if (priority_debug) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO) {
 		int i;
 		double *post_tres_factors =
 			job_ptr->prio_factors->priority_tres;
@@ -829,17 +818,10 @@ static void _handle_qos_tres_run_secs(long double *tres_run_decay,
 			qos->usage->grp_used_tres_run_secs[i] -=
 				tres_run_delta[i];
 
-		if (priority_debug) {
-			info("_handle_qos_tres_run_secs: job %u: "
-			     "Removed %"PRIu64" unused seconds "
-			     "from QOS %s TRES %s "
-			     "grp_used_tres_run_secs = %"PRIu64,
-			     job_id,
-			     tres_run_delta[i],
-			     qos->name,
-			     assoc_mgr_tres_name_array[i],
-			     qos->usage->grp_used_tres_run_secs[i]);
-		}
+		log_flag(PRIO, "%s: job %u: Removed %"PRIu64" unused seconds from QOS %s TRES %s grp_used_tres_run_secs = %"PRIu64,
+			 __func__, job_id, tres_run_delta[i], qos->name,
+			 assoc_mgr_tres_name_array[i],
+			 qos->usage->grp_used_tres_run_secs[i]);
 	}
 }
 
@@ -875,17 +857,10 @@ static void _handle_assoc_tres_run_secs(long double *tres_run_decay,
 			assoc->usage->grp_used_tres_run_secs[i] -=
 				tres_run_delta[i];
 
-		if (priority_debug) {
-			info("_handle_assoc_tres_run_secs: job %u: "
-			     "Removed %"PRIu64" unused seconds "
-			     "from assoc %d TRES %s "
-			     "grp_used_tres_run_secs = %"PRIu64,
-			     job_id,
-			     tres_run_delta[i],
-			     assoc->id,
-			     assoc_mgr_tres_name_array[i],
-			     assoc->usage->grp_used_tres_run_secs[i]);
-		}
+		log_flag(PRIO, "%s: job %u: Removed %"PRIu64" unused seconds from assoc %d TRES %s grp_used_tres_run_secs = %"PRIu64,
+			 __func__, job_id, tres_run_delta[i], assoc->id,
+			 assoc_mgr_tres_name_array[i],
+			 assoc->usage->grp_used_tres_run_secs[i]);
 	}
 }
 
@@ -897,6 +872,13 @@ static void _handle_tres_run_secs(uint64_t *tres_run_delta,
 
 	_handle_qos_tres_run_secs(NULL, tres_run_delta,
 				  job_ptr->job_id, job_ptr->qos_ptr);
+
+	/* Only update partition qos if not being used by job */
+	if (job_ptr->part_ptr &&
+	    (job_ptr->part_ptr->qos_ptr != job_ptr->qos_ptr))
+		_handle_qos_tres_run_secs(NULL, tres_run_delta, job_ptr->job_id,
+					  job_ptr->part_ptr->qos_ptr);
+
 	while (assoc) {
 		_handle_assoc_tres_run_secs(NULL, tres_run_delta,
 					    job_ptr->job_id, assoc);
@@ -926,10 +908,10 @@ static void _init_grp_used_tres_run_secs(time_t last_ran)
 	uint64_t tres_run_delta[slurmctld_tres_cnt];
 	int i;
 
-	if (priority_debug)
-		info("Initializing grp_used_tres_run_secs");
+	log_flag(PRIO, "Initializing grp_used_tres_run_secs");
 
-	if (!(enforce & ACCOUNTING_ENFORCE_LIMITS))
+	if (!(slurm_conf.accounting_storage_enforce &
+	      ACCOUNTING_ENFORCE_LIMITS))
 		return;
 	if (!(job_list && list_count(job_list)))
 		return;
@@ -940,8 +922,7 @@ static void _init_grp_used_tres_run_secs(time_t last_ran)
 	assoc_mgr_lock(&locks);
 	while ((job_ptr = list_next(itr))) {
 		double usage_factor = 1.0;
-		if (priority_debug)
-			debug2("job: %u", job_ptr->job_id);
+		log_flag(PRIO, "job: %u", job_ptr->job_id);
 
 		/* If end_time_exp is NO_VAL we have already ran the end for
 		 * this job.  We don't want to do it again, so just exit.
@@ -978,7 +959,7 @@ static void _init_grp_used_tres_run_secs(time_t last_ran)
 /* If the job is running then apply decay to the job.
  *
  * Return 0 if we don't need to process the job any further, 1 if
- * futher processing is needed.
+ * further processing is needed.
  */
 static int _apply_new_usage(job_record_t *job_ptr, time_t start_period,
 			    time_t end_period, bool adjust_for_end)
@@ -1062,7 +1043,7 @@ static int _apply_new_usage(job_record_t *job_ptr, time_t start_period,
 	if (adjust_for_end)
 		job_ptr->end_time_exp = (time_t)NO_VAL;
 
-	if (priority_debug) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO) {
 		info("job %u ran for %g seconds with TRES counts of",
 		     job_ptr->job_id, run_delta);
 		if (job_ptr->tres_alloc_cnt) {
@@ -1176,14 +1157,10 @@ static int _apply_new_usage(job_record_t *job_ptr, time_t start_period,
 	while (assoc) {
 		assoc->usage->grp_used_wall += run_decay;
 		assoc->usage->usage_raw += (long double)real_decay;
-		if (priority_debug)
-			info("Adding %f new usage to assoc %u (%s/%s/%s) "
-			     "raw usage is now %Lf.  Group wall "
-			     "added %f making it %f.",
-			     real_decay, assoc->id, assoc->acct,
-			     assoc->user, assoc->partition,
-			     assoc->usage->usage_raw, run_decay,
-			     assoc->usage->grp_used_wall);
+		log_flag(PRIO, "Adding %f new usage to assoc %u (%s/%s/%s) raw usage is now %Lf. Group wall added %f making it %f.",
+			 real_decay, assoc->id, assoc->acct, assoc->user,
+			 assoc->partition, assoc->usage->usage_raw, run_decay,
+			 assoc->usage->grp_used_wall);
 		_handle_assoc_tres_run_secs(tres_run_decay, tres_run_delta,
 					    job_ptr->job_id, assoc);
 
@@ -1213,9 +1190,8 @@ static void *_decay_thread(void *no_data)
 {
 	time_t start_time = time(NULL);
 	time_t last_reset = 0, next_reset = 0;
-	uint32_t calc_period = slurm_get_priority_calc_period();
-	double decay_hl = (double)slurm_get_priority_decay_hl();
-	uint16_t reset_period = slurm_get_priority_reset_period();
+	double decay_hl = (double) slurm_conf.priority_decay_hl;
+	uint16_t reset_period = slurm_conf.priority_reset_period;
 
 	time_t now;
 	double run_delta = 0.0, real_decay = 0.0;
@@ -1300,10 +1276,9 @@ static void *_decay_thread(void *no_data)
 			   flush the used time at a certain time
 			   set by PriorityUsageResetPeriod in the slurm.conf
 			*/
-			calc_period = slurm_get_priority_calc_period();
-			reset_period = slurm_get_priority_reset_period();
+			reset_period = slurm_conf.priority_reset_period;
 			next_reset = 0;
-			decay_hl = (double)slurm_get_priority_decay_hl();
+			decay_hl = (double) slurm_conf.priority_decay_hl;
 			if (decay_hl > 0)
 				decay_factor = 1 - (0.693 / decay_hl);
 			else
@@ -1361,10 +1336,8 @@ static void *_decay_thread(void *no_data)
 		if (real_decay < DBL_MIN)
 			real_decay = DBL_MIN;
 
-		if (priority_debug)
-			info("Decay factor over %g seconds goes "
-			     "from %.15f -> %.15f",
-			     run_delta, decay_factor, real_decay);
+		log_flag(PRIO, "Decay factor over %g seconds goes from %.15f -> %.15f",
+			 run_delta, decay_factor, real_decay);
 
 		/* first apply decay to used time */
 		if (_apply_decay(real_decay) != SLURM_SUCCESS) {
@@ -1403,7 +1376,7 @@ static void *_decay_thread(void *no_data)
 		running_decay = 0;
 
 		/* Sleep until the next time. */
-		abs.tv_sec += calc_period;
+		abs.tv_sec += slurm_conf.priority_calc_period;
 		slurm_cond_timedwait(&decay_cond, &decay_lock, &abs);
 		slurm_mutex_unlock(&decay_lock);
 
@@ -1546,43 +1519,30 @@ static void _filter_job(job_record_t *job_ptr,
 
 static void _internal_setup(void)
 {
-	char *tres_weights_str;
-	if (slurm_get_debug_flags() & DEBUG_FLAG_PRIO)
-		priority_debug = 1;
-	else
-		priority_debug = 0;
-
-	favor_small = slurm_get_priority_favor_small();
-	damp_factor = (long double)slurm_get_fs_dampening_factor();
-	enforce = slurm_get_accounting_storage_enforce();
-	max_age = slurm_get_priority_max_age();
-	weight_age = slurm_get_priority_weight_age();
-	weight_assoc = slurm_get_priority_weight_assoc();
-	weight_fs = slurm_get_priority_weight_fairshare();
-	weight_js = slurm_get_priority_weight_job_size();
-	weight_part = slurm_get_priority_weight_partition();
-	weight_qos = slurm_get_priority_weight_qos();
+	damp_factor = (long double) slurm_conf.fs_dampening_factor;
+	max_age = slurm_conf.priority_max_age;
+	weight_age = slurm_conf.priority_weight_age;
+	weight_assoc = slurm_conf.priority_weight_assoc;
+	weight_fs = slurm_conf.priority_weight_fs;
+	weight_js = slurm_conf.priority_weight_js;
+	weight_part = slurm_conf.priority_weight_part;
+	weight_qos = slurm_conf.priority_weight_qos;
 	xfree(weight_tres);
-	if ((tres_weights_str = slurm_get_priority_weight_tres())) {
-		weight_tres = slurm_get_tres_weight_array(tres_weights_str,
-							  slurmctld_tres_cnt,
-							  true);
-	}
-	xfree(tres_weights_str);
-	flags = slurm_get_priority_flags();
+	weight_tres = slurm_get_tres_weight_array(
+		slurm_conf.priority_weight_tres, slurmctld_tres_cnt, true);
+	flags = slurm_conf.priority_flags;
 
-	if (priority_debug) {
-		info("priority: Damp Factor is %u", damp_factor);
-		info("priority: AccountingStorageEnforce is %u", enforce);
-		info("priority: Max Age is %u", max_age);
-		info("priority: Weight Age is %u", weight_age);
-		info("priority: Weight Assoc is %u", weight_assoc);
-		info("priority: Weight Fairshare is %u", weight_fs);
-		info("priority: Weight JobSize is %u", weight_js);
-		info("priority: Weight Part is %u", weight_part);
-		info("priority: Weight QOS is %u", weight_qos);
-		info("priority: Flags is %u", flags);
-	}
+	log_flag(PRIO, "priority: Damp Factor is %u", damp_factor);
+	log_flag(PRIO, "priority: AccountingStorageEnforce is %u",
+		 slurm_conf.accounting_storage_enforce);
+	log_flag(PRIO, "priority: Max Age is %u", max_age);
+	log_flag(PRIO, "priority: Weight Age is %u", weight_age);
+	log_flag(PRIO, "priority: Weight Assoc is %u", weight_assoc);
+	log_flag(PRIO, "priority: Weight Fairshare is %u", weight_fs);
+	log_flag(PRIO, "priority: Weight JobSize is %u", weight_js);
+	log_flag(PRIO, "priority: Weight Part is %u", weight_part);
+	log_flag(PRIO, "priority: Weight QOS is %u", weight_qos);
+	log_flag(PRIO, "priority: Flags is %u", flags);
 }
 
 
@@ -1696,25 +1656,16 @@ static void _depth_oblivious_set_usage_efctv(slurmdb_assoc_rec_t *assoc)
 			assoc->usage->shares_norm;
 #endif
 
-		if (priority_debug) {
-			info("Effective usage for %s %s off %s(%s) "
-			     "(%Lf * %Lf ^ %Lf) * %f  = %Lf",
-			     child, child_str,
-			     assoc->usage->parent_assoc_ptr->acct,
-			     assoc->usage->fs_assoc_ptr->acct,
-			     ratio_p, ratio_l, k,
-			     assoc->usage->shares_norm,
-			     assoc->usage->usage_efctv);
-		}
+		log_flag(PRIO, "Effective usage for %s %s off %s(%s) (%Lf * %Lf ^ %Lf) * %f  = %Lf",
+			 child, child_str, assoc->usage->parent_assoc_ptr->acct,
+			 assoc->usage->fs_assoc_ptr->acct, ratio_p, ratio_l, k,
+			 assoc->usage->shares_norm, assoc->usage->usage_efctv);
 	} else {
 		assoc->usage->usage_efctv = assoc->usage->usage_norm;
-		if (priority_debug) {
-			info("Effective usage for %s %s off %s(%s) %Lf",
-			     child, child_str,
-			     assoc->usage->parent_assoc_ptr->acct,
-			     assoc->usage->fs_assoc_ptr->acct,
-			     assoc->usage->usage_efctv);
-		}
+		log_flag(PRIO, "Effective usage for %s %s off %s(%s) %Lf",
+			 child, child_str, assoc->usage->parent_assoc_ptr->acct,
+			 assoc->usage->fs_assoc_ptr->acct,
+			 assoc->usage->usage_efctv);
 	}
 }
 
@@ -1744,23 +1695,20 @@ static void _set_usage_efctv(slurmdb_assoc_rec_t *assoc)
  */
 int init ( void )
 {
-	char *temp = NULL;
 	/* Write lock on jobs, read lock on nodes and partitions */
 	slurmctld_lock_t job_write_lock =
 		{ NO_LOCK, WRITE_LOCK, READ_LOCK, READ_LOCK, NO_LOCK };
 
 	/* This means we aren't running from the controller so skip setup. */
 	if (cluster_cpus == NO_VAL) {
-		damp_factor = (long double)slurm_get_fs_dampening_factor();
+		damp_factor = (long double) slurm_conf.fs_dampening_factor;
 		return SLURM_SUCCESS;
 	}
 
 	_internal_setup();
 
 	/* Check to see if we are running a supported accounting plugin */
-	temp = slurm_get_accounting_storage_type();
-	if (xstrcasecmp(temp, "accounting_storage/slurmdbd")
-	    && xstrcasecmp(temp, "accounting_storage/mysql")) {
+	if (!slurm_with_slurmdbd()) {
 		time_t start_time = time(NULL);
 		if (weight_age)
 			error("PriorityWeightAge can only be used with SlurmDBD, ignoring");
@@ -1810,8 +1758,6 @@ int init ( void )
 		calc_fairshare = 0;
 	}
 
-	xfree(temp);
-
 	site_factor_plugin_init();
 
 	debug("%s loaded", plugin_name);
@@ -1868,19 +1814,20 @@ extern void priority_p_reconfig(bool assoc_clear)
 				   NO_LOCK, NO_LOCK, NO_LOCK };
 
 	reconfig = 1;
-	prevflags = flags;
 	_internal_setup();
 
 	/* Since Fair Tree uses a different shares calculation method, we
 	 * must reassign shares at reconfigure if the algorithm was switched to
 	 * or from Fair Tree */
 	if ((flags & PRIORITY_FLAGS_FAIR_TREE) !=
-	    (prevflags & PRIORITY_FLAGS_FAIR_TREE)) {
+	    (slurm_conf.priority_flags & PRIORITY_FLAGS_FAIR_TREE)) {
 		assoc_mgr_lock(&locks);
 		_set_norm_shares(assoc_mgr_root_assoc->usage->children_list);
 		assoc_mgr_unlock(&locks);
 	}
 
+	flags = slurm_conf.priority_flags;
+
 	/* Since the used_cpu_run_secs has been reset by the reconfig,
 	 * we need to remove the time that has past since the last
 	 * poll.  We can't just do the correct calculation in the
@@ -1928,7 +1875,7 @@ extern void priority_p_set_assoc_usage(slurmdb_assoc_rec_t *assoc)
 	set_assoc_usage_norm(assoc);
 	_set_assoc_usage_efctv(assoc);
 
-	if (priority_debug)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO)
 		_priority_p_set_assoc_usage_debug(assoc);
 }
 
@@ -2008,12 +1955,12 @@ extern List priority_p_get_priority_factors_list(
 			if (job_ptr->priority == 0)
 				continue;
 
-			if ((slurmctld_conf.private_data & PRIVATE_DATA_JOBS) &&
+			if ((slurm_conf.private_data & PRIVATE_DATA_JOBS) &&
 			    (job_ptr->user_id != uid) &&
 			    !validate_operator(uid) &&
 			    (((slurm_mcs_get_privatedata() == 0) &&
 			      !assoc_mgr_is_user_acct_coord(acct_db_conn, uid,
-							    job_ptr->account))||
+			                                    job_ptr->account))||
 			     ((slurm_mcs_get_privatedata() == 1) &&
 			      (mcs_g_check_mcs_label(uid, job_ptr->mcs_label)
 			       != 0))))
@@ -2036,8 +1983,7 @@ extern List priority_p_get_priority_factors_list(
  * READ_LOCK, READ_LOCK, NO_LOCK }; should be locked before calling this */
 extern void priority_p_job_end(job_record_t *job_ptr)
 {
-	if (priority_debug)
-		info("priority_p_job_end: called for job %u", job_ptr->job_id);
+	log_flag(PRIO, "%s: called for job %u", __func__, job_ptr->job_id);
 
 	_apply_new_usage(job_ptr, g_last_ran, time(NULL), 1);
 }
@@ -2096,7 +2042,7 @@ extern int decay_apply_weighted_factors(job_record_t *job_ptr,
 
 extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 {
-	slurmdb_qos_rec_t *qos_ptr = NULL;
+	assoc_mgr_lock_t locks = { .assoc = READ_LOCK, .qos = READ_LOCK };
 
 	xassert(job_ptr);
 
@@ -2110,8 +2056,6 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 		       sizeof(priority_factors_object_t));
 	}
 
-	qos_ptr = job_ptr->qos_ptr;
-
 	if (weight_age && job_ptr->details->accrue_time) {
 		uint32_t diff = 0;
 
@@ -2170,12 +2114,12 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 			job_ptr->prio_factors->priority_js /= time_limit;
 			/* Normalize to max value of 1.0 */
 			job_ptr->prio_factors->priority_js /= cluster_cpus;
-			if (favor_small) {
+			if (slurm_conf.priority_favor_small) {
 				job_ptr->prio_factors->priority_js =
 					(double) 1.0 -
 					job_ptr->prio_factors->priority_js;
 			}
-		} else if (favor_small) {
+		} else if (slurm_conf.priority_favor_small) {
 			job_ptr->prio_factors->priority_js =
 				(double)(node_record_count - min_nodes)
 				/ (double)node_record_count;
@@ -2210,18 +2154,20 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 
 	job_ptr->prio_factors->priority_site = job_ptr->site_factor;
 
+	assoc_mgr_lock(&locks);
 	if (job_ptr->assoc_ptr && weight_assoc)
 		job_ptr->prio_factors->priority_assoc =
 			(flags & PRIORITY_FLAGS_NO_NORMAL_ASSOC) ?
 			job_ptr->assoc_ptr->priority :
 			job_ptr->assoc_ptr->usage->priority_norm;
 
-	if (qos_ptr && qos_ptr->priority && weight_qos) {
+	if (job_ptr->qos_ptr && job_ptr->qos_ptr->priority && weight_qos) {
 		job_ptr->prio_factors->priority_qos =
 			(flags & PRIORITY_FLAGS_NO_NORMAL_QOS) ?
-			qos_ptr->priority :
-			qos_ptr->usage->norm_priority;
+			job_ptr->qos_ptr->priority :
+			job_ptr->qos_ptr->usage->norm_priority;
 	}
+	assoc_mgr_unlock(&locks);
 
 	if (job_ptr->details)
 		job_ptr->prio_factors->nice = job_ptr->details->nice;
