@@ -50,8 +50,10 @@
 #include "src/common/bitstring.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/proc_args.h"
+#include "src/common/read_config.h"
 #include "src/common/slurm_resource_info.h"
 #include "src/common/xstring.h"
+#include "src/common/cgroup.h"
 #include "src/slurmd/common/xcpuinfo.h"
 #include "src/slurmd/common/task_plugin.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
@@ -113,9 +115,6 @@ static inline int hwloc_bitmap_isequal(
 static hwloc_bitmap_t global_allowed_cpuset;
 #endif
 
-hwloc_obj_type_t obj_types[3] = {HWLOC_OBJ_SOCKET, HWLOC_OBJ_CORE,
-			         HWLOC_OBJ_PU};
-
 static uint16_t bind_mode = CPU_BIND_NONE   | CPU_BIND_MASK   |
 			    CPU_BIND_RANK   | CPU_BIND_MAP    |
 			    CPU_BIND_LDMASK | CPU_BIND_LDRANK |
@@ -126,97 +125,6 @@ static uint16_t bind_mode_ldom =
 
 #endif
 
-static bool cpuset_prefix_set = false;
-static char *cpuset_prefix = "";
-
-static char user_cgroup_path[PATH_MAX];
-static char job_cgroup_path[PATH_MAX];
-static char jobstep_cgroup_path[PATH_MAX];
-
-static xcgroup_ns_t cpuset_ns;
-
-static xcgroup_t user_cpuset_cg;
-static xcgroup_t job_cpuset_cg;
-static xcgroup_t step_cpuset_cg;
-
-static int _xcgroup_cpuset_init(xcgroup_t* cg);
-
-/* when cgroups are configured with cpuset, at least
- * cpuset.cpus and cpuset.mems must be set or the cgroup
- * will not be available at all.
- * we duplicate the ancestor configuration in the init step */
-static int _xcgroup_cpuset_init(xcgroup_t* cg)
-{
-	int fstatus,i;
-
-	char* cpuset_metafiles[] = {
-		"cpus",
-		"mems"
-	};
-	char cpuset_meta[PATH_MAX];
-	char* cpuset_conf;
-	size_t csize;
-	xcgroup_t acg;
-	char *acg_name, *p;
-
-	fstatus = XCGROUP_ERROR;
-
-	/* load ancestor cg */
-	acg_name = (char *)xstrdup(cg->name);
-	p = xstrrchr(acg_name, '/');
-	if (p == NULL) {
-		debug2("task/cgroup: unable to get ancestor path for "
-		       "cpuset cg '%s' : %m", cg->path);
-		xfree(acg_name);
-		return fstatus;
-	} else {
-		*p = '\0';
-	}
-	if (xcgroup_load(cg->ns, &acg, acg_name) != XCGROUP_SUCCESS) {
-		debug2("task/cgroup: unable to load ancestor for "
-		       "cpuset cg '%s' : %m", cg->path);
-		xfree(acg_name);
-		return fstatus;
-	}
-	xfree(acg_name);
-
-	/* inherits ancestor params */
-	for (i = 0; i < 2; i++) {
-	again:
-		snprintf(cpuset_meta, sizeof(cpuset_meta), "%s%s",
-			 cpuset_prefix, cpuset_metafiles[i]);
-		if (xcgroup_get_param(&acg,cpuset_meta,
-				      &cpuset_conf,&csize)
-		    != XCGROUP_SUCCESS) {
-			if (!cpuset_prefix_set) {
-				cpuset_prefix_set = 1;
-				cpuset_prefix = "cpuset.";
-				goto again;
-			}
-
-			debug("task/cgroup: assuming no cpuset cg "
-			       "support for '%s'",acg.path);
-			xcgroup_destroy(&acg);
-			return fstatus;
-		}
-		if (csize > 0)
-			cpuset_conf[csize-1]='\0';
-		if (xcgroup_set_param(cg,cpuset_meta,cpuset_conf)
-		    != XCGROUP_SUCCESS) {
-			debug("task/cgroup: unable to write %s configuration "
-			       "(%s) for cpuset cg '%s'",cpuset_meta,
-			       cpuset_conf,cg->path);
-			xcgroup_destroy(&acg);
-			xfree(cpuset_conf);
-			return fstatus;
-		}
-		xfree(cpuset_conf);
-	}
-
-	xcgroup_destroy(&acg);
-	return XCGROUP_SUCCESS;
-}
-
 #ifdef HAVE_HWLOC
 
 /*
@@ -245,7 +153,7 @@ static int _get_ldom_sched_cpuset(
 static int _get_sched_cpuset(
 		hwloc_topology_t topology,
 		hwloc_obj_type_t hwtype, hwloc_obj_type_t req_hwtype,
-		cpu_set_t *mask, stepd_step_rec_t *job);
+		cpu_set_t *mask, stepd_step_rec_t *job, uint32_t local_id);
 
 /*
  * Add hwloc cpuset for a hwloc object to the total cpuset for a task, using
@@ -272,7 +180,7 @@ static void _add_hwloc_cpuset(
 static int _task_cgroup_cpuset_dist_cyclic(
 	hwloc_topology_t topology, hwloc_obj_type_t hwtype,
 	hwloc_obj_type_t req_hwtype, stepd_step_rec_t *job, int bind_verbose,
-	hwloc_bitmap_t cpuset);
+	hwloc_bitmap_t cpuset, uint32_t taskid);
 
 /*
  * Distribute cpus to task using block distribution
@@ -287,7 +195,8 @@ static int _task_cgroup_cpuset_dist_cyclic(
 static int _task_cgroup_cpuset_dist_block(
 	hwloc_topology_t topology, hwloc_obj_type_t hwtype,
 	hwloc_obj_type_t req_hwtype, uint32_t nobj,
-	stepd_step_rec_t *job, int bind_verbose, hwloc_bitmap_t cpuset);
+	stepd_step_rec_t *job, int bind_verbose, hwloc_bitmap_t cpuset,
+	uint32_t taskid);
 
 static int _get_ldom_sched_cpuset(hwloc_topology_t topology,
 		hwloc_obj_type_t hwtype, hwloc_obj_type_t req_hwtype,
@@ -309,12 +218,11 @@ static int _get_ldom_sched_cpuset(hwloc_topology_t topology,
 
 static int _get_sched_cpuset(hwloc_topology_t topology,
 		hwloc_obj_type_t hwtype, hwloc_obj_type_t req_hwtype,
-		cpu_set_t *mask, stepd_step_rec_t *job)
+		cpu_set_t *mask, stepd_step_rec_t *job, uint32_t local_id)
 {
 	int nummasks, maskid, i, threads;
 	char *curstr, *selstr;
 	char mstr[1 + CPU_SETSIZE / 4];
-	uint32_t local_id = job->envtp->localid;
 	char buftype[1024];
 
 	/* For CPU_BIND_RANK, CPU_BIND_MASK and CPU_BIND_MAP, generate sched
@@ -323,7 +231,7 @@ static int _get_sched_cpuset(hwloc_topology_t topology,
 	 * sched cpuset from hwloc topology.
 	 */
 	slurm_sprint_cpu_bind_type(buftype, job->cpu_bind_type);
-	debug3("task/cgroup: (%s[%d]) %s", buftype,
+	debug3("(%s[%d]) %s", buftype,
 			job->cpu_bind_type, job->cpu_bind);
 	CPU_ZERO(mask);
 
@@ -333,7 +241,7 @@ static int _get_sched_cpuset(hwloc_topology_t topology,
 
 	if (job->cpu_bind_type & CPU_BIND_RANK) {
 		threads = MAX(conf->threads, 1);
-		CPU_SET(job->envtp->localid % (job->cpus*threads), mask);
+		CPU_SET(local_id % (job->cpus*threads), mask);
 		return true;
 	}
 
@@ -387,7 +295,7 @@ static int _get_sched_cpuset(hwloc_topology_t topology,
 	if (job->cpu_bind_type & CPU_BIND_MASK) {
 		/* convert mask string into cpu_set_t mask */
 		if (task_str_to_cpuset(mask, mstr) < 0) {
-			error("task/cgroup: task_str_to_cpuset %s", mstr);
+			error("task_str_to_cpuset %s", mstr);
 			return false;
 		}
 		return true;
@@ -478,7 +386,7 @@ static void _add_hwloc_cpuset(
 
 		if (pobj != NULL) {
 			if (bind_verbose)
-				info("task/cgroup: task[%u] higher level %s "
+				info("task[%u] higher level %s "
 				     "found", taskid,
 				     hwloc_obj_type_string(pobj->type));
 #if HWLOC_API_VERSION >= 0x00020000
@@ -493,7 +401,7 @@ static void _add_hwloc_cpuset(
 		} else {
 			/* should not be executed */
 			if (bind_verbose)
-				info("task/cgroup: task[%u] no higher level "
+				info("task[%u] no higher level "
 				     "found", taskid);
 #if HWLOC_API_VERSION >= 0x00020000
 			allowed_cpuset = hwloc_bitmap_alloc();
@@ -522,7 +430,7 @@ static void _add_hwloc_cpuset(
 static int _task_cgroup_cpuset_dist_cyclic(
 	hwloc_topology_t topology, hwloc_obj_type_t hwtype,
 	hwloc_obj_type_t req_hwtype, stepd_step_rec_t *job, int bind_verbose,
-	hwloc_bitmap_t cpuset)
+	hwloc_bitmap_t cpuset, uint32_t taskid)
 {
 #if HWLOC_API_VERSION >= 0x00020000
 	hwloc_bitmap_t allowed_cpuset;
@@ -534,12 +442,16 @@ static int _task_cgroup_cpuset_dist_cyclic(
 	uint32_t *c_ixn;	/* core index by socket (next taskid) */
 	uint32_t *t_ix;		/* thread index by core by socket */
 	uint16_t npus = 0, nboards = 0, nthreads = 0, ncores = 0, nsockets = 0;
-	uint32_t taskid = job->envtp->localid;
 	int spec_thread_cnt = 0;
 	bitstr_t *spec_threads = NULL;
 	uint32_t obj_idxs[3], cps, tpc, i, j, sock_loop, ntskip, npdist;
 	bool core_cyclic, core_fcyclic, sock_fcyclic;
 	bool hwloc_success = true;
+	hwloc_obj_type_t socket_type = HWLOC_OBJ_SOCKET;
+#if HWLOC_API_VERSION >= 0x00020000
+	if (xstrcasestr(slurm_conf.slurmd_params, "l3cache_as_socket"))
+		socket_type = HWLOC_OBJ_L3CACHE;
+#endif
 
 	/*
 	 * We can't trust the slurmd_conf_t *conf here as we need actual
@@ -555,7 +467,7 @@ static int _task_cgroup_cpuset_dist_cyclic(
 		 * (e.g. 4 cores on socket 0 and 3 cores on socket 1)
 		 */
 		nsockets = (uint16_t) hwloc_get_nbobjs_by_type(topology,
-							HWLOC_OBJ_SOCKET);
+							       socket_type);
 		ncores = (uint16_t) hwloc_get_nbobjs_by_type(topology,
 							HWLOC_OBJ_CORE);
 		nthreads = (uint16_t) hwloc_get_nbobjs_by_type(topology,
@@ -570,7 +482,7 @@ static int _task_cgroup_cpuset_dist_cyclic(
 	}
 
 	if ((nsockets == 0) || (ncores == 0))
-		return XCGROUP_ERROR;
+		return SLURM_ERROR;
 	cps = (ncores + nsockets - 1) / nsockets;
 	tpc = (nthreads + ncores - 1) / ncores;
 
@@ -582,7 +494,7 @@ static int _task_cgroup_cpuset_dist_cyclic(
 		SLURM_DIST_CORECFULL ? true : false;
 
 	if (bind_verbose) {
-		info("task/cgroup: task[%u] using %s distribution "
+		info("task[%u] using %s distribution "
 		     "(task_dist=0x%x)", taskid,
 		     format_task_dist_states(job->task_dist), job->task_dist);
 	}
@@ -644,7 +556,7 @@ static int _task_cgroup_cpuset_dist_cyclic(
 		 */
 		while ((s_ix < nsockets) && (j < npdist)) {
 			obj = hwloc_get_obj_below_by_type(
-				topology, HWLOC_OBJ_SOCKET, s_ix,
+				topology, socket_type, s_ix,
 				hwtype, c_ixc[s_ix]);
 #if HWLOC_API_VERSION >= 0x00020000
 			if (obj) {
@@ -665,6 +577,12 @@ static int _task_cgroup_cpuset_dist_cyclic(
 #endif
 				if (hwloc_compare_types(hwtype, HWLOC_OBJ_PU)
 									>= 0) {
+					hwloc_obj_type_t obj_types[3] = {
+						socket_type,
+						HWLOC_OBJ_CORE,
+						HWLOC_OBJ_PU
+					};
+
 					/* granularity is thread */
 					obj_idxs[0]=s_ix;
 					obj_idxs[1]=c_ixc[s_ix];
@@ -752,30 +670,30 @@ static int _task_cgroup_cpuset_dist_cyclic(
 	if ((sock_loop > npdist) && !hwloc_success) {
 		/* hwloc_get_obj_below_by_type() fails if no CPU set
 		 * configured, see hwloc documentation for details */
-		error("task/cgroup: hwloc_get_obj_below_by_type() failing, "
+		error("hwloc_get_obj_below_by_type() failing, "
 		      "task/affinity plugin may be required to address bug "
 		      "fixed in HWLOC version 1.11.5");
-		return XCGROUP_ERROR;
+		return SLURM_ERROR;
 	} else if (sock_loop > npdist) {
 		char buf[128] = "";
 		hwloc_bitmap_snprintf(buf, sizeof(buf), cpuset);
-		error("task/cgroup: task[%u] infinite loop broken while trying "
+		error("task[%u] infinite loop broken while trying "
 		      "to provision compute elements using %s (bitmap:%s)",
 		      taskid, format_task_dist_states(job->task_dist), buf);
-		return XCGROUP_ERROR;
+		return SLURM_ERROR;
 	} else
-		return XCGROUP_SUCCESS;
+		return SLURM_SUCCESS;
 }
 
 static int _task_cgroup_cpuset_dist_block(
 	hwloc_topology_t topology, hwloc_obj_type_t hwtype,
 	hwloc_obj_type_t req_hwtype, uint32_t nobj,
-	stepd_step_rec_t *job, int bind_verbose, hwloc_bitmap_t cpuset)
+	stepd_step_rec_t *job, int bind_verbose, hwloc_bitmap_t cpuset,
+	uint32_t taskid)
 {
 	hwloc_obj_t obj;
 	uint32_t core_loop, ntskip, npdist;
 	uint32_t i, j, pfirst, plast;
-	uint32_t taskid = job->envtp->localid;
 	int hwdepth;
 	uint32_t npus, ncores, nsockets;
 	int spec_thread_cnt = 0;
@@ -785,8 +703,14 @@ static int _task_cgroup_cpuset_dist_block(
 	uint32_t core_idx;
 	bool core_fcyclic, core_block;
 
+	hwloc_obj_type_t socket_type = HWLOC_OBJ_SOCKET;
+#if HWLOC_API_VERSION >= 0x00020000
+	if (xstrcasestr(slurm_conf.slurmd_params, "l3cache_as_socket"))
+		socket_type = HWLOC_OBJ_L3CACHE;
+#endif
+
 	nsockets = (uint32_t) hwloc_get_nbobjs_by_type(topology,
-						       HWLOC_OBJ_SOCKET);
+						       socket_type);
 	ncores = (uint32_t) hwloc_get_nbobjs_by_type(topology,
 						     HWLOC_OBJ_CORE);
 	npus = (uint32_t) hwloc_get_nbobjs_by_type(topology,
@@ -798,7 +722,7 @@ static int _task_cgroup_cpuset_dist_block(
 		SLURM_DIST_CORECFULL ? true : false;
 
 	if (bind_verbose) {
-		info("task/cgroup: task[%u] using block distribution, "
+		info("task[%u] using block distribution, "
 		     "task_dist 0x%x", taskid, job->task_dist);
 	}
 
@@ -846,20 +770,20 @@ static int _task_cgroup_cpuset_dist_block(
 		if ((core_loop > npdist) && !hwloc_success) {
 			/* hwloc_get_obj_below_by_type() fails if no CPU set
 			 * configured, see hwloc documentation for details */
-			error("task/cgroup: hwloc_get_obj_below_by_type() "
+			error("hwloc_get_obj_below_by_type() "
 			      "failing, task/affinity plugin may be required"
 			      "to address bug fixed in HWLOC version 1.11.5");
-			return XCGROUP_ERROR;
+			return SLURM_ERROR;
 		} else if (core_loop > npdist) {
 			char buf[128] = "";
 			hwloc_bitmap_snprintf(buf, sizeof(buf), cpuset);
-			error("task/cgroup: task[%u] infinite loop broken while "
+			error("task[%u] infinite loop broken while "
 			      "trying to provision compute elements using %s (bitmap:%s)",
 			      taskid, format_task_dist_states(job->task_dist),
 			      buf);
-			return XCGROUP_ERROR;
+			return SLURM_ERROR;
 		} else
-			return XCGROUP_SUCCESS;
+			return SLURM_SUCCESS;
 	}
 
 	if (hwloc_compare_types(hwtype, HWLOC_OBJ_CORE) >= 0) {
@@ -919,7 +843,7 @@ static int _task_cgroup_cpuset_dist_block(
 		FREE_NULL_BITMAP(spec_threads);
 	}
 
-	return XCGROUP_SUCCESS;
+	return SLURM_SUCCESS;
 }
 
 
@@ -968,7 +892,7 @@ static void _validate_mask(uint32_t task_id, hwloc_obj_t obj, cpu_set_t *ts)
 	}
 
 	if (!superset) {
-		info("task/cgroup: Ignoring user CPU binding outside of job "
+		info("Ignoring user CPU binding outside of job "
 		     "step allocation for task[%u]", task_id);
 		fprintf(stderr, "Requested cpu_bind option outside of job "
 			"step allocation for task[%u]\n", task_id);
@@ -982,348 +906,96 @@ static void _validate_mask(uint32_t task_id, hwloc_obj_t obj, cpu_set_t *ts)
 
 extern int task_cgroup_cpuset_init(void)
 {
-	/* initialize user/job/jobstep cgroup relative paths */
-	user_cgroup_path[0]='\0';
-	job_cgroup_path[0]='\0';
-	jobstep_cgroup_path[0]='\0';
-
-	/* initialize cpuset cgroup namespace */
-	if (xcgroup_ns_create(&cpuset_ns, "", "cpuset")
-	    != XCGROUP_SUCCESS) {
-		error("task/cgroup: unable to create cpuset namespace");
-		return SLURM_ERROR;
-	}
-
+	cgroup_g_initialize(CG_CPUS);
 	return SLURM_SUCCESS;
 }
 
 extern int task_cgroup_cpuset_fini(void)
 {
-	xcgroup_t cpuset_cg;
-
-	/* Similarly to task_cgroup_memory_fini(), we must lock the
-	 * root cgroup so we don't race with another job step that is
-	 * being started.  */
-        if (xcgroup_create(&cpuset_ns, &cpuset_cg,"",0,0) == XCGROUP_SUCCESS) {
-		if (xcgroup_lock(&cpuset_cg) == XCGROUP_SUCCESS) {
-			/* First move slurmstepd to the root cpuset cg
-			 * so we can remove the step/job/user cpuset
-			 * cg's.  */
-			xcgroup_move_process(&cpuset_cg, getpid());
-
-			xcgroup_wait_pid_moved(&step_cpuset_cg, "cpuset step");
-
-                        if (xcgroup_delete(&step_cpuset_cg) != SLURM_SUCCESS)
-                                debug2("task/cgroup: unable to remove step "
-                                       "cpuset : %m");
-                        if (xcgroup_delete(&job_cpuset_cg) != XCGROUP_SUCCESS)
-                                debug2("task/cgroup: not removing "
-                                       "job cpuset : %m");
-                        if (xcgroup_delete(&user_cpuset_cg) != XCGROUP_SUCCESS)
-                                debug2("task/cgroup: not removing "
-                                       "user cpuset : %m");
-                        xcgroup_unlock(&cpuset_cg);
-                } else
-                        error("task/cgroup: unable to lock root cpuset : %m");
-                xcgroup_destroy(&cpuset_cg);
-        } else
-                error("task/cgroup: unable to create root cpuset : %m");
-
-	if (user_cgroup_path[0] != '\0')
-		xcgroup_destroy(&user_cpuset_cg);
-	if (job_cgroup_path[0] != '\0')
-		xcgroup_destroy(&job_cpuset_cg);
-	if (jobstep_cgroup_path[0] != '\0')
-		xcgroup_destroy(&step_cpuset_cg);
-
-	user_cgroup_path[0]='\0';
-	job_cgroup_path[0]='\0';
-	jobstep_cgroup_path[0]='\0';
-
-	xcgroup_ns_destroy(&cpuset_ns);
-
-	return SLURM_SUCCESS;
+	return cgroup_g_step_destroy(CG_CPUS);
 }
 
 extern int task_cgroup_cpuset_create(stepd_step_rec_t *job)
 {
-	int rc;
-	int fstatus = SLURM_ERROR;
-	xcgroup_t cpuset_cg;
-	uint32_t jobid;
-	uint32_t stepid = job->stepid;
-	uid_t uid = job->uid;
-	uid_t gid = job->gid;
-	char* user_alloc_cores = NULL;
-	char* job_alloc_cores = NULL;
-	char* step_alloc_cores = NULL;
-	char cpuset_meta[PATH_MAX];
-	char *cpus = NULL;
-	size_t cpus_size;
-	char *slurm_cgpath;
-	xcgroup_t slurm_cg;
-#ifdef HAVE_NATIVE_CRAY
-	char expected_usage[32];
-#endif
+	cgroup_limits_t limits, *root_limits = NULL;
+	char *job_alloc_cpus = NULL;
+	char *step_alloc_cpus = NULL;
+	int rc = SLURM_SUCCESS;
 
-	/* create slurm root cg in this cg namespace */
-	slurm_cgpath = task_cgroup_create_slurm_cg(&cpuset_ns);
-	if (slurm_cgpath == NULL)
-		return SLURM_ERROR;
+	/* First create the cpuset hierarchy for this job */
+	if ((rc = cgroup_g_step_create(CG_CPUS, job)) != SLURM_SUCCESS)
+		return rc;
 
-	/* check that this cgroup has cpus allowed or initialize them */
-	if (xcgroup_load(&cpuset_ns,&slurm_cg,slurm_cgpath) != XCGROUP_SUCCESS){
-		error("task/cgroup: unable to load slurm cpuset xcgroup");
-		xfree(slurm_cgpath);
-		return SLURM_ERROR;
-	}
-again:
-	snprintf(cpuset_meta, sizeof(cpuset_meta), "%scpus", cpuset_prefix);
-	rc = xcgroup_get_param(&slurm_cg, cpuset_meta, &cpus, &cpus_size);
-	if ((rc != XCGROUP_SUCCESS) || (cpus_size == 1)) {
-		if (!cpuset_prefix_set && (rc != XCGROUP_SUCCESS)) {
-			cpuset_prefix_set = 1;
-			cpuset_prefix = "cpuset.";
-			xfree(cpus);
-			goto again;
-		}
+	/* Then constrain the user/job/step to the required cores/cpus */
 
-		/* initialize the cpusets as it was non-existent */
-		if (_xcgroup_cpuset_init(&slurm_cg) != XCGROUP_SUCCESS) {
-			xfree(cpus);
-			xfree(slurm_cgpath);
-			xcgroup_destroy(&slurm_cg);
-			return SLURM_ERROR;
-		}
-	}
-	xfree(cpus);
-	xcgroup_destroy(&slurm_cg);
+	/* build job and job steps allocated cores lists */
+	debug("job abstract cores are '%s'", job->job_alloc_cores);
+	debug("step abstract cores are '%s'", job->step_alloc_cores);
 
-	/* build user cgroup relative path if not set (should not be) */
-	if (*user_cgroup_path == '\0') {
-		if (snprintf(user_cgroup_path, PATH_MAX,
-			     "%s/uid_%u", slurm_cgpath, uid) >= PATH_MAX) {
-			error("task/cgroup: unable to build uid %u cgroup "
-			      "relative path : %m", uid);
-			xfree(slurm_cgpath);
-			return SLURM_ERROR;
-		}
-	}
-	xfree(slurm_cgpath);
-
-	/* build job cgroup relative path if no set (should not be) */
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
-	else
-		jobid = job->jobid;
-	if (*job_cgroup_path == '\0') {
-		if (snprintf(job_cgroup_path,PATH_MAX,"%s/job_%u",
-			     user_cgroup_path, jobid) >= PATH_MAX) {
-			error("task/cgroup: unable to build job %u cpuset "
-			      "cg relative path : %m", jobid);
-			return SLURM_ERROR;
-		}
-	}
-
-	/* build job step cgroup relative path (should not be) */
-	if (*jobstep_cgroup_path == '\0') {
-		int cc;
-		if (stepid == SLURM_BATCH_SCRIPT) {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				      "%s/step_batch", job_cgroup_path);
-		} else if (stepid == SLURM_EXTERN_CONT) {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				      "%s/step_extern", job_cgroup_path);
-		} else {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				      "%s/step_%u", job_cgroup_path, stepid);
-		}
-		if (cc >= PATH_MAX) {
-			error("task/cgroup: unable to build job step %u.%u "
-			      "cpuset cg relative path: %m",
-			      jobid, stepid);
-			return SLURM_ERROR;
-		}
-	}
-
-	/*
-	 * create cpuset root cg and lock it
-	 *
-	 * we will keep the lock until the end to avoid the effect of a release
-	 * agent that would remove an existing cgroup hierarchy while we are
-	 * setting it up. As soon as the step cgroup is created, we can release
-	 * the lock.
-	 * Indeed, consecutive slurm steps could result in cg being removed
-	 * between the next EEXIST instantiation and the first addition of
-	 * a task. The release_agent will have to lock the root cpuset cgroup
-	 * to avoid this scenario.
-	 */
-	if (xcgroup_create(&cpuset_ns, &cpuset_cg, "", 0,0) != XCGROUP_SUCCESS){
-		error("task/cgroup: unable to create root cpuset xcgroup");
-		return SLURM_ERROR;
-	}
-	if (xcgroup_lock(&cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&cpuset_cg);
-		error("task/cgroup: unable to lock root cpuset cg");
-		return SLURM_ERROR;
-	}
-
-	/*
-	 * build job and job steps allocated cores lists
-	 */
-	debug("task/cgroup: job abstract cores are '%s'",
-	      job->job_alloc_cores);
-	debug("task/cgroup: step abstract cores are '%s'",
-	      job->step_alloc_cores);
 	if (xcpuinfo_abs_to_mac(job->job_alloc_cores,
-				&job_alloc_cores) != SLURM_SUCCESS) {
-		error("task/cgroup: unable to build job physical cores");
-		goto error;
+				&job_alloc_cpus) != SLURM_SUCCESS) {
+		error("unable to build job physical cores");
+		goto endit;
 	}
 	if (xcpuinfo_abs_to_mac(job->step_alloc_cores,
-				&step_alloc_cores) != SLURM_SUCCESS) {
-		error("task/cgroup: unable to build step physical cores");
-		goto error;
+				&step_alloc_cpus) != SLURM_SUCCESS) {
+		error("unable to build step physical cores");
+		goto endit;
 	}
-	debug("task/cgroup: job physical cores are '%s'",
-	      job_alloc_cores);
-	debug("task/cgroup: step physical cores are '%s'",
-	      step_alloc_cores);
+	debug("job physical CPUs are '%s'", job_alloc_cpus);
+	debug("step physical CPUs are '%s'", step_alloc_cpus);
 
 	/*
-	 * create user cgroup in the cpuset ns (it could already exist)
+	 * check that user's cpuset cgroup is consistent and add the job's CPUs
 	 */
-	if (xcgroup_create(&cpuset_ns,&user_cpuset_cg, user_cgroup_path,
-			   getuid(), getgid()) != XCGROUP_SUCCESS) {
-		goto error;
-	}
-	if (xcgroup_instantiate(&user_cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		goto error;
-	}
+	root_limits = cgroup_g_root_constrain_get(CG_CPUS);
 
-	/*
-	 * Check that user's cpuset cgroup is consistent and add the job cores.
-	 * This will set cpuset.mems and cpuset.cpus to the ancestor value and
-	 * after we'll set it to job's allocated cores.
-	 */
-	if (_xcgroup_cpuset_init(&user_cpuset_cg) != XCGROUP_SUCCESS) {
-		(void)xcgroup_delete(&user_cpuset_cg);
-		xcgroup_destroy(&user_cpuset_cg);
-		xfree(cpus);
-		goto error;
-	}
-	user_alloc_cores = xstrdup(job_alloc_cores);
-	if ((cpus != NULL) && (cpus_size > 1)) {
-		cpus[cpus_size-1]='\0';
-		xstrcat(user_alloc_cores, ",");
-		xstrcat(user_alloc_cores, cpus);
-	}
-	xcgroup_set_param(&user_cpuset_cg, cpuset_meta, user_alloc_cores);
-	xfree(cpus);
+	if (!root_limits)
+		goto endit;
 
-	/*
-	 * create job cgroup in the cpuset ns (it could already exist)
-	 */
-	if (xcgroup_create(&cpuset_ns,&job_cpuset_cg,
-			   job_cgroup_path,
-			   getuid(),getgid()) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		goto error;
-	}
-	if (xcgroup_instantiate(&job_cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		xcgroup_destroy(&job_cpuset_cg);
-		goto error;
-	}
-	if (_xcgroup_cpuset_init(&job_cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		xcgroup_destroy(&job_cpuset_cg);
-		goto error;
-	}
-	xcgroup_set_param(&job_cpuset_cg, cpuset_meta, job_alloc_cores);
-	/*
-	 * create step cgroup in the cpuset ns (it should not exists)
-	 * use job's user uid/gid to enable tasks cgroups creation by
-	 * the user inside the step cgroup owned by root
-	 */
-	if (xcgroup_create(&cpuset_ns,&step_cpuset_cg,
-			   jobstep_cgroup_path,
-			   uid,gid) != XCGROUP_SUCCESS) {
-		/* do not delete user/job cgroup as */
-		/* they can exist for other steps */
-		xcgroup_destroy(&user_cpuset_cg);
-		xcgroup_destroy(&job_cpuset_cg);
-		goto error;
-	}
-	if (xcgroup_instantiate(&step_cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		xcgroup_destroy(&job_cpuset_cg);
-		xcgroup_destroy(&step_cpuset_cg);
-		goto error;
-	}
-	if (_xcgroup_cpuset_init(&step_cpuset_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_cpuset_cg);
-		xcgroup_destroy(&job_cpuset_cg);
-		(void)xcgroup_delete(&step_cpuset_cg);
-		xcgroup_destroy(&step_cpuset_cg);
-		goto error;
-	}
-	xcgroup_set_param(&step_cpuset_cg, cpuset_meta, step_alloc_cores);
+	memset(&limits, 0, sizeof(limits));
+	limits.allow_mems = root_limits->allow_mems;
 
-	/*
-	 * on Cray systems, set the expected usage in bytes.
-	 * This is used by the Cray OOM killer
-	 */
-#ifdef HAVE_NATIVE_CRAY
-	snprintf(expected_usage, sizeof(expected_usage), "%"PRIu64,
-		 (uint64_t)job->step_mem * 1024 * 1024);
-	xcgroup_set_param(&step_cpuset_cg, "expected_usage_in_bytes",
-			  expected_usage);
-#endif
+	/* User constrain */
+	limits.allow_cores = xstrdup_printf(
+		"%s,%s", job_alloc_cpus, root_limits->allow_cores);
+	rc = cgroup_g_user_constrain_set(CG_CPUS, job, &limits);
+	xfree(limits.allow_cores);
+	if (rc != SLURM_SUCCESS)
+		goto endit;
 
-	/* attach the slurmstepd to the step cpuset cgroup */
-	pid_t pid = getpid();
-	rc = xcgroup_add_pids(&step_cpuset_cg,&pid,1);
-	if (rc != XCGROUP_SUCCESS) {
-		error("task/cgroup: unable to add slurmstepd to cpuset cg '%s'",
-		      step_cpuset_cg.path);
-		fstatus = SLURM_ERROR;
-	} else
-		fstatus = SLURM_SUCCESS;
+	/* Job constrain */
+	limits.allow_cores = job_alloc_cpus;
+	rc = cgroup_g_job_constrain_set(CG_CPUS, job, &limits);
+	if (rc != SLURM_SUCCESS)
+		goto endit;
+
+	/* Step constrain */
+	limits.allow_cores = step_alloc_cpus;
+	rc = cgroup_g_step_constrain_set(CG_CPUS, job, &limits);
+	if (rc != SLURM_SUCCESS)
+		goto endit;
 
 	/* validate the requested cpu frequency and set it */
-	cpu_freq_cgroup_validate(job, step_alloc_cores);
-error:
-	xcgroup_unlock(&cpuset_cg);
-	xcgroup_destroy(&cpuset_cg);
+	cpu_freq_cgroup_validate(job, step_alloc_cpus);
 
-	xfree(user_alloc_cores);
-	xfree(job_alloc_cores);
-	xfree(step_alloc_cores);
-
-	return fstatus;
-}
-
-extern int task_cgroup_cpuset_attach_task(stepd_step_rec_t *job)
-{
-	int fstatus = SLURM_ERROR;
-
-	/* tasks are automatically attached as slurmstepd is in the step cg */
-	fstatus = SLURM_SUCCESS;
-
-	return fstatus;
+endit:
+	xfree(job_alloc_cpus);
+	xfree(step_alloc_cpus);
+	cgroup_free_limits(root_limits);
+	return rc;
 }
 
 /* affinity should be set using sched_setaffinity to not force */
 /* user to have to play with the cgroup hierarchy to modify it */
-extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
+extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job,
+						uint32_t taskid)
 {
 	int fstatus = SLURM_ERROR;
 
 #ifndef HAVE_HWLOC
 
-	error("task/cgroup: plugin not compiled with hwloc support, "
+	error("plugin not compiled with hwloc support, "
 	      "skipping affinity.");
 	return fstatus;
 
@@ -1339,18 +1011,23 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 	hwloc_obj_type_t req_hwtype;
 	int bind_verbose = 0;
 	int rc = SLURM_SUCCESS;
-	pid_t    pid = job->envtp->task_pid;
+	pid_t    pid = job->task[taskid]->pid;
 	size_t tssize;
 	uint32_t nldoms;
 	uint32_t nsockets;
 	uint32_t ncores;
 	uint32_t npus;
 	uint32_t nobj;
-	uint32_t taskid = job->envtp->localid;
 	uint32_t jntasks = job->node_tasks;
 	uint32_t jnpus;
 	int spec_threads = 0;
 
+	hwloc_obj_type_t socket_type = HWLOC_OBJ_SOCKET;
+#if HWLOC_API_VERSION >= 0x00020000
+	if (xstrcasestr(slurm_conf.slurmd_params, "l3cache_as_socket"))
+		socket_type = HWLOC_OBJ_L3CACHE;
+#endif
+
 	/* Allocate and initialize hwloc objects */
 	hwloc_topology_init(&topology);
 
@@ -1371,7 +1048,7 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 		jnpus = jntasks * job->cpus_per_task;
 
 	bind_type = job->cpu_bind_type;
-	if ((conf->task_plugin_param & CPU_BIND_VERBOSE) ||
+	if ((slurm_conf.task_plugin_param & CPU_BIND_VERBOSE) ||
 	    (bind_type & CPU_BIND_VERBOSE))
 		bind_verbose = 1 ;
 
@@ -1382,44 +1059,44 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 		 * In such case, use NUMA-node instead of socket. */
 		socket_or_node = HWLOC_OBJ_NODE;
 	} else {
-		socket_or_node = HWLOC_OBJ_SOCKET;
+		socket_or_node = socket_type;
 	}
 
 	if (bind_type & CPU_BIND_NONE) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting no affinity",
+			info("task[%u] is requesting no affinity",
 			     taskid);
 		return 0;
 	} else if (bind_type & CPU_BIND_TO_THREADS) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "thread level binding",taskid);
 		req_hwtype = HWLOC_OBJ_PU;
 	} else if (bind_type & CPU_BIND_TO_CORES) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "core level binding",taskid);
 		req_hwtype = HWLOC_OBJ_CORE;
 	} else if (bind_type & CPU_BIND_TO_SOCKETS) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "socket level binding",taskid);
 		req_hwtype = socket_or_node;
 	} else if (bind_type & CPU_BIND_TO_LDOMS) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "ldom level binding",taskid);
 		req_hwtype = HWLOC_OBJ_NODE;
 	} else if (bind_type & CPU_BIND_TO_BOARDS) {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "board level binding",taskid);
 		req_hwtype = HWLOC_OBJ_GROUP;
 	} else if (bind_type & bind_mode_ldom) {
 		req_hwtype = HWLOC_OBJ_NODE;
 	} else {
 		if (bind_verbose)
-			info("task/cgroup: task[%u] using core level binding"
+			info("task[%u] using core level binding"
 			     " by default",taskid);
 		req_hwtype = HWLOC_OBJ_CORE;
 	}
@@ -1498,19 +1175,19 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 	 * If not enough objects to do the job, revert to no affinity mode
 	 */
 	if (hwloc_compare_types(hwtype, HWLOC_OBJ_MACHINE) == 0) {
-		info("task/cgroup: task[%u] disabling affinity because of %s "
+		info("task[%u] disabling affinity because of %s "
 		     "granularity",taskid, hwloc_obj_type_string(hwtype));
 		if (job->cpu_bind_type & CPU_BIND_VERBOSE)
-			fprintf(stderr,"task/cgroup: task[%u] disabling "
+			fprintf(stderr,"task[%u] disabling "
 				"affinity because of %s granularity\n",
 				taskid, hwloc_obj_type_string(hwtype));
 	} else if ((hwloc_compare_types(hwtype, HWLOC_OBJ_CORE) >= 0) &&
 		   (nobj < jnpus)) {
-		info("task/cgroup: task[%u] not enough %s objects (%d < %d), "
+		info("task[%u] not enough %s objects (%d < %d), "
 		     "disabling affinity",
 		     taskid, hwloc_obj_type_string(hwtype), nobj, jnpus);
 		if (job->cpu_bind_type & CPU_BIND_VERBOSE)
-			fprintf(stderr, "task/cgroup: task[%u] not enough %s "
+			fprintf(stderr, "task[%u] not enough %s "
 				"objects (%d < %d), disabling affinity\n",
 				taskid, hwloc_obj_type_string(hwtype),
 				nobj, jnpus);
@@ -1521,24 +1198,25 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 		 */
 		obj = hwloc_get_obj_by_type(topology, HWLOC_OBJ_MACHINE, 0);
 		if (bind_verbose) {
-			info("task/cgroup: task[%u] is requesting "
+			info("task[%u] is requesting "
 			     "explicit binding mode", taskid);
 		}
-		_get_sched_cpuset(topology, hwtype, req_hwtype, &ts, job);
+		_get_sched_cpuset(topology, hwtype, req_hwtype, &ts, job,
+				  taskid);
 		tssize = sizeof(cpu_set_t);
 		fstatus = SLURM_SUCCESS;
 		_validate_mask(taskid, obj, &ts);
 		if ((rc = sched_setaffinity(pid, tssize, &ts))) {
-			error("task/cgroup: task[%u] unable to set "
+			error("task[%u] unable to set "
 			      "mask 0x%s", taskid,
 			      task_cpuset_to_str(&ts, mstr));
 			error("sched_setaffinity rc = %d", rc);
 			fstatus = SLURM_ERROR;
 		} else if (bind_verbose) {
-			info("task/cgroup: task[%u] mask 0x%s",
+			info("task[%u] mask 0x%s",
 			     taskid, task_cpuset_to_str(&ts, mstr));
 		}
-		task_slurm_chkaffinity(&ts, job, rc);
+		task_slurm_chkaffinity(&ts, job, rc, taskid);
 	} else {
 		/*
 		 * Bind the detected object to the taskid, respecting the
@@ -1548,7 +1226,7 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 		char *str = NULL;
 
 		if (bind_verbose) {
-			info("task/cgroup: task[%u] using %s granularity dist %u",
+			info("task[%u] using %s granularity dist %u",
 			     taskid, hwloc_obj_type_string(hwtype),
 			     job->task_dist);
 		}
@@ -1568,17 +1246,18 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 			/* tasks are distributed in blocks within a plane */
 			_task_cgroup_cpuset_dist_block(topology,
 				hwtype, req_hwtype,
-				nobj, job, bind_verbose, cpuset);
+				nobj, job, bind_verbose, cpuset, taskid);
 			break;
 		case SLURM_DIST_ARBITRARY:
 		case SLURM_DIST_BLOCK:
 		case SLURM_DIST_CYCLIC:
 		case SLURM_DIST_UNKNOWN:
-			if (slurm_get_select_type_param()
+			if (slurm_conf.select_type_param
 			    & CR_CORE_DEFAULT_DIST_BLOCK) {
 				_task_cgroup_cpuset_dist_block(topology,
 					hwtype, req_hwtype,
-					nobj, job, bind_verbose, cpuset);
+					nobj, job, bind_verbose, cpuset,
+					taskid);
 				break;
 			}
 			/*
@@ -1588,7 +1267,7 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 		default:
 			_task_cgroup_cpuset_dist_cyclic(topology,
 				hwtype, req_hwtype,
-				job, bind_verbose, cpuset);
+				job, bind_verbose, cpuset, taskid);
 			break;
 		}
 
@@ -1599,16 +1278,16 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
 							 &ts, tssize) == 0) {
 			fstatus = SLURM_SUCCESS;
 			if ((rc = sched_setaffinity(pid, tssize, &ts))) {
-				error("task/cgroup: task[%u] unable to set "
+				error("task[%u] unable to set "
 				      "taskset '%s'", taskid, str);
 				fstatus = SLURM_ERROR;
 			} else if (bind_verbose) {
-				info("task/cgroup: task[%u] set taskset '%s'",
+				info("task[%u] set taskset '%s'",
 				     taskid, str);
 			}
-			task_slurm_chkaffinity(&ts, job, rc);
+			task_slurm_chkaffinity(&ts, job, rc, taskid);
 		} else {
-			error("task/cgroup: task[%u] unable to build "
+			error("task[%u] unable to build "
 			      "taskset '%s'",taskid,str);
 			fstatus = SLURM_ERROR;
 		}
@@ -1634,7 +1313,7 @@ extern int task_cgroup_cpuset_set_task_affinity(stepd_step_rec_t *job)
  */
 extern int task_cgroup_cpuset_add_pid(pid_t pid)
 {
-	return xcgroup_add_pids(&step_cpuset_cg, &pid, 1);
+	return cgroup_g_step_addto(CG_CPUS, &pid, 1);
 }
 
 #endif
