@@ -167,6 +167,7 @@ typedef enum {
 	PRINT_PRXMPT,
 	PRINT_UF,
 	PRINT_UT,
+	PRINT_LF,
 
 	/* PROBLEM */
 	PRINT_PROBLEM = 7000,
@@ -226,6 +227,7 @@ extern List g_tres_list;
 
 extern bool user_case_norm;
 extern bool tree_display;
+extern bool have_db_conn;
 
 extern bool sacctmgr_check_default_qos(uint32_t qos_id,
 				       slurmdb_assoc_cond_t *assoc_cond);
@@ -253,7 +255,7 @@ extern int sacctmgr_list_assoc(int argc, char **argv);
 extern int sacctmgr_list_user(int argc, char **argv);
 extern int sacctmgr_list_account(int argc, char **argv);
 extern int sacctmgr_list_cluster(int argc, char **argv);
-extern int sacctmgr_list_config(bool have_db_conn);
+extern int sacctmgr_list_config(void);
 extern int sacctmgr_list_event(int argc, char **argv);
 extern int sacctmgr_list_federation(int argc, char **argv);
 extern int sacctmgr_list_problem(int argc, char **argv);
@@ -264,7 +266,6 @@ extern int sacctmgr_list_stats(int argc, char **argv);
 extern int sacctmgr_list_tres(int, char **);
 extern int sacctmgr_list_wckey(int argc, char **argv);
 
-extern int sacctmgr_modify_assoc(int argc, char **argv);
 extern int sacctmgr_modify_user(int argc, char **argv);
 extern int sacctmgr_modify_account(int argc, char **argv);
 extern int sacctmgr_modify_cluster(int argc, char **argv);
@@ -273,7 +274,6 @@ extern int sacctmgr_modify_job(int argc, char **argv);
 extern int sacctmgr_modify_qos(int argc, char **argv);
 extern int sacctmgr_modify_res(int argc, char **argv);
 
-extern int sacctmgr_delete_assoc(int argc, char **argv);
 extern int sacctmgr_delete_user(int argc, char **argv);
 extern int sacctmgr_delete_account(int argc, char **argv);
 extern int sacctmgr_delete_cluster(int argc, char **argv);
