@@ -386,7 +386,8 @@ static void _filter_job_records(void)
 			 * not begin with a '*', act on all wckeys with the same
 			 * name, default or not.
 			 */
-			if ((opt.wckey[0] != '*') && (job_key[0] == '*'))
+			if ((opt.wckey[0] != '*') && job_key &&
+			    (job_key[0] == '*'))
 				job_key++;
 
 			if (xstrcmp(job_key, opt.wckey) != 0) {
@@ -734,21 +735,12 @@ _cancel_job_id (void *ci)
 	}
 
 	for (i = 0; i < MAX_CANCEL_RETRY; i++) {
-		job_step_kill_msg_t kill_msg;
-
 		_add_delay();
 		START_TIMER;
 
-		memset(&kill_msg, 0, sizeof(job_step_kill_msg_t));
-		kill_msg.flags	= flags;
-		kill_msg.step_id.job_id = NO_VAL;
-		kill_msg.step_id.step_id = NO_VAL;
-		kill_msg.step_id.step_het_comp = NO_VAL;
-		kill_msg.sibling     = opt.sibling;
-		kill_msg.signal      = cancel_info->sig;
-		kill_msg.sjob_id     = cancel_info->job_id_str;
-
-		error_code = slurm_kill_job_msg(REQUEST_KILL_JOB, &kill_msg);
+		error_code = slurm_kill_job2(cancel_info->job_id_str,
+					     cancel_info->sig, flags,
+					     opt.sibling);
 
 		END_TIMER;
 		slurm_mutex_lock(&max_delay_lock);
@@ -776,7 +768,7 @@ _cancel_job_id (void *ci)
 		     (error_code == ESLURM_INVALID_JOB_ID)) &&
 		    (cancel_info->sig == SIGKILL)) {
 			error_code = 0;	/* Ignore error if job done */
-		}	
+		}
 	}
 
 	/* Purposely free the struct passed in here, so the caller doesn't have
