@@ -78,12 +78,15 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#include "src/common/env.h"
+#include "src/common/fd.h"
 #include "src/common/forward.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
 #include "src/common/node_select.h"
 #include "src/common/parse_time.h"
+#include "src/common/run_command.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_interface.h"
 #include "src/common/uid.h"
@@ -105,6 +108,7 @@
 #define RPC_PACK_MAX_AGE	30	/* Rebuild data over 30 seconds old */
 #define DUMP_RPC_COUNT 		25
 #define HOSTLIST_MAX_SIZE 	80
+#define MAIL_PROG_TIMEOUT 120*1000
 
 typedef enum {
 	DSH_NEW,        /* Request not yet started */
@@ -172,6 +176,7 @@ typedef struct queued_request {
 typedef struct mail_info {
 	char *user_name;
 	char *message;
+	char **environment; /* MailProg environment variables */
 } mail_info_t;
 
 static void _agent_defer(void);
@@ -199,7 +204,7 @@ static mail_info_t *_mail_alloc(void);
 static void  _mail_free(void *arg);
 static void *_mail_proc(void *arg);
 static char *_mail_type_str(uint16_t mail_type);
-static char **_build_mail_env(void);
+static char **_build_mail_env(job_record_t *job_ptr, uint32_t mail_type);
 
 static pthread_mutex_t defer_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mail_mutex  = PTHREAD_MUTEX_INITIALIZER;
@@ -260,26 +265,23 @@ void *agent(void *args)
 	}
 #endif
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: Agent_cnt=%d agent_thread_cnt=%d with msg_type=%s retry_list_size=%d",
-		     __func__, agent_cnt, agent_thread_cnt,
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     retry_list_size());
-	}
+	log_flag(AGENT, "%s: Agent_cnt=%d agent_thread_cnt=%d with msg_type=%s retry_list_size=%d",
+		 __func__, agent_cnt, agent_thread_cnt,
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 retry_list_size());
+
 	slurm_mutex_lock(&agent_cnt_mutex);
 
-	if (sched_update != slurmctld_conf.last_update) {
+	if (sched_update != slurm_conf.last_update) {
 #ifdef HAVE_NATIVE_CRAY
 		reboot_from_ctld = true;
 #else
-		char *ctld_params = slurm_get_slurmctld_params();
-
 		reboot_from_ctld = false;
-		if (xstrcasestr(ctld_params, "reboot_from_controller"))
+		if (xstrcasestr(slurm_conf.slurmctld_params,
+		                "reboot_from_controller"))
 			reboot_from_ctld = true;
-		xfree(ctld_params);
 #endif
-		sched_update = slurmctld_conf.last_update;
+		sched_update = slurm_conf.last_update;
 	}
 
 	rpc_thread_cnt = 2 + MIN(agent_arg_ptr->node_count, AGENT_THREAD_COUNT);
@@ -315,15 +317,14 @@ void *agent(void *args)
 	/* start the watchdog thread */
 	slurm_thread_create(&thread_wdog, _wdog, agent_info_ptr);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
-		     __func__, agent_info_ptr->thread_count,
-		     agent_info_ptr->threads_active,
-		     agent_info_ptr->retry ? 'T' : 'F',
-		     agent_info_ptr->get_reply ? 'T' : 'F',
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     agent_info_ptr->protocol_version);
-	}
+	log_flag(AGENT, "%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
+		 __func__, agent_info_ptr->thread_count,
+		 agent_info_ptr->threads_active,
+		 agent_info_ptr->retry ? 'T' : 'F',
+		 agent_info_ptr->get_reply ? 'T' : 'F',
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 agent_info_ptr->protocol_version);
+
 	/* start all the other threads (up to AGENT_THREAD_COUNT active) */
 	for (i = 0; i < agent_info_ptr->thread_count; i++) {
 		/* wait until "room" for another thread */
@@ -350,7 +351,7 @@ void *agent(void *args)
 	/* Wait for termination of remaining threads */
 	pthread_join(thread_wdog, NULL);
 	delay = (int) difftime(time(NULL), begin_time);
-	if (delay > (slurm_get_msg_timeout() * 2)) {
+	if (delay > (slurm_conf.msg_timeout * 2)) {
 		info("agent msg_type=%u ran for %d seconds",
 			agent_arg_ptr->msg_type,  delay);
 	}
@@ -361,15 +362,13 @@ void *agent(void *args)
 	}
 	slurm_mutex_unlock(&agent_info_ptr->thread_mutex);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: end agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
-		     __func__, agent_info_ptr->thread_count,
-		     agent_info_ptr->threads_active,
-		     agent_info_ptr->retry ? 'T' : 'F',
-		     agent_info_ptr->get_reply ? 'T' : 'F',
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     agent_info_ptr->protocol_version);
-	}
+	log_flag(AGENT, "%s: end agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
+		 __func__, agent_info_ptr->thread_count,
+		 agent_info_ptr->threads_active,
+		 agent_info_ptr->retry ? 'T' : 'F',
+		 agent_info_ptr->get_reply ? 'T' : 'F',
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 agent_info_ptr->protocol_version);
 
 cleanup:
 	_purge_agent_args(agent_arg_ptr);
@@ -506,12 +505,9 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 		thread_ptr[thr_count].nodelist =
 			hostlist_ranged_string_xmalloc(hl);
 		hostlist_destroy(hl);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-			info("%s: sending msg_type %s to nodes %s",
-			     __func__, rpc_num2string(agent_arg_ptr->msg_type),
-			     thread_ptr[thr_count].nodelist);
-
-		}
+		log_flag(AGENT, "%s: sending msg_type %s to nodes %s",
+			 __func__, rpc_num2string(agent_arg_ptr->msg_type),
+			 thread_ptr[thr_count].nodelist);
 		thr_count++;
 	}
 	xfree(span);
@@ -544,10 +540,8 @@ static void _update_wdog_state(thd_t *thread_ptr,
 	case DSH_ACTIVE:
 		thd_comp->work_done = false;
 		if (thread_ptr->end_time <= thd_comp->now) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-				info("%s: agent thread %lu timed out", __func__,
-				     (unsigned long) thread_ptr->thread);
-			}
+			log_flag(AGENT, "%s: agent thread %lu timed out",
+				 __func__, (unsigned long) thread_ptr->thread);
 			if (pthread_kill(thread_ptr->thread, SIGUSR1) == ESRCH)
 				*state = DSH_NO_RESP;
 			else
@@ -651,11 +645,9 @@ static void *_wdog(void *args)
 		xfree(thread_ptr[i].nodelist);
 	}
 
-	if (thd_comp.max_delay &&
-	    (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT)) {
-		info("%s: agent maximum delay %d seconds", __func__,
-		     thd_comp.max_delay);
-	}
+	if (thd_comp.max_delay)
+		log_flag(AGENT, "%s: agent maximum delay %d seconds",
+			 __func__, thd_comp.max_delay);
 
 	slurm_mutex_unlock(&agent_ptr->thread_mutex);
 	return (void *) NULL;
@@ -666,30 +658,30 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 	/* Locks: Write job */
 	slurmctld_lock_t job_write_lock =
 	    { NO_LOCK, WRITE_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
-	uint32_t job_id = 0, step_id = 0;
+	slurm_step_id_t step_id = {
+		.job_id = 0,
+		.step_id = NO_VAL,
+		.step_het_comp = NO_VAL,
+	};
 	thd_t *thread_ptr = agent_ptr->thread_struct;
 
 	if        (agent_ptr->msg_type == SRUN_PING) {
 		srun_ping_msg_t *msg = *agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = msg->step_id;
+		step_id.job_id  = msg->job_id;
 	} else if (agent_ptr->msg_type == SRUN_TIMEOUT) {
 		srun_timeout_msg_t *msg = *agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = msg->step_id;
+		memcpy(&step_id, &msg->step_id, sizeof(step_id));
 	} else if (agent_ptr->msg_type == RESPONSE_RESOURCE_ALLOCATION) {
 		resource_allocation_response_msg_t *msg =
 			*agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = NO_VAL;
+		step_id.job_id = msg->job_id;
 	} else if (agent_ptr->msg_type == RESPONSE_HET_JOB_ALLOCATION) {
 		List het_alloc_list = *agent_ptr->msg_args_pptr;
 		resource_allocation_response_msg_t *msg;
 		if (!het_alloc_list || (list_count(het_alloc_list) == 0))
 			return;
 		msg = list_peek(het_alloc_list);
-		job_id  = msg->job_id;
-		step_id = NO_VAL;
+		step_id.job_id  = msg->job_id;
 	} else if ((agent_ptr->msg_type == SRUN_JOB_COMPLETE)		||
 		   (agent_ptr->msg_type == SRUN_REQUEST_SUSPEND)	||
 		   (agent_ptr->msg_type == SRUN_STEP_MISSING)		||
@@ -705,7 +697,7 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 	}
 	lock_slurmctld(job_write_lock);
 	if  (thread_ptr[0].state == DSH_DONE) {
-		srun_response(job_id, step_id);
+		srun_response(&step_id);
 	}
 
 	unlock_slurmctld(job_write_lock);
@@ -739,7 +731,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 				  .fed  = READ_LOCK };
 
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
+			job_complete(job_id, slurm_conf.slurm_user_id,
 				     true, false, 0);
 			unlock_slurmctld(job_write_lock);
 		}
@@ -780,9 +772,8 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names,
-					    "Prolog/Epilog failure",
-					    slurmctld_conf.slurm_user_id);
+				drain_nodes(node_names, "Prolog/Epilog failure",
+				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Prolog/Epilog failure on nodes %s%s",
@@ -792,9 +783,8 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names,
-					    "Duplicate jobid",
-					    slurmctld_conf.slurm_user_id);
+				drain_nodes(node_names, "Duplicate jobid",
+				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Duplicate jobid on nodes %s%s",
@@ -818,10 +808,7 @@ finished:	;
 	if (run_scheduler) {
 		run_scheduler = false;
 		/* below functions all have their own locking */
-		if (schedule(0))	{
-			schedule_job_save();
-			schedule_node_save();
-		}
+		schedule(false);
 	}
 	if ((agent_ptr->msg_type == REQUEST_PING) ||
 	    (agent_ptr->msg_type == REQUEST_HEALTH_CHECK) ||
@@ -932,10 +919,8 @@ static void *_thread_per_group_rpc(void *args)
 	msg.msg_type = msg_type;
 	msg.data     = task_ptr->msg_args_ptr;
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: sending %s to %s", __func__, rpc_num2string(msg_type),
-		     thread_ptr->nodelist);
-	}
+	log_flag(AGENT, "%s: sending %s to %s",
+		 __func__, rpc_num2string(msg_type), thread_ptr->nodelist);
 
 	if (task_ptr->get_reply) {
 		if (thread_ptr->addr) {
@@ -1020,7 +1005,7 @@ static void *_thread_per_group_rpc(void *args)
 				task_ptr->msg_args_ptr;
 			rc = SLURM_SUCCESS;
 			lock_slurmctld(job_write_lock);
-			if (job_epilog_complete(kill_job->job_id,
+			if (job_epilog_complete(kill_job->step_id.job_id,
 						ret_data_info->node_name, rc))
 				run_scheduler = true;
 			unlock_slurmctld(job_write_lock);
@@ -1048,8 +1033,8 @@ static void *_thread_per_group_rpc(void *args)
 			thread_state = DSH_DONE;
 			ret_data_info->err = thread_state;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		} else if ((msg_type == RESPONSE_RESOURCE_ALLOCATION) &&
@@ -1064,8 +1049,8 @@ static void *_thread_per_group_rpc(void *args)
 			     job_id, slurm_strerror(rc));
 			thread_state = DSH_FAILED;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		} else if ((msg_type == RESPONSE_HET_JOB_ALLOCATION) &&
@@ -1084,8 +1069,8 @@ static void *_thread_per_group_rpc(void *args)
 			     job_id, slurm_strerror(rc));
 			thread_state = DSH_FAILED;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		}
@@ -1097,7 +1082,7 @@ static void *_thread_per_group_rpc(void *args)
 
 			if ((msg_ptr->signal == SIGCONT) ||
 			    (msg_ptr->signal == SIGSTOP)) {
-				job_id = msg_ptr->job_id;
+				job_id = msg_ptr->step_id.job_id;
 				lock_slurmctld(job_write_lock);
 				job_ptr = find_job_record(job_id);
 				if (job_ptr == NULL) {
@@ -1157,10 +1142,8 @@ static void *_thread_per_group_rpc(void *args)
 			/* Not indicative of a real error */
 		case ESLURMD_JOB_NOTRUNNING:
 			/* Not indicative of a real error */
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-				info("%s: RPC to node %s failed, job not running",
-				     __func__, ret_data_info->node_name);
-			}
+			log_flag(AGENT, "%s: RPC to node %s failed, job not running",
+				 __func__, ret_data_info->node_name);
 			thread_state = DSH_DONE;
 			break;
 		default:
@@ -1200,7 +1183,7 @@ cleanup:
 			task_ptr->msg_args_ptr;
 		if ((msg_ptr->signal == SIGCONT) ||
 		    (msg_ptr->signal == SIGSTOP)) {
-			job_id = msg_ptr->job_id;
+			job_id = msg_ptr->step_id.job_id;
 			lock_slurmctld(job_write_lock);
 			job_ptr = find_job_record(job_id);
 			if (job_ptr)
@@ -1244,8 +1227,8 @@ static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 
 	itr = list_iterator_create(thread_ptr->ret_list);
 	while ((ret_data_info = list_next(itr))) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT)
-			info("%s: got err of %d", __func__, ret_data_info->err);
+		log_flag(AGENT, "%s: got err of %d",
+			 __func__, ret_data_info->err);
 		if (ret_data_info->err != DSH_NO_RESP)
 			continue;
 
@@ -1255,7 +1238,9 @@ static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 		node_ptr = find_node_record(ret_data_info->node_name);
 #endif
 		if (node_ptr &&
-		    (IS_NODE_DOWN(node_ptr) || IS_NODE_POWER_SAVE(node_ptr))) {
+		    (IS_NODE_DOWN(node_ptr) ||
+		     IS_NODE_POWERING_DOWN(node_ptr) ||
+		     IS_NODE_POWERED_DOWN(node_ptr))) {
 			--(*count);
 		} else if (agent_arg_ptr) {
 			debug("%s: got the name %s to resend out of %d",
@@ -1320,7 +1305,8 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 #endif
 			if (node_ptr &&
 			    (IS_NODE_DOWN(node_ptr) ||
-			     IS_NODE_POWER_SAVE(node_ptr))) {
+			     IS_NODE_POWERING_DOWN(node_ptr) ||
+			     IS_NODE_POWERED_DOWN(node_ptr))) {
 				/* Do not re-send RPC to DOWN node */
 				if (count)
 					count--;
@@ -1443,14 +1429,10 @@ extern void agent_init(void)
  */
 extern void agent_trigger(int min_wait, bool mail_too)
 {
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: pending_wait_time=%d->%d mail_too=%c->%c Agent_cnt=%d agent_thread_cnt=%d retry_list_size=%d",
-		     __func__, pending_wait_time, min_wait,
-		     mail_too ?  'T' : 'F',
-		     pending_mail ? 'T' : 'F',
-		     agent_cnt, agent_thread_cnt,
-		     retry_list_size());
-	}
+	log_flag(AGENT, "%s: pending_wait_time=%d->%d mail_too=%c->%c Agent_cnt=%d agent_thread_cnt=%d retry_list_size=%d",
+		 __func__, pending_wait_time, min_wait,
+		 mail_too ?  'T' : 'F', pending_mail ? 'T' : 'F',
+		 agent_cnt, agent_thread_cnt, retry_list_size());
 
 	slurm_mutex_lock(&pending_mutex);
 	if ((pending_wait_time == NO_VAL16) ||
@@ -1463,7 +1445,7 @@ extern void agent_trigger(int min_wait, bool mail_too)
 }
 
 /* agent_pack_pending_rpc_stats - pack counts of pending RPCs into a buffer */
-extern void agent_pack_pending_rpc_stats(Buf buffer)
+extern void agent_pack_pending_rpc_stats(buf_t *buffer)
 {
 	time_t now;
 	int i;
@@ -1609,7 +1591,7 @@ static void _agent_retry(int min_wait, bool mail_too)
 		if (((list_size > 100) &&
 		     (difftime(now, last_msg_time) > 300)) ||
 		    ((list_size > 0) &&
-		     (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT))) {
+		     (slurm_conf.debug_flags & DEBUG_FLAG_AGENT))) {
 			/* Note sizable backlog (retry_list_size()) of work */
 			retry_iter = list_iterator_create(retry_list);
 			while ((queued_req_ptr = list_next(retry_iter))) {
@@ -1710,7 +1692,7 @@ void agent_queue_request(agent_arg_t *agent_arg_ptr)
 		fatal("AGENT_THREAD_COUNT value is too high relative to MAX_SERVER_THREADS");
 
 	if (message_timeout == NO_VAL16) {
-		message_timeout = MAX(slurm_get_msg_timeout(), 30);
+		message_timeout = MAX(slurm_conf.msg_timeout, 30);
 	}
 
 	if (agent_arg_ptr->msg_type == REQUEST_SHUTDOWN) {
@@ -1845,6 +1827,8 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 			slurm_free_suspend_int_msg(agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == REQUEST_LAUNCH_PROLOG)
 			slurm_free_prolog_launch_msg(agent_arg_ptr->msg_args);
+		else if (agent_arg_ptr->msg_type == REQUEST_REBOOT_NODES)
+			slurm_free_reboot_msg(agent_arg_ptr->msg_args);
 		else
 			xfree(agent_arg_ptr->msg_args);
 	}
@@ -1863,54 +1847,49 @@ static void _mail_free(void *arg)
 	if (mi) {
 		xfree(mi->user_name);
 		xfree(mi->message);
+		env_array_free(mi->environment);
 		xfree(mi);
 	}
 }
 
-static char **_build_mail_env(void)
+/*
+ * Initializes MailProg environment and sets main part of variables, however
+ * additional variables are set in _set_job_time and _set_job_term_info to avoid
+ * code duplication.
+ */
+static char **_build_mail_env(job_record_t *job_ptr, uint32_t mail_type)
 {
-	char **my_env;
+	char **my_env = job_common_env_vars(job_ptr,
+					    ((mail_type & MAIL_JOB_END) ||
+					     (mail_type & MAIL_JOB_FAIL)));
 
-        my_env = xmalloc(sizeof(char *));
-        my_env[0] = NULL;
-        if (slurmctld_conf.cluster_name) {
-                setenvf(&my_env, "SLURM_CLUSTER_NAME", "%s",
-                        slurmctld_conf.cluster_name);
-        }
+	setenvf(&my_env, "SLURM_JOB_STATE", "%s",
+		job_state_string(job_ptr->job_state & JOB_STATE_BASE));
 
-	 return my_env;
+	setenvf(&my_env, "SLURM_JOB_MAIL_TYPE", "%s",
+		_mail_type_str(mail_type));
+
+	return my_env;
 }
 
 /* process an email request and free the record */
 static void *_mail_proc(void *arg)
 {
 	mail_info_t *mi = (mail_info_t *) arg;
-	pid_t pid;
+	int status;
+	char *result = NULL;
+	char *argv[5] = {
+		slurm_conf.mail_prog, "-s", mi->message, mi->user_name, NULL};
 
-	pid = fork();
-	if (pid < 0) {		/* error */
-		error("fork(): %m");
-	} else if (pid == 0) {	/* child */
-		int fd_0, fd_1, fd_2, i;
-		char **my_env = NULL;
-		my_env = _build_mail_env();
-		for (i = 0; i < 1024; i++)
-			(void) close(i);
-		if ((fd_0 = open("/dev/null", O_RDWR)) == -1)	// fd = 0
-			error("Couldn't open /dev/null: %m");
-		if ((fd_1 = dup(fd_0)) == -1)			// fd = 1
-			error("Couldn't do a dup on fd 1: %m");
-		if ((fd_2 = dup(fd_0)) == -1)			// fd = 2
-			error("Couldn't do a dup on fd 2 %m");
-		execle(slurmctld_conf.mail_prog, "mail",
-			"-s", mi->message, mi->user_name,
-			NULL, my_env);
-		error("Failed to exec %s: %m",
-			slurmctld_conf.mail_prog);
-		_exit(1);
-	} else {		/* parent */
-		waitpid(pid, NULL, 0);
-	}
+	result = run_command("MailProg", slurm_conf.mail_prog, argv,
+			     mi->environment, MAIL_PROG_TIMEOUT, 0, &status);
+	if (status)
+		error("MailProg returned error, it's output was '%s'", result);
+	else if (result && (strlen(result) > 0))
+		debug("MailProg output was '%s'.", result);
+	else
+		debug2("No output from MailProg, exit code=%d", status);
+	xfree(result);
 	_mail_free(mi);
 	slurm_mutex_lock(&agent_cnt_mutex);
 	slurm_mutex_lock(&mail_mutex);
@@ -1930,6 +1909,8 @@ static void *_mail_proc(void *arg)
 
 static char *_mail_type_str(uint16_t mail_type)
 {
+	if (mail_type == MAIL_INVALID_DEPEND)
+		return "Invalid dependency";
 	if (mail_type == MAIL_JOB_BEGIN)
 		return "Began";
 	if (mail_type == MAIL_JOB_END)
@@ -1952,16 +1933,19 @@ static char *_mail_type_str(uint16_t mail_type)
 }
 
 static void _set_job_time(job_record_t *job_ptr, uint16_t mail_type,
-			  char *buf, int buf_len)
+			  char *buf, int buf_len, char ***env)
 {
 	time_t interval = NO_VAL;
+	int msg_len;
 
 	buf[0] = '\0';
 	if ((mail_type == MAIL_JOB_BEGIN) && job_ptr->start_time &&
 	    job_ptr->details && job_ptr->details->submit_time) {
 		interval = job_ptr->start_time - job_ptr->details->submit_time;
 		snprintf(buf, buf_len, ", Queued time ");
-		secs2time_str(interval, buf+14, buf_len-14);
+		msg_len = 14;
+		secs2time_str(interval, buf+msg_len, buf_len-msg_len);
+		setenvf(env, "SLURM_JOB_QUEUED_TIME", "%s", buf+msg_len);
 		return;
 	}
 
@@ -1974,7 +1958,9 @@ static void _set_job_time(job_record_t *job_ptr, uint16_t mail_type,
 		} else
 			interval = job_ptr->end_time - job_ptr->start_time;
 		snprintf(buf, buf_len, ", Run time ");
-		secs2time_str(interval, buf+11, buf_len-11);
+		msg_len = 11;
+		secs2time_str(interval, buf+msg_len, buf_len-msg_len);
+		setenvf(env, "SLURM_JOB_RUN_TIME", "%s", buf+msg_len);
 		return;
 	}
 
@@ -1988,20 +1974,24 @@ static void _set_job_time(job_record_t *job_ptr, uint16_t mail_type,
 		} else
 			interval = time(NULL) - job_ptr->start_time;
 		snprintf(buf, buf_len, ", Run time ");
-		secs2time_str(interval, buf+11, buf_len-11);
+		msg_len = 11;
+		secs2time_str(interval, buf+msg_len, buf_len-msg_len);
+		setenvf(env, "SLURM_JOB_RUN_TIME", "%s", buf+msg_len);
 		return;
 	}
 
 	if ((mail_type == MAIL_JOB_STAGE_OUT) && job_ptr->end_time) {
 		interval = time(NULL) - job_ptr->end_time;
 		snprintf(buf, buf_len, " time ");
-		secs2time_str(interval, buf + 6, buf_len - 6);
+		msg_len = 11;
+		secs2time_str(interval, buf+msg_len, buf_len-msg_len);
+		setenvf(env, "SLURM_JOB_STAGE_OUT_TIME", "%s", buf+msg_len);
 		return;
 	}
 }
 
 static void _set_job_term_info(job_record_t *job_ptr, uint16_t mail_type,
-			       char *buf, int buf_len)
+			       char *buf, int buf_len, char ***env)
 {
 	buf[0] = '\0';
 
@@ -2030,17 +2020,27 @@ static void _set_job_term_info(job_record_t *job_ptr, uint16_t mail_type,
 				snprintf(buf, buf_len, ", %s, ExitCode [%d-%d]",
 					 state_string, exit_code_min,
 					 exit_code_max);
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MIN", "%d",
+					exit_code_min);
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MAX", "%d",
+					exit_code_max);
 			} else if (WIFSIGNALED(exit_status_max)) {
 				exit_code_max = WTERMSIG(exit_status_max);
 				snprintf(buf, buf_len, ", %s, MaxSignal [%d]",
 					 "Mixed", exit_code_max);
+				setenvf(env, "SLURM_JOB_TERM_SIGNAL_MAX", "%d",
+					exit_code_max);
 			} else if (WIFEXITED(exit_status_max)) {
 				exit_code_max = WEXITSTATUS(exit_status_max);
 				snprintf(buf, buf_len, ", %s, MaxExitCode [%d]",
 					 "Mixed", exit_code_max);
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MAX", "%d",
+					exit_code_max);
 			} else {
 				snprintf(buf, buf_len, ", %s",
 					 job_state_string(base_state));
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MAX", "%s",
+					"0");
 			}
 
 			if (job_ptr->array_recs->array_flags &
@@ -2054,9 +2054,13 @@ static void _set_job_term_info(job_record_t *job_ptr, uint16_t mail_type,
 				snprintf(buf, buf_len, ", %s, ExitCode %d",
 					 job_state_string(base_state),
 					 exit_code_max);
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MAX", "%d",
+					exit_code_max);
 			} else {
 				snprintf(buf, buf_len, ", %s",
 					 job_state_string(base_state));
+				setenvf(env, "SLURM_JOB_EXIT_CODE_MAX", "%s",
+					"0");
 			}
 		}
 	} else if (buf_len > 0) {
@@ -2093,8 +2097,11 @@ extern void mail_job_info(job_record_t *job_ptr, uint16_t mail_type)
 			job_ptr = master_job_ptr;
 	}
 
-	_set_job_time(job_ptr, mail_type, job_time, sizeof(job_time));
-	_set_job_term_info(job_ptr, mail_type, term_msg, sizeof(term_msg));
+	mi->environment = _build_mail_env(job_ptr, mail_type);
+	_set_job_time(job_ptr, mail_type, job_time, sizeof(job_time),
+		      &mi->environment);
+	_set_job_term_info(job_ptr, mail_type, term_msg, sizeof(term_msg),
+			   &mi->environment);
 	if (job_ptr->array_recs && !(job_ptr->mail_type & MAIL_ARRAY_TASKS)) {
 		mi->message = xstrdup_printf("Slurm Array Summary Job_id=%u_* (%u) Name=%s "
 					     "%s%s",
@@ -2116,7 +2123,7 @@ extern void mail_job_info(job_record_t *job_ptr, uint16_t mail_type)
 					     _mail_type_str(mail_type),
 					     job_time, term_msg);
 	}
-	info("email msg to %s: %s", mi->user_name, mi->message);
+	debug("email msg to %s: %s", mi->user_name, mi->message);
 
 	slurm_mutex_lock(&mail_mutex);
 	if (!mail_list)
@@ -2163,7 +2170,8 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 
 	if (job_ptr->wait_all_nodes) {
 		(void) job_node_ready(launch_msg_ptr->job_id, &tmp);
-		if (tmp == (READY_JOB_STATE | READY_NODE_STATE)) {
+		if (tmp ==
+		    (READY_JOB_STATE | READY_NODE_STATE | READY_PROLOG_STATE)) {
 			nodes_ready = 1;
 			if (launch_msg_ptr->alias_list &&
 			    !xstrcmp(launch_msg_ptr->alias_list, "TBD")) {
@@ -2191,7 +2199,8 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 			return -1;	/* invalid request?? */
 		}
 		xfree(hostname);
-		if (!IS_NODE_POWER_SAVE(node_ptr) &&
+		if (!IS_NODE_POWERED_DOWN(node_ptr) &&
+		    !IS_NODE_POWERING_DOWN(node_ptr) &&
 		    !IS_NODE_NO_RESPOND(node_ptr)) {
 			nodes_ready = 1;
 		}
@@ -2209,7 +2218,7 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 		queued_req_ptr->first_attempt = now;
 		queued_req_ptr->last_attempt  = now;
 	} else if (difftime(now, queued_req_ptr->first_attempt) >=
-				 slurm_get_resume_timeout()) {
+		   slurm_conf.resume_timeout) {
 		/* Nodes will get marked DOWN and job requeued, if possible */
 		error("agent waited too long for nodes to respond, abort launch of JobId=%u",
 		      job_ptr->job_id);
@@ -2234,11 +2243,11 @@ static int _signal_defer(queued_request_t *queued_req_ptr)
 
 	agent_arg_ptr = queued_req_ptr->agent_arg_ptr;
 	signal_msg_ptr = (signal_tasks_msg_t *)agent_arg_ptr->msg_args;
-	job_ptr = find_job_record(signal_msg_ptr->job_id);
+	job_ptr = find_job_record(signal_msg_ptr->step_id.job_id);
 
 	if (job_ptr == NULL) {
 		info("agent(signal_task): removed pending request for cancelled JobId=%u",
-		     signal_msg_ptr->job_id);
+		     signal_msg_ptr->step_id.job_id);
 		return -1;	/* job cancelled while waiting */
 	}
 
@@ -2248,7 +2257,7 @@ static int _signal_defer(queued_request_t *queued_req_ptr)
 	if (queued_req_ptr->first_attempt == 0) {
 		queued_req_ptr->first_attempt = now;
 	} else if (difftime(now, queued_req_ptr->first_attempt) >=
-				 2 * slurm_get_batch_start_timeout()) {
+	           (2 * slurm_conf.batch_start_timeout)) {
 		error("agent waited too long for nodes to respond, abort signal of JobId=%u",
 		      job_ptr->job_id);
 		return -1;
@@ -2269,31 +2278,30 @@ static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 {
 	char *argv[3], *pname;
 	pid_t child;
-	int i, rc, status = 0;
+	int rc, status = 0;
 
 	if (!agent_arg_ptr->hostlist) {
 		error("%s: hostlist is NULL", __func__);
 		return;
 	}
-	if (!slurmctld_conf.reboot_program) {
+	if (!slurm_conf.reboot_program) {
 		error("%s: RebootProgram is NULL", __func__);
 		return;
 	}
 
-	pname = strrchr(slurmctld_conf.reboot_program, '/');
+	pname = strrchr(slurm_conf.reboot_program, '/');
 	if (pname)
 		argv[0] = pname + 1;
 	else
-		argv[0] = slurmctld_conf.reboot_program;
+		argv[0] = slurm_conf.reboot_program;
 	argv[1] = hostlist_deranged_string_xmalloc(agent_arg_ptr->hostlist);
 	argv[2] = NULL;
 
 	child = fork();
 	if (child == 0) {
-		for (i = 0; i < 1024; i++)
-			(void) close(i);
+		closeall(0);
 		(void) setpgid(0, 0);
-		(void) execv(slurmctld_conf.reboot_program, argv);
+		(void) execv(slurm_conf.reboot_program, argv);
 		_exit(1);
 	} else if (child < 0) {
 		error("fork: %m");
