@@ -60,7 +60,7 @@ typedef struct slurm_bb_ops {
 	uint64_t	(*get_system_size)	(void);
 	int		(*load_state)	(bool init_config);
 	char *		(*get_status)	(uint32_t argc, char **argv);
-	int		(*state_pack)	(uid_t uid, Buf buffer,
+	int		(*state_pack)	(uid_t uid, buf_t *buffer,
 					 uint16_t protocol_version);
 	int		(*reconfig)	(void);
 	int		(*job_validate)	(job_desc_msg_t *job_desc,
@@ -79,6 +79,9 @@ typedef struct slurm_bb_ops {
 	int		(*job_test_post_run) (job_record_t *job_ptr);
 	int		(*job_test_stage_out) (job_record_t *job_ptr);
 	int		(*job_cancel) (job_record_t *job_ptr);
+	int		(*run_script) (char *func, uint32_t job_id,
+				       uint32_t argc, char **argv,
+				       char **resp_msg);
 	char *		(*xlate_bb_2_tres_str) (char *burst_buffer);
 } slurm_bb_ops_t;
 
@@ -103,6 +106,7 @@ static const char *syms[] = {
 	"bb_p_job_test_post_run",
 	"bb_p_job_test_stage_out",
 	"bb_p_job_cancel",
+	"bb_p_run_script",
 	"bb_p_xlate_bb_2_tres_str"
 };
 
@@ -132,7 +136,7 @@ extern int bb_g_init(void)
 	if (g_context_cnt >= 0)
 		goto fini;
 
-	bb_plugin_list = slurm_get_bb_type();
+	bb_plugin_list = xstrdup(slurm_conf.bb_type);
 	g_context_cnt = 0;
 	if ((bb_plugin_list == NULL) || (bb_plugin_list[0] == '\0'))
 		goto fini;
@@ -270,7 +274,7 @@ extern char *bb_g_get_status(uint32_t argc, char **argv)
  *
  * Returns a Slurm errno.
  */
-extern int bb_g_state_pack(uid_t uid, Buf buffer, uint16_t protocol_version)
+extern int bb_g_state_pack(uid_t uid, buf_t *buffer, uint16_t protocol_version)
 {
 	DEF_TIMERS;
 	int i, rc, rc2;
@@ -776,6 +780,26 @@ extern int bb_g_job_cancel(job_record_t *job_ptr)
 	return rc;
 }
 
+extern int bb_g_run_script(char *func, uint32_t job_id, uint32_t argc,
+			   char **argv, char **resp_msg)
+{
+	int i, rc, rc2;
+
+	rc = bb_g_init();
+	slurm_mutex_lock(&g_context_lock);
+	for (i = 0; i < g_context_cnt; i++) {
+		rc2 = (*(ops[i].run_script))(func, job_id, argc, argv,
+					     resp_msg);
+		if (rc2 != SLURM_SUCCESS) {
+			rc = rc2;
+			break;
+		}
+	}
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
+}
+
 /*
  * Translate a burst buffer string to it's equivalent TRES string
  * (e.g. "cray:2G,generic:4M" -> "1004=2048,1005=4")
