@@ -73,7 +73,7 @@ time_t last_front_end_update = (time_t) 0;
  * IN/OUT buffer - location to store data, pointers automatically advanced
  */
 static void _dump_front_end_state(front_end_record_t *front_end_ptr,
-				  Buf buffer)
+				  buf_t *buffer)
 {
 	packstr  (front_end_ptr->name, buffer);
 	pack32   (front_end_ptr->node_state, buffer);
@@ -89,11 +89,11 @@ static void _dump_front_end_state(front_end_record_t *front_end_ptr,
  * state_file IN - the name of the state save file used
  * RET the file description to read from or error code
  */
-static Buf _open_front_end_state_file(char **state_file)
+static buf_t *_open_front_end_state_file(char **state_file)
 {
-	Buf buf;
+	buf_t *buf;
 
-	*state_file = xstrdup(slurmctld_conf.state_save_location);
+	*state_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(*state_file, "/front_end_state");
 
 	if (!(buf = create_mmap_buf(*state_file)))
@@ -119,7 +119,7 @@ static Buf _open_front_end_state_file(char **state_file)
  *	changes to load_front_end_config in api/node_info.c
  */
 static void _pack_front_end(front_end_record_t *dump_front_end_ptr,
-			    Buf buffer, uint16_t protocol_version)
+			    buf_t *buffer, uint16_t protocol_version)
 {
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(dump_front_end_ptr->allow_groups, buffer);
@@ -493,7 +493,7 @@ extern void restore_front_end_state(int recover)
 #ifdef HAVE_FRONT_END
 	slurm_conf_frontend_t *slurm_conf_fe_ptr;
 	ListIterator iter;
-	uint32_t state_base, state_flags, tree_width;
+	uint32_t state_base, state_flags;
 	int i;
 
 	last_front_end_update = time(NULL);
@@ -582,7 +582,7 @@ extern void restore_front_end_state(int recover)
 		if (slurm_conf_fe_ptr->port)
 			front_end_nodes[i].port = slurm_conf_fe_ptr->port;
 		else
-			front_end_nodes[i].port = slurmctld_conf.slurmd_port;
+			front_end_nodes[i].port = slurm_conf.slurmd_port;
 		slurm_set_addr(&front_end_nodes[i].slurm_addr,
 			       front_end_nodes[i].port,
 			       front_end_nodes[i].comm_name);
@@ -590,12 +590,11 @@ extern void restore_front_end_state(int recover)
 	list_iterator_destroy(iter);
 	if (front_end_node_cnt == 0)
 		fatal("No front end nodes defined");
-	tree_width = slurm_get_tree_width();
-	if (front_end_node_cnt > tree_width) {
+	if (front_end_node_cnt > slurm_conf.tree_width) {
 		fatal("front_end_node_cnt > tree_width (%u > %u)",
-		      front_end_node_cnt, tree_width);
+		      front_end_node_cnt, slurm_conf.tree_width);
 	}
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_FRONT_END)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_FRONT_END)
 		log_front_end_state();
 #endif
 }
@@ -614,7 +613,7 @@ extern void pack_all_front_end(char **buffer_ptr, int *buffer_size, uid_t uid,
 {
 	time_t now = time(NULL);
 	uint32_t nodes_packed = 0;
-	Buf buffer;
+	buf_t *buffer;
 #ifdef HAVE_FRONT_END
 	uint32_t tmp_offset;
 	front_end_record_t *front_end_ptr;
@@ -674,7 +673,7 @@ extern int dump_all_front_end_state(void)
 	/* Locks: Read config and node */
 	slurmctld_lock_t node_read_lock = { READ_LOCK, NO_LOCK, READ_LOCK,
 					    NO_LOCK, NO_LOCK };
-	Buf buffer = init_buf(high_buffer_size);
+	buf_t *buffer = init_buf(high_buffer_size);
 	DEF_TIMERS;
 
 	START_TIMER;
@@ -692,11 +691,11 @@ extern int dump_all_front_end_state(void)
 		_dump_front_end_state(front_end_ptr, buffer);
 	}
 
-	old_file = xstrdup (slurmctld_conf.state_save_location);
+	old_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat (old_file, "/front_end_state.old");
-	reg_file = xstrdup (slurmctld_conf.state_save_location);
+	reg_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat (reg_file, "/front_end_state");
-	new_file = xstrdup (slurmctld_conf.state_save_location);
+	new_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat (new_file, "/front_end_state.new");
 	unlock_slurmctld (node_read_lock);
 
@@ -771,7 +770,7 @@ extern int load_all_front_end_state(bool state_only)
 	time_t reason_time = 0;
 	front_end_record_t *front_end_ptr;
 	time_t time_stamp;
-	Buf buffer;
+	buf_t *buffer;
 	char *ver_str = NULL;
 	uint16_t protocol_version = NO_VAL16;
 
@@ -923,7 +922,7 @@ extern void set_front_end_down (front_end_record_t *front_end_ptr,
 		xfree(front_end_ptr->reason);
 		front_end_ptr->reason = xstrdup(reason);
 		front_end_ptr->reason_time = now;
-		front_end_ptr->reason_uid = slurmctld_conf.slurm_user_id;
+		front_end_ptr->reason_uid = slurm_conf.slurm_user_id;
 	}
 	last_front_end_update = now;
 #endif
@@ -1000,7 +999,7 @@ extern void sync_front_end_state(void)
 		}
 	}
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_FRONT_END)
+	if (slurm_conf.debug_flags & DEBUG_FLAG_FRONT_END)
 		log_front_end_state();
 #endif
 }
