@@ -74,6 +74,7 @@ extern void prep_prolog_slurmctld_callback(int rc, uint32_t job_id)
 
 	/* all async prologs have completed, continue on now */
 	if (job_ptr->prep_prolog_failed) {
+		job_ptr->prep_prolog_failed = false;
 		if ((rc = job_requeue(0, job_id, NULL, false, 0))) {
 			info("unable to requeue JobId=%u: %s", job_id,
 			     slurm_strerror(rc));
@@ -93,24 +94,13 @@ extern void prep_prolog_slurmctld_callback(int rc, uint32_t job_id)
 
 	prolog_running_decr(job_ptr);
 
-	if (power_save_test()) {
-		/* Wait for node to register after booting */
-	} else if (job_ptr->node_bitmap) {
-		for (int i=0; i < node_record_count; i++) {
-			if (bit_test(job_ptr->node_bitmap, i) == 0)
-				continue;
-			bit_clear(booting_node_bitmap, i);
-			node_record_table_ptr[i].node_state &=
-				(~NODE_STATE_POWER_UP);
-		}
-	}
-
 	unlock_slurmctld(job_write_lock);
 }
 
 extern void prep_epilog_slurmctld_callback(int rc, uint32_t job_id)
 {
-	slurmctld_lock_t job_write_lock = { .job = WRITE_LOCK };
+	slurmctld_lock_t job_write_lock = {
+		.job = WRITE_LOCK, .node = WRITE_LOCK};
 	job_record_t *job_ptr;
 
 	lock_slurmctld(job_write_lock);
@@ -134,6 +124,31 @@ extern void prep_epilog_slurmctld_callback(int rc, uint32_t job_id)
 
 	/* all async prologs have completed, continue on now */
 	job_ptr->epilog_running = false;
+
+	if (job_ptr->bit_flags & NOT_LAUNCHED) {
+		/*
+		 * Job was configuring when it was cancelled and epilog wasn't
+		 * run on the nodes, so cleanup the nodes after EpilogSlurmctld
+		 * is done.
+		 */
+		job_ptr->bit_flags &= ~NOT_LAUNCHED;
+		if (job_ptr->node_bitmap_cg) {
+			int i_first, i_last;
+			i_first = bit_ffs(job_ptr->node_bitmap_cg);
+			if (i_first >= 0)
+				i_last = bit_fls(job_ptr->node_bitmap_cg);
+			else
+				i_last = i_first - 1;
+			for (int i = i_first; i <= i_last; i++) {
+				if (!bit_test(job_ptr->node_bitmap_cg, i))
+					continue;
+				job_epilog_complete(
+					job_ptr->job_id,
+					node_record_table_ptr[i].name, 0);
+			}
+		}
+	}
+
 	/*
 	 * Clear the JOB_COMPLETING flag only if the node count is 0
 	 * meaning the slurmd epilogs have already completed.
