@@ -138,7 +138,7 @@ extern void srun_allocate(job_record_t *job_ptr)
 		return;
 
 	if (job_ptr->het_job_id == 0) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->alloc_resp_port,
 			job_ptr->resp_host);
 
@@ -149,7 +149,7 @@ extern void srun_allocate(job_record_t *job_ptr)
 	} else if (_pending_het_jobs(job_ptr)) {
 		return;
 	} else if ((het_job_leader = find_job_record(job_ptr->het_job_id))) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, het_job_leader->alloc_resp_port,
 			       het_job_leader->resp_host);
 		job_resp_list = list_create(_free_srun_alloc);
@@ -186,12 +186,13 @@ extern void srun_allocate_abort(job_record_t *job_ptr)
 	    job_ptr->resp_host) {
 		slurm_addr_t * addr;
 		srun_job_complete_msg_t *msg_arg;
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->alloc_resp_port,
 			       job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(srun_timeout_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = NO_VAL;
+		msg_arg->job_id = job_ptr->job_id;
+		msg_arg->step_id = NO_VAL;
+		msg_arg->step_het_comp = NO_VAL;
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   SRUN_JOB_COMPLETE,
 				   msg_arg,
@@ -199,6 +200,39 @@ extern void srun_allocate_abort(job_record_t *job_ptr)
 	}
 }
 
+typedef struct {
+	int bit_position;
+	char *node_name;
+} srun_node_fail_args_t;
+
+static int _srun_node_fail(void *x, void *arg)
+{
+	step_record_t *step_ptr = (step_record_t *) x;
+	srun_node_fail_args_t *args = (srun_node_fail_args_t *) arg;
+	slurm_addr_t *addr;
+	srun_node_fail_msg_t *msg_arg;
+
+	if (!step_ptr->step_node_bitmap)   /* pending step */
+		return 0;
+	if (step_ptr->step_id.step_id == SLURM_BATCH_SCRIPT)
+		return 0;
+	if ((args->bit_position >= 0) &&
+	    (!bit_test(step_ptr->step_node_bitmap, args->bit_position)))
+		return 0;	/* job step not on this node */
+	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
+		return 0;
+
+	addr = xmalloc(sizeof(*addr));
+	slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+	msg_arg = xmalloc(sizeof(*msg_arg));
+	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
+	msg_arg->nodelist = xstrdup(args->node_name);
+	_srun_agent_launch(addr, step_ptr->host, SRUN_NODE_FAIL,
+			   msg_arg, step_ptr->start_protocol_ver);
+
+	return 0;
+}
+
 /*
  * srun_node_fail - notify srun of a node's failure
  * IN job_ptr - job to notify
@@ -209,11 +243,12 @@ extern void srun_node_fail(job_record_t *job_ptr, char *node_name)
 #ifndef HAVE_FRONT_END
 	node_record_t *node_ptr;
 #endif
-	int bit_position = -1;
+	srun_node_fail_args_t args = {
+		.bit_position = -1,
+		.node_name = node_name,
+	};
 	slurm_addr_t * addr;
 	srun_node_fail_msg_t *msg_arg;
-	ListIterator step_iterator;
-	step_record_t *step_ptr;
 
 	xassert(job_ptr);
 	xassert(node_name);
@@ -225,106 +260,91 @@ extern void srun_node_fail(job_record_t *job_ptr, char *node_name)
 #else
 	if (!node_name || (node_ptr = find_node_record(node_name)) == NULL)
 		return;
-	bit_position = node_ptr - node_record_table_ptr;
+	args.bit_position = node_ptr - node_record_table_ptr;
 #endif
 
-	step_iterator = list_iterator_create(job_ptr->step_list);
-	while ((step_ptr = list_next(step_iterator))) {
-		if (step_ptr->step_node_bitmap == NULL)   /* pending step */
-			continue;
-		if ((bit_position >= 0) &&
-		    (!bit_test(step_ptr->step_node_bitmap, bit_position)))
-			continue;	/* job step not on this node */
-		if ( (step_ptr->port    == 0)    ||
-		     (step_ptr->host    == NULL) ||
-		     (step_ptr->batch_step)      ||
-		     (step_ptr->host[0] == '\0') )
-			continue;
-		addr = xmalloc(sizeof(struct sockaddr_in));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
-		msg_arg = xmalloc(sizeof(srun_node_fail_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = step_ptr->step_id;
-		msg_arg->nodelist = xstrdup(node_name);
-		_srun_agent_launch(addr, step_ptr->host, SRUN_NODE_FAIL,
-				   msg_arg, step_ptr->start_protocol_ver);
-	}
-	list_iterator_destroy(step_iterator);
+	list_for_each(job_ptr->step_list, _srun_node_fail, &args);
 
 	if (job_ptr->other_port && job_ptr->alloc_node && job_ptr->resp_host) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(srun_node_fail_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = NO_VAL;
+		msg_arg->step_id.job_id = job_ptr->job_id;
+		msg_arg->step_id.step_id  = NO_VAL;
+		msg_arg->step_id.step_het_comp = NO_VAL;
 		msg_arg->nodelist = xstrdup(node_name);
 		_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_NODE_FAIL,
 				   msg_arg, job_ptr->start_protocol_ver);
 	}
 }
 
-/* srun_ping - ping all srun commands that have not been heard from recently */
-extern void srun_ping (void)
+static int _srun_ping(void *x, void *arg)
 {
-	ListIterator job_iterator;
-	job_record_t *job_ptr;
-	slurm_addr_t * addr;
-	time_t now = time(NULL);
-	time_t old = now - (slurmctld_conf.inactive_limit / 3) +
-			   slurmctld_conf.msg_timeout + 1;
+	job_record_t *job_ptr = (job_record_t *) x;
+	time_t *old = (time_t *) arg;
+	slurm_addr_t *addr;
 	srun_ping_msg_t *msg_arg;
 
-	if (slurmctld_conf.inactive_limit == 0)
-		return;		/* No limit, don't bother pinging */
+	xassert(job_ptr->magic == JOB_MAGIC);
 
-	job_iterator = list_iterator_create(job_list);
-	while ((job_ptr = list_next(job_iterator))) {
-		xassert (job_ptr->magic == JOB_MAGIC);
+	if (!IS_JOB_RUNNING(job_ptr) || (job_ptr->time_last_active > *old))
+		return 0;
 
-		if (!IS_JOB_RUNNING(job_ptr))
-			continue;
+	if (!job_ptr->other_port || !job_ptr->alloc_node || !job_ptr->resp_host)
+		return 0;
 
-		if ((job_ptr->time_last_active <= old) && job_ptr->other_port
-		    &&  job_ptr->alloc_node && job_ptr->resp_host) {
-			addr = xmalloc(sizeof(struct sockaddr_in));
-			slurm_set_addr(addr, job_ptr->other_port,
-				job_ptr->resp_host);
-			msg_arg = xmalloc(sizeof(srun_ping_msg_t));
-			msg_arg->job_id  = job_ptr->job_id;
-			msg_arg->step_id = NO_VAL;
-			_srun_agent_launch(addr, job_ptr->alloc_node,
-					   SRUN_PING, msg_arg,
-					   job_ptr->start_protocol_ver);
-		}
-	}
+	addr = xmalloc(sizeof(*addr));
+	msg_arg = xmalloc(sizeof(*msg_arg));
 
-	list_iterator_destroy(job_iterator);
+	slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
+	msg_arg->job_id = job_ptr->job_id;
+
+	_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_PING, msg_arg,
+			   job_ptr->start_protocol_ver);
+
+	return 0;
 }
 
 /*
- * srun_step_timeout - notify srun of a job step's imminent timeout
- * IN step_ptr - pointer to the slurmctld step record
- * IN timeout_val - when it is going to time out
+ * srun_ping - Ping all allocations srun/salloc that have not been heard from
+ * recently. This does not ping sruns inside a allocation from sbatch or salloc.
  */
-extern void srun_step_timeout(step_record_t *step_ptr, time_t timeout_val)
+extern void srun_ping (void)
 {
+	time_t old = time(NULL) - (slurm_conf.inactive_limit / 3) +
+		     slurm_conf.msg_timeout + 1;
+
+	if (slurm_conf.inactive_limit == 0)
+		return;		/* No limit, don't bother pinging */
+
+	list_for_each(job_list, _srun_ping, &old);
+}
+
+static int _srun_step_timeout(void *x, void *arg)
+{
+	step_record_t *step_ptr = (step_record_t *) x;
 	slurm_addr_t *addr;
 	srun_timeout_msg_t *msg_arg;
 
 	xassert(step_ptr);
 
-	if (step_ptr->batch_step || !step_ptr->port
-	    || !step_ptr->host || (step_ptr->host[0] == '\0'))
-		return;
+	if (step_ptr->step_id.step_id == SLURM_BATCH_SCRIPT)
+		return 0;
+
+	if (!step_ptr->port || !step_ptr->host || (step_ptr->host[0] == '\0'))
+		return 0;
+
+	addr = xmalloc(sizeof(*addr));
+	msg_arg = xmalloc(sizeof(*msg_arg));
 
-	addr = xmalloc(sizeof(struct sockaddr_in));
 	slurm_set_addr(addr, step_ptr->port, step_ptr->host);
-	msg_arg = xmalloc(sizeof(srun_timeout_msg_t));
-	msg_arg->job_id   = step_ptr->job_ptr->job_id;
-	msg_arg->step_id  = step_ptr->step_id;
-	msg_arg->timeout  = timeout_val;
+	memcpy(&msg_arg->step_id, &step_ptr->step_id, sizeof(msg_arg->step_id));
+	msg_arg->timeout = step_ptr->job_ptr->end_time;
+
 	_srun_agent_launch(addr, step_ptr->host, SRUN_TIMEOUT, msg_arg,
 			   step_ptr->start_protocol_ver);
+
+	return 0;
 }
 
 /*
@@ -335,29 +355,24 @@ extern void srun_timeout(job_record_t *job_ptr)
 {
 	slurm_addr_t * addr;
 	srun_timeout_msg_t *msg_arg;
-	ListIterator step_iterator;
-	step_record_t *step_ptr;
 
 	xassert(job_ptr);
 	if (!IS_JOB_RUNNING(job_ptr))
 		return;
 
 	if (job_ptr->other_port && job_ptr->alloc_node && job_ptr->resp_host) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(srun_timeout_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = NO_VAL;
+		msg_arg->step_id.job_id   = job_ptr->job_id;
+		msg_arg->step_id.step_id  = NO_VAL;
+		msg_arg->step_id.step_het_comp = NO_VAL;
 		msg_arg->timeout  = job_ptr->end_time;
 		_srun_agent_launch(addr, job_ptr->alloc_node, SRUN_TIMEOUT,
 				   msg_arg, job_ptr->start_protocol_ver);
 	}
 
-
-	step_iterator = list_iterator_create(job_ptr->step_list);
-	while ((step_ptr = list_next(step_iterator)))
-		srun_step_timeout(step_ptr, job_ptr->end_time);
-	list_iterator_destroy(step_iterator);
+	list_for_each(job_ptr->step_list, _srun_step_timeout, NULL);
 }
 
 /*
@@ -374,7 +389,7 @@ extern int srun_user_message(job_record_t *job_ptr, char *msg)
 
 	if (job_ptr->other_port &&
 	    job_ptr->resp_host && job_ptr->resp_host[0]) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(srun_user_msg_t));
 		msg_arg->job_id = job_ptr->job_id;
@@ -412,7 +427,9 @@ extern int srun_user_message(job_record_t *job_ptr, char *msg)
 #endif
 		notify_msg_ptr = (job_notify_msg_t *)
 				 xmalloc(sizeof(job_notify_msg_t));
-		notify_msg_ptr->job_id = job_ptr->job_id;
+		notify_msg_ptr->step_id.job_id = job_ptr->job_id;
+		notify_msg_ptr->step_id.step_id = NO_VAL;
+		notify_msg_ptr->step_id.step_het_comp = NO_VAL;
 		notify_msg_ptr->message = xstrdup(msg);
 		agent_arg_ptr->node_count = 1;
 		agent_arg_ptr->retry = 0;
@@ -425,6 +442,16 @@ extern int srun_user_message(job_record_t *job_ptr, char *msg)
 	return ESLURM_DISABLED;
 }
 
+static int _srun_job_complete(void *x, void *arg)
+{
+	step_record_t *step_ptr = (step_record_t *) x;
+
+	if (step_ptr->step_id.step_id != SLURM_BATCH_SCRIPT)
+		srun_step_complete(step_ptr);
+
+	return 0;
+}
+
 /*
  * srun_job_complete - notify srun of a job's termination
  * IN job_ptr - pointer to the slurmctld job record
@@ -433,29 +460,22 @@ extern void srun_job_complete(job_record_t *job_ptr)
 {
 	slurm_addr_t * addr;
 	srun_job_complete_msg_t *msg_arg;
-	ListIterator step_iterator;
-	step_record_t *step_ptr;
 
 	xassert(job_ptr);
 
 	if (job_ptr->other_port && job_ptr->alloc_node && job_ptr->resp_host) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(srun_job_complete_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = NO_VAL;
+		msg_arg->job_id = job_ptr->job_id;
+		msg_arg->step_id = NO_VAL;
+		msg_arg->step_het_comp = NO_VAL;
 		_srun_agent_launch(addr, job_ptr->alloc_node,
 				   SRUN_JOB_COMPLETE, msg_arg,
 				   job_ptr->start_protocol_ver);
 	}
 
-	step_iterator = list_iterator_create(job_ptr->step_list);
-	while ((step_ptr = list_next(step_iterator))) {
-		if (step_ptr->batch_step)	/* batch script itself */
-			continue;
-		srun_step_complete(step_ptr);
-	}
-	list_iterator_destroy(step_iterator);
+	list_for_each(job_ptr->step_list, _srun_job_complete, NULL);
 }
 
 /*
@@ -473,7 +493,7 @@ extern bool srun_job_suspend(job_record_t *job_ptr, uint16_t op)
 	xassert(job_ptr);
 
 	if (job_ptr->other_port && job_ptr->alloc_node && job_ptr->resp_host) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, job_ptr->other_port, job_ptr->resp_host);
 		msg_arg = xmalloc(sizeof(suspend_msg_t));
 		msg_arg->job_id  = job_ptr->job_id;
@@ -497,11 +517,11 @@ extern void srun_step_complete(step_record_t *step_ptr)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
 		msg_arg = xmalloc(sizeof(srun_job_complete_msg_t));
-		msg_arg->job_id   = step_ptr->job_ptr->job_id;
-		msg_arg->step_id  = step_ptr->step_id;
+		memcpy(&msg_arg->step_id, &step_ptr->step_id,
+		       sizeof(msg_arg->step_id));
 		_srun_agent_launch(addr, step_ptr->host, SRUN_JOB_COMPLETE,
 				   msg_arg, step_ptr->start_protocol_ver);
 	}
@@ -520,11 +540,11 @@ extern void srun_step_missing(step_record_t *step_ptr, char *node_list)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
 		msg_arg = xmalloc(sizeof(srun_step_missing_msg_t));
-		msg_arg->job_id   = step_ptr->job_ptr->job_id;
-		msg_arg->step_id  = step_ptr->step_id;
+		memcpy(&msg_arg->step_id, &step_ptr->step_id,
+		       sizeof(msg_arg->step_id));
 		msg_arg->nodelist = xstrdup(node_list);
 		_srun_agent_launch(addr, step_ptr->host, SRUN_STEP_MISSING,
 				   msg_arg, step_ptr->start_protocol_ver);
@@ -544,11 +564,11 @@ extern void srun_step_signal(step_record_t *step_ptr, uint16_t signal)
 
 	xassert(step_ptr);
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
 		msg_arg = xmalloc(sizeof(job_step_kill_msg_t));
-		msg_arg->job_id      = step_ptr->job_ptr->job_id;
-		msg_arg->job_step_id = step_ptr->step_id;
+		memcpy(&msg_arg->step_id, &step_ptr->step_id,
+		       sizeof(msg_arg->step_id));
 		msg_arg->signal      = signal;
 		_srun_agent_launch(addr, step_ptr->host, SRUN_STEP_SIGNAL,
 				   msg_arg, step_ptr->start_protocol_ver);
@@ -572,11 +592,11 @@ extern void srun_exec(step_record_t *step_ptr, char **argv)
 	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
 		for (i=0; argv[i]; i++)
 			cnt++;	/* start at 1 to include trailing NULL */
-		addr = xmalloc(sizeof(struct sockaddr_in));
+		addr = xmalloc(sizeof(slurm_addr_t));
 		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
 		msg_arg = xmalloc(sizeof(srun_exec_msg_t));
-		msg_arg->job_id  = step_ptr->job_ptr->job_id;
-		msg_arg->step_id = step_ptr->step_id;
+		memcpy(&msg_arg->step_id, &step_ptr->step_id,
+		       sizeof(msg_arg->step_id));
 		msg_arg->argc    = cnt;
 		msg_arg->argv    = xmalloc(sizeof(char *) * cnt);
 		for (i=0; i<cnt ; i++)
@@ -591,15 +611,22 @@ extern void srun_exec(step_record_t *step_ptr, char **argv)
 
 /*
  * srun_response - note that srun has responded
- * IN job_id  - id of job responding
  * IN step_id - id of step responding or NO_VAL if not a step
  */
-extern void srun_response(uint32_t job_id, uint32_t step_id)
+extern void srun_response(slurm_step_id_t *step_id)
 {
-	job_record_t *job_ptr = find_job_record(job_id);
+	job_record_t *job_ptr = find_job_record(step_id->job_id);
+	step_record_t *step_ptr;
 	time_t now = time(NULL);
 
 	if (job_ptr == NULL)
 		return;
 	job_ptr->time_last_active = now;
+
+	if (step_id->step_id == NO_VAL)
+		return;
+
+	if ((step_ptr = find_step_record(job_ptr, step_id)))
+		step_ptr->time_last_active = now;
+
 }
