@@ -106,7 +106,6 @@ typedef struct trig_mgr_info {
 	char *   res_id;	/* node name or job_id (string) */
 	bitstr_t *nodes_bitmap;	/* bitmap of requested nodes (if applicable) */
 	uint32_t job_id;	/* job ID (if applicable) */
-	job_record_t *job_ptr;	/* pointer to job record (if applicable) */
 	uint32_t trig_type;	/* TRIGGER_TYPE_* */
 	time_t   trig_time;	/* offset (pending) or time stamp (complete) */
 	uint32_t user_id;	/* user requesting trigger */
@@ -143,7 +142,7 @@ static void _dump_trigger_msg(char *header, trigger_info_msg_t *msg)
 {
 	int i;
 
-	if ((slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) == 0)
+	if (!(slurm_conf.debug_flags & DEBUG_FLAG_TRIGGERS))
 		return;
 
 	info("%s", header);
@@ -406,8 +405,8 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 	lock_slurmctld(job_read_lock);
 	slurm_mutex_lock(&trigger_mutex);
 
-	if ((slurmctld_conf.slurm_user_id != 0) &&
-	    (slurmctld_conf.slurm_user_id != uid)) {
+	if ((slurm_conf.slurm_user_id != 0) &&
+	    (slurm_conf.slurm_user_id != uid)) {
 		/* If SlurmUser is not root, then it is unable to set the
 		 * appropriate user id and group id for the program to be
 		 * launched. To prevent the launched program for an arbitrary
@@ -421,7 +420,7 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 	if (trigger_list == NULL) {
 		trigger_list = list_create(_trig_del);
 	} else if ((uid != 0) &&
-		   (list_count(trigger_list) >= slurmctld_conf.max_job_cnt)) {
+	           (list_count(trigger_list) >= slurm_conf.max_job_cnt)) {
 		rc = EAGAIN;
 		goto fini;
 	}
@@ -471,7 +470,6 @@ extern int trigger_set(uid_t uid, gid_t gid, trigger_info_msg_t *msg)
 			bitmap = NULL;
 		}
 		trig_add->job_id = job_id;
-		trig_add->job_ptr = job_ptr;
 		if (msg->trigger_array[i].res_id) {
 			trig_add->res_id = msg->trigger_array[i].res_id;
 			trig_add->orig_res_id = xstrdup(trig_add->res_id);
@@ -710,7 +708,7 @@ extern void trigger_burst_buffer(void)
 	slurm_mutex_unlock(&trigger_mutex);
 }
 
-static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, Buf buffer)
+static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, buf_t *buffer)
 {
 	/* write trigger pull state flags */
 	pack8(ctld_failure,    buffer);
@@ -724,7 +722,6 @@ static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, Buf buffer)
 	packstr  (trig_ptr->orig_res_id, buffer);  /* restores res_id too */
 	/* rebuild nodes_bitmap as needed from res_id */
 	/* rebuild job_id as needed from res_id */
-	/* rebuild job_ptr as needed from res_id */
 	pack32   (trig_ptr->trig_type, buffer);
 	pack_time(trig_ptr->orig_time, buffer);    /* restores trig_time too */
 	pack32   (trig_ptr->user_id,   buffer);
@@ -733,7 +730,7 @@ static void _dump_trigger_state(trig_mgr_info_t *trig_ptr, Buf buffer)
 	pack8    (trig_ptr->state,     buffer);
 }
 
-static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
+static int _load_trigger_state(buf_t *buffer, uint16_t protocol_version)
 {
 	trig_mgr_info_t *trig_ptr;
 	uint32_t str_len;
@@ -755,7 +752,6 @@ static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 		safe_unpackstr_xmalloc(&trig_ptr->res_id, &str_len, buffer);
 		/* rebuild nodes_bitmap as needed from res_id */
 		/* rebuild job_id as needed from res_id */
-		/* rebuild job_ptr as needed from res_id */
 		safe_unpack32   (&trig_ptr->trig_type, buffer);
 		safe_unpack_time(&trig_ptr->trig_time, buffer);
 		safe_unpack32   (&trig_ptr->user_id,   buffer);
@@ -773,16 +769,14 @@ static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 	    (trig_ptr->state > 2))
 		goto unpack_error;
 	if (trig_ptr->res_type == TRIGGER_RES_TYPE_JOB) {
+		job_record_t *job_ptr;
 		trig_ptr->job_id = (uint32_t) atol(trig_ptr->res_id);
-		trig_ptr->job_ptr = find_job_record(trig_ptr->job_id);
-		if ((trig_ptr->job_id == 0)     ||
-		    (trig_ptr->job_ptr == NULL) ||
-		    (IS_JOB_COMPLETED(trig_ptr->job_ptr) &&
-		     trig_ptr->state != 2))
+		job_ptr = find_job_record(trig_ptr->job_id);
+		if (((trig_ptr->job_id == 0) || (job_ptr == NULL)) &&
+		    (trig_ptr->state != 2))
 			goto unpack_error;
 	} else if (trig_ptr->res_type == TRIGGER_RES_TYPE_NODE) {
 		trig_ptr->job_id = 0;
-		trig_ptr->job_ptr = NULL;
 		if ((trig_ptr->res_id != NULL)   &&
 		    (trig_ptr->res_id[0] != '*') &&
 		    (node_name2bitmap(trig_ptr->res_id, false,
@@ -819,7 +813,7 @@ extern int trigger_state_save(void)
 	static int high_buffer_size = (1024 * 1024);
 	int error_code = 0, log_fd;
 	char *old_file, *new_file, *reg_file;
-	Buf buffer = init_buf(high_buffer_size);
+	buf_t *buffer = init_buf(high_buffer_size);
 	ListIterator trig_iter;
 	trig_mgr_info_t *trig_in;
 	/* Locks: Read config */
@@ -844,11 +838,11 @@ extern int trigger_state_save(void)
 
 	/* write the buffer to file */
 	lock_slurmctld(config_read_lock);
-	old_file = xstrdup(slurmctld_conf.state_save_location);
+	old_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(old_file, "/trigger_state.old");
-	reg_file = xstrdup(slurmctld_conf.state_save_location);
+	reg_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(reg_file, "/trigger_state");
-	new_file = xstrdup(slurmctld_conf.state_save_location);
+	new_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(new_file, "/trigger_state.new");
 	unlock_slurmctld(config_read_lock);
 
@@ -904,11 +898,11 @@ extern int trigger_state_save(void)
  * state_file IN - the name of the state save file used
  * RET the file description to read from or error code
  */
-static Buf _open_trigger_state_file(char **state_file)
+static buf_t *_open_trigger_state_file(char **state_file)
 {
-	Buf buf;
+	buf_t *buf;
 
-	*state_file = xstrdup(slurmctld_conf.state_save_location);
+	*state_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(*state_file, "/trigger_state");
 	if (!(buf = create_mmap_buf(*state_file)))
 		error("Could not open trigger state file %s: %m",
@@ -926,7 +920,7 @@ extern void trigger_state_restore(void)
 	uint16_t protocol_version = NO_VAL16;
 	int trigger_cnt = 0;
 	char *state_file;
-	Buf buffer;
+	buf_t *buffer;
 	time_t buf_time;
 	char *ver_str = NULL;
 	uint32_t ver_str_len;
@@ -1003,53 +997,45 @@ static bool _front_end_job_test(bitstr_t *front_end_bitmap,
 /* Test if the event has been triggered, change trigger state as needed */
 static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 {
+	job_record_t *job_ptr;
 	xassert(verify_lock(JOB_LOCK, READ_LOCK));
 
-	trig_in->job_ptr = find_job_record(trig_in->job_id);
+	job_ptr = find_job_record(trig_in->job_id);
 
 	if ((trig_in->trig_type & TRIGGER_TYPE_FINI) &&
-	    ((trig_in->job_ptr == NULL) ||
-	     (IS_JOB_COMPLETED(trig_in->job_ptr)))) {
+	    ((job_ptr == NULL) || (IS_JOB_COMPLETED(job_ptr)))) {
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] event for job %u fini",
-			     trig_in->trig_id, trig_in->job_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] event for job %u fini",
+			 trig_in->trig_id, trig_in->job_id);
 		return;
 	}
 
-	if (trig_in->job_ptr == NULL) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for defunct job %u",
-			     trig_in->trig_id, trig_in->job_id);
-		}
+	if (job_ptr == NULL) {
+		log_flag(TRIGGERS, "trigger[%u] for defunct job %u",
+			 trig_in->trig_id, trig_in->job_id);
 		trig_in->state = 2;
 		trig_in->trig_time = now;
 		return;
 	}
 
-	if (!IS_JOB_PENDING(trig_in->job_ptr) &&
+	if (!IS_JOB_PENDING(job_ptr) &&
 	    (trig_in->trig_type & TRIGGER_TYPE_TIME)) {
-		long rem_time = (trig_in->job_ptr->end_time - now);
+		long rem_time = (job_ptr->end_time - now);
 		if (rem_time <= (0x8000 - trig_in->trig_time)) {
 			trig_in->state = 1;
 			trig_in->trig_time = now;
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for job %u time",
-				     trig_in->trig_id, trig_in->job_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for job %u time",
+				 trig_in->trig_id, trig_in->job_id);
 			return;
 		}
 	}
 
 	if (trig_in->trig_type & TRIGGER_TYPE_DOWN) {
 		if (_front_end_job_test(trigger_down_front_end_bitmap,
-					trig_in->job_ptr)) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for job %u down",
-				     trig_in->trig_id, trig_in->job_id);
-			}
+					job_ptr)) {
+			log_flag(TRIGGERS, "trigger[%u] for job %u down",
+				 trig_in->trig_id, trig_in->job_id);
 			trig_in->state = 1;
 			trig_in->trig_time = now +
 					    (trig_in->trig_time - 0x8000);
@@ -1059,12 +1045,10 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if (trig_in->trig_type & TRIGGER_TYPE_DOWN) {
 		if (trigger_down_nodes_bitmap &&
-		    bit_overlap_any(trig_in->job_ptr->node_bitmap,
+		    bit_overlap_any(job_ptr->node_bitmap,
 				    trigger_down_nodes_bitmap)) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for job %u down",
-				     trig_in->trig_id, trig_in->job_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for job %u down",
+				 trig_in->trig_id, trig_in->job_id);
 			trig_in->state = 1;
 			trig_in->trig_time = now +
 					    (trig_in->trig_time - 0x8000);
@@ -1074,12 +1058,10 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if (trig_in->trig_type & TRIGGER_TYPE_FAIL) {
 		if (trigger_fail_nodes_bitmap &&
-		    bit_overlap_any(trig_in->job_ptr->node_bitmap,
+		    bit_overlap_any(job_ptr->node_bitmap,
 				    trigger_fail_nodes_bitmap)) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for job %u node fail",
-				     trig_in->trig_id, trig_in->job_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for job %u node fail",
+				 trig_in->trig_id, trig_in->job_id);
 			trig_in->state = 1;
 			trig_in->trig_time = now +
 					     (trig_in->trig_time - 0x8000);
@@ -1089,15 +1071,13 @@ static void _trigger_job_event(trig_mgr_info_t *trig_in, time_t now)
 
 	if (trig_in->trig_type & TRIGGER_TYPE_UP) {
 		if (trigger_up_nodes_bitmap &&
-		    bit_overlap_any(trig_in->job_ptr->node_bitmap,
+		    bit_overlap_any(job_ptr->node_bitmap,
 				    trigger_up_nodes_bitmap)) {
 			trig_in->state = 1;
 			trig_in->trig_time = now +
 					    (0x8000 - trig_in->trig_time);
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for job %u up",
-				     trig_in->trig_id, trig_in->job_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for job %u up",
+				 trig_in->trig_id, trig_in->job_id);
 			return;
 		}
 	}
@@ -1123,10 +1103,8 @@ static void _trigger_front_end_event(trig_mgr_info_t *trig_in, time_t now)
 		}
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for node %s down",
-			     trig_in->trig_id, trig_in->res_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for node %s down",
+			 trig_in->trig_id, trig_in->res_id);
 		return;
 	}
 
@@ -1143,10 +1121,8 @@ static void _trigger_front_end_event(trig_mgr_info_t *trig_in, time_t now)
 		}
 		trig_in->state = 1;
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for node %s up",
-			     trig_in->trig_id, trig_in->res_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for node %s up",
+			 trig_in->trig_id, trig_in->res_id);
 		return;
 	}
 }
@@ -1157,8 +1133,8 @@ static void _trigger_other_event(trig_mgr_info_t *trig_in, time_t now)
 	    trigger_bb_error) {
 		trig_in->state = 1;
 		trig_in->trig_time = now;
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS)
-			info("trigger[%u] for burst buffer", trig_in->trig_id);
+		log_flag(TRIGGERS, "trigger[%u] for burst buffer",
+			 trig_in->trig_id);
 		return;
 	}
 }
@@ -1187,10 +1163,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
 					     (trig_in->trig_time - 0x8000);
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for node %s down",
-				     trig_in->trig_id, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for node %s down",
+				 trig_in->trig_id, trig_in->res_id);
 			return;
 		}
 	}
@@ -1215,10 +1189,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
 					     (trig_in->trig_time - 0x8000);
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for node %s drained",
-				     trig_in->trig_id, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for node %s drained",
+				 trig_in->trig_id, trig_in->res_id);
 			return;
 		}
 	}
@@ -1243,10 +1215,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
 					     (trig_in->trig_time - 0x8000);
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for node %s fail",
-				     trig_in->trig_id, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for node %s fail",
+				 trig_in->trig_id, trig_in->res_id);
 			return;
 		}
 	}
@@ -1262,7 +1232,7 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		trigger_idle_node_bitmap = bit_alloc(node_record_count);
 		for (i = 0; i < node_record_count; i++, node_ptr++) {
 			if (!IS_NODE_IDLE(node_ptr) ||
-			    (node_ptr->last_idle > min_idle))
+			    (node_ptr->last_busy > min_idle))
 				continue;
 			bit_set(trigger_idle_node_bitmap, i);
 		}
@@ -1283,10 +1253,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		FREE_NULL_BITMAP(trigger_idle_node_bitmap);
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now;
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for node %s idle",
-				     trig_in->trig_id, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for node %s idle",
+				 trig_in->trig_id, trig_in->res_id);
 			return;
 		}
 	}
@@ -1311,10 +1279,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		if (trig_in->state == 1) {
 			trig_in->trig_time = now +
 					     (trig_in->trig_time - 0x8000);
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("trigger[%u] for node %s up",
-				     trig_in->trig_id, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "trigger[%u] for node %s up",
+				 trig_in->trig_id, trig_in->res_id);
 			return;
 		}
 	}
@@ -1325,8 +1291,8 @@ static void _trigger_node_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("reconfig");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS)
-			info("trigger[%u] for reconfig", trig_in->trig_id);
+		log_flag(TRIGGERS, "trigger[%u] for reconfig",
+			 trig_in->trig_id);
 		return;
 	}
 }
@@ -1339,10 +1305,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_slurmctld_failure");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_slurmctld_failure",
-			     trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmctld_failure",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_RES_OP) &&
@@ -1352,10 +1316,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		xfree(trig_in->res_id);
 		trig_in->res_id =
 			xstrdup("primary_slurmctld_resumed_operation");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_slurmctld_resumed_"
-			     "operation", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmctld_resumed_operation",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_RES_CTRL) &&
@@ -1364,10 +1326,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_slurmctld_resumed_control");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_slurmctld_resumed_"
-			     "control", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmctld_resumed_control",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_CTLD_ACCT_FULL) &&
@@ -1376,10 +1336,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_slurmctld_acct_buffer_full");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_slurmctld_acct_"
-			     "buffer_full", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmctld_acct_buffer_full",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_FAIL) &&
@@ -1388,10 +1346,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("backup_slurmctld_failure");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for backup_slurmctld_failure",
-			     trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for backup_slurmctld_failure",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_RES_OP) &&
@@ -1400,10 +1356,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("backup_slurmctld_resumed_operation");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for backup_slurmctld_resumed_"
-			     "operation", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for backup_slurmctld_resumed_operation",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_BU_CTLD_AS_CTRL) &&
@@ -1412,10 +1366,8 @@ static void _trigger_slurmctld_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("backup_slurmctld_assumed_control");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for bu_slurmctld_assumed_control",
-			     trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for bu_slurmctld_assumed_control",
+			 trig_in->trig_id);
 		return;
 	}
 }
@@ -1428,10 +1380,8 @@ static void _trigger_slurmdbd_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_slurmdbd_failure");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS)
-			info("trigger[%u] for primary_slurmcdbd_failure",
-			     trig_in->trig_id);
-		return;
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmcdbd_failure",
+			 trig_in->trig_id);
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DBD_RES_OP) &&
 	    trigger_pri_dbd_res_op) {
@@ -1439,10 +1389,8 @@ static void _trigger_slurmdbd_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_slurmdbd_resumed_operation");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_slurmdbd_resumed_"
-			     "operation", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_slurmdbd_resumed_operation",
+			 trig_in->trig_id);
 		return;
 	}
 }
@@ -1455,10 +1403,8 @@ static void _trigger_database_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_database_failure");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_database_failure",
-			     trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_database_failure",
+			 trig_in->trig_id);
 		return;
 	}
 	if ((trig_in->trig_type & TRIGGER_TYPE_PRI_DB_RES_OP) &&
@@ -1467,10 +1413,8 @@ static void _trigger_database_event(trig_mgr_info_t *trig_in, time_t now)
 		trig_in->trig_time = now + (trig_in->trig_time - 0x8000);
 		xfree(trig_in->res_id);
 		trig_in->res_id = xstrdup("primary_database_resumed_operation");
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-			info("trigger[%u] for primary_database_resumed_"
-			     "operation", trig_in->trig_id);
-		}
+		log_flag(TRIGGERS, "trigger[%u] for primary_database_resumed_operation",
+			 trig_in->trig_id);
 		return;
 	}
 }
@@ -1525,11 +1469,9 @@ static void _trigger_run_program(trig_mgr_info_t *trig_in)
 	if (child_pid > 0) {
 		trig_in->child_pid = child_pid;
 	} else if (child_pid == 0) {
-		int i;
-		bool run_as_self = (uid == slurmctld_conf.slurm_user_id);
+		bool run_as_self = (uid == slurm_conf.slurm_user_id);
 
-		for (i = 0; i < 1024; i++)
-			(void) close(i);
+		closeall(0);
 		setpgid(0, 0);
 		setsid();
 		if ((initgroups(user_name, gid) == -1) && !run_as_self) {
@@ -1609,7 +1551,6 @@ static void _trigger_clone(trig_mgr_info_t *trig_in)
 		trig_add->orig_bitmap  = bit_copy(trig_in->orig_bitmap);
 	}
 	trig_add->job_id    = trig_in->job_id;
-	trig_add->job_ptr   = trig_in->job_ptr;
 	trig_add->trig_type = trig_in->trig_type;
 	trig_add->trig_time = trig_in->orig_time;
 	trig_add->orig_time = trig_in->orig_time;
@@ -1656,13 +1597,10 @@ extern void trigger_process(void)
 		}
 		if ((trig_in->state == 1) &&
 		    (trig_in->trig_time <= now)) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_TRIGGERS) {
-				info("launching program for trigger[%u]",
-				     trig_in->trig_id);
-				info("  uid=%u gid=%u program=%s arg=%s",
-				     trig_in->user_id, trig_in->group_id,
-				     trig_in->program, trig_in->res_id);
-			}
+			log_flag(TRIGGERS, "launching program for trigger[%u]: uid=%u gid=%u program=%s arg=%s",
+				 trig_in->trig_id, trig_in->user_id,
+				 trig_in->group_id, trig_in->program,
+				 trig_in->res_id);
 			if (trig_in->flags & TRIGGER_FLAG_PERM) {
 				_trigger_clone(trig_in);
 			}
@@ -1692,11 +1630,8 @@ extern void trigger_process(void)
 			}
 
 			if (trig_in->child_pid == 0) {
-				if (slurmctld_conf.debug_flags &
-				    DEBUG_FLAG_TRIGGERS) {
-					info("purging trigger[%u]",
-					     trig_in->trig_id);
-				}
+				log_flag(TRIGGERS, "purging trigger[%u]",
+					 trig_in->trig_id);
 				list_delete_item(trig_iter);
 				state_change = true;
 			}
