@@ -93,11 +93,9 @@ struct io_operations client_ops = {
 	.handle_write = &_client_write,
 };
 
+#define CLIENT_IO_MAGIC 0x10102
 struct client_io_info {
-#ifndef NDEBUG
-#define CLIENT_IO_MAGIC  0x10102
 	int                   magic;
-#endif
 	stepd_step_rec_t    *job;		 /* pointer back to job data   */
 
 	/* incoming variables */
@@ -145,11 +143,9 @@ struct io_operations task_write_ops = {
 	.handle_error = &_task_write_error,
 };
 
+#define TASK_IN_MAGIC 0x10103
 struct task_write_info {
-#ifndef NDEBUG
-#define TASK_IN_MAGIC  0x10103
 	int              magic;
-#endif
 	stepd_step_rec_t    *job;		 /* pointer back to job data   */
 
 	List msg_queue;
@@ -168,11 +164,9 @@ struct io_operations task_read_ops = {
 	.handle_read = &_task_read,
 };
 
-struct task_read_info {
-#ifndef NDEBUG
 #define TASK_OUT_MAGIC  0x10103
+struct task_read_info {
 	int              magic;
-#endif
 	uint16_t         type;           /* type of IO object          */
 	uint16_t         gtaskid;
 	uint16_t         ltaskid;
@@ -199,7 +193,8 @@ static void *_window_manager(void *arg);
  * General declarations
  **********************************************************************/
 static void *_io_thr(void *);
-static int _send_io_init_msg(int sock, srun_key_t *key, stepd_step_rec_t *job);
+static int _send_io_init_msg(int sock, srun_info_t *srun, stepd_step_rec_t *job,
+			     bool init);
 static void _send_eof_msg(struct task_read_info *out);
 static struct io_buf *_task_build_message(struct task_read_info *out,
 					  stepd_step_rec_t *job, cbuf_t *cbuf);
@@ -518,7 +513,7 @@ _local_file_write(eio_obj_t *obj, List objs)
 	void *buf;
 	int n;
 	struct slurm_io_header header;
-	Buf header_tmp_buf;
+	buf_t *header_tmp_buf;
 
 	xassert(client->magic == CLIENT_IO_MAGIC);
 	/*
@@ -591,13 +586,10 @@ _local_file_write(eio_obj_t *obj, List objs)
 static eio_obj_t *
 _create_task_in_eio(int fd, stepd_step_rec_t *job)
 {
-	struct task_write_info *t = NULL;
+	struct task_write_info *t = xmalloc(sizeof(*t));
 	eio_obj_t *eio = NULL;
 
-	t = (struct task_write_info *)xmalloc(sizeof(struct task_write_info));
-#ifndef NDEBUG
 	t->magic = TASK_IN_MAGIC;
-#endif
 	t->job = job;
 	t->msg_queue = list_create(NULL); /* FIXME! Add destructor */
 	t->msg = NULL;
@@ -708,13 +700,10 @@ static eio_obj_t *
 _create_task_out_eio(int fd, uint16_t type,
 		     stepd_step_rec_t *job, stepd_step_task_info_t *task)
 {
-	struct task_read_info *out = NULL;
+	struct task_read_info *out = xmalloc(sizeof(*out));
 	eio_obj_t *eio = NULL;
 
-	out = (struct task_read_info *)xmalloc(sizeof(struct task_read_info));
-#ifndef NDEBUG
 	out->magic = TASK_OUT_MAGIC;
-#endif
 	out->type = type;
 	out->gtaskid = task->gtid;
 	out->ltaskid = task->id;
@@ -937,7 +926,6 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *job)
 		 * task gets an eio object for stdin.
 		 * Its not clear why that is. */
 		if (task->gtid == 0) {
-			int amaster, aslave;
 			debug("  stdin uses a pty object");
 #if HAVE_SETRESUID
 			/*
@@ -953,7 +941,8 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *job)
 			if (setresuid(geteuid(), geteuid(), 0) < 0)
 				error ("pre openpty: setresuid: %m");
 #endif
-			if (openpty(&amaster, &aslave, NULL, NULL, NULL) < 0) {
+			if (openpty(&task->to_stdin, &task->stdin_fd,
+				    NULL, NULL, NULL) < 0) {
 				error("stdin openpty: %m");
 				return SLURM_ERROR;
 			}
@@ -961,9 +950,7 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *job)
 			if (setresuid(0, getuid(), 0) < 0)
 				error ("post openpty: setresuid: %m");
 #endif
-			task->stdin_fd = aslave;
 			fd_set_close_on_exec(task->stdin_fd);
-			task->to_stdin = amaster;
 			fd_set_close_on_exec(task->to_stdin);
 			fd_set_nonblocking(task->to_stdin);
 			_spawn_window_manager(task, job);
@@ -1254,7 +1241,7 @@ static struct io_buf *
 _build_connection_okay_message(stepd_step_rec_t *job)
 {
 	struct io_buf *msg;
-	Buf packbuf;
+	buf_t *packbuf;
 	struct slurm_io_header header;
 
 	if (_outgoing_buf_free(job)) {
@@ -1277,7 +1264,7 @@ _build_connection_okay_message(stepd_step_rec_t *job)
 	msg->length = io_hdr_packed_size();
 	msg->ref_count = 0; /* make certain it is initialized */
 
-	/* free the Buf packbuf, but not the memory to which it points */
+	/* free packbuf, but not the memory to which it points */
 	packbuf->head = NULL;	/* CLANG false positive bug here */
 	free_buf(packbuf);
 
@@ -1512,10 +1499,8 @@ io_create_local_client(const char *filename, int file_flags,
 	fd_set_close_on_exec(fd);
 
 	/* Now set up the eio object */
-	client = xmalloc(sizeof(struct client_io_info));
-#ifndef NDEBUG
+	client = xmalloc(sizeof(*client));
 	client->magic = CLIENT_IO_MAGIC;
-#endif
 	client->job = job;
 	client->msg_queue = list_create(NULL); /* FIXME - destructor */
 
@@ -1558,15 +1543,12 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *job,
 
 	debug4 ("adding IO connection (logical node rank %d)", job->nodeid);
 
-	if (srun->ioaddr.sin_addr.s_addr) {
-		char         ip[256];
-		uint16_t     port;
-		slurm_get_ip_str(&srun->ioaddr, &port, ip, sizeof(ip));
-		if (ntohs(port) == 0) {
+	if (!slurm_addr_is_unspec(&srun->ioaddr)) {
+		if (slurm_get_port(&srun->ioaddr) == 0) {
 			debug3("No IO connection requested");
 			return SLURM_SUCCESS;
 		}
-		debug4("connecting IO back to %s:%d", ip, ntohs(port));
+		debug4("connecting IO back to %pA", &srun->ioaddr);
 	}
 
 	if ((sock = (int) slurm_open_stream(&srun->ioaddr, true)) < 0) {
@@ -1578,18 +1560,15 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *job,
 	}
 
 	fd_set_blocking(sock);  /* just in case... */
-
-	_send_io_init_msg(sock, srun->key, job);
+	_send_io_init_msg(sock, srun, job, true);
 
 	debug5("  back from _send_io_init_msg");
 	fd_set_nonblocking(sock);
 	fd_set_close_on_exec(sock);
 
 	/* Now set up the eio object */
-	client = xmalloc(sizeof(struct client_io_info));
-#ifndef NDEBUG
+	client = xmalloc(sizeof(*client));
 	client->magic = CLIENT_IO_MAGIC;
-#endif
 	client->job = job;
 	client->msg_queue = list_create(NULL); /* FIXME - destructor */
 
@@ -1622,11 +1601,8 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *job)
 
 	debug4 ("adding IO connection (logical node rank %d)", job->nodeid);
 
-	if (srun->ioaddr.sin_addr.s_addr) {
-		char         ip[256];
-		uint16_t     port;
-		slurm_get_ip_str(&srun->ioaddr, &port, ip, sizeof(ip));
-		debug4("connecting IO back to %s:%d", ip, ntohs(port));
+	if (!slurm_addr_is_unspec(&srun->ioaddr)) {
+		debug4("connecting IO back to %pA", &srun->ioaddr);
 	}
 
 	if ((sock = (int) slurm_open_stream(&srun->ioaddr, true)) < 0) {
@@ -1638,18 +1614,15 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *job)
 	}
 
 	fd_set_blocking(sock);  /* just in case... */
-
-	_send_io_init_msg(sock, srun->key, job);
+	_send_io_init_msg(sock, srun, job, false);
 
 	debug5("  back from _send_io_init_msg");
 	fd_set_nonblocking(sock);
 	fd_set_close_on_exec(sock);
 
 	/* Now set up the eio object */
-	client = xmalloc(sizeof(struct client_io_info));
-#ifndef NDEBUG
+	client = xmalloc(sizeof(*client));
 	client->magic = CLIENT_IO_MAGIC;
-#endif
 	client->job = job;
 	client->msg_queue = NULL; /* initialized in _client_writable */
 
@@ -1670,12 +1643,27 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *job)
 }
 
 static int
-_send_io_init_msg(int sock, srun_key_t *key, stepd_step_rec_t *job)
+_send_io_init_msg(int sock, srun_info_t *srun, stepd_step_rec_t *job, bool init)
 {
-	struct slurm_io_init_msg msg;
+	io_init_msg_t msg;
 
-	memcpy(msg.cred_signature, key->data, SLURM_IO_KEY_SIZE);
+	msg.io_key = xmalloc(srun->key->len);
+	msg.io_key_len = srun->key->len;
+	memcpy(msg.io_key, srun->key->data, srun->key->len);
 	msg.nodeid = job->nodeid;
+
+	if (srun->protocol_version >= SLURM_21_08_PROTOCOL_VERSION)
+		msg.version = SLURM_PROTOCOL_VERSION;
+	else
+		msg.version = IO_PROTOCOL_VERSION;
+
+	/*
+	 * The initial message does not need the node_offset it is needed for
+	 * sattach
+	 */
+	if (!init && (job->step_id.step_het_comp != NO_VAL))
+		msg.nodeid += job->het_job_node_offset;
+
 	if (job->stdout_eio_objs == NULL)
 		msg.stdout_objs = 0;
 	else
@@ -1687,9 +1675,11 @@ _send_io_init_msg(int sock, srun_key_t *key, stepd_step_rec_t *job)
 
 	if (io_init_msg_write_to_fd(sock, &msg) != SLURM_SUCCESS) {
 		error("Couldn't sent slurm_io_init_msg");
+		xfree(msg.io_key);
 		return SLURM_ERROR;
 	}
 
+	xfree(msg.io_key);
 
 	return SLURM_SUCCESS;
 }
@@ -1732,7 +1722,7 @@ _send_eof_msg(struct task_read_info *out)
 	eio_obj_t *eio;
 	ListIterator clients;
 	struct slurm_io_header header;
-	Buf packbuf;
+	buf_t *packbuf;
 
 	debug4("Entering _send_eof_msg");
 	out->eof_msg_sent = true;
@@ -1764,7 +1754,7 @@ _send_eof_msg(struct task_read_info *out)
 	msg->length = io_hdr_packed_size() + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 
-	/* free the Buf packbuf, but not the memory to which it points */
+	/* free packbuf, but not the memory to which it points */
 	packbuf->head = NULL;	/* CLANG false positive bug here */
 	free_buf(packbuf);
 
@@ -1793,7 +1783,7 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 {
 	struct io_buf *msg;
 	char *ptr;
-	Buf packbuf;
+	buf_t *packbuf;
 	bool must_truncate = false;
 	int avail;
 	struct slurm_io_header header;
@@ -1855,7 +1845,7 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 	msg->length = io_hdr_packed_size() + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 
-	/* free the Buf packbuf, but not the memory to which it points */
+	/* free packbuf, but not the memory to which it points */
 	packbuf->head = NULL;	/* CLANG false positive bug here */
 	free_buf(packbuf);
 
@@ -1866,9 +1856,8 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 struct io_buf *
 alloc_io_buf(void)
 {
-	struct io_buf *buf;
+	struct io_buf *buf = xmalloc(sizeof(*buf));
 
-	buf = xmalloc(sizeof(struct io_buf));
 	buf->ref_count = 0;
 	buf->length = 0;
 	/* The following "+ 1" is just temporary so I can stick a \0 at
@@ -2100,7 +2089,6 @@ io_find_filename_pattern( stepd_step_rec_t *job,
 int
 io_get_file_flags(stepd_step_rec_t *job)
 {
-	slurm_ctl_conf_t *conf;
 	int file_flags;
 
 	/* set files for opening stdout/err */
@@ -2109,7 +2097,7 @@ io_get_file_flags(stepd_step_rec_t *job)
 	else if (job->open_mode == OPEN_MODE_TRUNCATE)
 		file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
 	else {
-		conf = slurm_conf_lock();
+		slurm_conf_t *conf = slurm_conf_lock();
 		if (conf->job_file_append)
 			file_flags = O_CREAT|O_WRONLY|O_APPEND;
 		else
