@@ -54,12 +54,9 @@
 #include "src/common/job_options.h"
 #include "src/common/stepd_api.h"
 
-#ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN	64
-#endif
-
 typedef struct {
-	unsigned char data[SLURM_IO_KEY_SIZE];
+	char *data;
+	uint32_t len;
 } srun_key_t;
 
 typedef struct {
@@ -123,11 +120,11 @@ typedef struct {		/* MPMD specifications, needed for Cray */
 } mpmd_set_t;
 
 typedef struct {
+	char *container;		/* OCI Container Bundle path	*/
 	slurmstepd_state_t state;	/* Job state			*/
 	pthread_cond_t state_cond;	/* Job state conditional	*/
 	pthread_mutex_t state_mutex;	/* Job state mutex		*/
-	uint32_t       jobid;  /* Current Slurm job id                      */
-	uint32_t       stepid; /* Current step id (or NO_VAL)               */
+	slurm_step_id_t step_id; /* Current step id (or NO_VAL)               */
 	uint32_t       array_job_id;  /* job array master job ID            */
 	uint32_t       array_task_id; /* job array ID                       */
 	uint32_t       nnodes; /* number of nodes in current job            */
@@ -226,7 +223,6 @@ typedef struct {
 	jobacctinfo_t *jobacct;
 	uint8_t        open_mode;	/* stdout/err append or truncate */
 	job_options_t  options;
-	uint32_t       resv_id;		/* Cray/BASIL reservation ID	*/
 	uint16_t       restart_cnt;	/* batch job restart count	*/
 	char	      *job_alloc_cores;	/* needed by the SPANK cpuset plugin */
 	char	      *step_alloc_cores;/* needed by the SPANK cpuset plugin */
@@ -253,6 +249,8 @@ typedef struct {
 	char *x11_target;		/* remote target. unix socket if port == 0 */
 	uint16_t x11_target_port;	/* remote x11 port to connect back to */
 	char *x11_xauthority;		/* temporary XAUTHORITY location, or NULL */
+
+	char *selinux_context;
 } stepd_step_rec_t;
 
 
@@ -267,6 +265,8 @@ srun_info_t * srun_info_create(slurm_cred_t *cred, slurm_addr_t *respaddr,
 
 void  srun_info_destroy(srun_info_t *srun);
 
+void srun_key_destroy(srun_key_t *key);
+
 stepd_step_task_info_t * task_info_create(int taskid, int gtaskid,
 					  char *ifname, char *ofname,
 					  char *efname);
