@@ -95,6 +95,8 @@ int job_format_add_function(List list, int width, bool right_justify,
 	job_format_add_function(list,wid,right,suffix,_print_job_burst_buffer_state)
 #define job_format_add_cluster_name(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_cluster_name)
+#define job_format_add_container(list,wid,right,suffix) \
+	job_format_add_function(list,wid,right,suffix,_print_job_container)
 #define job_format_add_core_spec(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_core_spec)
 #define job_format_add_delay_boot(list,wid,right,suffix) \
@@ -153,6 +155,9 @@ int job_format_add_function(List list, int width, bool right_justify,
 	job_format_add_function(list,wid,right,suffix,_print_job_time_used)
 #define job_format_add_time_submit(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_time_submit)
+#define job_format_add_time_pending(list, wid, right, suffix) \
+	job_format_add_function(list, wid, right, suffix, \
+				_print_job_time_pending)
 #define job_format_add_time_start(list,wid,right,suffix) \
 	job_format_add_function(list,wid,right,suffix,_print_job_time_start)
 #define job_format_add_deadline(list,wid,right,suffix) \
@@ -353,6 +358,7 @@ int _print_job_burst_buffer_state(job_info_t * job, int width,
 				  bool right_justify, char* suffix);
 int _print_job_cluster_name(job_info_t * job, int width, bool right,
 			    char* suffix);
+int _print_job_container(job_info_t *job, int width, bool right, char *suffix);
 int _print_job_core_spec(job_info_t * job, int width, bool right_justify,
 			 char* suffix);
 int _print_job_delay_boot(job_info_t * job, int width, bool right_justify,
@@ -395,6 +401,8 @@ int _print_job_time_used(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_time_submit(job_info_t * job, int width, bool right_justify,
 			char* suffix);
+int _print_job_time_pending(job_info_t *job, int width, bool right_justify,
+			    char *suffix);
 int _print_job_time_start(job_info_t * job, int width, bool right_justify,
 			char* suffix);
 int _print_job_time_end(job_info_t * job, int width, bool right_justify,
@@ -583,6 +591,8 @@ int step_format_add_function(List list, int width, bool right_justify,
 
 #define step_format_add_cluster_name(list,wid,right,suffix) \
 	step_format_add_function(list,wid,right,suffix,_print_step_cluster_name)
+#define step_format_add_container(list,wid,right,suffix) \
+	step_format_add_function(list,wid,right,suffix,_print_step_container)
 #define step_format_add_id(list,wid,right,suffix) \
 	step_format_add_function(list,wid,right,suffix,_print_step_id)
 #define step_format_add_partition(list,wid,right,suffix) \
@@ -651,6 +661,8 @@ int step_format_add_function(List list, int width, bool right_justify,
  *****************************************************************************/
 int _print_step_cluster_name(job_step_info_t * step, int width,
 			     bool right_justify, char *suffix);
+int _print_step_container(job_step_info_t *step, int width, bool right_justify,
+			  char *suffix);
 int _print_step_id(job_step_info_t * step, int width, bool right_justify,
 		   char *suffix);
 int _print_step_partition(job_step_info_t * step, int width,
