@@ -48,6 +48,7 @@
 #include "src/common/env.h"
 #include "src/common/fd.h"
 #include "src/common/forward.h"
+#include "src/common/gres.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
@@ -100,14 +101,10 @@ static void *_safe_signal_while_allocating(void *in_data)
 
 	debug("Got signal %d", signo);
 	xfree(in_data);
-	if (signo == SIGCONT)
-		return NULL;
 
-	destroy_job = 1;
 	if (pending_job_id != 0) {
 		info("Job allocation %u has been revoked", pending_job_id);
 		slurm_complete_job(pending_job_id, NO_VAL);
-		destroy_job = 1;
 	}
 
 	return NULL;
@@ -127,6 +124,11 @@ static void _signal_while_allocating(int signo)
 	 *
 	 * SO, DON'T PRINT ANYTHING IN THIS FUNCTION.
 	 */
+	if (signo == SIGCONT)
+		return;
+
+	destroy_job = 1;
+
 	local_signal = xmalloc(sizeof(int));
 	*local_signal = signo;
 	slurm_thread_create_detached(NULL, _safe_signal_while_allocating,
@@ -145,7 +147,7 @@ static void _job_complete_handler(srun_job_complete_msg_t *msg)
 	if (msg->step_id == NO_VAL)
 		info("Force Terminated job %u", msg->job_id);
 	else
-		info("Force Terminated job %u.%u", msg->job_id, msg->step_id);
+		info("Force Terminated %ps", msg);
 }
 
 /*
@@ -172,12 +174,6 @@ static void _user_msg_handler(srun_user_msg_t *msg)
 	info("%s", msg->msg);
 }
 
-static void _ping_handler(srun_ping_msg_t *msg)
-{
-	/* the api will respond so there really isn't anything to do
-	   here */
-}
-
 static void _node_fail_handler(srun_node_fail_msg_t *msg)
 {
 	error("Node failure on %s", msg->nodelist);
@@ -238,14 +234,12 @@ static int _wait_nodes_ready(resource_allocation_response_msg_t *alloc)
 	int is_ready = 0, i, rc;
 	double cur_delay = 0;
 	double cur_sleep = 0;
-	int suspend_time, resume_time, max_delay;
+	int max_delay;
 	bool job_killed = false;
 
-	suspend_time = slurm_get_suspend_timeout();
-	resume_time  = slurm_get_resume_timeout();
-	if ((suspend_time == 0) || (resume_time == 0))
+	if (!slurm_conf.suspend_timeout || !slurm_conf.resume_timeout)
 		return 1;	/* Power save mode disabled */
-	max_delay = suspend_time + resume_time;
+	max_delay = slurm_conf.suspend_timeout + slurm_conf.resume_timeout;
 	max_delay *= 5;		/* Allow for ResumeRate support */
 
 	pending_job_id = alloc->job_id;
@@ -273,7 +267,8 @@ static int _wait_nodes_ready(resource_allocation_response_msg_t *alloc)
 			job_killed = true;
 			break;
 		}
-		if (rc & READY_NODE_STATE) {		/* job and node ready */
+		if ((rc & READY_NODE_STATE) &&
+		    (rc & READY_PROLOG_STATE)) {
 			is_ready = 1;
 			break;
 		}
@@ -384,9 +379,8 @@ extern resource_allocation_response_msg_t *
 		return NULL;
 	}
 
-	j->origin_cluster = xstrdup(slurmctld_conf.cluster_name);
+	j->origin_cluster = xstrdup(slurm_conf.cluster_name);
 
-	callbacks.ping = _ping_handler;
 	callbacks.timeout = _timeout_handler;
 	callbacks.job_complete = _job_complete_handler;
 	callbacks.job_suspend = NULL;
@@ -467,7 +461,7 @@ extern resource_allocation_response_msg_t *
 
 relinquish:
 	if (resp) {
-		if (!destroy_job)
+		if (destroy_job)
 			slurm_complete_job(resp->job_id, 1);
 		slurm_free_resource_allocation_response_msg(resp);
 	}
@@ -475,6 +469,14 @@ relinquish:
 	return NULL;
 }
 
+static int _copy_other_port(void *x, void *arg)
+{
+	job_desc_msg_t *desc = x;
+	desc->other_port = *(uint16_t *)arg;
+
+	return SLURM_SUCCESS;
+}
+
 /*
  * Allocate nodes for heterogeneous job from the slurm controller --
  * retrying the attempt if the controller appears to be down, and optionally
@@ -511,7 +513,7 @@ List allocate_het_job_nodes(bool handle_signals)
 		if (!first_job)
 			first_job = j;
 
-		j->origin_cluster = xstrdup(slurmctld_conf.cluster_name);
+		j->origin_cluster = xstrdup(slurm_conf.cluster_name);
 
 		list_append(job_req_list, j);
 	}
@@ -533,7 +535,6 @@ List allocate_het_job_nodes(bool handle_signals)
 		return NULL;
 	}
 
-	callbacks.ping = _ping_handler;
 	callbacks.timeout = _timeout_handler;
 	callbacks.job_complete = _job_complete_handler;
 	callbacks.job_suspend = NULL;
@@ -543,6 +544,7 @@ List allocate_het_job_nodes(bool handle_signals)
 	/* create message thread to handle pings and such from slurmctld */
 	msg_thr = slurm_allocation_msg_thr_create(&first_job->other_port,
 						  &callbacks);
+	list_for_each(job_req_list, _copy_other_port, &first_job->other_port);
 
 	/* NOTE: Do not process signals in separate pthread. The signal will
 	 * cause slurm_allocate_resources_blocking() to exit immediately. */
@@ -629,8 +631,15 @@ List allocate_het_job_nodes(bool handle_signals)
 
 relinquish:
 	if (job_resp_list) {
-		if (!destroy_job && my_job_id)
+		if (my_job_id == 0) {
+			resp = (resource_allocation_response_msg_t *)
+			       list_peek(job_resp_list);
+			my_job_id = resp->job_id;
+		}
+
+		if (destroy_job && my_job_id) {
 			slurm_complete_job(my_job_id, 1);
+		}
 		list_destroy(job_resp_list);
 	}
 	exit(error_exit);
@@ -713,9 +722,7 @@ int slurmctld_msg_init(void)
 		exit(error_exit);
 	}
 	fd_set_nonblocking(slurmctld_fd);
-	/* hostname is not set,  so slurm_get_addr fails
-	   slurm_get_addr(&slurm_address, &port, hostname, sizeof(hostname)); */
-	slurmctld_comm_port = ntohs(slurm_address.sin_port);
+	slurmctld_comm_port = slurm_get_port(&slurm_address);
 	debug2("srun PMI messages to port=%u", slurmctld_comm_port);
 
 	return slurmctld_fd;
@@ -728,227 +735,45 @@ int slurmctld_msg_init(void)
 static job_desc_msg_t *_job_desc_msg_create_from_opts(slurm_opt_t *opt_local)
 {
 	srun_opt_t *srun_opt = opt_local->srun_opt;
-	job_desc_msg_t *j = xmalloc(sizeof(*j));
-	hostlist_t hl = NULL;
+	job_desc_msg_t *j = slurm_opt_create_job_desc(opt_local, true);
+
+	if (!j) {
+		return NULL;
+	}
+
 	xassert(srun_opt);
 
-	slurm_init_job_desc_msg(j);
-	j->contiguous     = opt_local->contiguous;
-	if (opt_local->core_spec != NO_VAL16)
-		j->core_spec      = opt_local->core_spec;
-	j->features       = opt_local->constraint;
-	j->cluster_features = opt_local->c_constraint;
-	if (opt_local->immediate == 1)
-		j->immediate = opt_local->immediate;
-	if (opt_local->job_name)
-		j->name   = opt_local->job_name;
-	else
-		j->name = srun_opt->cmd_name;
+	if (!j->name)
+		j->name = xstrdup(srun_opt->cmd_name);
+
 	if (srun_opt->argc > 0) {
 		j->argc    = 1;
 		j->argv    = (char **) xmalloc(sizeof(char *) * 2);
 		j->argv[0] = xstrdup(srun_opt->argv[0]);
 	}
-	if (opt_local->acctg_freq)
-		j->acctg_freq     = xstrdup(opt_local->acctg_freq);
-	j->reservation    = opt_local->reservation;
-	j->wckey          = opt_local->wckey;
-	j->x11 = opt.x11;
-	if (j->x11) {
+
+	j->container = xstrdup(opt_local->container);
+
+	if (srun_opt->cpu_bind)
+		j->cpu_bind = xstrdup(srun_opt->cpu_bind);
+	if (srun_opt->cpu_bind_type)
+		j->cpu_bind_type = srun_opt->cpu_bind_type;
+
+	if (!j->x11 && opt.x11) {
 		j->x11_magic_cookie = xstrdup(opt.x11_magic_cookie);
 		j->x11_target = xstrdup(opt.x11_target);
 		j->x11_target_port = opt.x11_target_port;
 	}
 
-	j->req_nodes      = xstrdup(opt_local->nodelist);
-
-	/* simplify the job allocation nodelist,
-	 * not laying out tasks until step */
-	if (j->req_nodes) {
-		hl = hostlist_create(j->req_nodes);
-		xfree(opt_local->nodelist);
-		opt_local->nodelist = hostlist_ranged_string_xmalloc(hl);
-		hostlist_uniq(hl);
-		xfree(j->req_nodes);
-		j->req_nodes = hostlist_ranged_string_xmalloc(hl);
-		hostlist_destroy(hl);
-
-	}
-
-	if (((opt_local->distribution & SLURM_DIST_STATE_BASE) ==
-	     SLURM_DIST_ARBITRARY) && !j->req_nodes) {
-		error("With Arbitrary distribution you need to "
-		      "specify a nodelist or hostfile with the -w option");
-		return NULL;
-	}
-	j->extra = opt_local->extra;
-	j->exc_nodes      = opt_local->exclude;
-	j->partition      = opt_local->partition;
-	j->min_nodes      = opt_local->min_nodes;
-	if (opt_local->sockets_per_node != NO_VAL)
-		j->sockets_per_node    = opt_local->sockets_per_node;
-	if (opt_local->cores_per_socket != NO_VAL)
-		j->cores_per_socket      = opt_local->cores_per_socket;
-	if (opt_local->threads_per_core != NO_VAL) {
-		j->threads_per_core    = opt_local->threads_per_core;
-		/* if 1 always make sure affinity knows about it */
-		if (j->threads_per_core == 1)
-			srun_opt->cpu_bind_type |= CPU_BIND_ONE_THREAD_PER_CORE;
-	}
-	j->user_id        = opt_local->uid;
-	j->dependency     = opt_local->dependency;
-	if (opt_local->nice != NO_VAL)
-		j->nice   = NICE_OFFSET + opt_local->nice;
-	if (opt_local->priority)
-		j->priority = opt_local->priority;
-	if (srun_opt->cpu_bind)
-		j->cpu_bind = srun_opt->cpu_bind;
-	if (srun_opt->cpu_bind_type)
-		j->cpu_bind_type = srun_opt->cpu_bind_type;
-	if (opt_local->delay_boot != NO_VAL)
-		j->delay_boot = opt_local->delay_boot;
-	if (opt_local->mem_bind)
-		j->mem_bind       = opt_local->mem_bind;
-	if (opt_local->mem_bind_type)
-		j->mem_bind_type  = opt_local->mem_bind_type;
-	if (opt_local->plane_size != NO_VAL)
-		j->plane_size     = opt_local->plane_size;
-	j->task_dist      = opt_local->distribution;
-
-	j->group_id       = opt_local->gid;
-	j->mail_type      = opt_local->mail_type;
-
-	if (opt_local->ntasks_per_node != NO_VAL)
-		j->ntasks_per_node   = opt_local->ntasks_per_node;
-	if (opt_local->ntasks_per_socket != NO_VAL)
-		j->ntasks_per_socket = opt_local->ntasks_per_socket;
-	if (opt_local->ntasks_per_core != NO_VAL)
-		j->ntasks_per_core   = opt_local->ntasks_per_core;
-
-	if (opt_local->mail_user)
-		j->mail_user = opt_local->mail_user;
-	if (opt_local->burst_buffer)
-		j->burst_buffer = opt_local->burst_buffer;
-	if (opt_local->begin)
-		j->begin_time = opt_local->begin;
-	if (opt_local->deadline)
-		j->deadline = opt_local->deadline;
-	if (opt_local->licenses)
-		j->licenses = opt_local->licenses;
-	if (opt_local->network)
-		j->network = opt_local->network;
-	if (opt_local->profile)
-		j->profile = opt_local->profile;
-	if (opt_local->account)
-		j->account = opt_local->account;
-	if (opt_local->comment)
-		j->comment = opt_local->comment;
-	if (opt_local->qos)
-		j->qos = opt_local->qos;
-	if (opt_local->chdir)
-		j->work_dir = opt_local->chdir;
-
-	if (opt_local->hold)
-		j->priority     = 0;
-	if (opt_local->reboot)
-		j->reboot = 1;
-
-	if (opt_local->max_nodes)
-		j->max_nodes    = opt_local->max_nodes;
-	else if (opt_local->nodes_set) {
-		/* On an allocation if the max nodes isn't set set it
-		 * to do the same behavior as with salloc or sbatch.
-		 */
-		j->max_nodes    = opt_local->min_nodes;
-	}
-	if (opt_local->pn_min_cpus > -1)
-		j->pn_min_cpus = opt_local->pn_min_cpus;
-	if (opt_local->pn_min_memory != NO_VAL64)
-		j->pn_min_memory = opt_local->pn_min_memory;
-	else if (opt_local->mem_per_cpu != NO_VAL64)
-		j->pn_min_memory = opt_local->mem_per_cpu | MEM_PER_CPU;
-	if (opt_local->pn_min_tmp_disk != NO_VAL64)
-		j->pn_min_tmp_disk = opt_local->pn_min_tmp_disk;
-	if (opt_local->overcommit) {
-		j->min_cpus    = opt_local->min_nodes;
-		j->overcommit  = opt_local->overcommit;
-	} else if (opt_local->cpus_set)
-		j->min_cpus    = opt_local->ntasks * opt_local->cpus_per_task;
-	else
-		j->min_cpus    = opt_local->ntasks;
-	if (opt_local->ntasks_set)
-		j->num_tasks   = opt_local->ntasks;
-
-	if (opt_local->cpus_set)
-		j->cpus_per_task = opt_local->cpus_per_task;
-
-	if (opt_local->no_kill)
-		j->kill_on_node_fail   = 0;
-	if (opt_local->time_limit != NO_VAL)
-		j->time_limit          = opt_local->time_limit;
-	if (opt_local->time_min != NO_VAL)
-		j->time_min            = opt_local->time_min;
-	if (opt_local->shared != NO_VAL16)
-		j->shared = opt_local->shared;
-
-	if (opt_local->warn_flags)
-		j->warn_flags = opt_local->warn_flags;
-	if (opt_local->warn_signal)
-		j->warn_signal = opt_local->warn_signal;
-	if (opt_local->warn_time)
-		j->warn_time = opt_local->warn_time;
-	if (opt_local->job_flags)
-		j->bitflags = opt_local->job_flags;
-
-	if (opt_local->cpu_freq_min != NO_VAL)
-		j->cpu_freq_min = opt_local->cpu_freq_min;
-	if (opt_local->cpu_freq_max != NO_VAL)
-		j->cpu_freq_max = opt_local->cpu_freq_max;
-	if (opt_local->cpu_freq_gov != NO_VAL)
-		j->cpu_freq_gov = opt_local->cpu_freq_gov;
-
-	if (opt_local->req_switch >= 0)
-		j->req_switch = opt_local->req_switch;
-	if (opt_local->wait4switch >= 0)
-		j->wait4switch = opt_local->wait4switch;
-
-	/* srun uses the same listening port for the allocation response
-	 * message as all other messages */
+	/*
+	 * srun uses the same listening port for the allocation response
+	 * message as all other messages
+	 */
 	j->alloc_resp_port = slurmctld_comm_port;
 	j->other_port = slurmctld_comm_port;
 
-	if (opt_local->spank_job_env_size) {
-		j->spank_job_env      = opt_local->spank_job_env;
-		j->spank_job_env_size = opt_local->spank_job_env_size;
-	}
-
-	j->power_flags = opt_local->power;
-	if (opt_local->mcs_label)
-		j->mcs_label = opt_local->mcs_label;
 	j->wait_all_nodes = 1;
 
-	/* If can run on multiple clusters find the earliest run time
-	 * and run it there */
-	j->clusters = xstrdup(opt_local->clusters);
-
-	if (opt_local->cpus_per_gpu)
-		xstrfmtcat(j->cpus_per_tres, "gpu:%d", opt_local->cpus_per_gpu);
-	j->tres_bind = xstrdup(opt_local->tres_bind);
-	j->tres_freq = xstrdup(opt_local->tres_freq);
-	xfmt_tres(&j->tres_per_job,    "gpu", opt_local->gpus);
-	xfmt_tres(&j->tres_per_node,   "gpu", opt_local->gpus_per_node);
-	if (opt_local->gres && xstrcasecmp(opt_local->gres, "NONE")) {
-		if (j->tres_per_node)
-			xstrfmtcat(j->tres_per_node, ",%s", opt_local->gres);
-		else
-			j->tres_per_node = xstrdup(opt_local->gres);
-	}
-	xfmt_tres(&j->tres_per_socket, "gpu", opt_local->gpus_per_socket);
-	xfmt_tres(&j->tres_per_task,   "gpu", opt_local->gpus_per_task);
-	if (opt_local->mem_per_gpu != NO_VAL64)
-		xstrfmtcat(j->mem_per_tres, "gpu:%"PRIu64,
-			   opt_local->mem_per_gpu);
-
 	return j;
 }
 
